BUILTIN_ORDERBOOK_CONFIGS = {
    'bchbtc': {
          'price_currency': 'BTC',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-BTC',
          'trading_pairs': 'bitstamp_bch_btc',
          'DEFAULT_WIDTH': '0.01',
    },
    'bchusd': {
          'price_currency': 'USD',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-USD',
          'trading_pairs': 'bitstamp_bch_usd',
          'DEFAULT_WIDTH': '100',
    },
    'bcheur': {
          'price_currency': 'EUR',
          'volume_currency': 'BCH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BCH-EUR',
          'trading_pairs': 'bitstamp_bch_eur',
          'DEFAULT_WIDTH': '20',
    },
    'btccad': {
          'price_currency': 'CAD',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-CAD',
          'trading_pairs': 'kraken_btc_cad',
          'DEFAULT_WIDTH': '100',
    },
    'btceur': {
          'price_currency': 'EUR',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-EUR',
          'trading_pairs': 'bitstamp_btc_eur, kraken_btc_eur',
          'DEFAULT_WIDTH': '100',
    },
    'btcusd': {
          'price_currency': 'USD',
          'volume_currency': 'BTC',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'BTC-USD',
          'trading_pairs': 'bitstamp_btc_usd, coinbase_btc_usd, gemini_btc_usd, itbit_btc_usd, kraken_btc_usd',
          'DEFAULT_WIDTH': '50',
    },
    'ethbtc': {
          'price_currency': 'BTC',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 100,
          'zero': 0,
          'pair_name': 'ETH-BTC',
          'trading_pairs': 'gemini_eth_btc, bitstamp_eth_btc, poloniex_eth_btc',
          'DEFAULT_WIDTH': '0.02',
    },
    'etheur': {
          'price_currency': 'EUR',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'ETH-EUR',
          'trading_pairs': 'bitstamp_eth_eur',
          'DEFAULT_WIDTH': '90',
    },
    'ethusd': {
          'price_currency': 'USD',
          'volume_currency': 'ETH',
          'infinitisimal': 0.00001,
          'infinity': 10000,
          'zero': 0,
          'pair_name': 'ETH-USD',
          'trading_pairs': 'bitstamp_eth_usd, gemini_eth_usd',
          'DEFAULT_WIDTH': '10',
    },
}


