"""
This module was used for some configuration of the execution platform but is mostly
vestigial. It hasn't been removed yet because it is still referenced in the crypto
balancer module and also the 'fundamental values' library. Once those two libraries
are removed or rehabilitated this file should be removed and configurations should be
taken from .conf files.
"""

fv_v3_weights = {}

fv_v2_weights = {}

withdrawals_enabled = []

deposits_enabled = []

proportions = {}

account_value_limits = {}

forbidden_targets = {}

