"""
This package collects code for calculating various features of market data in a single
place. These `metrics` can be used in machine learning, in dashboards, or directly in
strategies. Examples are: the midpoint of an orderbook, or the strength of the bid in an
orderbook at any given depth.

Much of this code use to reside in exchange.base, but it's far outgrown it's place
there.
"""
