"""Initial Migration

Revision ID: 5169cd803822
Revises: None
Create Date: 2019-03-18 15:12:05.563812

"""

# revision identifiers, used by Alembic.
revision = '5169cd803822'
down_revision = None

from alembic import op
import sqlalchemy as sa

from gryphon.dashboards.models.columns.password_column import PasswordColumn
sa.PasswordColumn = PasswordColumn

def upgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.create_table('user',
    sa.Column('user_id', sa.Integer(), nullable=False),
    sa.Column('unique_id', sa.Unicode(length=64), nullable=False),
    sa.Column('username', sa.Unicode(length=64), nullable=False),
    sa.Column('password', sa.PasswordColumn(length=100), nullable=False),
    sa.Column('user_type', sa.Unicode(length=64), nullable=False),
    sa.Column('time_created', sa.DateTime(), nullable=False),
    sa.PrimaryKeyConstraint('user_id')
    )
    ### end Alembic commands ###


def downgrade():
    ### commands auto generated by Alembic - please adjust! ###
    op.drop_table('user')
    ### end Alembic commands ###
