from gryphon.lib.exchange.exchange_factory import ALL_EXCHANGE_KEYS as all_exchanges
from gryphon.lib.util.list import distinct

# Initial Set of Time series

# - Revenue - 1hr
# - Volatility 1hr/10m buckets on core FV
# - 0 BTC, 1BTC, 20BTC - Midprice, Bid, Ask and Spread @ 1 hr samples BY exchange
# - Volume 1hr window by exchange
# - VWAP 1hour window by exchange
#
# Naming Convention - series_type-source-data_source-interval-other_info

# All aggregate metrics represent the hour before the timestamp. For example
# 8:00 represents [7:00 - 8:00) worth of data

# All sampled metrics are sampled in the last data point before the timestamp
# For example the last orderbook before 8:00 ist timestamped with 8:00

class MetricTypeException(Exception):
    pass

metric_types_map = {
    'test': 0,
    'revenue-gryphon-trades-1hr': 1,
    'volatility-gryphon-corefv-1hr-10m_bucket': 2,
    'volume-bitstamp-trades-1hr': 135,
    'volume-bitfinex-trades-1hr': 136,
    'volume-kraken-trades-1hr': 137,
    'volume-itbit-trades-1hr': 138,
    'volume-okcoin-trades-1hr': 139,
    'volume-coinbase-trades-1hr': 140,
    'volume-quadriga-trades-1hr': 141,
    'volume-cavirtex-trades-1hr': 142,
    'volume-coinsetter-trades-1hr': 143,
    'volume-coinbase_cad-trades-1hr': 144,
    'volume-gemini-trades-1hr': 145,
    'vwap-bitstamp-trades-1hr': 146,
    'vwap-bitfinex-trades-1hr': 147,
    'vwap-kraken-trades-1hr': 148,
    'vwap-itbit-trades-1hr': 149,
    'vwap-okcoin-trades-1hr': 150,
    'vwap-coinbase-trades-1hr': 151,
    'vwap-quadriga-trades-1hr': 152,
    'vwap-cavirtex-trades-1hr': 153,
    'vwap-coinsetter-trades-1hr': 154,
    'vwap-coinbase_cad-trades-1hr': 155,
    'vwap-gemini-trades-1hr': 156,
    'midpoint-bitstamp-orderbook-6s-1btc_depth-tmp': 157,
    'midpoint-bitstamp-orderbook-6s-1btc_depth': 158,
    'midpoint-bitfinex-orderbook-6s-1btc_depth': 159,
    'midpoint-coinbase-orderbook-6s-1btc_depth': 160,
    'midpoint-kraken-orderbook-6s-1btc_depth': 161,
    'midpoint-itbit-orderbook-6s-1btc_depth': 162,
    'midpoint-quadriga-orderbook-6s-1btc_depth': 163,
    'midpoint-okcoin-orderbook-6s-1btc_depth': 164,
    'midpoint-coinbase_cad-orderbook-6s-1btc_depth': 165,
    'midpoint-gemini-orderbook-6s-1btc_depth': 166,
    'midpoint-bitstamp-orderbook-6s-20btc_depth': 167,
    'news_signal-reddit-1d': 168,
    'last_trade_price-bitstamp-trades-1s': 169,
    'last_trade_price-bitstamp-trades-60s': 170,
    'last_trade_price-bitstamp-trades-10m': 171,
    'last_trade_price-bitstamp-trades-60ms': 172,
    'volume-kraken-tradeblock-1d-eth_eur': 173,
    'volume-kraken-tradeblock-1d-eth_btc': 174,
    'volume-bitfinex-tradeblock-1d-eth_usd': 175,
    'volume-bitfinex-tradeblock-1d-eth_btc': 176,
    'volume-bitfinex-tradeblock-1d-eth_btc': 176,
    ### new, pristine, series id's begin here. ###
    # Quote based series', 10m/5m/1m, 0, 1, 5, 10, 20 depths.
    'bid-bitstamp-orderbook-10m-0.00btc_depth': 243,
    'bid-bitstamp-orderbook-10m-1.00btc_depth': 244,
    'bid-bitstamp-orderbook-10m-5.00btc_depth': 245,
    'bid-bitstamp-orderbook-10m-10.00btc_depth': 246,
    'bid-bitstamp-orderbook-10m-20.00btc_depth': 247,
    'bid-bitstamp-orderbook-5m-0.00btc_depth': 248,
    'bid-bitstamp-orderbook-5m-1.00btc_depth': 249,
    'bid-bitstamp-orderbook-5m-5.00btc_depth': 250,
    'bid-bitstamp-orderbook-5m-10.00btc_depth': 251,
    'bid-bitstamp-orderbook-5m-20.00btc_depth': 252,
    'bid-bitstamp-orderbook-1m-0.00btc_depth': 253,
    'bid-bitstamp-orderbook-1m-1.00btc_depth': 254,
    'bid-bitstamp-orderbook-1m-5.00btc_depth': 255,
    'bid-bitstamp-orderbook-1m-10.00btc_depth': 256,
    'bid-bitstamp-orderbook-1m-20.00btc_depth': 257,
    'bid-bitfinex-orderbook-10m-0.00btc_depth': 258,
    'bid-bitfinex-orderbook-10m-1.00btc_depth': 259,
    'bid-bitfinex-orderbook-10m-5.00btc_depth': 260,
    'bid-bitfinex-orderbook-10m-10.00btc_depth': 261,
    'bid-bitfinex-orderbook-10m-20.00btc_depth': 262,
    'bid-bitfinex-orderbook-5m-0.00btc_depth': 263,
    'bid-bitfinex-orderbook-5m-1.00btc_depth': 264,
    'bid-bitfinex-orderbook-5m-5.00btc_depth': 265,
    'bid-bitfinex-orderbook-5m-10.00btc_depth': 266,
    'bid-bitfinex-orderbook-5m-20.00btc_depth': 267,
    'bid-bitfinex-orderbook-1m-0.00btc_depth': 268,
    'bid-bitfinex-orderbook-1m-1.00btc_depth': 269,
    'bid-bitfinex-orderbook-1m-5.00btc_depth': 270,
    'bid-bitfinex-orderbook-1m-10.00btc_depth': 271,
    'bid-bitfinex-orderbook-1m-20.00btc_depth': 272,
    'bid-coinbase-orderbook-10m-0.00btc_depth': 273,
    'bid-coinbase-orderbook-10m-1.00btc_depth': 274,
    'bid-coinbase-orderbook-10m-5.00btc_depth': 275,
    'bid-coinbase-orderbook-10m-10.00btc_depth': 276,
    'bid-coinbase-orderbook-10m-20.00btc_depth': 277,
    'bid-coinbase-orderbook-5m-0.00btc_depth': 278,
    'bid-coinbase-orderbook-5m-1.00btc_depth': 279,
    'bid-coinbase-orderbook-5m-5.00btc_depth': 280,
    'bid-coinbase-orderbook-5m-10.00btc_depth': 281,
    'bid-coinbase-orderbook-5m-20.00btc_depth': 282,
    'bid-coinbase-orderbook-1m-0.00btc_depth': 283,
    'bid-coinbase-orderbook-1m-1.00btc_depth': 284,
    'bid-coinbase-orderbook-1m-5.00btc_depth': 285,
    'bid-coinbase-orderbook-1m-10.00btc_depth': 286,
    'bid-coinbase-orderbook-1m-20.00btc_depth': 287,
    'bid-itbit-orderbook-10m-0.00btc_depth': 288,
    'bid-itbit-orderbook-10m-1.00btc_depth': 289,
    'bid-itbit-orderbook-10m-5.00btc_depth': 290,
    'bid-itbit-orderbook-10m-10.00btc_depth': 291,
    'bid-itbit-orderbook-10m-20.00btc_depth': 292,
    'bid-itbit-orderbook-5m-0.00btc_depth': 293,
    'bid-itbit-orderbook-5m-1.00btc_depth': 294,
    'bid-itbit-orderbook-5m-5.00btc_depth': 295,
    'bid-itbit-orderbook-5m-10.00btc_depth': 296,
    'bid-itbit-orderbook-5m-20.00btc_depth': 297,
    'bid-itbit-orderbook-1m-0.00btc_depth': 298,
    'bid-itbit-orderbook-1m-1.00btc_depth': 299,
    'bid-itbit-orderbook-1m-5.00btc_depth': 300,
    'bid-itbit-orderbook-1m-10.00btc_depth': 301,
    'bid-itbit-orderbook-1m-20.00btc_depth': 302,
    'bid-kraken-orderbook-10m-0.00btc_depth': 303,
    'bid-kraken-orderbook-10m-1.00btc_depth': 304,
    'bid-kraken-orderbook-10m-5.00btc_depth': 305,
    'bid-kraken-orderbook-10m-10.00btc_depth': 306,
    'bid-kraken-orderbook-10m-20.00btc_depth': 307,
    'bid-kraken-orderbook-5m-0.00btc_depth': 308,
    'bid-kraken-orderbook-5m-1.00btc_depth': 309,
    'bid-kraken-orderbook-5m-5.00btc_depth': 310,
    'bid-kraken-orderbook-5m-10.00btc_depth': 311,
    'bid-kraken-orderbook-5m-20.00btc_depth': 312,
    'bid-kraken-orderbook-1m-0.00btc_depth': 313,
    'bid-kraken-orderbook-1m-1.00btc_depth': 314,
    'bid-kraken-orderbook-1m-5.00btc_depth': 315,
    'bid-kraken-orderbook-1m-10.00btc_depth': 316,
    'bid-kraken-orderbook-1m-20.00btc_depth': 317,
    'bid-gemini-orderbook-10m-0.00btc_depth': 318,
    'bid-gemini-orderbook-10m-1.00btc_depth': 319,
    'bid-gemini-orderbook-10m-5.00btc_depth': 320,
    'bid-gemini-orderbook-10m-10.00btc_depth': 321,
    'bid-gemini-orderbook-10m-20.00btc_depth': 322,
    'bid-gemini-orderbook-5m-0.00btc_depth': 323,
    'bid-gemini-orderbook-5m-1.00btc_depth': 324,
    'bid-gemini-orderbook-5m-5.00btc_depth': 325,
    'bid-gemini-orderbook-5m-10.00btc_depth': 326,
    'bid-gemini-orderbook-5m-20.00btc_depth': 327,
    'bid-gemini-orderbook-1m-0.00btc_depth': 328,
    'bid-gemini-orderbook-1m-1.00btc_depth': 329,
    'bid-gemini-orderbook-1m-5.00btc_depth': 330,
    'bid-gemini-orderbook-1m-10.00btc_depth': 331,
    'bid-gemini-orderbook-1m-20.00btc_depth': 332,
    'bid-okcoin-orderbook-10m-0.00btc_depth': 333,
    'bid-okcoin-orderbook-10m-1.00btc_depth': 334,
    'bid-okcoin-orderbook-10m-5.00btc_depth': 335,
    'bid-okcoin-orderbook-10m-10.00btc_depth': 336,
    'bid-okcoin-orderbook-10m-20.00btc_depth': 337,
    'bid-okcoin-orderbook-5m-0.00btc_depth': 338,
    'bid-okcoin-orderbook-5m-1.00btc_depth': 339,
    'bid-okcoin-orderbook-5m-5.00btc_depth': 340,
    'bid-okcoin-orderbook-5m-10.00btc_depth': 341,
    'bid-okcoin-orderbook-5m-20.00btc_depth': 342,
    'bid-okcoin-orderbook-1m-0.00btc_depth': 343,
    'bid-okcoin-orderbook-1m-1.00btc_depth': 344,
    'bid-okcoin-orderbook-1m-5.00btc_depth': 345,
    'bid-okcoin-orderbook-1m-10.00btc_depth': 346,
    'bid-okcoin-orderbook-1m-20.00btc_depth': 347,
    'ask-bitstamp-orderbook-10m-0.00btc_depth': 348,
    'ask-bitstamp-orderbook-10m-1.00btc_depth': 349,
    'ask-bitstamp-orderbook-10m-5.00btc_depth': 350,
    'ask-bitstamp-orderbook-10m-10.00btc_depth': 351,
    'ask-bitstamp-orderbook-10m-20.00btc_depth': 352,
    'ask-bitstamp-orderbook-5m-0.00btc_depth': 353,
    'ask-bitstamp-orderbook-5m-1.00btc_depth': 354,
    'ask-bitstamp-orderbook-5m-5.00btc_depth': 355,
    'ask-bitstamp-orderbook-5m-10.00btc_depth': 356,
    'ask-bitstamp-orderbook-5m-20.00btc_depth': 357,
    'ask-bitstamp-orderbook-1m-0.00btc_depth': 358,
    'ask-bitstamp-orderbook-1m-1.00btc_depth': 359,
    'ask-bitstamp-orderbook-1m-5.00btc_depth': 360,
    'ask-bitstamp-orderbook-1m-10.00btc_depth': 361,
    'ask-bitstamp-orderbook-1m-20.00btc_depth': 362,
    'ask-bitfinex-orderbook-10m-0.00btc_depth': 363,
    'ask-bitfinex-orderbook-10m-1.00btc_depth': 364,
    'ask-bitfinex-orderbook-10m-5.00btc_depth': 365,
    'ask-bitfinex-orderbook-10m-10.00btc_depth': 366,
    'ask-bitfinex-orderbook-10m-20.00btc_depth': 367,
    'ask-bitfinex-orderbook-5m-0.00btc_depth': 368,
    'ask-bitfinex-orderbook-5m-1.00btc_depth': 369,
    'ask-bitfinex-orderbook-5m-5.00btc_depth': 370,
    'ask-bitfinex-orderbook-5m-10.00btc_depth': 371,
    'ask-bitfinex-orderbook-5m-20.00btc_depth': 372,
    'ask-bitfinex-orderbook-1m-0.00btc_depth': 373,
    'ask-bitfinex-orderbook-1m-1.00btc_depth': 374,
    'ask-bitfinex-orderbook-1m-5.00btc_depth': 375,
    'ask-bitfinex-orderbook-1m-10.00btc_depth': 376,
    'ask-bitfinex-orderbook-1m-20.00btc_depth': 377,
    'ask-coinbase-orderbook-10m-0.00btc_depth': 378,
    'ask-coinbase-orderbook-10m-1.00btc_depth': 379,
    'ask-coinbase-orderbook-10m-5.00btc_depth': 380,
    'ask-coinbase-orderbook-10m-10.00btc_depth': 381,
    'ask-coinbase-orderbook-10m-20.00btc_depth': 382,
    'ask-coinbase-orderbook-5m-0.00btc_depth': 383,
    'ask-coinbase-orderbook-5m-1.00btc_depth': 384,
    'ask-coinbase-orderbook-5m-5.00btc_depth': 385,
    'ask-coinbase-orderbook-5m-10.00btc_depth': 386,
    'ask-coinbase-orderbook-5m-20.00btc_depth': 387,
    'ask-coinbase-orderbook-1m-0.00btc_depth': 388,
    'ask-coinbase-orderbook-1m-1.00btc_depth': 389,
    'ask-coinbase-orderbook-1m-5.00btc_depth': 390,
    'ask-coinbase-orderbook-1m-10.00btc_depth': 391,
    'ask-coinbase-orderbook-1m-20.00btc_depth': 392,
    'ask-itbit-orderbook-10m-0.00btc_depth': 393,
    'ask-itbit-orderbook-10m-1.00btc_depth': 394,
    'ask-itbit-orderbook-10m-5.00btc_depth': 395,
    'ask-itbit-orderbook-10m-10.00btc_depth': 396,
    'ask-itbit-orderbook-10m-20.00btc_depth': 397,
    'ask-itbit-orderbook-5m-0.00btc_depth': 398,
    'ask-itbit-orderbook-5m-1.00btc_depth': 399,
    'ask-itbit-orderbook-5m-5.00btc_depth': 400,
    'ask-itbit-orderbook-5m-10.00btc_depth': 401,
    'ask-itbit-orderbook-5m-20.00btc_depth': 402,
    'ask-itbit-orderbook-1m-0.00btc_depth': 403,
    'ask-itbit-orderbook-1m-1.00btc_depth': 404,
    'ask-itbit-orderbook-1m-5.00btc_depth': 405,
    'ask-itbit-orderbook-1m-10.00btc_depth': 406,
    'ask-itbit-orderbook-1m-20.00btc_depth': 407,
    'ask-kraken-orderbook-10m-0.00btc_depth': 408,
    'ask-kraken-orderbook-10m-1.00btc_depth': 409,
    'ask-kraken-orderbook-10m-5.00btc_depth': 410,
    'ask-kraken-orderbook-10m-10.00btc_depth': 411,
    'ask-kraken-orderbook-10m-20.00btc_depth': 412,
    'ask-kraken-orderbook-5m-0.00btc_depth': 413,
    'ask-kraken-orderbook-5m-1.00btc_depth': 414,
    'ask-kraken-orderbook-5m-5.00btc_depth': 415,
    'ask-kraken-orderbook-5m-10.00btc_depth': 416,
    'ask-kraken-orderbook-5m-20.00btc_depth': 417,
    'ask-kraken-orderbook-1m-0.00btc_depth': 418,
    'ask-kraken-orderbook-1m-1.00btc_depth': 419,
    'ask-kraken-orderbook-1m-5.00btc_depth': 420,
    'ask-kraken-orderbook-1m-10.00btc_depth': 421,
    'ask-kraken-orderbook-1m-20.00btc_depth': 422,
    'ask-gemini-orderbook-10m-0.00btc_depth': 423,
    'ask-gemini-orderbook-10m-1.00btc_depth': 424,
    'ask-gemini-orderbook-10m-5.00btc_depth': 425,
    'ask-gemini-orderbook-10m-10.00btc_depth': 426,
    'ask-gemini-orderbook-10m-20.00btc_depth': 427,
    'ask-gemini-orderbook-5m-0.00btc_depth': 428,
    'ask-gemini-orderbook-5m-1.00btc_depth': 429,
    'ask-gemini-orderbook-5m-5.00btc_depth': 430,
    'ask-gemini-orderbook-5m-10.00btc_depth': 431,
    'ask-gemini-orderbook-5m-20.00btc_depth': 432,
    'ask-gemini-orderbook-1m-0.00btc_depth': 433,
    'ask-gemini-orderbook-1m-1.00btc_depth': 434,
    'ask-gemini-orderbook-1m-5.00btc_depth': 435,
    'ask-gemini-orderbook-1m-10.00btc_depth': 436,
    'ask-gemini-orderbook-1m-20.00btc_depth': 437,
    'ask-okcoin-orderbook-10m-0.00btc_depth': 438,
    'ask-okcoin-orderbook-10m-1.00btc_depth': 439,
    'ask-okcoin-orderbook-10m-5.00btc_depth': 440,
    'ask-okcoin-orderbook-10m-10.00btc_depth': 441,
    'ask-okcoin-orderbook-10m-20.00btc_depth': 442,
    'ask-okcoin-orderbook-5m-0.00btc_depth': 443,
    'ask-okcoin-orderbook-5m-1.00btc_depth': 444,
    'ask-okcoin-orderbook-5m-5.00btc_depth': 445,
    'ask-okcoin-orderbook-5m-10.00btc_depth': 446,
    'ask-okcoin-orderbook-5m-20.00btc_depth': 447,
    'ask-okcoin-orderbook-1m-0.00btc_depth': 448,
    'ask-okcoin-orderbook-1m-1.00btc_depth': 449,
    'ask-okcoin-orderbook-1m-5.00btc_depth': 450,
    'ask-okcoin-orderbook-1m-10.00btc_depth': 451,
    'ask-okcoin-orderbook-1m-20.00btc_depth': 452,
    'spread-bitstamp-orderbook-10m-0.00btc_depth': 453,
    'spread-bitstamp-orderbook-10m-1.00btc_depth': 454,
    'spread-bitstamp-orderbook-10m-5.00btc_depth': 455,
    'spread-bitstamp-orderbook-10m-10.00btc_depth': 456,
    'spread-bitstamp-orderbook-10m-20.00btc_depth': 457,
    'spread-bitstamp-orderbook-5m-0.00btc_depth': 458,
    'spread-bitstamp-orderbook-5m-1.00btc_depth': 459,
    'spread-bitstamp-orderbook-5m-5.00btc_depth': 460,
    'spread-bitstamp-orderbook-5m-10.00btc_depth': 461,
    'spread-bitstamp-orderbook-5m-20.00btc_depth': 462,
    'spread-bitstamp-orderbook-1m-0.00btc_depth': 463,
    'spread-bitstamp-orderbook-1m-1.00btc_depth': 464,
    'spread-bitstamp-orderbook-1m-5.00btc_depth': 465,
    'spread-bitstamp-orderbook-1m-10.00btc_depth': 466,
    'spread-bitstamp-orderbook-1m-20.00btc_depth': 467,
    'spread-bitfinex-orderbook-10m-0.00btc_depth': 468,
    'spread-bitfinex-orderbook-10m-1.00btc_depth': 469,
    'spread-bitfinex-orderbook-10m-5.00btc_depth': 470,
    'spread-bitfinex-orderbook-10m-10.00btc_depth': 471,
    'spread-bitfinex-orderbook-10m-20.00btc_depth': 472,
    'spread-bitfinex-orderbook-5m-0.00btc_depth': 473,
    'spread-bitfinex-orderbook-5m-1.00btc_depth': 474,
    'spread-bitfinex-orderbook-5m-5.00btc_depth': 475,
    'spread-bitfinex-orderbook-5m-10.00btc_depth': 476,
    'spread-bitfinex-orderbook-5m-20.00btc_depth': 477,
    'spread-bitfinex-orderbook-1m-0.00btc_depth': 478,
    'spread-bitfinex-orderbook-1m-1.00btc_depth': 479,
    'spread-bitfinex-orderbook-1m-5.00btc_depth': 480,
    'spread-bitfinex-orderbook-1m-10.00btc_depth': 481,
    'spread-bitfinex-orderbook-1m-20.00btc_depth': 482,
    'spread-coinbase-orderbook-10m-0.00btc_depth': 483,
    'spread-coinbase-orderbook-10m-1.00btc_depth': 484,
    'spread-coinbase-orderbook-10m-5.00btc_depth': 485,
    'spread-coinbase-orderbook-10m-10.00btc_depth': 486,
    'spread-coinbase-orderbook-10m-20.00btc_depth': 487,
    'spread-coinbase-orderbook-5m-0.00btc_depth': 488,
    'spread-coinbase-orderbook-5m-1.00btc_depth': 489,
    'spread-coinbase-orderbook-5m-5.00btc_depth': 490,
    'spread-coinbase-orderbook-5m-10.00btc_depth': 491,
    'spread-coinbase-orderbook-5m-20.00btc_depth': 492,
    'spread-coinbase-orderbook-1m-0.00btc_depth': 493,
    'spread-coinbase-orderbook-1m-1.00btc_depth': 494,
    'spread-coinbase-orderbook-1m-5.00btc_depth': 495,
    'spread-coinbase-orderbook-1m-10.00btc_depth': 496,
    'spread-coinbase-orderbook-1m-20.00btc_depth': 497,
    'spread-itbit-orderbook-10m-0.00btc_depth': 498,
    'spread-itbit-orderbook-10m-1.00btc_depth': 499,
    'spread-itbit-orderbook-10m-5.00btc_depth': 500,
    'spread-itbit-orderbook-10m-10.00btc_depth': 501,
    'spread-itbit-orderbook-10m-20.00btc_depth': 502,
    'spread-itbit-orderbook-5m-0.00btc_depth': 503,
    'spread-itbit-orderbook-5m-1.00btc_depth': 504,
    'spread-itbit-orderbook-5m-5.00btc_depth': 505,
    'spread-itbit-orderbook-5m-10.00btc_depth': 506,
    'spread-itbit-orderbook-5m-20.00btc_depth': 507,
    'spread-itbit-orderbook-1m-0.00btc_depth': 508,
    'spread-itbit-orderbook-1m-1.00btc_depth': 509,
    'spread-itbit-orderbook-1m-5.00btc_depth': 510,
    'spread-itbit-orderbook-1m-10.00btc_depth': 511,
    'spread-itbit-orderbook-1m-20.00btc_depth': 512,
    'spread-kraken-orderbook-10m-0.00btc_depth': 513,
    'spread-kraken-orderbook-10m-1.00btc_depth': 514,
    'spread-kraken-orderbook-10m-5.00btc_depth': 515,
    'spread-kraken-orderbook-10m-10.00btc_depth': 516,
    'spread-kraken-orderbook-10m-20.00btc_depth': 517,
    'spread-kraken-orderbook-5m-0.00btc_depth': 518,
    'spread-kraken-orderbook-5m-1.00btc_depth': 519,
    'spread-kraken-orderbook-5m-5.00btc_depth': 520,
    'spread-kraken-orderbook-5m-10.00btc_depth': 521,
    'spread-kraken-orderbook-5m-20.00btc_depth': 522,
    'spread-kraken-orderbook-1m-0.00btc_depth': 523,
    'spread-kraken-orderbook-1m-1.00btc_depth': 524,
    'spread-kraken-orderbook-1m-5.00btc_depth': 525,
    'spread-kraken-orderbook-1m-10.00btc_depth': 526,
    'spread-kraken-orderbook-1m-20.00btc_depth': 527,
    'spread-gemini-orderbook-10m-0.00btc_depth': 528,
    'spread-gemini-orderbook-10m-1.00btc_depth': 529,
    'spread-gemini-orderbook-10m-5.00btc_depth': 530,
    'spread-gemini-orderbook-10m-10.00btc_depth': 531,
    'spread-gemini-orderbook-10m-20.00btc_depth': 532,
    'spread-gemini-orderbook-5m-0.00btc_depth': 533,
    'spread-gemini-orderbook-5m-1.00btc_depth': 534,
    'spread-gemini-orderbook-5m-5.00btc_depth': 535,
    'spread-gemini-orderbook-5m-10.00btc_depth': 536,
    'spread-gemini-orderbook-5m-20.00btc_depth': 537,
    'spread-gemini-orderbook-1m-0.00btc_depth': 538,
    'spread-gemini-orderbook-1m-1.00btc_depth': 539,
    'spread-gemini-orderbook-1m-5.00btc_depth': 540,
    'spread-gemini-orderbook-1m-10.00btc_depth': 541,
    'spread-gemini-orderbook-1m-20.00btc_depth': 542,
    'spread-okcoin-orderbook-10m-0.00btc_depth': 543,
    'spread-okcoin-orderbook-10m-1.00btc_depth': 544,
    'spread-okcoin-orderbook-10m-5.00btc_depth': 545,
    'spread-okcoin-orderbook-10m-10.00btc_depth': 546,
    'spread-okcoin-orderbook-10m-20.00btc_depth': 547,
    'spread-okcoin-orderbook-5m-0.00btc_depth': 548,
    'spread-okcoin-orderbook-5m-1.00btc_depth': 549,
    'spread-okcoin-orderbook-5m-5.00btc_depth': 550,
    'spread-okcoin-orderbook-5m-10.00btc_depth': 551,
    'spread-okcoin-orderbook-5m-20.00btc_depth': 552,
    'spread-okcoin-orderbook-1m-0.00btc_depth': 553,
    'spread-okcoin-orderbook-1m-1.00btc_depth': 554,
    'spread-okcoin-orderbook-1m-5.00btc_depth': 555,
    'spread-okcoin-orderbook-1m-10.00btc_depth': 556,
    'spread-okcoin-orderbook-1m-20.00btc_depth': 557,
    'midpoint-bitstamp-orderbook-10m-0.00btc_depth': 558,
    'midpoint-bitstamp-orderbook-10m-1.00btc_depth': 559,
    'midpoint-bitstamp-orderbook-10m-5.00btc_depth': 560,
    'midpoint-bitstamp-orderbook-10m-10.00btc_depth': 561,
    'midpoint-bitstamp-orderbook-10m-20.00btc_depth': 562,
    'midpoint-bitstamp-orderbook-5m-0.00btc_depth': 563,
    'midpoint-bitstamp-orderbook-5m-1.00btc_depth': 564,
    'midpoint-bitstamp-orderbook-5m-5.00btc_depth': 565,
    'midpoint-bitstamp-orderbook-5m-10.00btc_depth': 566,
    'midpoint-bitstamp-orderbook-5m-20.00btc_depth': 567,
    'midpoint-bitstamp-orderbook-1m-0.00btc_depth': 568,
    'midpoint-bitstamp-orderbook-1m-1.00btc_depth': 569,
    'midpoint-bitstamp-orderbook-1m-5.00btc_depth': 570,
    'midpoint-bitstamp-orderbook-1m-10.00btc_depth': 571,
    'midpoint-bitstamp-orderbook-1m-20.00btc_depth': 572,
    'midpoint-bitfinex-orderbook-10m-0.00btc_depth': 573,
    'midpoint-bitfinex-orderbook-10m-1.00btc_depth': 574,
    'midpoint-bitfinex-orderbook-10m-5.00btc_depth': 575,
    'midpoint-bitfinex-orderbook-10m-10.00btc_depth': 576,
    'midpoint-bitfinex-orderbook-10m-20.00btc_depth': 577,
    'midpoint-bitfinex-orderbook-5m-0.00btc_depth': 578,
    'midpoint-bitfinex-orderbook-5m-1.00btc_depth': 579,
    'midpoint-bitfinex-orderbook-5m-5.00btc_depth': 580,
    'midpoint-bitfinex-orderbook-5m-10.00btc_depth': 581,
    'midpoint-bitfinex-orderbook-5m-20.00btc_depth': 582,
    'midpoint-bitfinex-orderbook-1m-0.00btc_depth': 583,
    'midpoint-bitfinex-orderbook-1m-1.00btc_depth': 584,
    'midpoint-bitfinex-orderbook-1m-5.00btc_depth': 585,
    'midpoint-bitfinex-orderbook-1m-10.00btc_depth': 586,
    'midpoint-bitfinex-orderbook-1m-20.00btc_depth': 587,
    'midpoint-coinbase-orderbook-10m-0.00btc_depth': 588,
    'midpoint-coinbase-orderbook-10m-1.00btc_depth': 589,
    'midpoint-coinbase-orderbook-10m-5.00btc_depth': 590,
    'midpoint-coinbase-orderbook-10m-10.00btc_depth': 591,
    'midpoint-coinbase-orderbook-10m-20.00btc_depth': 592,
    'midpoint-coinbase-orderbook-5m-0.00btc_depth': 593,
    'midpoint-coinbase-orderbook-5m-1.00btc_depth': 594,
    'midpoint-coinbase-orderbook-5m-5.00btc_depth': 595,
    'midpoint-coinbase-orderbook-5m-10.00btc_depth': 596,
    'midpoint-coinbase-orderbook-5m-20.00btc_depth': 597,
    'midpoint-coinbase-orderbook-1m-0.00btc_depth': 598,
    'midpoint-coinbase-orderbook-1m-1.00btc_depth': 599,
    'midpoint-coinbase-orderbook-1m-5.00btc_depth': 600,
    'midpoint-coinbase-orderbook-1m-10.00btc_depth': 601,
    'midpoint-coinbase-orderbook-1m-20.00btc_depth': 602,
    'midpoint-itbit-orderbook-10m-0.00btc_depth': 603,
    'midpoint-itbit-orderbook-10m-1.00btc_depth': 604,
    'midpoint-itbit-orderbook-10m-5.00btc_depth': 605,
    'midpoint-itbit-orderbook-10m-10.00btc_depth': 606,
    'midpoint-itbit-orderbook-10m-20.00btc_depth': 607,
    'midpoint-itbit-orderbook-5m-0.00btc_depth': 608,
    'midpoint-itbit-orderbook-5m-1.00btc_depth': 609,
    'midpoint-itbit-orderbook-5m-5.00btc_depth': 610,
    'midpoint-itbit-orderbook-5m-10.00btc_depth': 611,
    'midpoint-itbit-orderbook-5m-20.00btc_depth': 612,
    'midpoint-itbit-orderbook-1m-0.00btc_depth': 613,
    'midpoint-itbit-orderbook-1m-1.00btc_depth': 614,
    'midpoint-itbit-orderbook-1m-5.00btc_depth': 615,
    'midpoint-itbit-orderbook-1m-10.00btc_depth': 616,
    'midpoint-itbit-orderbook-1m-20.00btc_depth': 617,
    'midpoint-kraken-orderbook-10m-0.00btc_depth': 618,
    'midpoint-kraken-orderbook-10m-1.00btc_depth': 619,
    'midpoint-kraken-orderbook-10m-5.00btc_depth': 620,
    'midpoint-kraken-orderbook-10m-10.00btc_depth': 621,
    'midpoint-kraken-orderbook-10m-20.00btc_depth': 622,
    'midpoint-kraken-orderbook-5m-0.00btc_depth': 623,
    'midpoint-kraken-orderbook-5m-1.00btc_depth': 624,
    'midpoint-kraken-orderbook-5m-5.00btc_depth': 625,
    'midpoint-kraken-orderbook-5m-10.00btc_depth': 626,
    'midpoint-kraken-orderbook-5m-20.00btc_depth': 627,
    'midpoint-kraken-orderbook-1m-0.00btc_depth': 628,
    'midpoint-kraken-orderbook-1m-1.00btc_depth': 629,
    'midpoint-kraken-orderbook-1m-5.00btc_depth': 630,
    'midpoint-kraken-orderbook-1m-10.00btc_depth': 631,
    'midpoint-kraken-orderbook-1m-20.00btc_depth': 632,
    'midpoint-gemini-orderbook-10m-0.00btc_depth': 633,
    'midpoint-gemini-orderbook-10m-1.00btc_depth': 634,
    'midpoint-gemini-orderbook-10m-5.00btc_depth': 635,
    'midpoint-gemini-orderbook-10m-10.00btc_depth': 636,
    'midpoint-gemini-orderbook-10m-20.00btc_depth': 637,
    'midpoint-gemini-orderbook-5m-0.00btc_depth': 638,
    'midpoint-gemini-orderbook-5m-1.00btc_depth': 639,
    'midpoint-gemini-orderbook-5m-5.00btc_depth': 640,
    'midpoint-gemini-orderbook-5m-10.00btc_depth': 641,
    'midpoint-gemini-orderbook-5m-20.00btc_depth': 642,
    'midpoint-gemini-orderbook-1m-0.00btc_depth': 643,
    'midpoint-gemini-orderbook-1m-1.00btc_depth': 644,
    'midpoint-gemini-orderbook-1m-5.00btc_depth': 645,
    'midpoint-gemini-orderbook-1m-10.00btc_depth': 646,
    'midpoint-gemini-orderbook-1m-20.00btc_depth': 647,
    'midpoint-okcoin-orderbook-10m-0.00btc_depth': 648,
    'midpoint-okcoin-orderbook-10m-1.00btc_depth': 649,
    'midpoint-okcoin-orderbook-10m-5.00btc_depth': 650,
    'midpoint-okcoin-orderbook-10m-10.00btc_depth': 651,
    'midpoint-okcoin-orderbook-10m-20.00btc_depth': 652,
    'midpoint-okcoin-orderbook-5m-0.00btc_depth': 653,
    'midpoint-okcoin-orderbook-5m-1.00btc_depth': 654,
    'midpoint-okcoin-orderbook-5m-5.00btc_depth': 655,
    'midpoint-okcoin-orderbook-5m-10.00btc_depth': 656,
    'midpoint-okcoin-orderbook-5m-20.00btc_depth': 657,
    'midpoint-okcoin-orderbook-1m-0.00btc_depth': 658,
    'midpoint-okcoin-orderbook-1m-1.00btc_depth': 659,
    'midpoint-okcoin-orderbook-1m-5.00btc_depth': 660,
    'midpoint-okcoin-orderbook-1m-10.00btc_depth': 661,
    'midpoint-okcoin-orderbook-1m-20.00btc_depth': 662,
    # Strength series' np.arange(0, 5.1, 0.1) and 10, 20 usd slippages, btc and usd.
    # 10m/5m/1m resolutions.
    'bid_strength-bitstamp-orderbook-10m-0.00usd_slippage': 663,
    'bid_strength-bitstamp-orderbook-10m-0.10usd_slippage': 664,
    'bid_strength-bitstamp-orderbook-10m-0.20usd_slippage': 665,
    'bid_strength-bitstamp-orderbook-10m-0.30usd_slippage': 666,
    'bid_strength-bitstamp-orderbook-10m-0.40usd_slippage': 667,
    'bid_strength-bitstamp-orderbook-10m-0.50usd_slippage': 668,
    'bid_strength-bitstamp-orderbook-10m-0.60usd_slippage': 669,
    'bid_strength-bitstamp-orderbook-10m-0.70usd_slippage': 670,
    'bid_strength-bitstamp-orderbook-10m-0.80usd_slippage': 671,
    'bid_strength-bitstamp-orderbook-10m-0.90usd_slippage': 672,
    'bid_strength-bitstamp-orderbook-10m-1.00usd_slippage': 673,
    'bid_strength-bitstamp-orderbook-10m-1.10usd_slippage': 674,
    'bid_strength-bitstamp-orderbook-10m-1.20usd_slippage': 675,
    'bid_strength-bitstamp-orderbook-10m-1.30usd_slippage': 676,
    'bid_strength-bitstamp-orderbook-10m-1.40usd_slippage': 677,
    'bid_strength-bitstamp-orderbook-10m-1.50usd_slippage': 678,
    'bid_strength-bitstamp-orderbook-10m-1.60usd_slippage': 679,
    'bid_strength-bitstamp-orderbook-10m-1.70usd_slippage': 680,
    'bid_strength-bitstamp-orderbook-10m-1.80usd_slippage': 681,
    'bid_strength-bitstamp-orderbook-10m-1.90usd_slippage': 682,
    'bid_strength-bitstamp-orderbook-10m-2.00usd_slippage': 683,
    'bid_strength-bitstamp-orderbook-10m-2.10usd_slippage': 684,
    'bid_strength-bitstamp-orderbook-10m-2.20usd_slippage': 685,
    'bid_strength-bitstamp-orderbook-10m-2.30usd_slippage': 686,
    'bid_strength-bitstamp-orderbook-10m-2.40usd_slippage': 687,
    'bid_strength-bitstamp-orderbook-10m-2.50usd_slippage': 688,
    'bid_strength-bitstamp-orderbook-10m-2.60usd_slippage': 689,
    'bid_strength-bitstamp-orderbook-10m-2.70usd_slippage': 690,
    'bid_strength-bitstamp-orderbook-10m-2.80usd_slippage': 691,
    'bid_strength-bitstamp-orderbook-10m-2.90usd_slippage': 692,
    'bid_strength-bitstamp-orderbook-10m-3.00usd_slippage': 693,
    'bid_strength-bitstamp-orderbook-10m-3.10usd_slippage': 694,
    'bid_strength-bitstamp-orderbook-10m-3.20usd_slippage': 695,
    'bid_strength-bitstamp-orderbook-10m-3.30usd_slippage': 696,
    'bid_strength-bitstamp-orderbook-10m-3.40usd_slippage': 697,
    'bid_strength-bitstamp-orderbook-10m-3.50usd_slippage': 698,
    'bid_strength-bitstamp-orderbook-10m-3.60usd_slippage': 699,
    'bid_strength-bitstamp-orderbook-10m-3.70usd_slippage': 700,
    'bid_strength-bitstamp-orderbook-10m-3.80usd_slippage': 701,
    'bid_strength-bitstamp-orderbook-10m-3.90usd_slippage': 702,
    'bid_strength-bitstamp-orderbook-10m-4.00usd_slippage': 703,
    'bid_strength-bitstamp-orderbook-10m-4.10usd_slippage': 704,
    'bid_strength-bitstamp-orderbook-10m-4.20usd_slippage': 705,
    'bid_strength-bitstamp-orderbook-10m-4.30usd_slippage': 706,
    'bid_strength-bitstamp-orderbook-10m-4.40usd_slippage': 707,
    'bid_strength-bitstamp-orderbook-10m-4.50usd_slippage': 708,
    'bid_strength-bitstamp-orderbook-10m-4.60usd_slippage': 709,
    'bid_strength-bitstamp-orderbook-10m-4.70usd_slippage': 710,
    'bid_strength-bitstamp-orderbook-10m-4.80usd_slippage': 711,
    'bid_strength-bitstamp-orderbook-10m-4.90usd_slippage': 712,
    'bid_strength-bitstamp-orderbook-10m-5.00usd_slippage': 713,
    'bid_strength-bitstamp-orderbook-10m-10.00usd_slippage': 714,
    'bid_strength-bitstamp-orderbook-10m-20.00usd_slippage': 715,
    'bid_strength-bitstamp-orderbook-5m-0.00usd_slippage': 716,
    'bid_strength-bitstamp-orderbook-5m-0.10usd_slippage': 717,
    'bid_strength-bitstamp-orderbook-5m-0.20usd_slippage': 718,
    'bid_strength-bitstamp-orderbook-5m-0.30usd_slippage': 719,
    'bid_strength-bitstamp-orderbook-5m-0.40usd_slippage': 720,
    'bid_strength-bitstamp-orderbook-5m-0.50usd_slippage': 721,
    'bid_strength-bitstamp-orderbook-5m-0.60usd_slippage': 722,
    'bid_strength-bitstamp-orderbook-5m-0.70usd_slippage': 723,
    'bid_strength-bitstamp-orderbook-5m-0.80usd_slippage': 724,
    'bid_strength-bitstamp-orderbook-5m-0.90usd_slippage': 725,
    'bid_strength-bitstamp-orderbook-5m-1.00usd_slippage': 726,
    'bid_strength-bitstamp-orderbook-5m-1.10usd_slippage': 727,
    'bid_strength-bitstamp-orderbook-5m-1.20usd_slippage': 728,
    'bid_strength-bitstamp-orderbook-5m-1.30usd_slippage': 729,
    'bid_strength-bitstamp-orderbook-5m-1.40usd_slippage': 730,
    'bid_strength-bitstamp-orderbook-5m-1.50usd_slippage': 731,
    'bid_strength-bitstamp-orderbook-5m-1.60usd_slippage': 732,
    'bid_strength-bitstamp-orderbook-5m-1.70usd_slippage': 733,
    'bid_strength-bitstamp-orderbook-5m-1.80usd_slippage': 734,
    'bid_strength-bitstamp-orderbook-5m-1.90usd_slippage': 735,
    'bid_strength-bitstamp-orderbook-5m-2.00usd_slippage': 736,
    'bid_strength-bitstamp-orderbook-5m-2.10usd_slippage': 737,
    'bid_strength-bitstamp-orderbook-5m-2.20usd_slippage': 738,
    'bid_strength-bitstamp-orderbook-5m-2.30usd_slippage': 739,
    'bid_strength-bitstamp-orderbook-5m-2.40usd_slippage': 740,
    'bid_strength-bitstamp-orderbook-5m-2.50usd_slippage': 741,
    'bid_strength-bitstamp-orderbook-5m-2.60usd_slippage': 742,
    'bid_strength-bitstamp-orderbook-5m-2.70usd_slippage': 743,
    'bid_strength-bitstamp-orderbook-5m-2.80usd_slippage': 744,
    'bid_strength-bitstamp-orderbook-5m-2.90usd_slippage': 745,
    'bid_strength-bitstamp-orderbook-5m-3.00usd_slippage': 746,
    'bid_strength-bitstamp-orderbook-5m-3.10usd_slippage': 747,
    'bid_strength-bitstamp-orderbook-5m-3.20usd_slippage': 748,
    'bid_strength-bitstamp-orderbook-5m-3.30usd_slippage': 749,
    'bid_strength-bitstamp-orderbook-5m-3.40usd_slippage': 750,
    'bid_strength-bitstamp-orderbook-5m-3.50usd_slippage': 751,
    'bid_strength-bitstamp-orderbook-5m-3.60usd_slippage': 752,
    'bid_strength-bitstamp-orderbook-5m-3.70usd_slippage': 753,
    'bid_strength-bitstamp-orderbook-5m-3.80usd_slippage': 754,
    'bid_strength-bitstamp-orderbook-5m-3.90usd_slippage': 755,
    'bid_strength-bitstamp-orderbook-5m-4.00usd_slippage': 756,
    'bid_strength-bitstamp-orderbook-5m-4.10usd_slippage': 757,
    'bid_strength-bitstamp-orderbook-5m-4.20usd_slippage': 758,
    'bid_strength-bitstamp-orderbook-5m-4.30usd_slippage': 759,
    'bid_strength-bitstamp-orderbook-5m-4.40usd_slippage': 760,
    'bid_strength-bitstamp-orderbook-5m-4.50usd_slippage': 761,
    'bid_strength-bitstamp-orderbook-5m-4.60usd_slippage': 762,
    'bid_strength-bitstamp-orderbook-5m-4.70usd_slippage': 763,
    'bid_strength-bitstamp-orderbook-5m-4.80usd_slippage': 764,
    'bid_strength-bitstamp-orderbook-5m-4.90usd_slippage': 765,
    'bid_strength-bitstamp-orderbook-5m-5.00usd_slippage': 766,
    'bid_strength-bitstamp-orderbook-5m-10.00usd_slippage': 767,
    'bid_strength-bitstamp-orderbook-5m-20.00usd_slippage': 768,
    'bid_strength-bitstamp-orderbook-1m-0.00usd_slippage': 769,
    'bid_strength-bitstamp-orderbook-1m-0.10usd_slippage': 770,
    'bid_strength-bitstamp-orderbook-1m-0.20usd_slippage': 771,
    'bid_strength-bitstamp-orderbook-1m-0.30usd_slippage': 772,
    'bid_strength-bitstamp-orderbook-1m-0.40usd_slippage': 773,
    'bid_strength-bitstamp-orderbook-1m-0.50usd_slippage': 774,
    'bid_strength-bitstamp-orderbook-1m-0.60usd_slippage': 775,
    'bid_strength-bitstamp-orderbook-1m-0.70usd_slippage': 776,
    'bid_strength-bitstamp-orderbook-1m-0.80usd_slippage': 777,
    'bid_strength-bitstamp-orderbook-1m-0.90usd_slippage': 778,
    'bid_strength-bitstamp-orderbook-1m-1.00usd_slippage': 779,
    'bid_strength-bitstamp-orderbook-1m-1.10usd_slippage': 780,
    'bid_strength-bitstamp-orderbook-1m-1.20usd_slippage': 781,
    'bid_strength-bitstamp-orderbook-1m-1.30usd_slippage': 782,
    'bid_strength-bitstamp-orderbook-1m-1.40usd_slippage': 783,
    'bid_strength-bitstamp-orderbook-1m-1.50usd_slippage': 784,
    'bid_strength-bitstamp-orderbook-1m-1.60usd_slippage': 785,
    'bid_strength-bitstamp-orderbook-1m-1.70usd_slippage': 786,
    'bid_strength-bitstamp-orderbook-1m-1.80usd_slippage': 787,
    'bid_strength-bitstamp-orderbook-1m-1.90usd_slippage': 788,
    'bid_strength-bitstamp-orderbook-1m-2.00usd_slippage': 789,
    'bid_strength-bitstamp-orderbook-1m-2.10usd_slippage': 790,
    'bid_strength-bitstamp-orderbook-1m-2.20usd_slippage': 791,
    'bid_strength-bitstamp-orderbook-1m-2.30usd_slippage': 792,
    'bid_strength-bitstamp-orderbook-1m-2.40usd_slippage': 793,
    'bid_strength-bitstamp-orderbook-1m-2.50usd_slippage': 794,
    'bid_strength-bitstamp-orderbook-1m-2.60usd_slippage': 795,
    'bid_strength-bitstamp-orderbook-1m-2.70usd_slippage': 796,
    'bid_strength-bitstamp-orderbook-1m-2.80usd_slippage': 797,
    'bid_strength-bitstamp-orderbook-1m-2.90usd_slippage': 798,
    'bid_strength-bitstamp-orderbook-1m-3.00usd_slippage': 799,
    'bid_strength-bitstamp-orderbook-1m-3.10usd_slippage': 800,
    'bid_strength-bitstamp-orderbook-1m-3.20usd_slippage': 801,
    'bid_strength-bitstamp-orderbook-1m-3.30usd_slippage': 802,
    'bid_strength-bitstamp-orderbook-1m-3.40usd_slippage': 803,
    'bid_strength-bitstamp-orderbook-1m-3.50usd_slippage': 804,
    'bid_strength-bitstamp-orderbook-1m-3.60usd_slippage': 805,
    'bid_strength-bitstamp-orderbook-1m-3.70usd_slippage': 806,
    'bid_strength-bitstamp-orderbook-1m-3.80usd_slippage': 807,
    'bid_strength-bitstamp-orderbook-1m-3.90usd_slippage': 808,
    'bid_strength-bitstamp-orderbook-1m-4.00usd_slippage': 809,
    'bid_strength-bitstamp-orderbook-1m-4.10usd_slippage': 810,
    'bid_strength-bitstamp-orderbook-1m-4.20usd_slippage': 811,
    'bid_strength-bitstamp-orderbook-1m-4.30usd_slippage': 812,
    'bid_strength-bitstamp-orderbook-1m-4.40usd_slippage': 813,
    'bid_strength-bitstamp-orderbook-1m-4.50usd_slippage': 814,
    'bid_strength-bitstamp-orderbook-1m-4.60usd_slippage': 815,
    'bid_strength-bitstamp-orderbook-1m-4.70usd_slippage': 816,
    'bid_strength-bitstamp-orderbook-1m-4.80usd_slippage': 817,
    'bid_strength-bitstamp-orderbook-1m-4.90usd_slippage': 818,
    'bid_strength-bitstamp-orderbook-1m-5.00usd_slippage': 819,
    'bid_strength-bitstamp-orderbook-1m-10.00usd_slippage': 820,
    'bid_strength-bitstamp-orderbook-1m-20.00usd_slippage': 821,
    'bid_strength-bitfinex-orderbook-10m-0.00usd_slippage': 822,
    'bid_strength-bitfinex-orderbook-10m-0.10usd_slippage': 823,
    'bid_strength-bitfinex-orderbook-10m-0.20usd_slippage': 824,
    'bid_strength-bitfinex-orderbook-10m-0.30usd_slippage': 825,
    'bid_strength-bitfinex-orderbook-10m-0.40usd_slippage': 826,
    'bid_strength-bitfinex-orderbook-10m-0.50usd_slippage': 827,
    'bid_strength-bitfinex-orderbook-10m-0.60usd_slippage': 828,
    'bid_strength-bitfinex-orderbook-10m-0.70usd_slippage': 829,
    'bid_strength-bitfinex-orderbook-10m-0.80usd_slippage': 830,
    'bid_strength-bitfinex-orderbook-10m-0.90usd_slippage': 831,
    'bid_strength-bitfinex-orderbook-10m-1.00usd_slippage': 832,
    'bid_strength-bitfinex-orderbook-10m-1.10usd_slippage': 833,
    'bid_strength-bitfinex-orderbook-10m-1.20usd_slippage': 834,
    'bid_strength-bitfinex-orderbook-10m-1.30usd_slippage': 835,
    'bid_strength-bitfinex-orderbook-10m-1.40usd_slippage': 836,
    'bid_strength-bitfinex-orderbook-10m-1.50usd_slippage': 837,
    'bid_strength-bitfinex-orderbook-10m-1.60usd_slippage': 838,
    'bid_strength-bitfinex-orderbook-10m-1.70usd_slippage': 839,
    'bid_strength-bitfinex-orderbook-10m-1.80usd_slippage': 840,
    'bid_strength-bitfinex-orderbook-10m-1.90usd_slippage': 841,
    'bid_strength-bitfinex-orderbook-10m-2.00usd_slippage': 842,
    'bid_strength-bitfinex-orderbook-10m-2.10usd_slippage': 843,
    'bid_strength-bitfinex-orderbook-10m-2.20usd_slippage': 844,
    'bid_strength-bitfinex-orderbook-10m-2.30usd_slippage': 845,
    'bid_strength-bitfinex-orderbook-10m-2.40usd_slippage': 846,
    'bid_strength-bitfinex-orderbook-10m-2.50usd_slippage': 847,
    'bid_strength-bitfinex-orderbook-10m-2.60usd_slippage': 848,
    'bid_strength-bitfinex-orderbook-10m-2.70usd_slippage': 849,
    'bid_strength-bitfinex-orderbook-10m-2.80usd_slippage': 850,
    'bid_strength-bitfinex-orderbook-10m-2.90usd_slippage': 851,
    'bid_strength-bitfinex-orderbook-10m-3.00usd_slippage': 852,
    'bid_strength-bitfinex-orderbook-10m-3.10usd_slippage': 853,
    'bid_strength-bitfinex-orderbook-10m-3.20usd_slippage': 854,
    'bid_strength-bitfinex-orderbook-10m-3.30usd_slippage': 855,
    'bid_strength-bitfinex-orderbook-10m-3.40usd_slippage': 856,
    'bid_strength-bitfinex-orderbook-10m-3.50usd_slippage': 857,
    'bid_strength-bitfinex-orderbook-10m-3.60usd_slippage': 858,
    'bid_strength-bitfinex-orderbook-10m-3.70usd_slippage': 859,
    'bid_strength-bitfinex-orderbook-10m-3.80usd_slippage': 860,
    'bid_strength-bitfinex-orderbook-10m-3.90usd_slippage': 861,
    'bid_strength-bitfinex-orderbook-10m-4.00usd_slippage': 862,
    'bid_strength-bitfinex-orderbook-10m-4.10usd_slippage': 863,
    'bid_strength-bitfinex-orderbook-10m-4.20usd_slippage': 864,
    'bid_strength-bitfinex-orderbook-10m-4.30usd_slippage': 865,
    'bid_strength-bitfinex-orderbook-10m-4.40usd_slippage': 866,
    'bid_strength-bitfinex-orderbook-10m-4.50usd_slippage': 867,
    'bid_strength-bitfinex-orderbook-10m-4.60usd_slippage': 868,
    'bid_strength-bitfinex-orderbook-10m-4.70usd_slippage': 869,
    'bid_strength-bitfinex-orderbook-10m-4.80usd_slippage': 870,
    'bid_strength-bitfinex-orderbook-10m-4.90usd_slippage': 871,
    'bid_strength-bitfinex-orderbook-10m-5.00usd_slippage': 872,
    'bid_strength-bitfinex-orderbook-10m-10.00usd_slippage': 873,
    'bid_strength-bitfinex-orderbook-10m-20.00usd_slippage': 874,
    'bid_strength-bitfinex-orderbook-5m-0.00usd_slippage': 875,
    'bid_strength-bitfinex-orderbook-5m-0.10usd_slippage': 876,
    'bid_strength-bitfinex-orderbook-5m-0.20usd_slippage': 877,
    'bid_strength-bitfinex-orderbook-5m-0.30usd_slippage': 878,
    'bid_strength-bitfinex-orderbook-5m-0.40usd_slippage': 879,
    'bid_strength-bitfinex-orderbook-5m-0.50usd_slippage': 880,
    'bid_strength-bitfinex-orderbook-5m-0.60usd_slippage': 881,
    'bid_strength-bitfinex-orderbook-5m-0.70usd_slippage': 882,
    'bid_strength-bitfinex-orderbook-5m-0.80usd_slippage': 883,
    'bid_strength-bitfinex-orderbook-5m-0.90usd_slippage': 884,
    'bid_strength-bitfinex-orderbook-5m-1.00usd_slippage': 885,
    'bid_strength-bitfinex-orderbook-5m-1.10usd_slippage': 886,
    'bid_strength-bitfinex-orderbook-5m-1.20usd_slippage': 887,
    'bid_strength-bitfinex-orderbook-5m-1.30usd_slippage': 888,
    'bid_strength-bitfinex-orderbook-5m-1.40usd_slippage': 889,
    'bid_strength-bitfinex-orderbook-5m-1.50usd_slippage': 890,
    'bid_strength-bitfinex-orderbook-5m-1.60usd_slippage': 891,
    'bid_strength-bitfinex-orderbook-5m-1.70usd_slippage': 892,
    'bid_strength-bitfinex-orderbook-5m-1.80usd_slippage': 893,
    'bid_strength-bitfinex-orderbook-5m-1.90usd_slippage': 894,
    'bid_strength-bitfinex-orderbook-5m-2.00usd_slippage': 895,
    'bid_strength-bitfinex-orderbook-5m-2.10usd_slippage': 896,
    'bid_strength-bitfinex-orderbook-5m-2.20usd_slippage': 897,
    'bid_strength-bitfinex-orderbook-5m-2.30usd_slippage': 898,
    'bid_strength-bitfinex-orderbook-5m-2.40usd_slippage': 899,
    'bid_strength-bitfinex-orderbook-5m-2.50usd_slippage': 900,
    'bid_strength-bitfinex-orderbook-5m-2.60usd_slippage': 901,
    'bid_strength-bitfinex-orderbook-5m-2.70usd_slippage': 902,
    'bid_strength-bitfinex-orderbook-5m-2.80usd_slippage': 903,
    'bid_strength-bitfinex-orderbook-5m-2.90usd_slippage': 904,
    'bid_strength-bitfinex-orderbook-5m-3.00usd_slippage': 905,
    'bid_strength-bitfinex-orderbook-5m-3.10usd_slippage': 906,
    'bid_strength-bitfinex-orderbook-5m-3.20usd_slippage': 907,
    'bid_strength-bitfinex-orderbook-5m-3.30usd_slippage': 908,
    'bid_strength-bitfinex-orderbook-5m-3.40usd_slippage': 909,
    'bid_strength-bitfinex-orderbook-5m-3.50usd_slippage': 910,
    'bid_strength-bitfinex-orderbook-5m-3.60usd_slippage': 911,
    'bid_strength-bitfinex-orderbook-5m-3.70usd_slippage': 912,
    'bid_strength-bitfinex-orderbook-5m-3.80usd_slippage': 913,
    'bid_strength-bitfinex-orderbook-5m-3.90usd_slippage': 914,
    'bid_strength-bitfinex-orderbook-5m-4.00usd_slippage': 915,
    'bid_strength-bitfinex-orderbook-5m-4.10usd_slippage': 916,
    'bid_strength-bitfinex-orderbook-5m-4.20usd_slippage': 917,
    'bid_strength-bitfinex-orderbook-5m-4.30usd_slippage': 918,
    'bid_strength-bitfinex-orderbook-5m-4.40usd_slippage': 919,
    'bid_strength-bitfinex-orderbook-5m-4.50usd_slippage': 920,
    'bid_strength-bitfinex-orderbook-5m-4.60usd_slippage': 921,
    'bid_strength-bitfinex-orderbook-5m-4.70usd_slippage': 922,
    'bid_strength-bitfinex-orderbook-5m-4.80usd_slippage': 923,
    'bid_strength-bitfinex-orderbook-5m-4.90usd_slippage': 924,
    'bid_strength-bitfinex-orderbook-5m-5.00usd_slippage': 925,
    'bid_strength-bitfinex-orderbook-5m-10.00usd_slippage': 926,
    'bid_strength-bitfinex-orderbook-5m-20.00usd_slippage': 927,
    'bid_strength-bitfinex-orderbook-1m-0.00usd_slippage': 928,
    'bid_strength-bitfinex-orderbook-1m-0.10usd_slippage': 929,
    'bid_strength-bitfinex-orderbook-1m-0.20usd_slippage': 930,
    'bid_strength-bitfinex-orderbook-1m-0.30usd_slippage': 931,
    'bid_strength-bitfinex-orderbook-1m-0.40usd_slippage': 932,
    'bid_strength-bitfinex-orderbook-1m-0.50usd_slippage': 933,
    'bid_strength-bitfinex-orderbook-1m-0.60usd_slippage': 934,
    'bid_strength-bitfinex-orderbook-1m-0.70usd_slippage': 935,
    'bid_strength-bitfinex-orderbook-1m-0.80usd_slippage': 936,
    'bid_strength-bitfinex-orderbook-1m-0.90usd_slippage': 937,
    'bid_strength-bitfinex-orderbook-1m-1.00usd_slippage': 938,
    'bid_strength-bitfinex-orderbook-1m-1.10usd_slippage': 939,
    'bid_strength-bitfinex-orderbook-1m-1.20usd_slippage': 940,
    'bid_strength-bitfinex-orderbook-1m-1.30usd_slippage': 941,
    'bid_strength-bitfinex-orderbook-1m-1.40usd_slippage': 942,
    'bid_strength-bitfinex-orderbook-1m-1.50usd_slippage': 943,
    'bid_strength-bitfinex-orderbook-1m-1.60usd_slippage': 944,
    'bid_strength-bitfinex-orderbook-1m-1.70usd_slippage': 945,
    'bid_strength-bitfinex-orderbook-1m-1.80usd_slippage': 946,
    'bid_strength-bitfinex-orderbook-1m-1.90usd_slippage': 947,
    'bid_strength-bitfinex-orderbook-1m-2.00usd_slippage': 948,
    'bid_strength-bitfinex-orderbook-1m-2.10usd_slippage': 949,
    'bid_strength-bitfinex-orderbook-1m-2.20usd_slippage': 950,
    'bid_strength-bitfinex-orderbook-1m-2.30usd_slippage': 951,
    'bid_strength-bitfinex-orderbook-1m-2.40usd_slippage': 952,
    'bid_strength-bitfinex-orderbook-1m-2.50usd_slippage': 953,
    'bid_strength-bitfinex-orderbook-1m-2.60usd_slippage': 954,
    'bid_strength-bitfinex-orderbook-1m-2.70usd_slippage': 955,
    'bid_strength-bitfinex-orderbook-1m-2.80usd_slippage': 956,
    'bid_strength-bitfinex-orderbook-1m-2.90usd_slippage': 957,
    'bid_strength-bitfinex-orderbook-1m-3.00usd_slippage': 958,
    'bid_strength-bitfinex-orderbook-1m-3.10usd_slippage': 959,
    'bid_strength-bitfinex-orderbook-1m-3.20usd_slippage': 960,
    'bid_strength-bitfinex-orderbook-1m-3.30usd_slippage': 961,
    'bid_strength-bitfinex-orderbook-1m-3.40usd_slippage': 962,
    'bid_strength-bitfinex-orderbook-1m-3.50usd_slippage': 963,
    'bid_strength-bitfinex-orderbook-1m-3.60usd_slippage': 964,
    'bid_strength-bitfinex-orderbook-1m-3.70usd_slippage': 965,
    'bid_strength-bitfinex-orderbook-1m-3.80usd_slippage': 966,
    'bid_strength-bitfinex-orderbook-1m-3.90usd_slippage': 967,
    'bid_strength-bitfinex-orderbook-1m-4.00usd_slippage': 968,
    'bid_strength-bitfinex-orderbook-1m-4.10usd_slippage': 969,
    'bid_strength-bitfinex-orderbook-1m-4.20usd_slippage': 970,
    'bid_strength-bitfinex-orderbook-1m-4.30usd_slippage': 971,
    'bid_strength-bitfinex-orderbook-1m-4.40usd_slippage': 972,
    'bid_strength-bitfinex-orderbook-1m-4.50usd_slippage': 973,
    'bid_strength-bitfinex-orderbook-1m-4.60usd_slippage': 974,
    'bid_strength-bitfinex-orderbook-1m-4.70usd_slippage': 975,
    'bid_strength-bitfinex-orderbook-1m-4.80usd_slippage': 976,
    'bid_strength-bitfinex-orderbook-1m-4.90usd_slippage': 977,
    'bid_strength-bitfinex-orderbook-1m-5.00usd_slippage': 978,
    'bid_strength-bitfinex-orderbook-1m-10.00usd_slippage': 979,
    'bid_strength-bitfinex-orderbook-1m-20.00usd_slippage': 980,
    'bid_strength-coinbase-orderbook-10m-0.00usd_slippage': 981,
    'bid_strength-coinbase-orderbook-10m-0.10usd_slippage': 982,
    'bid_strength-coinbase-orderbook-10m-0.20usd_slippage': 983,
    'bid_strength-coinbase-orderbook-10m-0.30usd_slippage': 984,
    'bid_strength-coinbase-orderbook-10m-0.40usd_slippage': 985,
    'bid_strength-coinbase-orderbook-10m-0.50usd_slippage': 986,
    'bid_strength-coinbase-orderbook-10m-0.60usd_slippage': 987,
    'bid_strength-coinbase-orderbook-10m-0.70usd_slippage': 988,
    'bid_strength-coinbase-orderbook-10m-0.80usd_slippage': 989,
    'bid_strength-coinbase-orderbook-10m-0.90usd_slippage': 990,
    'bid_strength-coinbase-orderbook-10m-1.00usd_slippage': 991,
    'bid_strength-coinbase-orderbook-10m-1.10usd_slippage': 992,
    'bid_strength-coinbase-orderbook-10m-1.20usd_slippage': 993,
    'bid_strength-coinbase-orderbook-10m-1.30usd_slippage': 994,
    'bid_strength-coinbase-orderbook-10m-1.40usd_slippage': 995,
    'bid_strength-coinbase-orderbook-10m-1.50usd_slippage': 996,
    'bid_strength-coinbase-orderbook-10m-1.60usd_slippage': 997,
    'bid_strength-coinbase-orderbook-10m-1.70usd_slippage': 998,
    'bid_strength-coinbase-orderbook-10m-1.80usd_slippage': 999,
    'bid_strength-coinbase-orderbook-10m-1.90usd_slippage': 1000,
    'bid_strength-coinbase-orderbook-10m-2.00usd_slippage': 1001,
    'bid_strength-coinbase-orderbook-10m-2.10usd_slippage': 1002,
    'bid_strength-coinbase-orderbook-10m-2.20usd_slippage': 1003,
    'bid_strength-coinbase-orderbook-10m-2.30usd_slippage': 1004,
    'bid_strength-coinbase-orderbook-10m-2.40usd_slippage': 1005,
    'bid_strength-coinbase-orderbook-10m-2.50usd_slippage': 1006,
    'bid_strength-coinbase-orderbook-10m-2.60usd_slippage': 1007,
    'bid_strength-coinbase-orderbook-10m-2.70usd_slippage': 1008,
    'bid_strength-coinbase-orderbook-10m-2.80usd_slippage': 1009,
    'bid_strength-coinbase-orderbook-10m-2.90usd_slippage': 1010,
    'bid_strength-coinbase-orderbook-10m-3.00usd_slippage': 1011,
    'bid_strength-coinbase-orderbook-10m-3.10usd_slippage': 1012,
    'bid_strength-coinbase-orderbook-10m-3.20usd_slippage': 1013,
    'bid_strength-coinbase-orderbook-10m-3.30usd_slippage': 1014,
    'bid_strength-coinbase-orderbook-10m-3.40usd_slippage': 1015,
    'bid_strength-coinbase-orderbook-10m-3.50usd_slippage': 1016,
    'bid_strength-coinbase-orderbook-10m-3.60usd_slippage': 1017,
    'bid_strength-coinbase-orderbook-10m-3.70usd_slippage': 1018,
    'bid_strength-coinbase-orderbook-10m-3.80usd_slippage': 1019,
    'bid_strength-coinbase-orderbook-10m-3.90usd_slippage': 1020,
    'bid_strength-coinbase-orderbook-10m-4.00usd_slippage': 1021,
    'bid_strength-coinbase-orderbook-10m-4.10usd_slippage': 1022,
    'bid_strength-coinbase-orderbook-10m-4.20usd_slippage': 1023,
    'bid_strength-coinbase-orderbook-10m-4.30usd_slippage': 1024,
    'bid_strength-coinbase-orderbook-10m-4.40usd_slippage': 1025,
    'bid_strength-coinbase-orderbook-10m-4.50usd_slippage': 1026,
    'bid_strength-coinbase-orderbook-10m-4.60usd_slippage': 1027,
    'bid_strength-coinbase-orderbook-10m-4.70usd_slippage': 1028,
    'bid_strength-coinbase-orderbook-10m-4.80usd_slippage': 1029,
    'bid_strength-coinbase-orderbook-10m-4.90usd_slippage': 1030,
    'bid_strength-coinbase-orderbook-10m-5.00usd_slippage': 1031,
    'bid_strength-coinbase-orderbook-10m-10.00usd_slippage': 1032,
    'bid_strength-coinbase-orderbook-10m-20.00usd_slippage': 1033,
    'bid_strength-coinbase-orderbook-5m-0.00usd_slippage': 1034,
    'bid_strength-coinbase-orderbook-5m-0.10usd_slippage': 1035,
    'bid_strength-coinbase-orderbook-5m-0.20usd_slippage': 1036,
    'bid_strength-coinbase-orderbook-5m-0.30usd_slippage': 1037,
    'bid_strength-coinbase-orderbook-5m-0.40usd_slippage': 1038,
    'bid_strength-coinbase-orderbook-5m-0.50usd_slippage': 1039,
    'bid_strength-coinbase-orderbook-5m-0.60usd_slippage': 1040,
    'bid_strength-coinbase-orderbook-5m-0.70usd_slippage': 1041,
    'bid_strength-coinbase-orderbook-5m-0.80usd_slippage': 1042,
    'bid_strength-coinbase-orderbook-5m-0.90usd_slippage': 1043,
    'bid_strength-coinbase-orderbook-5m-1.00usd_slippage': 1044,
    'bid_strength-coinbase-orderbook-5m-1.10usd_slippage': 1045,
    'bid_strength-coinbase-orderbook-5m-1.20usd_slippage': 1046,
    'bid_strength-coinbase-orderbook-5m-1.30usd_slippage': 1047,
    'bid_strength-coinbase-orderbook-5m-1.40usd_slippage': 1048,
    'bid_strength-coinbase-orderbook-5m-1.50usd_slippage': 1049,
    'bid_strength-coinbase-orderbook-5m-1.60usd_slippage': 1050,
    'bid_strength-coinbase-orderbook-5m-1.70usd_slippage': 1051,
    'bid_strength-coinbase-orderbook-5m-1.80usd_slippage': 1052,
    'bid_strength-coinbase-orderbook-5m-1.90usd_slippage': 1053,
    'bid_strength-coinbase-orderbook-5m-2.00usd_slippage': 1054,
    'bid_strength-coinbase-orderbook-5m-2.10usd_slippage': 1055,
    'bid_strength-coinbase-orderbook-5m-2.20usd_slippage': 1056,
    'bid_strength-coinbase-orderbook-5m-2.30usd_slippage': 1057,
    'bid_strength-coinbase-orderbook-5m-2.40usd_slippage': 1058,
    'bid_strength-coinbase-orderbook-5m-2.50usd_slippage': 1059,
    'bid_strength-coinbase-orderbook-5m-2.60usd_slippage': 1060,
    'bid_strength-coinbase-orderbook-5m-2.70usd_slippage': 1061,
    'bid_strength-coinbase-orderbook-5m-2.80usd_slippage': 1062,
    'bid_strength-coinbase-orderbook-5m-2.90usd_slippage': 1063,
    'bid_strength-coinbase-orderbook-5m-3.00usd_slippage': 1064,
    'bid_strength-coinbase-orderbook-5m-3.10usd_slippage': 1065,
    'bid_strength-coinbase-orderbook-5m-3.20usd_slippage': 1066,
    'bid_strength-coinbase-orderbook-5m-3.30usd_slippage': 1067,
    'bid_strength-coinbase-orderbook-5m-3.40usd_slippage': 1068,
    'bid_strength-coinbase-orderbook-5m-3.50usd_slippage': 1069,
    'bid_strength-coinbase-orderbook-5m-3.60usd_slippage': 1070,
    'bid_strength-coinbase-orderbook-5m-3.70usd_slippage': 1071,
    'bid_strength-coinbase-orderbook-5m-3.80usd_slippage': 1072,
    'bid_strength-coinbase-orderbook-5m-3.90usd_slippage': 1073,
    'bid_strength-coinbase-orderbook-5m-4.00usd_slippage': 1074,
    'bid_strength-coinbase-orderbook-5m-4.10usd_slippage': 1075,
    'bid_strength-coinbase-orderbook-5m-4.20usd_slippage': 1076,
    'bid_strength-coinbase-orderbook-5m-4.30usd_slippage': 1077,
    'bid_strength-coinbase-orderbook-5m-4.40usd_slippage': 1078,
    'bid_strength-coinbase-orderbook-5m-4.50usd_slippage': 1079,
    'bid_strength-coinbase-orderbook-5m-4.60usd_slippage': 1080,
    'bid_strength-coinbase-orderbook-5m-4.70usd_slippage': 1081,
    'bid_strength-coinbase-orderbook-5m-4.80usd_slippage': 1082,
    'bid_strength-coinbase-orderbook-5m-4.90usd_slippage': 1083,
    'bid_strength-coinbase-orderbook-5m-5.00usd_slippage': 1084,
    'bid_strength-coinbase-orderbook-5m-10.00usd_slippage': 1085,
    'bid_strength-coinbase-orderbook-5m-20.00usd_slippage': 1086,
    'bid_strength-coinbase-orderbook-1m-0.00usd_slippage': 1087,
    'bid_strength-coinbase-orderbook-1m-0.10usd_slippage': 1088,
    'bid_strength-coinbase-orderbook-1m-0.20usd_slippage': 1089,
    'bid_strength-coinbase-orderbook-1m-0.30usd_slippage': 1090,
    'bid_strength-coinbase-orderbook-1m-0.40usd_slippage': 1091,
    'bid_strength-coinbase-orderbook-1m-0.50usd_slippage': 1092,
    'bid_strength-coinbase-orderbook-1m-0.60usd_slippage': 1093,
    'bid_strength-coinbase-orderbook-1m-0.70usd_slippage': 1094,
    'bid_strength-coinbase-orderbook-1m-0.80usd_slippage': 1095,
    'bid_strength-coinbase-orderbook-1m-0.90usd_slippage': 1096,
    'bid_strength-coinbase-orderbook-1m-1.00usd_slippage': 1097,
    'bid_strength-coinbase-orderbook-1m-1.10usd_slippage': 1098,
    'bid_strength-coinbase-orderbook-1m-1.20usd_slippage': 1099,
    'bid_strength-coinbase-orderbook-1m-1.30usd_slippage': 1100,
    'bid_strength-coinbase-orderbook-1m-1.40usd_slippage': 1101,
    'bid_strength-coinbase-orderbook-1m-1.50usd_slippage': 1102,
    'bid_strength-coinbase-orderbook-1m-1.60usd_slippage': 1103,
    'bid_strength-coinbase-orderbook-1m-1.70usd_slippage': 1104,
    'bid_strength-coinbase-orderbook-1m-1.80usd_slippage': 1105,
    'bid_strength-coinbase-orderbook-1m-1.90usd_slippage': 1106,
    'bid_strength-coinbase-orderbook-1m-2.00usd_slippage': 1107,
    'bid_strength-coinbase-orderbook-1m-2.10usd_slippage': 1108,
    'bid_strength-coinbase-orderbook-1m-2.20usd_slippage': 1109,
    'bid_strength-coinbase-orderbook-1m-2.30usd_slippage': 1110,
    'bid_strength-coinbase-orderbook-1m-2.40usd_slippage': 1111,
    'bid_strength-coinbase-orderbook-1m-2.50usd_slippage': 1112,
    'bid_strength-coinbase-orderbook-1m-2.60usd_slippage': 1113,
    'bid_strength-coinbase-orderbook-1m-2.70usd_slippage': 1114,
    'bid_strength-coinbase-orderbook-1m-2.80usd_slippage': 1115,
    'bid_strength-coinbase-orderbook-1m-2.90usd_slippage': 1116,
    'bid_strength-coinbase-orderbook-1m-3.00usd_slippage': 1117,
    'bid_strength-coinbase-orderbook-1m-3.10usd_slippage': 1118,
    'bid_strength-coinbase-orderbook-1m-3.20usd_slippage': 1119,
    'bid_strength-coinbase-orderbook-1m-3.30usd_slippage': 1120,
    'bid_strength-coinbase-orderbook-1m-3.40usd_slippage': 1121,
    'bid_strength-coinbase-orderbook-1m-3.50usd_slippage': 1122,
    'bid_strength-coinbase-orderbook-1m-3.60usd_slippage': 1123,
    'bid_strength-coinbase-orderbook-1m-3.70usd_slippage': 1124,
    'bid_strength-coinbase-orderbook-1m-3.80usd_slippage': 1125,
    'bid_strength-coinbase-orderbook-1m-3.90usd_slippage': 1126,
    'bid_strength-coinbase-orderbook-1m-4.00usd_slippage': 1127,
    'bid_strength-coinbase-orderbook-1m-4.10usd_slippage': 1128,
    'bid_strength-coinbase-orderbook-1m-4.20usd_slippage': 1129,
    'bid_strength-coinbase-orderbook-1m-4.30usd_slippage': 1130,
    'bid_strength-coinbase-orderbook-1m-4.40usd_slippage': 1131,
    'bid_strength-coinbase-orderbook-1m-4.50usd_slippage': 1132,
    'bid_strength-coinbase-orderbook-1m-4.60usd_slippage': 1133,
    'bid_strength-coinbase-orderbook-1m-4.70usd_slippage': 1134,
    'bid_strength-coinbase-orderbook-1m-4.80usd_slippage': 1135,
    'bid_strength-coinbase-orderbook-1m-4.90usd_slippage': 1136,
    'bid_strength-coinbase-orderbook-1m-5.00usd_slippage': 1137,
    'bid_strength-coinbase-orderbook-1m-10.00usd_slippage': 1138,
    'bid_strength-coinbase-orderbook-1m-20.00usd_slippage': 1139,
    'bid_strength-itbit-orderbook-10m-0.00usd_slippage': 1140,
    'bid_strength-itbit-orderbook-10m-0.10usd_slippage': 1141,
    'bid_strength-itbit-orderbook-10m-0.20usd_slippage': 1142,
    'bid_strength-itbit-orderbook-10m-0.30usd_slippage': 1143,
    'bid_strength-itbit-orderbook-10m-0.40usd_slippage': 1144,
    'bid_strength-itbit-orderbook-10m-0.50usd_slippage': 1145,
    'bid_strength-itbit-orderbook-10m-0.60usd_slippage': 1146,
    'bid_strength-itbit-orderbook-10m-0.70usd_slippage': 1147,
    'bid_strength-itbit-orderbook-10m-0.80usd_slippage': 1148,
    'bid_strength-itbit-orderbook-10m-0.90usd_slippage': 1149,
    'bid_strength-itbit-orderbook-10m-1.00usd_slippage': 1150,
    'bid_strength-itbit-orderbook-10m-1.10usd_slippage': 1151,
    'bid_strength-itbit-orderbook-10m-1.20usd_slippage': 1152,
    'bid_strength-itbit-orderbook-10m-1.30usd_slippage': 1153,
    'bid_strength-itbit-orderbook-10m-1.40usd_slippage': 1154,
    'bid_strength-itbit-orderbook-10m-1.50usd_slippage': 1155,
    'bid_strength-itbit-orderbook-10m-1.60usd_slippage': 1156,
    'bid_strength-itbit-orderbook-10m-1.70usd_slippage': 1157,
    'bid_strength-itbit-orderbook-10m-1.80usd_slippage': 1158,
    'bid_strength-itbit-orderbook-10m-1.90usd_slippage': 1159,
    'bid_strength-itbit-orderbook-10m-2.00usd_slippage': 1160,
    'bid_strength-itbit-orderbook-10m-2.10usd_slippage': 1161,
    'bid_strength-itbit-orderbook-10m-2.20usd_slippage': 1162,
    'bid_strength-itbit-orderbook-10m-2.30usd_slippage': 1163,
    'bid_strength-itbit-orderbook-10m-2.40usd_slippage': 1164,
    'bid_strength-itbit-orderbook-10m-2.50usd_slippage': 1165,
    'bid_strength-itbit-orderbook-10m-2.60usd_slippage': 1166,
    'bid_strength-itbit-orderbook-10m-2.70usd_slippage': 1167,
    'bid_strength-itbit-orderbook-10m-2.80usd_slippage': 1168,
    'bid_strength-itbit-orderbook-10m-2.90usd_slippage': 1169,
    'bid_strength-itbit-orderbook-10m-3.00usd_slippage': 1170,
    'bid_strength-itbit-orderbook-10m-3.10usd_slippage': 1171,
    'bid_strength-itbit-orderbook-10m-3.20usd_slippage': 1172,
    'bid_strength-itbit-orderbook-10m-3.30usd_slippage': 1173,
    'bid_strength-itbit-orderbook-10m-3.40usd_slippage': 1174,
    'bid_strength-itbit-orderbook-10m-3.50usd_slippage': 1175,
    'bid_strength-itbit-orderbook-10m-3.60usd_slippage': 1176,
    'bid_strength-itbit-orderbook-10m-3.70usd_slippage': 1177,
    'bid_strength-itbit-orderbook-10m-3.80usd_slippage': 1178,
    'bid_strength-itbit-orderbook-10m-3.90usd_slippage': 1179,
    'bid_strength-itbit-orderbook-10m-4.00usd_slippage': 1180,
    'bid_strength-itbit-orderbook-10m-4.10usd_slippage': 1181,
    'bid_strength-itbit-orderbook-10m-4.20usd_slippage': 1182,
    'bid_strength-itbit-orderbook-10m-4.30usd_slippage': 1183,
    'bid_strength-itbit-orderbook-10m-4.40usd_slippage': 1184,
    'bid_strength-itbit-orderbook-10m-4.50usd_slippage': 1185,
    'bid_strength-itbit-orderbook-10m-4.60usd_slippage': 1186,
    'bid_strength-itbit-orderbook-10m-4.70usd_slippage': 1187,
    'bid_strength-itbit-orderbook-10m-4.80usd_slippage': 1188,
    'bid_strength-itbit-orderbook-10m-4.90usd_slippage': 1189,
    'bid_strength-itbit-orderbook-10m-5.00usd_slippage': 1190,
    'bid_strength-itbit-orderbook-10m-10.00usd_slippage': 1191,
    'bid_strength-itbit-orderbook-10m-20.00usd_slippage': 1192,
    'bid_strength-itbit-orderbook-5m-0.00usd_slippage': 1193,
    'bid_strength-itbit-orderbook-5m-0.10usd_slippage': 1194,
    'bid_strength-itbit-orderbook-5m-0.20usd_slippage': 1195,
    'bid_strength-itbit-orderbook-5m-0.30usd_slippage': 1196,
    'bid_strength-itbit-orderbook-5m-0.40usd_slippage': 1197,
    'bid_strength-itbit-orderbook-5m-0.50usd_slippage': 1198,
    'bid_strength-itbit-orderbook-5m-0.60usd_slippage': 1199,
    'bid_strength-itbit-orderbook-5m-0.70usd_slippage': 1200,
    'bid_strength-itbit-orderbook-5m-0.80usd_slippage': 1201,
    'bid_strength-itbit-orderbook-5m-0.90usd_slippage': 1202,
    'bid_strength-itbit-orderbook-5m-1.00usd_slippage': 1203,
    'bid_strength-itbit-orderbook-5m-1.10usd_slippage': 1204,
    'bid_strength-itbit-orderbook-5m-1.20usd_slippage': 1205,
    'bid_strength-itbit-orderbook-5m-1.30usd_slippage': 1206,
    'bid_strength-itbit-orderbook-5m-1.40usd_slippage': 1207,
    'bid_strength-itbit-orderbook-5m-1.50usd_slippage': 1208,
    'bid_strength-itbit-orderbook-5m-1.60usd_slippage': 1209,
    'bid_strength-itbit-orderbook-5m-1.70usd_slippage': 1210,
    'bid_strength-itbit-orderbook-5m-1.80usd_slippage': 1211,
    'bid_strength-itbit-orderbook-5m-1.90usd_slippage': 1212,
    'bid_strength-itbit-orderbook-5m-2.00usd_slippage': 1213,
    'bid_strength-itbit-orderbook-5m-2.10usd_slippage': 1214,
    'bid_strength-itbit-orderbook-5m-2.20usd_slippage': 1215,
    'bid_strength-itbit-orderbook-5m-2.30usd_slippage': 1216,
    'bid_strength-itbit-orderbook-5m-2.40usd_slippage': 1217,
    'bid_strength-itbit-orderbook-5m-2.50usd_slippage': 1218,
    'bid_strength-itbit-orderbook-5m-2.60usd_slippage': 1219,
    'bid_strength-itbit-orderbook-5m-2.70usd_slippage': 1220,
    'bid_strength-itbit-orderbook-5m-2.80usd_slippage': 1221,
    'bid_strength-itbit-orderbook-5m-2.90usd_slippage': 1222,
    'bid_strength-itbit-orderbook-5m-3.00usd_slippage': 1223,
    'bid_strength-itbit-orderbook-5m-3.10usd_slippage': 1224,
    'bid_strength-itbit-orderbook-5m-3.20usd_slippage': 1225,
    'bid_strength-itbit-orderbook-5m-3.30usd_slippage': 1226,
    'bid_strength-itbit-orderbook-5m-3.40usd_slippage': 1227,
    'bid_strength-itbit-orderbook-5m-3.50usd_slippage': 1228,
    'bid_strength-itbit-orderbook-5m-3.60usd_slippage': 1229,
    'bid_strength-itbit-orderbook-5m-3.70usd_slippage': 1230,
    'bid_strength-itbit-orderbook-5m-3.80usd_slippage': 1231,
    'bid_strength-itbit-orderbook-5m-3.90usd_slippage': 1232,
    'bid_strength-itbit-orderbook-5m-4.00usd_slippage': 1233,
    'bid_strength-itbit-orderbook-5m-4.10usd_slippage': 1234,
    'bid_strength-itbit-orderbook-5m-4.20usd_slippage': 1235,
    'bid_strength-itbit-orderbook-5m-4.30usd_slippage': 1236,
    'bid_strength-itbit-orderbook-5m-4.40usd_slippage': 1237,
    'bid_strength-itbit-orderbook-5m-4.50usd_slippage': 1238,
    'bid_strength-itbit-orderbook-5m-4.60usd_slippage': 1239,
    'bid_strength-itbit-orderbook-5m-4.70usd_slippage': 1240,
    'bid_strength-itbit-orderbook-5m-4.80usd_slippage': 1241,
    'bid_strength-itbit-orderbook-5m-4.90usd_slippage': 1242,
    'bid_strength-itbit-orderbook-5m-5.00usd_slippage': 1243,
    'bid_strength-itbit-orderbook-5m-10.00usd_slippage': 1244,
    'bid_strength-itbit-orderbook-5m-20.00usd_slippage': 1245,
    'bid_strength-itbit-orderbook-1m-0.00usd_slippage': 1246,
    'bid_strength-itbit-orderbook-1m-0.10usd_slippage': 1247,
    'bid_strength-itbit-orderbook-1m-0.20usd_slippage': 1248,
    'bid_strength-itbit-orderbook-1m-0.30usd_slippage': 1249,
    'bid_strength-itbit-orderbook-1m-0.40usd_slippage': 1250,
    'bid_strength-itbit-orderbook-1m-0.50usd_slippage': 1251,
    'bid_strength-itbit-orderbook-1m-0.60usd_slippage': 1252,
    'bid_strength-itbit-orderbook-1m-0.70usd_slippage': 1253,
    'bid_strength-itbit-orderbook-1m-0.80usd_slippage': 1254,
    'bid_strength-itbit-orderbook-1m-0.90usd_slippage': 1255,
    'bid_strength-itbit-orderbook-1m-1.00usd_slippage': 1256,
    'bid_strength-itbit-orderbook-1m-1.10usd_slippage': 1257,
    'bid_strength-itbit-orderbook-1m-1.20usd_slippage': 1258,
    'bid_strength-itbit-orderbook-1m-1.30usd_slippage': 1259,
    'bid_strength-itbit-orderbook-1m-1.40usd_slippage': 1260,
    'bid_strength-itbit-orderbook-1m-1.50usd_slippage': 1261,
    'bid_strength-itbit-orderbook-1m-1.60usd_slippage': 1262,
    'bid_strength-itbit-orderbook-1m-1.70usd_slippage': 1263,
    'bid_strength-itbit-orderbook-1m-1.80usd_slippage': 1264,
    'bid_strength-itbit-orderbook-1m-1.90usd_slippage': 1265,
    'bid_strength-itbit-orderbook-1m-2.00usd_slippage': 1266,
    'bid_strength-itbit-orderbook-1m-2.10usd_slippage': 1267,
    'bid_strength-itbit-orderbook-1m-2.20usd_slippage': 1268,
    'bid_strength-itbit-orderbook-1m-2.30usd_slippage': 1269,
    'bid_strength-itbit-orderbook-1m-2.40usd_slippage': 1270,
    'bid_strength-itbit-orderbook-1m-2.50usd_slippage': 1271,
    'bid_strength-itbit-orderbook-1m-2.60usd_slippage': 1272,
    'bid_strength-itbit-orderbook-1m-2.70usd_slippage': 1273,
    'bid_strength-itbit-orderbook-1m-2.80usd_slippage': 1274,
    'bid_strength-itbit-orderbook-1m-2.90usd_slippage': 1275,
    'bid_strength-itbit-orderbook-1m-3.00usd_slippage': 1276,
    'bid_strength-itbit-orderbook-1m-3.10usd_slippage': 1277,
    'bid_strength-itbit-orderbook-1m-3.20usd_slippage': 1278,
    'bid_strength-itbit-orderbook-1m-3.30usd_slippage': 1279,
    'bid_strength-itbit-orderbook-1m-3.40usd_slippage': 1280,
    'bid_strength-itbit-orderbook-1m-3.50usd_slippage': 1281,
    'bid_strength-itbit-orderbook-1m-3.60usd_slippage': 1282,
    'bid_strength-itbit-orderbook-1m-3.70usd_slippage': 1283,
    'bid_strength-itbit-orderbook-1m-3.80usd_slippage': 1284,
    'bid_strength-itbit-orderbook-1m-3.90usd_slippage': 1285,
    'bid_strength-itbit-orderbook-1m-4.00usd_slippage': 1286,
    'bid_strength-itbit-orderbook-1m-4.10usd_slippage': 1287,
    'bid_strength-itbit-orderbook-1m-4.20usd_slippage': 1288,
    'bid_strength-itbit-orderbook-1m-4.30usd_slippage': 1289,
    'bid_strength-itbit-orderbook-1m-4.40usd_slippage': 1290,
    'bid_strength-itbit-orderbook-1m-4.50usd_slippage': 1291,
    'bid_strength-itbit-orderbook-1m-4.60usd_slippage': 1292,
    'bid_strength-itbit-orderbook-1m-4.70usd_slippage': 1293,
    'bid_strength-itbit-orderbook-1m-4.80usd_slippage': 1294,
    'bid_strength-itbit-orderbook-1m-4.90usd_slippage': 1295,
    'bid_strength-itbit-orderbook-1m-5.00usd_slippage': 1296,
    'bid_strength-itbit-orderbook-1m-10.00usd_slippage': 1297,
    'bid_strength-itbit-orderbook-1m-20.00usd_slippage': 1298,
    'bid_strength-kraken-orderbook-10m-0.00usd_slippage': 1299,
    'bid_strength-kraken-orderbook-10m-0.10usd_slippage': 1300,
    'bid_strength-kraken-orderbook-10m-0.20usd_slippage': 1301,
    'bid_strength-kraken-orderbook-10m-0.30usd_slippage': 1302,
    'bid_strength-kraken-orderbook-10m-0.40usd_slippage': 1303,
    'bid_strength-kraken-orderbook-10m-0.50usd_slippage': 1304,
    'bid_strength-kraken-orderbook-10m-0.60usd_slippage': 1305,
    'bid_strength-kraken-orderbook-10m-0.70usd_slippage': 1306,
    'bid_strength-kraken-orderbook-10m-0.80usd_slippage': 1307,
    'bid_strength-kraken-orderbook-10m-0.90usd_slippage': 1308,
    'bid_strength-kraken-orderbook-10m-1.00usd_slippage': 1309,
    'bid_strength-kraken-orderbook-10m-1.10usd_slippage': 1310,
    'bid_strength-kraken-orderbook-10m-1.20usd_slippage': 1311,
    'bid_strength-kraken-orderbook-10m-1.30usd_slippage': 1312,
    'bid_strength-kraken-orderbook-10m-1.40usd_slippage': 1313,
    'bid_strength-kraken-orderbook-10m-1.50usd_slippage': 1314,
    'bid_strength-kraken-orderbook-10m-1.60usd_slippage': 1315,
    'bid_strength-kraken-orderbook-10m-1.70usd_slippage': 1316,
    'bid_strength-kraken-orderbook-10m-1.80usd_slippage': 1317,
    'bid_strength-kraken-orderbook-10m-1.90usd_slippage': 1318,
    'bid_strength-kraken-orderbook-10m-2.00usd_slippage': 1319,
    'bid_strength-kraken-orderbook-10m-2.10usd_slippage': 1320,
    'bid_strength-kraken-orderbook-10m-2.20usd_slippage': 1321,
    'bid_strength-kraken-orderbook-10m-2.30usd_slippage': 1322,
    'bid_strength-kraken-orderbook-10m-2.40usd_slippage': 1323,
    'bid_strength-kraken-orderbook-10m-2.50usd_slippage': 1324,
    'bid_strength-kraken-orderbook-10m-2.60usd_slippage': 1325,
    'bid_strength-kraken-orderbook-10m-2.70usd_slippage': 1326,
    'bid_strength-kraken-orderbook-10m-2.80usd_slippage': 1327,
    'bid_strength-kraken-orderbook-10m-2.90usd_slippage': 1328,
    'bid_strength-kraken-orderbook-10m-3.00usd_slippage': 1329,
    'bid_strength-kraken-orderbook-10m-3.10usd_slippage': 1330,
    'bid_strength-kraken-orderbook-10m-3.20usd_slippage': 1331,
    'bid_strength-kraken-orderbook-10m-3.30usd_slippage': 1332,
    'bid_strength-kraken-orderbook-10m-3.40usd_slippage': 1333,
    'bid_strength-kraken-orderbook-10m-3.50usd_slippage': 1334,
    'bid_strength-kraken-orderbook-10m-3.60usd_slippage': 1335,
    'bid_strength-kraken-orderbook-10m-3.70usd_slippage': 1336,
    'bid_strength-kraken-orderbook-10m-3.80usd_slippage': 1337,
    'bid_strength-kraken-orderbook-10m-3.90usd_slippage': 1338,
    'bid_strength-kraken-orderbook-10m-4.00usd_slippage': 1339,
    'bid_strength-kraken-orderbook-10m-4.10usd_slippage': 1340,
    'bid_strength-kraken-orderbook-10m-4.20usd_slippage': 1341,
    'bid_strength-kraken-orderbook-10m-4.30usd_slippage': 1342,
    'bid_strength-kraken-orderbook-10m-4.40usd_slippage': 1343,
    'bid_strength-kraken-orderbook-10m-4.50usd_slippage': 1344,
    'bid_strength-kraken-orderbook-10m-4.60usd_slippage': 1345,
    'bid_strength-kraken-orderbook-10m-4.70usd_slippage': 1346,
    'bid_strength-kraken-orderbook-10m-4.80usd_slippage': 1347,
    'bid_strength-kraken-orderbook-10m-4.90usd_slippage': 1348,
    'bid_strength-kraken-orderbook-10m-5.00usd_slippage': 1349,
    'bid_strength-kraken-orderbook-10m-10.00usd_slippage': 1350,
    'bid_strength-kraken-orderbook-10m-20.00usd_slippage': 1351,
    'bid_strength-kraken-orderbook-5m-0.00usd_slippage': 1352,
    'bid_strength-kraken-orderbook-5m-0.10usd_slippage': 1353,
    'bid_strength-kraken-orderbook-5m-0.20usd_slippage': 1354,
    'bid_strength-kraken-orderbook-5m-0.30usd_slippage': 1355,
    'bid_strength-kraken-orderbook-5m-0.40usd_slippage': 1356,
    'bid_strength-kraken-orderbook-5m-0.50usd_slippage': 1357,
    'bid_strength-kraken-orderbook-5m-0.60usd_slippage': 1358,
    'bid_strength-kraken-orderbook-5m-0.70usd_slippage': 1359,
    'bid_strength-kraken-orderbook-5m-0.80usd_slippage': 1360,
    'bid_strength-kraken-orderbook-5m-0.90usd_slippage': 1361,
    'bid_strength-kraken-orderbook-5m-1.00usd_slippage': 1362,
    'bid_strength-kraken-orderbook-5m-1.10usd_slippage': 1363,
    'bid_strength-kraken-orderbook-5m-1.20usd_slippage': 1364,
    'bid_strength-kraken-orderbook-5m-1.30usd_slippage': 1365,
    'bid_strength-kraken-orderbook-5m-1.40usd_slippage': 1366,
    'bid_strength-kraken-orderbook-5m-1.50usd_slippage': 1367,
    'bid_strength-kraken-orderbook-5m-1.60usd_slippage': 1368,
    'bid_strength-kraken-orderbook-5m-1.70usd_slippage': 1369,
    'bid_strength-kraken-orderbook-5m-1.80usd_slippage': 1370,
    'bid_strength-kraken-orderbook-5m-1.90usd_slippage': 1371,
    'bid_strength-kraken-orderbook-5m-2.00usd_slippage': 1372,
    'bid_strength-kraken-orderbook-5m-2.10usd_slippage': 1373,
    'bid_strength-kraken-orderbook-5m-2.20usd_slippage': 1374,
    'bid_strength-kraken-orderbook-5m-2.30usd_slippage': 1375,
    'bid_strength-kraken-orderbook-5m-2.40usd_slippage': 1376,
    'bid_strength-kraken-orderbook-5m-2.50usd_slippage': 1377,
    'bid_strength-kraken-orderbook-5m-2.60usd_slippage': 1378,
    'bid_strength-kraken-orderbook-5m-2.70usd_slippage': 1379,
    'bid_strength-kraken-orderbook-5m-2.80usd_slippage': 1380,
    'bid_strength-kraken-orderbook-5m-2.90usd_slippage': 1381,
    'bid_strength-kraken-orderbook-5m-3.00usd_slippage': 1382,
    'bid_strength-kraken-orderbook-5m-3.10usd_slippage': 1383,
    'bid_strength-kraken-orderbook-5m-3.20usd_slippage': 1384,
    'bid_strength-kraken-orderbook-5m-3.30usd_slippage': 1385,
    'bid_strength-kraken-orderbook-5m-3.40usd_slippage': 1386,
    'bid_strength-kraken-orderbook-5m-3.50usd_slippage': 1387,
    'bid_strength-kraken-orderbook-5m-3.60usd_slippage': 1388,
    'bid_strength-kraken-orderbook-5m-3.70usd_slippage': 1389,
    'bid_strength-kraken-orderbook-5m-3.80usd_slippage': 1390,
    'bid_strength-kraken-orderbook-5m-3.90usd_slippage': 1391,
    'bid_strength-kraken-orderbook-5m-4.00usd_slippage': 1392,
    'bid_strength-kraken-orderbook-5m-4.10usd_slippage': 1393,
    'bid_strength-kraken-orderbook-5m-4.20usd_slippage': 1394,
    'bid_strength-kraken-orderbook-5m-4.30usd_slippage': 1395,
    'bid_strength-kraken-orderbook-5m-4.40usd_slippage': 1396,
    'bid_strength-kraken-orderbook-5m-4.50usd_slippage': 1397,
    'bid_strength-kraken-orderbook-5m-4.60usd_slippage': 1398,
    'bid_strength-kraken-orderbook-5m-4.70usd_slippage': 1399,
    'bid_strength-kraken-orderbook-5m-4.80usd_slippage': 1400,
    'bid_strength-kraken-orderbook-5m-4.90usd_slippage': 1401,
    'bid_strength-kraken-orderbook-5m-5.00usd_slippage': 1402,
    'bid_strength-kraken-orderbook-5m-10.00usd_slippage': 1403,
    'bid_strength-kraken-orderbook-5m-20.00usd_slippage': 1404,
    'bid_strength-kraken-orderbook-1m-0.00usd_slippage': 1405,
    'bid_strength-kraken-orderbook-1m-0.10usd_slippage': 1406,
    'bid_strength-kraken-orderbook-1m-0.20usd_slippage': 1407,
    'bid_strength-kraken-orderbook-1m-0.30usd_slippage': 1408,
    'bid_strength-kraken-orderbook-1m-0.40usd_slippage': 1409,
    'bid_strength-kraken-orderbook-1m-0.50usd_slippage': 1410,
    'bid_strength-kraken-orderbook-1m-0.60usd_slippage': 1411,
    'bid_strength-kraken-orderbook-1m-0.70usd_slippage': 1412,
    'bid_strength-kraken-orderbook-1m-0.80usd_slippage': 1413,
    'bid_strength-kraken-orderbook-1m-0.90usd_slippage': 1414,
    'bid_strength-kraken-orderbook-1m-1.00usd_slippage': 1415,
    'bid_strength-kraken-orderbook-1m-1.10usd_slippage': 1416,
    'bid_strength-kraken-orderbook-1m-1.20usd_slippage': 1417,
    'bid_strength-kraken-orderbook-1m-1.30usd_slippage': 1418,
    'bid_strength-kraken-orderbook-1m-1.40usd_slippage': 1419,
    'bid_strength-kraken-orderbook-1m-1.50usd_slippage': 1420,
    'bid_strength-kraken-orderbook-1m-1.60usd_slippage': 1421,
    'bid_strength-kraken-orderbook-1m-1.70usd_slippage': 1422,
    'bid_strength-kraken-orderbook-1m-1.80usd_slippage': 1423,
    'bid_strength-kraken-orderbook-1m-1.90usd_slippage': 1424,
    'bid_strength-kraken-orderbook-1m-2.00usd_slippage': 1425,
    'bid_strength-kraken-orderbook-1m-2.10usd_slippage': 1426,
    'bid_strength-kraken-orderbook-1m-2.20usd_slippage': 1427,
    'bid_strength-kraken-orderbook-1m-2.30usd_slippage': 1428,
    'bid_strength-kraken-orderbook-1m-2.40usd_slippage': 1429,
    'bid_strength-kraken-orderbook-1m-2.50usd_slippage': 1430,
    'bid_strength-kraken-orderbook-1m-2.60usd_slippage': 1431,
    'bid_strength-kraken-orderbook-1m-2.70usd_slippage': 1432,
    'bid_strength-kraken-orderbook-1m-2.80usd_slippage': 1433,
    'bid_strength-kraken-orderbook-1m-2.90usd_slippage': 1434,
    'bid_strength-kraken-orderbook-1m-3.00usd_slippage': 1435,
    'bid_strength-kraken-orderbook-1m-3.10usd_slippage': 1436,
    'bid_strength-kraken-orderbook-1m-3.20usd_slippage': 1437,
    'bid_strength-kraken-orderbook-1m-3.30usd_slippage': 1438,
    'bid_strength-kraken-orderbook-1m-3.40usd_slippage': 1439,
    'bid_strength-kraken-orderbook-1m-3.50usd_slippage': 1440,
    'bid_strength-kraken-orderbook-1m-3.60usd_slippage': 1441,
    'bid_strength-kraken-orderbook-1m-3.70usd_slippage': 1442,
    'bid_strength-kraken-orderbook-1m-3.80usd_slippage': 1443,
    'bid_strength-kraken-orderbook-1m-3.90usd_slippage': 1444,
    'bid_strength-kraken-orderbook-1m-4.00usd_slippage': 1445,
    'bid_strength-kraken-orderbook-1m-4.10usd_slippage': 1446,
    'bid_strength-kraken-orderbook-1m-4.20usd_slippage': 1447,
    'bid_strength-kraken-orderbook-1m-4.30usd_slippage': 1448,
    'bid_strength-kraken-orderbook-1m-4.40usd_slippage': 1449,
    'bid_strength-kraken-orderbook-1m-4.50usd_slippage': 1450,
    'bid_strength-kraken-orderbook-1m-4.60usd_slippage': 1451,
    'bid_strength-kraken-orderbook-1m-4.70usd_slippage': 1452,
    'bid_strength-kraken-orderbook-1m-4.80usd_slippage': 1453,
    'bid_strength-kraken-orderbook-1m-4.90usd_slippage': 1454,
    'bid_strength-kraken-orderbook-1m-5.00usd_slippage': 1455,
    'bid_strength-kraken-orderbook-1m-10.00usd_slippage': 1456,
    'bid_strength-kraken-orderbook-1m-20.00usd_slippage': 1457,
    'bid_strength-gemini-orderbook-10m-0.00usd_slippage': 1458,
    'bid_strength-gemini-orderbook-10m-0.10usd_slippage': 1459,
    'bid_strength-gemini-orderbook-10m-0.20usd_slippage': 1460,
    'bid_strength-gemini-orderbook-10m-0.30usd_slippage': 1461,
    'bid_strength-gemini-orderbook-10m-0.40usd_slippage': 1462,
    'bid_strength-gemini-orderbook-10m-0.50usd_slippage': 1463,
    'bid_strength-gemini-orderbook-10m-0.60usd_slippage': 1464,
    'bid_strength-gemini-orderbook-10m-0.70usd_slippage': 1465,
    'bid_strength-gemini-orderbook-10m-0.80usd_slippage': 1466,
    'bid_strength-gemini-orderbook-10m-0.90usd_slippage': 1467,
    'bid_strength-gemini-orderbook-10m-1.00usd_slippage': 1468,
    'bid_strength-gemini-orderbook-10m-1.10usd_slippage': 1469,
    'bid_strength-gemini-orderbook-10m-1.20usd_slippage': 1470,
    'bid_strength-gemini-orderbook-10m-1.30usd_slippage': 1471,
    'bid_strength-gemini-orderbook-10m-1.40usd_slippage': 1472,
    'bid_strength-gemini-orderbook-10m-1.50usd_slippage': 1473,
    'bid_strength-gemini-orderbook-10m-1.60usd_slippage': 1474,
    'bid_strength-gemini-orderbook-10m-1.70usd_slippage': 1475,
    'bid_strength-gemini-orderbook-10m-1.80usd_slippage': 1476,
    'bid_strength-gemini-orderbook-10m-1.90usd_slippage': 1477,
    'bid_strength-gemini-orderbook-10m-2.00usd_slippage': 1478,
    'bid_strength-gemini-orderbook-10m-2.10usd_slippage': 1479,
    'bid_strength-gemini-orderbook-10m-2.20usd_slippage': 1480,
    'bid_strength-gemini-orderbook-10m-2.30usd_slippage': 1481,
    'bid_strength-gemini-orderbook-10m-2.40usd_slippage': 1482,
    'bid_strength-gemini-orderbook-10m-2.50usd_slippage': 1483,
    'bid_strength-gemini-orderbook-10m-2.60usd_slippage': 1484,
    'bid_strength-gemini-orderbook-10m-2.70usd_slippage': 1485,
    'bid_strength-gemini-orderbook-10m-2.80usd_slippage': 1486,
    'bid_strength-gemini-orderbook-10m-2.90usd_slippage': 1487,
    'bid_strength-gemini-orderbook-10m-3.00usd_slippage': 1488,
    'bid_strength-gemini-orderbook-10m-3.10usd_slippage': 1489,
    'bid_strength-gemini-orderbook-10m-3.20usd_slippage': 1490,
    'bid_strength-gemini-orderbook-10m-3.30usd_slippage': 1491,
    'bid_strength-gemini-orderbook-10m-3.40usd_slippage': 1492,
    'bid_strength-gemini-orderbook-10m-3.50usd_slippage': 1493,
    'bid_strength-gemini-orderbook-10m-3.60usd_slippage': 1494,
    'bid_strength-gemini-orderbook-10m-3.70usd_slippage': 1495,
    'bid_strength-gemini-orderbook-10m-3.80usd_slippage': 1496,
    'bid_strength-gemini-orderbook-10m-3.90usd_slippage': 1497,
    'bid_strength-gemini-orderbook-10m-4.00usd_slippage': 1498,
    'bid_strength-gemini-orderbook-10m-4.10usd_slippage': 1499,
    'bid_strength-gemini-orderbook-10m-4.20usd_slippage': 1500,
    'bid_strength-gemini-orderbook-10m-4.30usd_slippage': 1501,
    'bid_strength-gemini-orderbook-10m-4.40usd_slippage': 1502,
    'bid_strength-gemini-orderbook-10m-4.50usd_slippage': 1503,
    'bid_strength-gemini-orderbook-10m-4.60usd_slippage': 1504,
    'bid_strength-gemini-orderbook-10m-4.70usd_slippage': 1505,
    'bid_strength-gemini-orderbook-10m-4.80usd_slippage': 1506,
    'bid_strength-gemini-orderbook-10m-4.90usd_slippage': 1507,
    'bid_strength-gemini-orderbook-10m-5.00usd_slippage': 1508,
    'bid_strength-gemini-orderbook-10m-10.00usd_slippage': 1509,
    'bid_strength-gemini-orderbook-10m-20.00usd_slippage': 1510,
    'bid_strength-gemini-orderbook-5m-0.00usd_slippage': 1511,
    'bid_strength-gemini-orderbook-5m-0.10usd_slippage': 1512,
    'bid_strength-gemini-orderbook-5m-0.20usd_slippage': 1513,
    'bid_strength-gemini-orderbook-5m-0.30usd_slippage': 1514,
    'bid_strength-gemini-orderbook-5m-0.40usd_slippage': 1515,
    'bid_strength-gemini-orderbook-5m-0.50usd_slippage': 1516,
    'bid_strength-gemini-orderbook-5m-0.60usd_slippage': 1517,
    'bid_strength-gemini-orderbook-5m-0.70usd_slippage': 1518,
    'bid_strength-gemini-orderbook-5m-0.80usd_slippage': 1519,
    'bid_strength-gemini-orderbook-5m-0.90usd_slippage': 1520,
    'bid_strength-gemini-orderbook-5m-1.00usd_slippage': 1521,
    'bid_strength-gemini-orderbook-5m-1.10usd_slippage': 1522,
    'bid_strength-gemini-orderbook-5m-1.20usd_slippage': 1523,
    'bid_strength-gemini-orderbook-5m-1.30usd_slippage': 1524,
    'bid_strength-gemini-orderbook-5m-1.40usd_slippage': 1525,
    'bid_strength-gemini-orderbook-5m-1.50usd_slippage': 1526,
    'bid_strength-gemini-orderbook-5m-1.60usd_slippage': 1527,
    'bid_strength-gemini-orderbook-5m-1.70usd_slippage': 1528,
    'bid_strength-gemini-orderbook-5m-1.80usd_slippage': 1529,
    'bid_strength-gemini-orderbook-5m-1.90usd_slippage': 1530,
    'bid_strength-gemini-orderbook-5m-2.00usd_slippage': 1531,
    'bid_strength-gemini-orderbook-5m-2.10usd_slippage': 1532,
    'bid_strength-gemini-orderbook-5m-2.20usd_slippage': 1533,
    'bid_strength-gemini-orderbook-5m-2.30usd_slippage': 1534,
    'bid_strength-gemini-orderbook-5m-2.40usd_slippage': 1535,
    'bid_strength-gemini-orderbook-5m-2.50usd_slippage': 1536,
    'bid_strength-gemini-orderbook-5m-2.60usd_slippage': 1537,
    'bid_strength-gemini-orderbook-5m-2.70usd_slippage': 1538,
    'bid_strength-gemini-orderbook-5m-2.80usd_slippage': 1539,
    'bid_strength-gemini-orderbook-5m-2.90usd_slippage': 1540,
    'bid_strength-gemini-orderbook-5m-3.00usd_slippage': 1541,
    'bid_strength-gemini-orderbook-5m-3.10usd_slippage': 1542,
    'bid_strength-gemini-orderbook-5m-3.20usd_slippage': 1543,
    'bid_strength-gemini-orderbook-5m-3.30usd_slippage': 1544,
    'bid_strength-gemini-orderbook-5m-3.40usd_slippage': 1545,
    'bid_strength-gemini-orderbook-5m-3.50usd_slippage': 1546,
    'bid_strength-gemini-orderbook-5m-3.60usd_slippage': 1547,
    'bid_strength-gemini-orderbook-5m-3.70usd_slippage': 1548,
    'bid_strength-gemini-orderbook-5m-3.80usd_slippage': 1549,
    'bid_strength-gemini-orderbook-5m-3.90usd_slippage': 1550,
    'bid_strength-gemini-orderbook-5m-4.00usd_slippage': 1551,
    'bid_strength-gemini-orderbook-5m-4.10usd_slippage': 1552,
    'bid_strength-gemini-orderbook-5m-4.20usd_slippage': 1553,
    'bid_strength-gemini-orderbook-5m-4.30usd_slippage': 1554,
    'bid_strength-gemini-orderbook-5m-4.40usd_slippage': 1555,
    'bid_strength-gemini-orderbook-5m-4.50usd_slippage': 1556,
    'bid_strength-gemini-orderbook-5m-4.60usd_slippage': 1557,
    'bid_strength-gemini-orderbook-5m-4.70usd_slippage': 1558,
    'bid_strength-gemini-orderbook-5m-4.80usd_slippage': 1559,
    'bid_strength-gemini-orderbook-5m-4.90usd_slippage': 1560,
    'bid_strength-gemini-orderbook-5m-5.00usd_slippage': 1561,
    'bid_strength-gemini-orderbook-5m-10.00usd_slippage': 1562,
    'bid_strength-gemini-orderbook-5m-20.00usd_slippage': 1563,
    'bid_strength-gemini-orderbook-1m-0.00usd_slippage': 1564,
    'bid_strength-gemini-orderbook-1m-0.10usd_slippage': 1565,
    'bid_strength-gemini-orderbook-1m-0.20usd_slippage': 1566,
    'bid_strength-gemini-orderbook-1m-0.30usd_slippage': 1567,
    'bid_strength-gemini-orderbook-1m-0.40usd_slippage': 1568,
    'bid_strength-gemini-orderbook-1m-0.50usd_slippage': 1569,
    'bid_strength-gemini-orderbook-1m-0.60usd_slippage': 1570,
    'bid_strength-gemini-orderbook-1m-0.70usd_slippage': 1571,
    'bid_strength-gemini-orderbook-1m-0.80usd_slippage': 1572,
    'bid_strength-gemini-orderbook-1m-0.90usd_slippage': 1573,
    'bid_strength-gemini-orderbook-1m-1.00usd_slippage': 1574,
    'bid_strength-gemini-orderbook-1m-1.10usd_slippage': 1575,
    'bid_strength-gemini-orderbook-1m-1.20usd_slippage': 1576,
    'bid_strength-gemini-orderbook-1m-1.30usd_slippage': 1577,
    'bid_strength-gemini-orderbook-1m-1.40usd_slippage': 1578,
    'bid_strength-gemini-orderbook-1m-1.50usd_slippage': 1579,
    'bid_strength-gemini-orderbook-1m-1.60usd_slippage': 1580,
    'bid_strength-gemini-orderbook-1m-1.70usd_slippage': 1581,
    'bid_strength-gemini-orderbook-1m-1.80usd_slippage': 1582,
    'bid_strength-gemini-orderbook-1m-1.90usd_slippage': 1583,
    'bid_strength-gemini-orderbook-1m-2.00usd_slippage': 1584,
    'bid_strength-gemini-orderbook-1m-2.10usd_slippage': 1585,
    'bid_strength-gemini-orderbook-1m-2.20usd_slippage': 1586,
    'bid_strength-gemini-orderbook-1m-2.30usd_slippage': 1587,
    'bid_strength-gemini-orderbook-1m-2.40usd_slippage': 1588,
    'bid_strength-gemini-orderbook-1m-2.50usd_slippage': 1589,
    'bid_strength-gemini-orderbook-1m-2.60usd_slippage': 1590,
    'bid_strength-gemini-orderbook-1m-2.70usd_slippage': 1591,
    'bid_strength-gemini-orderbook-1m-2.80usd_slippage': 1592,
    'bid_strength-gemini-orderbook-1m-2.90usd_slippage': 1593,
    'bid_strength-gemini-orderbook-1m-3.00usd_slippage': 1594,
    'bid_strength-gemini-orderbook-1m-3.10usd_slippage': 1595,
    'bid_strength-gemini-orderbook-1m-3.20usd_slippage': 1596,
    'bid_strength-gemini-orderbook-1m-3.30usd_slippage': 1597,
    'bid_strength-gemini-orderbook-1m-3.40usd_slippage': 1598,
    'bid_strength-gemini-orderbook-1m-3.50usd_slippage': 1599,
    'bid_strength-gemini-orderbook-1m-3.60usd_slippage': 1600,
    'bid_strength-gemini-orderbook-1m-3.70usd_slippage': 1601,
    'bid_strength-gemini-orderbook-1m-3.80usd_slippage': 1602,
    'bid_strength-gemini-orderbook-1m-3.90usd_slippage': 1603,
    'bid_strength-gemini-orderbook-1m-4.00usd_slippage': 1604,
    'bid_strength-gemini-orderbook-1m-4.10usd_slippage': 1605,
    'bid_strength-gemini-orderbook-1m-4.20usd_slippage': 1606,
    'bid_strength-gemini-orderbook-1m-4.30usd_slippage': 1607,
    'bid_strength-gemini-orderbook-1m-4.40usd_slippage': 1608,
    'bid_strength-gemini-orderbook-1m-4.50usd_slippage': 1609,
    'bid_strength-gemini-orderbook-1m-4.60usd_slippage': 1610,
    'bid_strength-gemini-orderbook-1m-4.70usd_slippage': 1611,
    'bid_strength-gemini-orderbook-1m-4.80usd_slippage': 1612,
    'bid_strength-gemini-orderbook-1m-4.90usd_slippage': 1613,
    'bid_strength-gemini-orderbook-1m-5.00usd_slippage': 1614,
    'bid_strength-gemini-orderbook-1m-10.00usd_slippage': 1615,
    'bid_strength-gemini-orderbook-1m-20.00usd_slippage': 1616,
    'bid_strength-okcoin-orderbook-10m-0.00usd_slippage': 1617,
    'bid_strength-okcoin-orderbook-10m-0.10usd_slippage': 1618,
    'bid_strength-okcoin-orderbook-10m-0.20usd_slippage': 1619,
    'bid_strength-okcoin-orderbook-10m-0.30usd_slippage': 1620,
    'bid_strength-okcoin-orderbook-10m-0.40usd_slippage': 1621,
    'bid_strength-okcoin-orderbook-10m-0.50usd_slippage': 1622,
    'bid_strength-okcoin-orderbook-10m-0.60usd_slippage': 1623,
    'bid_strength-okcoin-orderbook-10m-0.70usd_slippage': 1624,
    'bid_strength-okcoin-orderbook-10m-0.80usd_slippage': 1625,
    'bid_strength-okcoin-orderbook-10m-0.90usd_slippage': 1626,
    'bid_strength-okcoin-orderbook-10m-1.00usd_slippage': 1627,
    'bid_strength-okcoin-orderbook-10m-1.10usd_slippage': 1628,
    'bid_strength-okcoin-orderbook-10m-1.20usd_slippage': 1629,
    'bid_strength-okcoin-orderbook-10m-1.30usd_slippage': 1630,
    'bid_strength-okcoin-orderbook-10m-1.40usd_slippage': 1631,
    'bid_strength-okcoin-orderbook-10m-1.50usd_slippage': 1632,
    'bid_strength-okcoin-orderbook-10m-1.60usd_slippage': 1633,
    'bid_strength-okcoin-orderbook-10m-1.70usd_slippage': 1634,
    'bid_strength-okcoin-orderbook-10m-1.80usd_slippage': 1635,
    'bid_strength-okcoin-orderbook-10m-1.90usd_slippage': 1636,
    'bid_strength-okcoin-orderbook-10m-2.00usd_slippage': 1637,
    'bid_strength-okcoin-orderbook-10m-2.10usd_slippage': 1638,
    'bid_strength-okcoin-orderbook-10m-2.20usd_slippage': 1639,
    'bid_strength-okcoin-orderbook-10m-2.30usd_slippage': 1640,
    'bid_strength-okcoin-orderbook-10m-2.40usd_slippage': 1641,
    'bid_strength-okcoin-orderbook-10m-2.50usd_slippage': 1642,
    'bid_strength-okcoin-orderbook-10m-2.60usd_slippage': 1643,
    'bid_strength-okcoin-orderbook-10m-2.70usd_slippage': 1644,
    'bid_strength-okcoin-orderbook-10m-2.80usd_slippage': 1645,
    'bid_strength-okcoin-orderbook-10m-2.90usd_slippage': 1646,
    'bid_strength-okcoin-orderbook-10m-3.00usd_slippage': 1647,
    'bid_strength-okcoin-orderbook-10m-3.10usd_slippage': 1648,
    'bid_strength-okcoin-orderbook-10m-3.20usd_slippage': 1649,
    'bid_strength-okcoin-orderbook-10m-3.30usd_slippage': 1650,
    'bid_strength-okcoin-orderbook-10m-3.40usd_slippage': 1651,
    'bid_strength-okcoin-orderbook-10m-3.50usd_slippage': 1652,
    'bid_strength-okcoin-orderbook-10m-3.60usd_slippage': 1653,
    'bid_strength-okcoin-orderbook-10m-3.70usd_slippage': 1654,
    'bid_strength-okcoin-orderbook-10m-3.80usd_slippage': 1655,
    'bid_strength-okcoin-orderbook-10m-3.90usd_slippage': 1656,
    'bid_strength-okcoin-orderbook-10m-4.00usd_slippage': 1657,
    'bid_strength-okcoin-orderbook-10m-4.10usd_slippage': 1658,
    'bid_strength-okcoin-orderbook-10m-4.20usd_slippage': 1659,
    'bid_strength-okcoin-orderbook-10m-4.30usd_slippage': 1660,
    'bid_strength-okcoin-orderbook-10m-4.40usd_slippage': 1661,
    'bid_strength-okcoin-orderbook-10m-4.50usd_slippage': 1662,
    'bid_strength-okcoin-orderbook-10m-4.60usd_slippage': 1663,
    'bid_strength-okcoin-orderbook-10m-4.70usd_slippage': 1664,
    'bid_strength-okcoin-orderbook-10m-4.80usd_slippage': 1665,
    'bid_strength-okcoin-orderbook-10m-4.90usd_slippage': 1666,
    'bid_strength-okcoin-orderbook-10m-5.00usd_slippage': 1667,
    'bid_strength-okcoin-orderbook-10m-10.00usd_slippage': 1668,
    'bid_strength-okcoin-orderbook-10m-20.00usd_slippage': 1669,
    'bid_strength-okcoin-orderbook-5m-0.00usd_slippage': 1670,
    'bid_strength-okcoin-orderbook-5m-0.10usd_slippage': 1671,
    'bid_strength-okcoin-orderbook-5m-0.20usd_slippage': 1672,
    'bid_strength-okcoin-orderbook-5m-0.30usd_slippage': 1673,
    'bid_strength-okcoin-orderbook-5m-0.40usd_slippage': 1674,
    'bid_strength-okcoin-orderbook-5m-0.50usd_slippage': 1675,
    'bid_strength-okcoin-orderbook-5m-0.60usd_slippage': 1676,
    'bid_strength-okcoin-orderbook-5m-0.70usd_slippage': 1677,
    'bid_strength-okcoin-orderbook-5m-0.80usd_slippage': 1678,
    'bid_strength-okcoin-orderbook-5m-0.90usd_slippage': 1679,
    'bid_strength-okcoin-orderbook-5m-1.00usd_slippage': 1680,
    'bid_strength-okcoin-orderbook-5m-1.10usd_slippage': 1681,
    'bid_strength-okcoin-orderbook-5m-1.20usd_slippage': 1682,
    'bid_strength-okcoin-orderbook-5m-1.30usd_slippage': 1683,
    'bid_strength-okcoin-orderbook-5m-1.40usd_slippage': 1684,
    'bid_strength-okcoin-orderbook-5m-1.50usd_slippage': 1685,
    'bid_strength-okcoin-orderbook-5m-1.60usd_slippage': 1686,
    'bid_strength-okcoin-orderbook-5m-1.70usd_slippage': 1687,
    'bid_strength-okcoin-orderbook-5m-1.80usd_slippage': 1688,
    'bid_strength-okcoin-orderbook-5m-1.90usd_slippage': 1689,
    'bid_strength-okcoin-orderbook-5m-2.00usd_slippage': 1690,
    'bid_strength-okcoin-orderbook-5m-2.10usd_slippage': 1691,
    'bid_strength-okcoin-orderbook-5m-2.20usd_slippage': 1692,
    'bid_strength-okcoin-orderbook-5m-2.30usd_slippage': 1693,
    'bid_strength-okcoin-orderbook-5m-2.40usd_slippage': 1694,
    'bid_strength-okcoin-orderbook-5m-2.50usd_slippage': 1695,
    'bid_strength-okcoin-orderbook-5m-2.60usd_slippage': 1696,
    'bid_strength-okcoin-orderbook-5m-2.70usd_slippage': 1697,
    'bid_strength-okcoin-orderbook-5m-2.80usd_slippage': 1698,
    'bid_strength-okcoin-orderbook-5m-2.90usd_slippage': 1699,
    'bid_strength-okcoin-orderbook-5m-3.00usd_slippage': 1700,
    'bid_strength-okcoin-orderbook-5m-3.10usd_slippage': 1701,
    'bid_strength-okcoin-orderbook-5m-3.20usd_slippage': 1702,
    'bid_strength-okcoin-orderbook-5m-3.30usd_slippage': 1703,
    'bid_strength-okcoin-orderbook-5m-3.40usd_slippage': 1704,
    'bid_strength-okcoin-orderbook-5m-3.50usd_slippage': 1705,
    'bid_strength-okcoin-orderbook-5m-3.60usd_slippage': 1706,
    'bid_strength-okcoin-orderbook-5m-3.70usd_slippage': 1707,
    'bid_strength-okcoin-orderbook-5m-3.80usd_slippage': 1708,
    'bid_strength-okcoin-orderbook-5m-3.90usd_slippage': 1709,
    'bid_strength-okcoin-orderbook-5m-4.00usd_slippage': 1710,
    'bid_strength-okcoin-orderbook-5m-4.10usd_slippage': 1711,
    'bid_strength-okcoin-orderbook-5m-4.20usd_slippage': 1712,
    'bid_strength-okcoin-orderbook-5m-4.30usd_slippage': 1713,
    'bid_strength-okcoin-orderbook-5m-4.40usd_slippage': 1714,
    'bid_strength-okcoin-orderbook-5m-4.50usd_slippage': 1715,
    'bid_strength-okcoin-orderbook-5m-4.60usd_slippage': 1716,
    'bid_strength-okcoin-orderbook-5m-4.70usd_slippage': 1717,
    'bid_strength-okcoin-orderbook-5m-4.80usd_slippage': 1718,
    'bid_strength-okcoin-orderbook-5m-4.90usd_slippage': 1719,
    'bid_strength-okcoin-orderbook-5m-5.00usd_slippage': 1720,
    'bid_strength-okcoin-orderbook-5m-10.00usd_slippage': 1721,
    'bid_strength-okcoin-orderbook-5m-20.00usd_slippage': 1722,
    'bid_strength-okcoin-orderbook-1m-0.00usd_slippage': 1723,
    'bid_strength-okcoin-orderbook-1m-0.10usd_slippage': 1724,
    'bid_strength-okcoin-orderbook-1m-0.20usd_slippage': 1725,
    'bid_strength-okcoin-orderbook-1m-0.30usd_slippage': 1726,
    'bid_strength-okcoin-orderbook-1m-0.40usd_slippage': 1727,
    'bid_strength-okcoin-orderbook-1m-0.50usd_slippage': 1728,
    'bid_strength-okcoin-orderbook-1m-0.60usd_slippage': 1729,
    'bid_strength-okcoin-orderbook-1m-0.70usd_slippage': 1730,
    'bid_strength-okcoin-orderbook-1m-0.80usd_slippage': 1731,
    'bid_strength-okcoin-orderbook-1m-0.90usd_slippage': 1732,
    'bid_strength-okcoin-orderbook-1m-1.00usd_slippage': 1733,
    'bid_strength-okcoin-orderbook-1m-1.10usd_slippage': 1734,
    'bid_strength-okcoin-orderbook-1m-1.20usd_slippage': 1735,
    'bid_strength-okcoin-orderbook-1m-1.30usd_slippage': 1736,
    'bid_strength-okcoin-orderbook-1m-1.40usd_slippage': 1737,
    'bid_strength-okcoin-orderbook-1m-1.50usd_slippage': 1738,
    'bid_strength-okcoin-orderbook-1m-1.60usd_slippage': 1739,
    'bid_strength-okcoin-orderbook-1m-1.70usd_slippage': 1740,
    'bid_strength-okcoin-orderbook-1m-1.80usd_slippage': 1741,
    'bid_strength-okcoin-orderbook-1m-1.90usd_slippage': 1742,
    'bid_strength-okcoin-orderbook-1m-2.00usd_slippage': 1743,
    'bid_strength-okcoin-orderbook-1m-2.10usd_slippage': 1744,
    'bid_strength-okcoin-orderbook-1m-2.20usd_slippage': 1745,
    'bid_strength-okcoin-orderbook-1m-2.30usd_slippage': 1746,
    'bid_strength-okcoin-orderbook-1m-2.40usd_slippage': 1747,
    'bid_strength-okcoin-orderbook-1m-2.50usd_slippage': 1748,
    'bid_strength-okcoin-orderbook-1m-2.60usd_slippage': 1749,
    'bid_strength-okcoin-orderbook-1m-2.70usd_slippage': 1750,
    'bid_strength-okcoin-orderbook-1m-2.80usd_slippage': 1751,
    'bid_strength-okcoin-orderbook-1m-2.90usd_slippage': 1752,
    'bid_strength-okcoin-orderbook-1m-3.00usd_slippage': 1753,
    'bid_strength-okcoin-orderbook-1m-3.10usd_slippage': 1754,
    'bid_strength-okcoin-orderbook-1m-3.20usd_slippage': 1755,
    'bid_strength-okcoin-orderbook-1m-3.30usd_slippage': 1756,
    'bid_strength-okcoin-orderbook-1m-3.40usd_slippage': 1757,
    'bid_strength-okcoin-orderbook-1m-3.50usd_slippage': 1758,
    'bid_strength-okcoin-orderbook-1m-3.60usd_slippage': 1759,
    'bid_strength-okcoin-orderbook-1m-3.70usd_slippage': 1760,
    'bid_strength-okcoin-orderbook-1m-3.80usd_slippage': 1761,
    'bid_strength-okcoin-orderbook-1m-3.90usd_slippage': 1762,
    'bid_strength-okcoin-orderbook-1m-4.00usd_slippage': 1763,
    'bid_strength-okcoin-orderbook-1m-4.10usd_slippage': 1764,
    'bid_strength-okcoin-orderbook-1m-4.20usd_slippage': 1765,
    'bid_strength-okcoin-orderbook-1m-4.30usd_slippage': 1766,
    'bid_strength-okcoin-orderbook-1m-4.40usd_slippage': 1767,
    'bid_strength-okcoin-orderbook-1m-4.50usd_slippage': 1768,
    'bid_strength-okcoin-orderbook-1m-4.60usd_slippage': 1769,
    'bid_strength-okcoin-orderbook-1m-4.70usd_slippage': 1770,
    'bid_strength-okcoin-orderbook-1m-4.80usd_slippage': 1771,
    'bid_strength-okcoin-orderbook-1m-4.90usd_slippage': 1772,
    'bid_strength-okcoin-orderbook-1m-5.00usd_slippage': 1773,
    'bid_strength-okcoin-orderbook-1m-10.00usd_slippage': 1774,
    'bid_strength-okcoin-orderbook-1m-20.00usd_slippage': 1775,
    'ask_strength-bitstamp-orderbook-10m-0.00usd_slippage': 1776,
    'ask_strength-bitstamp-orderbook-10m-0.10usd_slippage': 1777,
    'ask_strength-bitstamp-orderbook-10m-0.20usd_slippage': 1778,
    'ask_strength-bitstamp-orderbook-10m-0.30usd_slippage': 1779,
    'ask_strength-bitstamp-orderbook-10m-0.40usd_slippage': 1780,
    'ask_strength-bitstamp-orderbook-10m-0.50usd_slippage': 1781,
    'ask_strength-bitstamp-orderbook-10m-0.60usd_slippage': 1782,
    'ask_strength-bitstamp-orderbook-10m-0.70usd_slippage': 1783,
    'ask_strength-bitstamp-orderbook-10m-0.80usd_slippage': 1784,
    'ask_strength-bitstamp-orderbook-10m-0.90usd_slippage': 1785,
    'ask_strength-bitstamp-orderbook-10m-1.00usd_slippage': 1786,
    'ask_strength-bitstamp-orderbook-10m-1.10usd_slippage': 1787,
    'ask_strength-bitstamp-orderbook-10m-1.20usd_slippage': 1788,
    'ask_strength-bitstamp-orderbook-10m-1.30usd_slippage': 1789,
    'ask_strength-bitstamp-orderbook-10m-1.40usd_slippage': 1790,
    'ask_strength-bitstamp-orderbook-10m-1.50usd_slippage': 1791,
    'ask_strength-bitstamp-orderbook-10m-1.60usd_slippage': 1792,
    'ask_strength-bitstamp-orderbook-10m-1.70usd_slippage': 1793,
    'ask_strength-bitstamp-orderbook-10m-1.80usd_slippage': 1794,
    'ask_strength-bitstamp-orderbook-10m-1.90usd_slippage': 1795,
    'ask_strength-bitstamp-orderbook-10m-2.00usd_slippage': 1796,
    'ask_strength-bitstamp-orderbook-10m-2.10usd_slippage': 1797,
    'ask_strength-bitstamp-orderbook-10m-2.20usd_slippage': 1798,
    'ask_strength-bitstamp-orderbook-10m-2.30usd_slippage': 1799,
    'ask_strength-bitstamp-orderbook-10m-2.40usd_slippage': 1800,
    'ask_strength-bitstamp-orderbook-10m-2.50usd_slippage': 1801,
    'ask_strength-bitstamp-orderbook-10m-2.60usd_slippage': 1802,
    'ask_strength-bitstamp-orderbook-10m-2.70usd_slippage': 1803,
    'ask_strength-bitstamp-orderbook-10m-2.80usd_slippage': 1804,
    'ask_strength-bitstamp-orderbook-10m-2.90usd_slippage': 1805,
    'ask_strength-bitstamp-orderbook-10m-3.00usd_slippage': 1806,
    'ask_strength-bitstamp-orderbook-10m-3.10usd_slippage': 1807,
    'ask_strength-bitstamp-orderbook-10m-3.20usd_slippage': 1808,
    'ask_strength-bitstamp-orderbook-10m-3.30usd_slippage': 1809,
    'ask_strength-bitstamp-orderbook-10m-3.40usd_slippage': 1810,
    'ask_strength-bitstamp-orderbook-10m-3.50usd_slippage': 1811,
    'ask_strength-bitstamp-orderbook-10m-3.60usd_slippage': 1812,
    'ask_strength-bitstamp-orderbook-10m-3.70usd_slippage': 1813,
    'ask_strength-bitstamp-orderbook-10m-3.80usd_slippage': 1814,
    'ask_strength-bitstamp-orderbook-10m-3.90usd_slippage': 1815,
    'ask_strength-bitstamp-orderbook-10m-4.00usd_slippage': 1816,
    'ask_strength-bitstamp-orderbook-10m-4.10usd_slippage': 1817,
    'ask_strength-bitstamp-orderbook-10m-4.20usd_slippage': 1818,
    'ask_strength-bitstamp-orderbook-10m-4.30usd_slippage': 1819,
    'ask_strength-bitstamp-orderbook-10m-4.40usd_slippage': 1820,
    'ask_strength-bitstamp-orderbook-10m-4.50usd_slippage': 1821,
    'ask_strength-bitstamp-orderbook-10m-4.60usd_slippage': 1822,
    'ask_strength-bitstamp-orderbook-10m-4.70usd_slippage': 1823,
    'ask_strength-bitstamp-orderbook-10m-4.80usd_slippage': 1824,
    'ask_strength-bitstamp-orderbook-10m-4.90usd_slippage': 1825,
    'ask_strength-bitstamp-orderbook-10m-5.00usd_slippage': 1826,
    'ask_strength-bitstamp-orderbook-10m-10.00usd_slippage': 1827,
    'ask_strength-bitstamp-orderbook-10m-20.00usd_slippage': 1828,
    'ask_strength-bitstamp-orderbook-5m-0.00usd_slippage': 1829,
    'ask_strength-bitstamp-orderbook-5m-0.10usd_slippage': 1830,
    'ask_strength-bitstamp-orderbook-5m-0.20usd_slippage': 1831,
    'ask_strength-bitstamp-orderbook-5m-0.30usd_slippage': 1832,
    'ask_strength-bitstamp-orderbook-5m-0.40usd_slippage': 1833,
    'ask_strength-bitstamp-orderbook-5m-0.50usd_slippage': 1834,
    'ask_strength-bitstamp-orderbook-5m-0.60usd_slippage': 1835,
    'ask_strength-bitstamp-orderbook-5m-0.70usd_slippage': 1836,
    'ask_strength-bitstamp-orderbook-5m-0.80usd_slippage': 1837,
    'ask_strength-bitstamp-orderbook-5m-0.90usd_slippage': 1838,
    'ask_strength-bitstamp-orderbook-5m-1.00usd_slippage': 1839,
    'ask_strength-bitstamp-orderbook-5m-1.10usd_slippage': 1840,
    'ask_strength-bitstamp-orderbook-5m-1.20usd_slippage': 1841,
    'ask_strength-bitstamp-orderbook-5m-1.30usd_slippage': 1842,
    'ask_strength-bitstamp-orderbook-5m-1.40usd_slippage': 1843,
    'ask_strength-bitstamp-orderbook-5m-1.50usd_slippage': 1844,
    'ask_strength-bitstamp-orderbook-5m-1.60usd_slippage': 1845,
    'ask_strength-bitstamp-orderbook-5m-1.70usd_slippage': 1846,
    'ask_strength-bitstamp-orderbook-5m-1.80usd_slippage': 1847,
    'ask_strength-bitstamp-orderbook-5m-1.90usd_slippage': 1848,
    'ask_strength-bitstamp-orderbook-5m-2.00usd_slippage': 1849,
    'ask_strength-bitstamp-orderbook-5m-2.10usd_slippage': 1850,
    'ask_strength-bitstamp-orderbook-5m-2.20usd_slippage': 1851,
    'ask_strength-bitstamp-orderbook-5m-2.30usd_slippage': 1852,
    'ask_strength-bitstamp-orderbook-5m-2.40usd_slippage': 1853,
    'ask_strength-bitstamp-orderbook-5m-2.50usd_slippage': 1854,
    'ask_strength-bitstamp-orderbook-5m-2.60usd_slippage': 1855,
    'ask_strength-bitstamp-orderbook-5m-2.70usd_slippage': 1856,
    'ask_strength-bitstamp-orderbook-5m-2.80usd_slippage': 1857,
    'ask_strength-bitstamp-orderbook-5m-2.90usd_slippage': 1858,
    'ask_strength-bitstamp-orderbook-5m-3.00usd_slippage': 1859,
    'ask_strength-bitstamp-orderbook-5m-3.10usd_slippage': 1860,
    'ask_strength-bitstamp-orderbook-5m-3.20usd_slippage': 1861,
    'ask_strength-bitstamp-orderbook-5m-3.30usd_slippage': 1862,
    'ask_strength-bitstamp-orderbook-5m-3.40usd_slippage': 1863,
    'ask_strength-bitstamp-orderbook-5m-3.50usd_slippage': 1864,
    'ask_strength-bitstamp-orderbook-5m-3.60usd_slippage': 1865,
    'ask_strength-bitstamp-orderbook-5m-3.70usd_slippage': 1866,
    'ask_strength-bitstamp-orderbook-5m-3.80usd_slippage': 1867,
    'ask_strength-bitstamp-orderbook-5m-3.90usd_slippage': 1868,
    'ask_strength-bitstamp-orderbook-5m-4.00usd_slippage': 1869,
    'ask_strength-bitstamp-orderbook-5m-4.10usd_slippage': 1870,
    'ask_strength-bitstamp-orderbook-5m-4.20usd_slippage': 1871,
    'ask_strength-bitstamp-orderbook-5m-4.30usd_slippage': 1872,
    'ask_strength-bitstamp-orderbook-5m-4.40usd_slippage': 1873,
    'ask_strength-bitstamp-orderbook-5m-4.50usd_slippage': 1874,
    'ask_strength-bitstamp-orderbook-5m-4.60usd_slippage': 1875,
    'ask_strength-bitstamp-orderbook-5m-4.70usd_slippage': 1876,
    'ask_strength-bitstamp-orderbook-5m-4.80usd_slippage': 1877,
    'ask_strength-bitstamp-orderbook-5m-4.90usd_slippage': 1878,
    'ask_strength-bitstamp-orderbook-5m-5.00usd_slippage': 1879,
    'ask_strength-bitstamp-orderbook-5m-10.00usd_slippage': 1880,
    'ask_strength-bitstamp-orderbook-5m-20.00usd_slippage': 1881,
    'ask_strength-bitstamp-orderbook-1m-0.00usd_slippage': 1882,
    'ask_strength-bitstamp-orderbook-1m-0.10usd_slippage': 1883,
    'ask_strength-bitstamp-orderbook-1m-0.20usd_slippage': 1884,
    'ask_strength-bitstamp-orderbook-1m-0.30usd_slippage': 1885,
    'ask_strength-bitstamp-orderbook-1m-0.40usd_slippage': 1886,
    'ask_strength-bitstamp-orderbook-1m-0.50usd_slippage': 1887,
    'ask_strength-bitstamp-orderbook-1m-0.60usd_slippage': 1888,
    'ask_strength-bitstamp-orderbook-1m-0.70usd_slippage': 1889,
    'ask_strength-bitstamp-orderbook-1m-0.80usd_slippage': 1890,
    'ask_strength-bitstamp-orderbook-1m-0.90usd_slippage': 1891,
    'ask_strength-bitstamp-orderbook-1m-1.00usd_slippage': 1892,
    'ask_strength-bitstamp-orderbook-1m-1.10usd_slippage': 1893,
    'ask_strength-bitstamp-orderbook-1m-1.20usd_slippage': 1894,
    'ask_strength-bitstamp-orderbook-1m-1.30usd_slippage': 1895,
    'ask_strength-bitstamp-orderbook-1m-1.40usd_slippage': 1896,
    'ask_strength-bitstamp-orderbook-1m-1.50usd_slippage': 1897,
    'ask_strength-bitstamp-orderbook-1m-1.60usd_slippage': 1898,
    'ask_strength-bitstamp-orderbook-1m-1.70usd_slippage': 1899,
    'ask_strength-bitstamp-orderbook-1m-1.80usd_slippage': 1900,
    'ask_strength-bitstamp-orderbook-1m-1.90usd_slippage': 1901,
    'ask_strength-bitstamp-orderbook-1m-2.00usd_slippage': 1902,
    'ask_strength-bitstamp-orderbook-1m-2.10usd_slippage': 1903,
    'ask_strength-bitstamp-orderbook-1m-2.20usd_slippage': 1904,
    'ask_strength-bitstamp-orderbook-1m-2.30usd_slippage': 1905,
    'ask_strength-bitstamp-orderbook-1m-2.40usd_slippage': 1906,
    'ask_strength-bitstamp-orderbook-1m-2.50usd_slippage': 1907,
    'ask_strength-bitstamp-orderbook-1m-2.60usd_slippage': 1908,
    'ask_strength-bitstamp-orderbook-1m-2.70usd_slippage': 1909,
    'ask_strength-bitstamp-orderbook-1m-2.80usd_slippage': 1910,
    'ask_strength-bitstamp-orderbook-1m-2.90usd_slippage': 1911,
    'ask_strength-bitstamp-orderbook-1m-3.00usd_slippage': 1912,
    'ask_strength-bitstamp-orderbook-1m-3.10usd_slippage': 1913,
    'ask_strength-bitstamp-orderbook-1m-3.20usd_slippage': 1914,
    'ask_strength-bitstamp-orderbook-1m-3.30usd_slippage': 1915,
    'ask_strength-bitstamp-orderbook-1m-3.40usd_slippage': 1916,
    'ask_strength-bitstamp-orderbook-1m-3.50usd_slippage': 1917,
    'ask_strength-bitstamp-orderbook-1m-3.60usd_slippage': 1918,
    'ask_strength-bitstamp-orderbook-1m-3.70usd_slippage': 1919,
    'ask_strength-bitstamp-orderbook-1m-3.80usd_slippage': 1920,
    'ask_strength-bitstamp-orderbook-1m-3.90usd_slippage': 1921,
    'ask_strength-bitstamp-orderbook-1m-4.00usd_slippage': 1922,
    'ask_strength-bitstamp-orderbook-1m-4.10usd_slippage': 1923,
    'ask_strength-bitstamp-orderbook-1m-4.20usd_slippage': 1924,
    'ask_strength-bitstamp-orderbook-1m-4.30usd_slippage': 1925,
    'ask_strength-bitstamp-orderbook-1m-4.40usd_slippage': 1926,
    'ask_strength-bitstamp-orderbook-1m-4.50usd_slippage': 1927,
    'ask_strength-bitstamp-orderbook-1m-4.60usd_slippage': 1928,
    'ask_strength-bitstamp-orderbook-1m-4.70usd_slippage': 1929,
    'ask_strength-bitstamp-orderbook-1m-4.80usd_slippage': 1930,
    'ask_strength-bitstamp-orderbook-1m-4.90usd_slippage': 1931,
    'ask_strength-bitstamp-orderbook-1m-5.00usd_slippage': 1932,
    'ask_strength-bitstamp-orderbook-1m-10.00usd_slippage': 1933,
    'ask_strength-bitstamp-orderbook-1m-20.00usd_slippage': 1934,
    'ask_strength-bitfinex-orderbook-10m-0.00usd_slippage': 1935,
    'ask_strength-bitfinex-orderbook-10m-0.10usd_slippage': 1936,
    'ask_strength-bitfinex-orderbook-10m-0.20usd_slippage': 1937,
    'ask_strength-bitfinex-orderbook-10m-0.30usd_slippage': 1938,
    'ask_strength-bitfinex-orderbook-10m-0.40usd_slippage': 1939,
    'ask_strength-bitfinex-orderbook-10m-0.50usd_slippage': 1940,
    'ask_strength-bitfinex-orderbook-10m-0.60usd_slippage': 1941,
    'ask_strength-bitfinex-orderbook-10m-0.70usd_slippage': 1942,
    'ask_strength-bitfinex-orderbook-10m-0.80usd_slippage': 1943,
    'ask_strength-bitfinex-orderbook-10m-0.90usd_slippage': 1944,
    'ask_strength-bitfinex-orderbook-10m-1.00usd_slippage': 1945,
    'ask_strength-bitfinex-orderbook-10m-1.10usd_slippage': 1946,
    'ask_strength-bitfinex-orderbook-10m-1.20usd_slippage': 1947,
    'ask_strength-bitfinex-orderbook-10m-1.30usd_slippage': 1948,
    'ask_strength-bitfinex-orderbook-10m-1.40usd_slippage': 1949,
    'ask_strength-bitfinex-orderbook-10m-1.50usd_slippage': 1950,
    'ask_strength-bitfinex-orderbook-10m-1.60usd_slippage': 1951,
    'ask_strength-bitfinex-orderbook-10m-1.70usd_slippage': 1952,
    'ask_strength-bitfinex-orderbook-10m-1.80usd_slippage': 1953,
    'ask_strength-bitfinex-orderbook-10m-1.90usd_slippage': 1954,
    'ask_strength-bitfinex-orderbook-10m-2.00usd_slippage': 1955,
    'ask_strength-bitfinex-orderbook-10m-2.10usd_slippage': 1956,
    'ask_strength-bitfinex-orderbook-10m-2.20usd_slippage': 1957,
    'ask_strength-bitfinex-orderbook-10m-2.30usd_slippage': 1958,
    'ask_strength-bitfinex-orderbook-10m-2.40usd_slippage': 1959,
    'ask_strength-bitfinex-orderbook-10m-2.50usd_slippage': 1960,
    'ask_strength-bitfinex-orderbook-10m-2.60usd_slippage': 1961,
    'ask_strength-bitfinex-orderbook-10m-2.70usd_slippage': 1962,
    'ask_strength-bitfinex-orderbook-10m-2.80usd_slippage': 1963,
    'ask_strength-bitfinex-orderbook-10m-2.90usd_slippage': 1964,
    'ask_strength-bitfinex-orderbook-10m-3.00usd_slippage': 1965,
    'ask_strength-bitfinex-orderbook-10m-3.10usd_slippage': 1966,
    'ask_strength-bitfinex-orderbook-10m-3.20usd_slippage': 1967,
    'ask_strength-bitfinex-orderbook-10m-3.30usd_slippage': 1968,
    'ask_strength-bitfinex-orderbook-10m-3.40usd_slippage': 1969,
    'ask_strength-bitfinex-orderbook-10m-3.50usd_slippage': 1970,
    'ask_strength-bitfinex-orderbook-10m-3.60usd_slippage': 1971,
    'ask_strength-bitfinex-orderbook-10m-3.70usd_slippage': 1972,
    'ask_strength-bitfinex-orderbook-10m-3.80usd_slippage': 1973,
    'ask_strength-bitfinex-orderbook-10m-3.90usd_slippage': 1974,
    'ask_strength-bitfinex-orderbook-10m-4.00usd_slippage': 1975,
    'ask_strength-bitfinex-orderbook-10m-4.10usd_slippage': 1976,
    'ask_strength-bitfinex-orderbook-10m-4.20usd_slippage': 1977,
    'ask_strength-bitfinex-orderbook-10m-4.30usd_slippage': 1978,
    'ask_strength-bitfinex-orderbook-10m-4.40usd_slippage': 1979,
    'ask_strength-bitfinex-orderbook-10m-4.50usd_slippage': 1980,
    'ask_strength-bitfinex-orderbook-10m-4.60usd_slippage': 1981,
    'ask_strength-bitfinex-orderbook-10m-4.70usd_slippage': 1982,
    'ask_strength-bitfinex-orderbook-10m-4.80usd_slippage': 1983,
    'ask_strength-bitfinex-orderbook-10m-4.90usd_slippage': 1984,
    'ask_strength-bitfinex-orderbook-10m-5.00usd_slippage': 1985,
    'ask_strength-bitfinex-orderbook-10m-10.00usd_slippage': 1986,
    'ask_strength-bitfinex-orderbook-10m-20.00usd_slippage': 1987,
    'ask_strength-bitfinex-orderbook-5m-0.00usd_slippage': 1988,
    'ask_strength-bitfinex-orderbook-5m-0.10usd_slippage': 1989,
    'ask_strength-bitfinex-orderbook-5m-0.20usd_slippage': 1990,
    'ask_strength-bitfinex-orderbook-5m-0.30usd_slippage': 1991,
    'ask_strength-bitfinex-orderbook-5m-0.40usd_slippage': 1992,
    'ask_strength-bitfinex-orderbook-5m-0.50usd_slippage': 1993,
    'ask_strength-bitfinex-orderbook-5m-0.60usd_slippage': 1994,
    'ask_strength-bitfinex-orderbook-5m-0.70usd_slippage': 1995,
    'ask_strength-bitfinex-orderbook-5m-0.80usd_slippage': 1996,
    'ask_strength-bitfinex-orderbook-5m-0.90usd_slippage': 1997,
    'ask_strength-bitfinex-orderbook-5m-1.00usd_slippage': 1998,
    'ask_strength-bitfinex-orderbook-5m-1.10usd_slippage': 1999,
    'ask_strength-bitfinex-orderbook-5m-1.20usd_slippage': 2000,
    'ask_strength-bitfinex-orderbook-5m-1.30usd_slippage': 2001,
    'ask_strength-bitfinex-orderbook-5m-1.40usd_slippage': 2002,
    'ask_strength-bitfinex-orderbook-5m-1.50usd_slippage': 2003,
    'ask_strength-bitfinex-orderbook-5m-1.60usd_slippage': 2004,
    'ask_strength-bitfinex-orderbook-5m-1.70usd_slippage': 2005,
    'ask_strength-bitfinex-orderbook-5m-1.80usd_slippage': 2006,
    'ask_strength-bitfinex-orderbook-5m-1.90usd_slippage': 2007,
    'ask_strength-bitfinex-orderbook-5m-2.00usd_slippage': 2008,
    'ask_strength-bitfinex-orderbook-5m-2.10usd_slippage': 2009,
    'ask_strength-bitfinex-orderbook-5m-2.20usd_slippage': 2010,
    'ask_strength-bitfinex-orderbook-5m-2.30usd_slippage': 2011,
    'ask_strength-bitfinex-orderbook-5m-2.40usd_slippage': 2012,
    'ask_strength-bitfinex-orderbook-5m-2.50usd_slippage': 2013,
    'ask_strength-bitfinex-orderbook-5m-2.60usd_slippage': 2014,
    'ask_strength-bitfinex-orderbook-5m-2.70usd_slippage': 2015,
    'ask_strength-bitfinex-orderbook-5m-2.80usd_slippage': 2016,
    'ask_strength-bitfinex-orderbook-5m-2.90usd_slippage': 2017,
    'ask_strength-bitfinex-orderbook-5m-3.00usd_slippage': 2018,
    'ask_strength-bitfinex-orderbook-5m-3.10usd_slippage': 2019,
    'ask_strength-bitfinex-orderbook-5m-3.20usd_slippage': 2020,
    'ask_strength-bitfinex-orderbook-5m-3.30usd_slippage': 2021,
    'ask_strength-bitfinex-orderbook-5m-3.40usd_slippage': 2022,
    'ask_strength-bitfinex-orderbook-5m-3.50usd_slippage': 2023,
    'ask_strength-bitfinex-orderbook-5m-3.60usd_slippage': 2024,
    'ask_strength-bitfinex-orderbook-5m-3.70usd_slippage': 2025,
    'ask_strength-bitfinex-orderbook-5m-3.80usd_slippage': 2026,
    'ask_strength-bitfinex-orderbook-5m-3.90usd_slippage': 2027,
    'ask_strength-bitfinex-orderbook-5m-4.00usd_slippage': 2028,
    'ask_strength-bitfinex-orderbook-5m-4.10usd_slippage': 2029,
    'ask_strength-bitfinex-orderbook-5m-4.20usd_slippage': 2030,
    'ask_strength-bitfinex-orderbook-5m-4.30usd_slippage': 2031,
    'ask_strength-bitfinex-orderbook-5m-4.40usd_slippage': 2032,
    'ask_strength-bitfinex-orderbook-5m-4.50usd_slippage': 2033,
    'ask_strength-bitfinex-orderbook-5m-4.60usd_slippage': 2034,
    'ask_strength-bitfinex-orderbook-5m-4.70usd_slippage': 2035,
    'ask_strength-bitfinex-orderbook-5m-4.80usd_slippage': 2036,
    'ask_strength-bitfinex-orderbook-5m-4.90usd_slippage': 2037,
    'ask_strength-bitfinex-orderbook-5m-5.00usd_slippage': 2038,
    'ask_strength-bitfinex-orderbook-5m-10.00usd_slippage': 2039,
    'ask_strength-bitfinex-orderbook-5m-20.00usd_slippage': 2040,
    'ask_strength-bitfinex-orderbook-1m-0.00usd_slippage': 2041,
    'ask_strength-bitfinex-orderbook-1m-0.10usd_slippage': 2042,
    'ask_strength-bitfinex-orderbook-1m-0.20usd_slippage': 2043,
    'ask_strength-bitfinex-orderbook-1m-0.30usd_slippage': 2044,
    'ask_strength-bitfinex-orderbook-1m-0.40usd_slippage': 2045,
    'ask_strength-bitfinex-orderbook-1m-0.50usd_slippage': 2046,
    'ask_strength-bitfinex-orderbook-1m-0.60usd_slippage': 2047,
    'ask_strength-bitfinex-orderbook-1m-0.70usd_slippage': 2048,
    'ask_strength-bitfinex-orderbook-1m-0.80usd_slippage': 2049,
    'ask_strength-bitfinex-orderbook-1m-0.90usd_slippage': 2050,
    'ask_strength-bitfinex-orderbook-1m-1.00usd_slippage': 2051,
    'ask_strength-bitfinex-orderbook-1m-1.10usd_slippage': 2052,
    'ask_strength-bitfinex-orderbook-1m-1.20usd_slippage': 2053,
    'ask_strength-bitfinex-orderbook-1m-1.30usd_slippage': 2054,
    'ask_strength-bitfinex-orderbook-1m-1.40usd_slippage': 2055,
    'ask_strength-bitfinex-orderbook-1m-1.50usd_slippage': 2056,
    'ask_strength-bitfinex-orderbook-1m-1.60usd_slippage': 2057,
    'ask_strength-bitfinex-orderbook-1m-1.70usd_slippage': 2058,
    'ask_strength-bitfinex-orderbook-1m-1.80usd_slippage': 2059,
    'ask_strength-bitfinex-orderbook-1m-1.90usd_slippage': 2060,
    'ask_strength-bitfinex-orderbook-1m-2.00usd_slippage': 2061,
    'ask_strength-bitfinex-orderbook-1m-2.10usd_slippage': 2062,
    'ask_strength-bitfinex-orderbook-1m-2.20usd_slippage': 2063,
    'ask_strength-bitfinex-orderbook-1m-2.30usd_slippage': 2064,
    'ask_strength-bitfinex-orderbook-1m-2.40usd_slippage': 2065,
    'ask_strength-bitfinex-orderbook-1m-2.50usd_slippage': 2066,
    'ask_strength-bitfinex-orderbook-1m-2.60usd_slippage': 2067,
    'ask_strength-bitfinex-orderbook-1m-2.70usd_slippage': 2068,
    'ask_strength-bitfinex-orderbook-1m-2.80usd_slippage': 2069,
    'ask_strength-bitfinex-orderbook-1m-2.90usd_slippage': 2070,
    'ask_strength-bitfinex-orderbook-1m-3.00usd_slippage': 2071,
    'ask_strength-bitfinex-orderbook-1m-3.10usd_slippage': 2072,
    'ask_strength-bitfinex-orderbook-1m-3.20usd_slippage': 2073,
    'ask_strength-bitfinex-orderbook-1m-3.30usd_slippage': 2074,
    'ask_strength-bitfinex-orderbook-1m-3.40usd_slippage': 2075,
    'ask_strength-bitfinex-orderbook-1m-3.50usd_slippage': 2076,
    'ask_strength-bitfinex-orderbook-1m-3.60usd_slippage': 2077,
    'ask_strength-bitfinex-orderbook-1m-3.70usd_slippage': 2078,
    'ask_strength-bitfinex-orderbook-1m-3.80usd_slippage': 2079,
    'ask_strength-bitfinex-orderbook-1m-3.90usd_slippage': 2080,
    'ask_strength-bitfinex-orderbook-1m-4.00usd_slippage': 2081,
    'ask_strength-bitfinex-orderbook-1m-4.10usd_slippage': 2082,
    'ask_strength-bitfinex-orderbook-1m-4.20usd_slippage': 2083,
    'ask_strength-bitfinex-orderbook-1m-4.30usd_slippage': 2084,
    'ask_strength-bitfinex-orderbook-1m-4.40usd_slippage': 2085,
    'ask_strength-bitfinex-orderbook-1m-4.50usd_slippage': 2086,
    'ask_strength-bitfinex-orderbook-1m-4.60usd_slippage': 2087,
    'ask_strength-bitfinex-orderbook-1m-4.70usd_slippage': 2088,
    'ask_strength-bitfinex-orderbook-1m-4.80usd_slippage': 2089,
    'ask_strength-bitfinex-orderbook-1m-4.90usd_slippage': 2090,
    'ask_strength-bitfinex-orderbook-1m-5.00usd_slippage': 2091,
    'ask_strength-bitfinex-orderbook-1m-10.00usd_slippage': 2092,
    'ask_strength-bitfinex-orderbook-1m-20.00usd_slippage': 2093,
    'ask_strength-coinbase-orderbook-10m-0.00usd_slippage': 2094,
    'ask_strength-coinbase-orderbook-10m-0.10usd_slippage': 2095,
    'ask_strength-coinbase-orderbook-10m-0.20usd_slippage': 2096,
    'ask_strength-coinbase-orderbook-10m-0.30usd_slippage': 2097,
    'ask_strength-coinbase-orderbook-10m-0.40usd_slippage': 2098,
    'ask_strength-coinbase-orderbook-10m-0.50usd_slippage': 2099,
    'ask_strength-coinbase-orderbook-10m-0.60usd_slippage': 2100,
    'ask_strength-coinbase-orderbook-10m-0.70usd_slippage': 2101,
    'ask_strength-coinbase-orderbook-10m-0.80usd_slippage': 2102,
    'ask_strength-coinbase-orderbook-10m-0.90usd_slippage': 2103,
    'ask_strength-coinbase-orderbook-10m-1.00usd_slippage': 2104,
    'ask_strength-coinbase-orderbook-10m-1.10usd_slippage': 2105,
    'ask_strength-coinbase-orderbook-10m-1.20usd_slippage': 2106,
    'ask_strength-coinbase-orderbook-10m-1.30usd_slippage': 2107,
    'ask_strength-coinbase-orderbook-10m-1.40usd_slippage': 2108,
    'ask_strength-coinbase-orderbook-10m-1.50usd_slippage': 2109,
    'ask_strength-coinbase-orderbook-10m-1.60usd_slippage': 2110,
    'ask_strength-coinbase-orderbook-10m-1.70usd_slippage': 2111,
    'ask_strength-coinbase-orderbook-10m-1.80usd_slippage': 2112,
    'ask_strength-coinbase-orderbook-10m-1.90usd_slippage': 2113,
    'ask_strength-coinbase-orderbook-10m-2.00usd_slippage': 2114,
    'ask_strength-coinbase-orderbook-10m-2.10usd_slippage': 2115,
    'ask_strength-coinbase-orderbook-10m-2.20usd_slippage': 2116,
    'ask_strength-coinbase-orderbook-10m-2.30usd_slippage': 2117,
    'ask_strength-coinbase-orderbook-10m-2.40usd_slippage': 2118,
    'ask_strength-coinbase-orderbook-10m-2.50usd_slippage': 2119,
    'ask_strength-coinbase-orderbook-10m-2.60usd_slippage': 2120,
    'ask_strength-coinbase-orderbook-10m-2.70usd_slippage': 2121,
    'ask_strength-coinbase-orderbook-10m-2.80usd_slippage': 2122,
    'ask_strength-coinbase-orderbook-10m-2.90usd_slippage': 2123,
    'ask_strength-coinbase-orderbook-10m-3.00usd_slippage': 2124,
    'ask_strength-coinbase-orderbook-10m-3.10usd_slippage': 2125,
    'ask_strength-coinbase-orderbook-10m-3.20usd_slippage': 2126,
    'ask_strength-coinbase-orderbook-10m-3.30usd_slippage': 2127,
    'ask_strength-coinbase-orderbook-10m-3.40usd_slippage': 2128,
    'ask_strength-coinbase-orderbook-10m-3.50usd_slippage': 2129,
    'ask_strength-coinbase-orderbook-10m-3.60usd_slippage': 2130,
    'ask_strength-coinbase-orderbook-10m-3.70usd_slippage': 2131,
    'ask_strength-coinbase-orderbook-10m-3.80usd_slippage': 2132,
    'ask_strength-coinbase-orderbook-10m-3.90usd_slippage': 2133,
    'ask_strength-coinbase-orderbook-10m-4.00usd_slippage': 2134,
    'ask_strength-coinbase-orderbook-10m-4.10usd_slippage': 2135,
    'ask_strength-coinbase-orderbook-10m-4.20usd_slippage': 2136,
    'ask_strength-coinbase-orderbook-10m-4.30usd_slippage': 2137,
    'ask_strength-coinbase-orderbook-10m-4.40usd_slippage': 2138,
    'ask_strength-coinbase-orderbook-10m-4.50usd_slippage': 2139,
    'ask_strength-coinbase-orderbook-10m-4.60usd_slippage': 2140,
    'ask_strength-coinbase-orderbook-10m-4.70usd_slippage': 2141,
    'ask_strength-coinbase-orderbook-10m-4.80usd_slippage': 2142,
    'ask_strength-coinbase-orderbook-10m-4.90usd_slippage': 2143,
    'ask_strength-coinbase-orderbook-10m-5.00usd_slippage': 2144,
    'ask_strength-coinbase-orderbook-10m-10.00usd_slippage': 2145,
    'ask_strength-coinbase-orderbook-10m-20.00usd_slippage': 2146,
    'ask_strength-coinbase-orderbook-5m-0.00usd_slippage': 2147,
    'ask_strength-coinbase-orderbook-5m-0.10usd_slippage': 2148,
    'ask_strength-coinbase-orderbook-5m-0.20usd_slippage': 2149,
    'ask_strength-coinbase-orderbook-5m-0.30usd_slippage': 2150,
    'ask_strength-coinbase-orderbook-5m-0.40usd_slippage': 2151,
    'ask_strength-coinbase-orderbook-5m-0.50usd_slippage': 2152,
    'ask_strength-coinbase-orderbook-5m-0.60usd_slippage': 2153,
    'ask_strength-coinbase-orderbook-5m-0.70usd_slippage': 2154,
    'ask_strength-coinbase-orderbook-5m-0.80usd_slippage': 2155,
    'ask_strength-coinbase-orderbook-5m-0.90usd_slippage': 2156,
    'ask_strength-coinbase-orderbook-5m-1.00usd_slippage': 2157,
    'ask_strength-coinbase-orderbook-5m-1.10usd_slippage': 2158,
    'ask_strength-coinbase-orderbook-5m-1.20usd_slippage': 2159,
    'ask_strength-coinbase-orderbook-5m-1.30usd_slippage': 2160,
    'ask_strength-coinbase-orderbook-5m-1.40usd_slippage': 2161,
    'ask_strength-coinbase-orderbook-5m-1.50usd_slippage': 2162,
    'ask_strength-coinbase-orderbook-5m-1.60usd_slippage': 2163,
    'ask_strength-coinbase-orderbook-5m-1.70usd_slippage': 2164,
    'ask_strength-coinbase-orderbook-5m-1.80usd_slippage': 2165,
    'ask_strength-coinbase-orderbook-5m-1.90usd_slippage': 2166,
    'ask_strength-coinbase-orderbook-5m-2.00usd_slippage': 2167,
    'ask_strength-coinbase-orderbook-5m-2.10usd_slippage': 2168,
    'ask_strength-coinbase-orderbook-5m-2.20usd_slippage': 2169,
    'ask_strength-coinbase-orderbook-5m-2.30usd_slippage': 2170,
    'ask_strength-coinbase-orderbook-5m-2.40usd_slippage': 2171,
    'ask_strength-coinbase-orderbook-5m-2.50usd_slippage': 2172,
    'ask_strength-coinbase-orderbook-5m-2.60usd_slippage': 2173,
    'ask_strength-coinbase-orderbook-5m-2.70usd_slippage': 2174,
    'ask_strength-coinbase-orderbook-5m-2.80usd_slippage': 2175,
    'ask_strength-coinbase-orderbook-5m-2.90usd_slippage': 2176,
    'ask_strength-coinbase-orderbook-5m-3.00usd_slippage': 2177,
    'ask_strength-coinbase-orderbook-5m-3.10usd_slippage': 2178,
    'ask_strength-coinbase-orderbook-5m-3.20usd_slippage': 2179,
    'ask_strength-coinbase-orderbook-5m-3.30usd_slippage': 2180,
    'ask_strength-coinbase-orderbook-5m-3.40usd_slippage': 2181,
    'ask_strength-coinbase-orderbook-5m-3.50usd_slippage': 2182,
    'ask_strength-coinbase-orderbook-5m-3.60usd_slippage': 2183,
    'ask_strength-coinbase-orderbook-5m-3.70usd_slippage': 2184,
    'ask_strength-coinbase-orderbook-5m-3.80usd_slippage': 2185,
    'ask_strength-coinbase-orderbook-5m-3.90usd_slippage': 2186,
    'ask_strength-coinbase-orderbook-5m-4.00usd_slippage': 2187,
    'ask_strength-coinbase-orderbook-5m-4.10usd_slippage': 2188,
    'ask_strength-coinbase-orderbook-5m-4.20usd_slippage': 2189,
    'ask_strength-coinbase-orderbook-5m-4.30usd_slippage': 2190,
    'ask_strength-coinbase-orderbook-5m-4.40usd_slippage': 2191,
    'ask_strength-coinbase-orderbook-5m-4.50usd_slippage': 2192,
    'ask_strength-coinbase-orderbook-5m-4.60usd_slippage': 2193,
    'ask_strength-coinbase-orderbook-5m-4.70usd_slippage': 2194,
    'ask_strength-coinbase-orderbook-5m-4.80usd_slippage': 2195,
    'ask_strength-coinbase-orderbook-5m-4.90usd_slippage': 2196,
    'ask_strength-coinbase-orderbook-5m-5.00usd_slippage': 2197,
    'ask_strength-coinbase-orderbook-5m-10.00usd_slippage': 2198,
    'ask_strength-coinbase-orderbook-5m-20.00usd_slippage': 2199,
    'ask_strength-coinbase-orderbook-1m-0.00usd_slippage': 2200,
    'ask_strength-coinbase-orderbook-1m-0.10usd_slippage': 2201,
    'ask_strength-coinbase-orderbook-1m-0.20usd_slippage': 2202,
    'ask_strength-coinbase-orderbook-1m-0.30usd_slippage': 2203,
    'ask_strength-coinbase-orderbook-1m-0.40usd_slippage': 2204,
    'ask_strength-coinbase-orderbook-1m-0.50usd_slippage': 2205,
    'ask_strength-coinbase-orderbook-1m-0.60usd_slippage': 2206,
    'ask_strength-coinbase-orderbook-1m-0.70usd_slippage': 2207,
    'ask_strength-coinbase-orderbook-1m-0.80usd_slippage': 2208,
    'ask_strength-coinbase-orderbook-1m-0.90usd_slippage': 2209,
    'ask_strength-coinbase-orderbook-1m-1.00usd_slippage': 2210,
    'ask_strength-coinbase-orderbook-1m-1.10usd_slippage': 2211,
    'ask_strength-coinbase-orderbook-1m-1.20usd_slippage': 2212,
    'ask_strength-coinbase-orderbook-1m-1.30usd_slippage': 2213,
    'ask_strength-coinbase-orderbook-1m-1.40usd_slippage': 2214,
    'ask_strength-coinbase-orderbook-1m-1.50usd_slippage': 2215,
    'ask_strength-coinbase-orderbook-1m-1.60usd_slippage': 2216,
    'ask_strength-coinbase-orderbook-1m-1.70usd_slippage': 2217,
    'ask_strength-coinbase-orderbook-1m-1.80usd_slippage': 2218,
    'ask_strength-coinbase-orderbook-1m-1.90usd_slippage': 2219,
    'ask_strength-coinbase-orderbook-1m-2.00usd_slippage': 2220,
    'ask_strength-coinbase-orderbook-1m-2.10usd_slippage': 2221,
    'ask_strength-coinbase-orderbook-1m-2.20usd_slippage': 2222,
    'ask_strength-coinbase-orderbook-1m-2.30usd_slippage': 2223,
    'ask_strength-coinbase-orderbook-1m-2.40usd_slippage': 2224,
    'ask_strength-coinbase-orderbook-1m-2.50usd_slippage': 2225,
    'ask_strength-coinbase-orderbook-1m-2.60usd_slippage': 2226,
    'ask_strength-coinbase-orderbook-1m-2.70usd_slippage': 2227,
    'ask_strength-coinbase-orderbook-1m-2.80usd_slippage': 2228,
    'ask_strength-coinbase-orderbook-1m-2.90usd_slippage': 2229,
    'ask_strength-coinbase-orderbook-1m-3.00usd_slippage': 2230,
    'ask_strength-coinbase-orderbook-1m-3.10usd_slippage': 2231,
    'ask_strength-coinbase-orderbook-1m-3.20usd_slippage': 2232,
    'ask_strength-coinbase-orderbook-1m-3.30usd_slippage': 2233,
    'ask_strength-coinbase-orderbook-1m-3.40usd_slippage': 2234,
    'ask_strength-coinbase-orderbook-1m-3.50usd_slippage': 2235,
    'ask_strength-coinbase-orderbook-1m-3.60usd_slippage': 2236,
    'ask_strength-coinbase-orderbook-1m-3.70usd_slippage': 2237,
    'ask_strength-coinbase-orderbook-1m-3.80usd_slippage': 2238,
    'ask_strength-coinbase-orderbook-1m-3.90usd_slippage': 2239,
    'ask_strength-coinbase-orderbook-1m-4.00usd_slippage': 2240,
    'ask_strength-coinbase-orderbook-1m-4.10usd_slippage': 2241,
    'ask_strength-coinbase-orderbook-1m-4.20usd_slippage': 2242,
    'ask_strength-coinbase-orderbook-1m-4.30usd_slippage': 2243,
    'ask_strength-coinbase-orderbook-1m-4.40usd_slippage': 2244,
    'ask_strength-coinbase-orderbook-1m-4.50usd_slippage': 2245,
    'ask_strength-coinbase-orderbook-1m-4.60usd_slippage': 2246,
    'ask_strength-coinbase-orderbook-1m-4.70usd_slippage': 2247,
    'ask_strength-coinbase-orderbook-1m-4.80usd_slippage': 2248,
    'ask_strength-coinbase-orderbook-1m-4.90usd_slippage': 2249,
    'ask_strength-coinbase-orderbook-1m-5.00usd_slippage': 2250,
    'ask_strength-coinbase-orderbook-1m-10.00usd_slippage': 2251,
    'ask_strength-coinbase-orderbook-1m-20.00usd_slippage': 2252,
    'ask_strength-itbit-orderbook-10m-0.00usd_slippage': 2253,
    'ask_strength-itbit-orderbook-10m-0.10usd_slippage': 2254,
    'ask_strength-itbit-orderbook-10m-0.20usd_slippage': 2255,
    'ask_strength-itbit-orderbook-10m-0.30usd_slippage': 2256,
    'ask_strength-itbit-orderbook-10m-0.40usd_slippage': 2257,
    'ask_strength-itbit-orderbook-10m-0.50usd_slippage': 2258,
    'ask_strength-itbit-orderbook-10m-0.60usd_slippage': 2259,
    'ask_strength-itbit-orderbook-10m-0.70usd_slippage': 2260,
    'ask_strength-itbit-orderbook-10m-0.80usd_slippage': 2261,
    'ask_strength-itbit-orderbook-10m-0.90usd_slippage': 2262,
    'ask_strength-itbit-orderbook-10m-1.00usd_slippage': 2263,
    'ask_strength-itbit-orderbook-10m-1.10usd_slippage': 2264,
    'ask_strength-itbit-orderbook-10m-1.20usd_slippage': 2265,
    'ask_strength-itbit-orderbook-10m-1.30usd_slippage': 2266,
    'ask_strength-itbit-orderbook-10m-1.40usd_slippage': 2267,
    'ask_strength-itbit-orderbook-10m-1.50usd_slippage': 2268,
    'ask_strength-itbit-orderbook-10m-1.60usd_slippage': 2269,
    'ask_strength-itbit-orderbook-10m-1.70usd_slippage': 2270,
    'ask_strength-itbit-orderbook-10m-1.80usd_slippage': 2271,
    'ask_strength-itbit-orderbook-10m-1.90usd_slippage': 2272,
    'ask_strength-itbit-orderbook-10m-2.00usd_slippage': 2273,
    'ask_strength-itbit-orderbook-10m-2.10usd_slippage': 2274,
    'ask_strength-itbit-orderbook-10m-2.20usd_slippage': 2275,
    'ask_strength-itbit-orderbook-10m-2.30usd_slippage': 2276,
    'ask_strength-itbit-orderbook-10m-2.40usd_slippage': 2277,
    'ask_strength-itbit-orderbook-10m-2.50usd_slippage': 2278,
    'ask_strength-itbit-orderbook-10m-2.60usd_slippage': 2279,
    'ask_strength-itbit-orderbook-10m-2.70usd_slippage': 2280,
    'ask_strength-itbit-orderbook-10m-2.80usd_slippage': 2281,
    'ask_strength-itbit-orderbook-10m-2.90usd_slippage': 2282,
    'ask_strength-itbit-orderbook-10m-3.00usd_slippage': 2283,
    'ask_strength-itbit-orderbook-10m-3.10usd_slippage': 2284,
    'ask_strength-itbit-orderbook-10m-3.20usd_slippage': 2285,
    'ask_strength-itbit-orderbook-10m-3.30usd_slippage': 2286,
    'ask_strength-itbit-orderbook-10m-3.40usd_slippage': 2287,
    'ask_strength-itbit-orderbook-10m-3.50usd_slippage': 2288,
    'ask_strength-itbit-orderbook-10m-3.60usd_slippage': 2289,
    'ask_strength-itbit-orderbook-10m-3.70usd_slippage': 2290,
    'ask_strength-itbit-orderbook-10m-3.80usd_slippage': 2291,
    'ask_strength-itbit-orderbook-10m-3.90usd_slippage': 2292,
    'ask_strength-itbit-orderbook-10m-4.00usd_slippage': 2293,
    'ask_strength-itbit-orderbook-10m-4.10usd_slippage': 2294,
    'ask_strength-itbit-orderbook-10m-4.20usd_slippage': 2295,
    'ask_strength-itbit-orderbook-10m-4.30usd_slippage': 2296,
    'ask_strength-itbit-orderbook-10m-4.40usd_slippage': 2297,
    'ask_strength-itbit-orderbook-10m-4.50usd_slippage': 2298,
    'ask_strength-itbit-orderbook-10m-4.60usd_slippage': 2299,
    'ask_strength-itbit-orderbook-10m-4.70usd_slippage': 2300,
    'ask_strength-itbit-orderbook-10m-4.80usd_slippage': 2301,
    'ask_strength-itbit-orderbook-10m-4.90usd_slippage': 2302,
    'ask_strength-itbit-orderbook-10m-5.00usd_slippage': 2303,
    'ask_strength-itbit-orderbook-10m-10.00usd_slippage': 2304,
    'ask_strength-itbit-orderbook-10m-20.00usd_slippage': 2305,
    'ask_strength-itbit-orderbook-5m-0.00usd_slippage': 2306,
    'ask_strength-itbit-orderbook-5m-0.10usd_slippage': 2307,
    'ask_strength-itbit-orderbook-5m-0.20usd_slippage': 2308,
    'ask_strength-itbit-orderbook-5m-0.30usd_slippage': 2309,
    'ask_strength-itbit-orderbook-5m-0.40usd_slippage': 2310,
    'ask_strength-itbit-orderbook-5m-0.50usd_slippage': 2311,
    'ask_strength-itbit-orderbook-5m-0.60usd_slippage': 2312,
    'ask_strength-itbit-orderbook-5m-0.70usd_slippage': 2313,
    'ask_strength-itbit-orderbook-5m-0.80usd_slippage': 2314,
    'ask_strength-itbit-orderbook-5m-0.90usd_slippage': 2315,
    'ask_strength-itbit-orderbook-5m-1.00usd_slippage': 2316,
    'ask_strength-itbit-orderbook-5m-1.10usd_slippage': 2317,
    'ask_strength-itbit-orderbook-5m-1.20usd_slippage': 2318,
    'ask_strength-itbit-orderbook-5m-1.30usd_slippage': 2319,
    'ask_strength-itbit-orderbook-5m-1.40usd_slippage': 2320,
    'ask_strength-itbit-orderbook-5m-1.50usd_slippage': 2321,
    'ask_strength-itbit-orderbook-5m-1.60usd_slippage': 2322,
    'ask_strength-itbit-orderbook-5m-1.70usd_slippage': 2323,
    'ask_strength-itbit-orderbook-5m-1.80usd_slippage': 2324,
    'ask_strength-itbit-orderbook-5m-1.90usd_slippage': 2325,
    'ask_strength-itbit-orderbook-5m-2.00usd_slippage': 2326,
    'ask_strength-itbit-orderbook-5m-2.10usd_slippage': 2327,
    'ask_strength-itbit-orderbook-5m-2.20usd_slippage': 2328,
    'ask_strength-itbit-orderbook-5m-2.30usd_slippage': 2329,
    'ask_strength-itbit-orderbook-5m-2.40usd_slippage': 2330,
    'ask_strength-itbit-orderbook-5m-2.50usd_slippage': 2331,
    'ask_strength-itbit-orderbook-5m-2.60usd_slippage': 2332,
    'ask_strength-itbit-orderbook-5m-2.70usd_slippage': 2333,
    'ask_strength-itbit-orderbook-5m-2.80usd_slippage': 2334,
    'ask_strength-itbit-orderbook-5m-2.90usd_slippage': 2335,
    'ask_strength-itbit-orderbook-5m-3.00usd_slippage': 2336,
    'ask_strength-itbit-orderbook-5m-3.10usd_slippage': 2337,
    'ask_strength-itbit-orderbook-5m-3.20usd_slippage': 2338,
    'ask_strength-itbit-orderbook-5m-3.30usd_slippage': 2339,
    'ask_strength-itbit-orderbook-5m-3.40usd_slippage': 2340,
    'ask_strength-itbit-orderbook-5m-3.50usd_slippage': 2341,
    'ask_strength-itbit-orderbook-5m-3.60usd_slippage': 2342,
    'ask_strength-itbit-orderbook-5m-3.70usd_slippage': 2343,
    'ask_strength-itbit-orderbook-5m-3.80usd_slippage': 2344,
    'ask_strength-itbit-orderbook-5m-3.90usd_slippage': 2345,
    'ask_strength-itbit-orderbook-5m-4.00usd_slippage': 2346,
    'ask_strength-itbit-orderbook-5m-4.10usd_slippage': 2347,
    'ask_strength-itbit-orderbook-5m-4.20usd_slippage': 2348,
    'ask_strength-itbit-orderbook-5m-4.30usd_slippage': 2349,
    'ask_strength-itbit-orderbook-5m-4.40usd_slippage': 2350,
    'ask_strength-itbit-orderbook-5m-4.50usd_slippage': 2351,
    'ask_strength-itbit-orderbook-5m-4.60usd_slippage': 2352,
    'ask_strength-itbit-orderbook-5m-4.70usd_slippage': 2353,
    'ask_strength-itbit-orderbook-5m-4.80usd_slippage': 2354,
    'ask_strength-itbit-orderbook-5m-4.90usd_slippage': 2355,
    'ask_strength-itbit-orderbook-5m-5.00usd_slippage': 2356,
    'ask_strength-itbit-orderbook-5m-10.00usd_slippage': 2357,
    'ask_strength-itbit-orderbook-5m-20.00usd_slippage': 2358,
    'ask_strength-itbit-orderbook-1m-0.00usd_slippage': 2359,
    'ask_strength-itbit-orderbook-1m-0.10usd_slippage': 2360,
    'ask_strength-itbit-orderbook-1m-0.20usd_slippage': 2361,
    'ask_strength-itbit-orderbook-1m-0.30usd_slippage': 2362,
    'ask_strength-itbit-orderbook-1m-0.40usd_slippage': 2363,
    'ask_strength-itbit-orderbook-1m-0.50usd_slippage': 2364,
    'ask_strength-itbit-orderbook-1m-0.60usd_slippage': 2365,
    'ask_strength-itbit-orderbook-1m-0.70usd_slippage': 2366,
    'ask_strength-itbit-orderbook-1m-0.80usd_slippage': 2367,
    'ask_strength-itbit-orderbook-1m-0.90usd_slippage': 2368,
    'ask_strength-itbit-orderbook-1m-1.00usd_slippage': 2369,
    'ask_strength-itbit-orderbook-1m-1.10usd_slippage': 2370,
    'ask_strength-itbit-orderbook-1m-1.20usd_slippage': 2371,
    'ask_strength-itbit-orderbook-1m-1.30usd_slippage': 2372,
    'ask_strength-itbit-orderbook-1m-1.40usd_slippage': 2373,
    'ask_strength-itbit-orderbook-1m-1.50usd_slippage': 2374,
    'ask_strength-itbit-orderbook-1m-1.60usd_slippage': 2375,
    'ask_strength-itbit-orderbook-1m-1.70usd_slippage': 2376,
    'ask_strength-itbit-orderbook-1m-1.80usd_slippage': 2377,
    'ask_strength-itbit-orderbook-1m-1.90usd_slippage': 2378,
    'ask_strength-itbit-orderbook-1m-2.00usd_slippage': 2379,
    'ask_strength-itbit-orderbook-1m-2.10usd_slippage': 2380,
    'ask_strength-itbit-orderbook-1m-2.20usd_slippage': 2381,
    'ask_strength-itbit-orderbook-1m-2.30usd_slippage': 2382,
    'ask_strength-itbit-orderbook-1m-2.40usd_slippage': 2383,
    'ask_strength-itbit-orderbook-1m-2.50usd_slippage': 2384,
    'ask_strength-itbit-orderbook-1m-2.60usd_slippage': 2385,
    'ask_strength-itbit-orderbook-1m-2.70usd_slippage': 2386,
    'ask_strength-itbit-orderbook-1m-2.80usd_slippage': 2387,
    'ask_strength-itbit-orderbook-1m-2.90usd_slippage': 2388,
    'ask_strength-itbit-orderbook-1m-3.00usd_slippage': 2389,
    'ask_strength-itbit-orderbook-1m-3.10usd_slippage': 2390,
    'ask_strength-itbit-orderbook-1m-3.20usd_slippage': 2391,
    'ask_strength-itbit-orderbook-1m-3.30usd_slippage': 2392,
    'ask_strength-itbit-orderbook-1m-3.40usd_slippage': 2393,
    'ask_strength-itbit-orderbook-1m-3.50usd_slippage': 2394,
    'ask_strength-itbit-orderbook-1m-3.60usd_slippage': 2395,
    'ask_strength-itbit-orderbook-1m-3.70usd_slippage': 2396,
    'ask_strength-itbit-orderbook-1m-3.80usd_slippage': 2397,
    'ask_strength-itbit-orderbook-1m-3.90usd_slippage': 2398,
    'ask_strength-itbit-orderbook-1m-4.00usd_slippage': 2399,
    'ask_strength-itbit-orderbook-1m-4.10usd_slippage': 2400,
    'ask_strength-itbit-orderbook-1m-4.20usd_slippage': 2401,
    'ask_strength-itbit-orderbook-1m-4.30usd_slippage': 2402,
    'ask_strength-itbit-orderbook-1m-4.40usd_slippage': 2403,
    'ask_strength-itbit-orderbook-1m-4.50usd_slippage': 2404,
    'ask_strength-itbit-orderbook-1m-4.60usd_slippage': 2405,
    'ask_strength-itbit-orderbook-1m-4.70usd_slippage': 2406,
    'ask_strength-itbit-orderbook-1m-4.80usd_slippage': 2407,
    'ask_strength-itbit-orderbook-1m-4.90usd_slippage': 2408,
    'ask_strength-itbit-orderbook-1m-5.00usd_slippage': 2409,
    'ask_strength-itbit-orderbook-1m-10.00usd_slippage': 2410,
    'ask_strength-itbit-orderbook-1m-20.00usd_slippage': 2411,
    'ask_strength-kraken-orderbook-10m-0.00usd_slippage': 2412,
    'ask_strength-kraken-orderbook-10m-0.10usd_slippage': 2413,
    'ask_strength-kraken-orderbook-10m-0.20usd_slippage': 2414,
    'ask_strength-kraken-orderbook-10m-0.30usd_slippage': 2415,
    'ask_strength-kraken-orderbook-10m-0.40usd_slippage': 2416,
    'ask_strength-kraken-orderbook-10m-0.50usd_slippage': 2417,
    'ask_strength-kraken-orderbook-10m-0.60usd_slippage': 2418,
    'ask_strength-kraken-orderbook-10m-0.70usd_slippage': 2419,
    'ask_strength-kraken-orderbook-10m-0.80usd_slippage': 2420,
    'ask_strength-kraken-orderbook-10m-0.90usd_slippage': 2421,
    'ask_strength-kraken-orderbook-10m-1.00usd_slippage': 2422,
    'ask_strength-kraken-orderbook-10m-1.10usd_slippage': 2423,
    'ask_strength-kraken-orderbook-10m-1.20usd_slippage': 2424,
    'ask_strength-kraken-orderbook-10m-1.30usd_slippage': 2425,
    'ask_strength-kraken-orderbook-10m-1.40usd_slippage': 2426,
    'ask_strength-kraken-orderbook-10m-1.50usd_slippage': 2427,
    'ask_strength-kraken-orderbook-10m-1.60usd_slippage': 2428,
    'ask_strength-kraken-orderbook-10m-1.70usd_slippage': 2429,
    'ask_strength-kraken-orderbook-10m-1.80usd_slippage': 2430,
    'ask_strength-kraken-orderbook-10m-1.90usd_slippage': 2431,
    'ask_strength-kraken-orderbook-10m-2.00usd_slippage': 2432,
    'ask_strength-kraken-orderbook-10m-2.10usd_slippage': 2433,
    'ask_strength-kraken-orderbook-10m-2.20usd_slippage': 2434,
    'ask_strength-kraken-orderbook-10m-2.30usd_slippage': 2435,
    'ask_strength-kraken-orderbook-10m-2.40usd_slippage': 2436,
    'ask_strength-kraken-orderbook-10m-2.50usd_slippage': 2437,
    'ask_strength-kraken-orderbook-10m-2.60usd_slippage': 2438,
    'ask_strength-kraken-orderbook-10m-2.70usd_slippage': 2439,
    'ask_strength-kraken-orderbook-10m-2.80usd_slippage': 2440,
    'ask_strength-kraken-orderbook-10m-2.90usd_slippage': 2441,
    'ask_strength-kraken-orderbook-10m-3.00usd_slippage': 2442,
    'ask_strength-kraken-orderbook-10m-3.10usd_slippage': 2443,
    'ask_strength-kraken-orderbook-10m-3.20usd_slippage': 2444,
    'ask_strength-kraken-orderbook-10m-3.30usd_slippage': 2445,
    'ask_strength-kraken-orderbook-10m-3.40usd_slippage': 2446,
    'ask_strength-kraken-orderbook-10m-3.50usd_slippage': 2447,
    'ask_strength-kraken-orderbook-10m-3.60usd_slippage': 2448,
    'ask_strength-kraken-orderbook-10m-3.70usd_slippage': 2449,
    'ask_strength-kraken-orderbook-10m-3.80usd_slippage': 2450,
    'ask_strength-kraken-orderbook-10m-3.90usd_slippage': 2451,
    'ask_strength-kraken-orderbook-10m-4.00usd_slippage': 2452,
    'ask_strength-kraken-orderbook-10m-4.10usd_slippage': 2453,
    'ask_strength-kraken-orderbook-10m-4.20usd_slippage': 2454,
    'ask_strength-kraken-orderbook-10m-4.30usd_slippage': 2455,
    'ask_strength-kraken-orderbook-10m-4.40usd_slippage': 2456,
    'ask_strength-kraken-orderbook-10m-4.50usd_slippage': 2457,
    'ask_strength-kraken-orderbook-10m-4.60usd_slippage': 2458,
    'ask_strength-kraken-orderbook-10m-4.70usd_slippage': 2459,
    'ask_strength-kraken-orderbook-10m-4.80usd_slippage': 2460,
    'ask_strength-kraken-orderbook-10m-4.90usd_slippage': 2461,
    'ask_strength-kraken-orderbook-10m-5.00usd_slippage': 2462,
    'ask_strength-kraken-orderbook-10m-10.00usd_slippage': 2463,
    'ask_strength-kraken-orderbook-10m-20.00usd_slippage': 2464,
    'ask_strength-kraken-orderbook-5m-0.00usd_slippage': 2465,
    'ask_strength-kraken-orderbook-5m-0.10usd_slippage': 2466,
    'ask_strength-kraken-orderbook-5m-0.20usd_slippage': 2467,
    'ask_strength-kraken-orderbook-5m-0.30usd_slippage': 2468,
    'ask_strength-kraken-orderbook-5m-0.40usd_slippage': 2469,
    'ask_strength-kraken-orderbook-5m-0.50usd_slippage': 2470,
    'ask_strength-kraken-orderbook-5m-0.60usd_slippage': 2471,
    'ask_strength-kraken-orderbook-5m-0.70usd_slippage': 2472,
    'ask_strength-kraken-orderbook-5m-0.80usd_slippage': 2473,
    'ask_strength-kraken-orderbook-5m-0.90usd_slippage': 2474,
    'ask_strength-kraken-orderbook-5m-1.00usd_slippage': 2475,
    'ask_strength-kraken-orderbook-5m-1.10usd_slippage': 2476,
    'ask_strength-kraken-orderbook-5m-1.20usd_slippage': 2477,
    'ask_strength-kraken-orderbook-5m-1.30usd_slippage': 2478,
    'ask_strength-kraken-orderbook-5m-1.40usd_slippage': 2479,
    'ask_strength-kraken-orderbook-5m-1.50usd_slippage': 2480,
    'ask_strength-kraken-orderbook-5m-1.60usd_slippage': 2481,
    'ask_strength-kraken-orderbook-5m-1.70usd_slippage': 2482,
    'ask_strength-kraken-orderbook-5m-1.80usd_slippage': 2483,
    'ask_strength-kraken-orderbook-5m-1.90usd_slippage': 2484,
    'ask_strength-kraken-orderbook-5m-2.00usd_slippage': 2485,
    'ask_strength-kraken-orderbook-5m-2.10usd_slippage': 2486,
    'ask_strength-kraken-orderbook-5m-2.20usd_slippage': 2487,
    'ask_strength-kraken-orderbook-5m-2.30usd_slippage': 2488,
    'ask_strength-kraken-orderbook-5m-2.40usd_slippage': 2489,
    'ask_strength-kraken-orderbook-5m-2.50usd_slippage': 2490,
    'ask_strength-kraken-orderbook-5m-2.60usd_slippage': 2491,
    'ask_strength-kraken-orderbook-5m-2.70usd_slippage': 2492,
    'ask_strength-kraken-orderbook-5m-2.80usd_slippage': 2493,
    'ask_strength-kraken-orderbook-5m-2.90usd_slippage': 2494,
    'ask_strength-kraken-orderbook-5m-3.00usd_slippage': 2495,
    'ask_strength-kraken-orderbook-5m-3.10usd_slippage': 2496,
    'ask_strength-kraken-orderbook-5m-3.20usd_slippage': 2497,
    'ask_strength-kraken-orderbook-5m-3.30usd_slippage': 2498,
    'ask_strength-kraken-orderbook-5m-3.40usd_slippage': 2499,
    'ask_strength-kraken-orderbook-5m-3.50usd_slippage': 2500,
    'ask_strength-kraken-orderbook-5m-3.60usd_slippage': 2501,
    'ask_strength-kraken-orderbook-5m-3.70usd_slippage': 2502,
    'ask_strength-kraken-orderbook-5m-3.80usd_slippage': 2503,
    'ask_strength-kraken-orderbook-5m-3.90usd_slippage': 2504,
    'ask_strength-kraken-orderbook-5m-4.00usd_slippage': 2505,
    'ask_strength-kraken-orderbook-5m-4.10usd_slippage': 2506,
    'ask_strength-kraken-orderbook-5m-4.20usd_slippage': 2507,
    'ask_strength-kraken-orderbook-5m-4.30usd_slippage': 2508,
    'ask_strength-kraken-orderbook-5m-4.40usd_slippage': 2509,
    'ask_strength-kraken-orderbook-5m-4.50usd_slippage': 2510,
    'ask_strength-kraken-orderbook-5m-4.60usd_slippage': 2511,
    'ask_strength-kraken-orderbook-5m-4.70usd_slippage': 2512,
    'ask_strength-kraken-orderbook-5m-4.80usd_slippage': 2513,
    'ask_strength-kraken-orderbook-5m-4.90usd_slippage': 2514,
    'ask_strength-kraken-orderbook-5m-5.00usd_slippage': 2515,
    'ask_strength-kraken-orderbook-5m-10.00usd_slippage': 2516,
    'ask_strength-kraken-orderbook-5m-20.00usd_slippage': 2517,
    'ask_strength-kraken-orderbook-1m-0.00usd_slippage': 2518,
    'ask_strength-kraken-orderbook-1m-0.10usd_slippage': 2519,
    'ask_strength-kraken-orderbook-1m-0.20usd_slippage': 2520,
    'ask_strength-kraken-orderbook-1m-0.30usd_slippage': 2521,
    'ask_strength-kraken-orderbook-1m-0.40usd_slippage': 2522,
    'ask_strength-kraken-orderbook-1m-0.50usd_slippage': 2523,
    'ask_strength-kraken-orderbook-1m-0.60usd_slippage': 2524,
    'ask_strength-kraken-orderbook-1m-0.70usd_slippage': 2525,
    'ask_strength-kraken-orderbook-1m-0.80usd_slippage': 2526,
    'ask_strength-kraken-orderbook-1m-0.90usd_slippage': 2527,
    'ask_strength-kraken-orderbook-1m-1.00usd_slippage': 2528,
    'ask_strength-kraken-orderbook-1m-1.10usd_slippage': 2529,
    'ask_strength-kraken-orderbook-1m-1.20usd_slippage': 2530,
    'ask_strength-kraken-orderbook-1m-1.30usd_slippage': 2531,
    'ask_strength-kraken-orderbook-1m-1.40usd_slippage': 2532,
    'ask_strength-kraken-orderbook-1m-1.50usd_slippage': 2533,
    'ask_strength-kraken-orderbook-1m-1.60usd_slippage': 2534,
    'ask_strength-kraken-orderbook-1m-1.70usd_slippage': 2535,
    'ask_strength-kraken-orderbook-1m-1.80usd_slippage': 2536,
    'ask_strength-kraken-orderbook-1m-1.90usd_slippage': 2537,
    'ask_strength-kraken-orderbook-1m-2.00usd_slippage': 2538,
    'ask_strength-kraken-orderbook-1m-2.10usd_slippage': 2539,
    'ask_strength-kraken-orderbook-1m-2.20usd_slippage': 2540,
    'ask_strength-kraken-orderbook-1m-2.30usd_slippage': 2541,
    'ask_strength-kraken-orderbook-1m-2.40usd_slippage': 2542,
    'ask_strength-kraken-orderbook-1m-2.50usd_slippage': 2543,
    'ask_strength-kraken-orderbook-1m-2.60usd_slippage': 2544,
    'ask_strength-kraken-orderbook-1m-2.70usd_slippage': 2545,
    'ask_strength-kraken-orderbook-1m-2.80usd_slippage': 2546,
    'ask_strength-kraken-orderbook-1m-2.90usd_slippage': 2547,
    'ask_strength-kraken-orderbook-1m-3.00usd_slippage': 2548,
    'ask_strength-kraken-orderbook-1m-3.10usd_slippage': 2549,
    'ask_strength-kraken-orderbook-1m-3.20usd_slippage': 2550,
    'ask_strength-kraken-orderbook-1m-3.30usd_slippage': 2551,
    'ask_strength-kraken-orderbook-1m-3.40usd_slippage': 2552,
    'ask_strength-kraken-orderbook-1m-3.50usd_slippage': 2553,
    'ask_strength-kraken-orderbook-1m-3.60usd_slippage': 2554,
    'ask_strength-kraken-orderbook-1m-3.70usd_slippage': 2555,
    'ask_strength-kraken-orderbook-1m-3.80usd_slippage': 2556,
    'ask_strength-kraken-orderbook-1m-3.90usd_slippage': 2557,
    'ask_strength-kraken-orderbook-1m-4.00usd_slippage': 2558,
    'ask_strength-kraken-orderbook-1m-4.10usd_slippage': 2559,
    'ask_strength-kraken-orderbook-1m-4.20usd_slippage': 2560,
    'ask_strength-kraken-orderbook-1m-4.30usd_slippage': 2561,
    'ask_strength-kraken-orderbook-1m-4.40usd_slippage': 2562,
    'ask_strength-kraken-orderbook-1m-4.50usd_slippage': 2563,
    'ask_strength-kraken-orderbook-1m-4.60usd_slippage': 2564,
    'ask_strength-kraken-orderbook-1m-4.70usd_slippage': 2565,
    'ask_strength-kraken-orderbook-1m-4.80usd_slippage': 2566,
    'ask_strength-kraken-orderbook-1m-4.90usd_slippage': 2567,
    'ask_strength-kraken-orderbook-1m-5.00usd_slippage': 2568,
    'ask_strength-kraken-orderbook-1m-10.00usd_slippage': 2569,
    'ask_strength-kraken-orderbook-1m-20.00usd_slippage': 2570,
    'ask_strength-gemini-orderbook-10m-0.00usd_slippage': 2571,
    'ask_strength-gemini-orderbook-10m-0.10usd_slippage': 2572,
    'ask_strength-gemini-orderbook-10m-0.20usd_slippage': 2573,
    'ask_strength-gemini-orderbook-10m-0.30usd_slippage': 2574,
    'ask_strength-gemini-orderbook-10m-0.40usd_slippage': 2575,
    'ask_strength-gemini-orderbook-10m-0.50usd_slippage': 2576,
    'ask_strength-gemini-orderbook-10m-0.60usd_slippage': 2577,
    'ask_strength-gemini-orderbook-10m-0.70usd_slippage': 2578,
    'ask_strength-gemini-orderbook-10m-0.80usd_slippage': 2579,
    'ask_strength-gemini-orderbook-10m-0.90usd_slippage': 2580,
    'ask_strength-gemini-orderbook-10m-1.00usd_slippage': 2581,
    'ask_strength-gemini-orderbook-10m-1.10usd_slippage': 2582,
    'ask_strength-gemini-orderbook-10m-1.20usd_slippage': 2583,
    'ask_strength-gemini-orderbook-10m-1.30usd_slippage': 2584,
    'ask_strength-gemini-orderbook-10m-1.40usd_slippage': 2585,
    'ask_strength-gemini-orderbook-10m-1.50usd_slippage': 2586,
    'ask_strength-gemini-orderbook-10m-1.60usd_slippage': 2587,
    'ask_strength-gemini-orderbook-10m-1.70usd_slippage': 2588,
    'ask_strength-gemini-orderbook-10m-1.80usd_slippage': 2589,
    'ask_strength-gemini-orderbook-10m-1.90usd_slippage': 2590,
    'ask_strength-gemini-orderbook-10m-2.00usd_slippage': 2591,
    'ask_strength-gemini-orderbook-10m-2.10usd_slippage': 2592,
    'ask_strength-gemini-orderbook-10m-2.20usd_slippage': 2593,
    'ask_strength-gemini-orderbook-10m-2.30usd_slippage': 2594,
    'ask_strength-gemini-orderbook-10m-2.40usd_slippage': 2595,
    'ask_strength-gemini-orderbook-10m-2.50usd_slippage': 2596,
    'ask_strength-gemini-orderbook-10m-2.60usd_slippage': 2597,
    'ask_strength-gemini-orderbook-10m-2.70usd_slippage': 2598,
    'ask_strength-gemini-orderbook-10m-2.80usd_slippage': 2599,
    'ask_strength-gemini-orderbook-10m-2.90usd_slippage': 2600,
    'ask_strength-gemini-orderbook-10m-3.00usd_slippage': 2601,
    'ask_strength-gemini-orderbook-10m-3.10usd_slippage': 2602,
    'ask_strength-gemini-orderbook-10m-3.20usd_slippage': 2603,
    'ask_strength-gemini-orderbook-10m-3.30usd_slippage': 2604,
    'ask_strength-gemini-orderbook-10m-3.40usd_slippage': 2605,
    'ask_strength-gemini-orderbook-10m-3.50usd_slippage': 2606,
    'ask_strength-gemini-orderbook-10m-3.60usd_slippage': 2607,
    'ask_strength-gemini-orderbook-10m-3.70usd_slippage': 2608,
    'ask_strength-gemini-orderbook-10m-3.80usd_slippage': 2609,
    'ask_strength-gemini-orderbook-10m-3.90usd_slippage': 2610,
    'ask_strength-gemini-orderbook-10m-4.00usd_slippage': 2611,
    'ask_strength-gemini-orderbook-10m-4.10usd_slippage': 2612,
    'ask_strength-gemini-orderbook-10m-4.20usd_slippage': 2613,
    'ask_strength-gemini-orderbook-10m-4.30usd_slippage': 2614,
    'ask_strength-gemini-orderbook-10m-4.40usd_slippage': 2615,
    'ask_strength-gemini-orderbook-10m-4.50usd_slippage': 2616,
    'ask_strength-gemini-orderbook-10m-4.60usd_slippage': 2617,
    'ask_strength-gemini-orderbook-10m-4.70usd_slippage': 2618,
    'ask_strength-gemini-orderbook-10m-4.80usd_slippage': 2619,
    'ask_strength-gemini-orderbook-10m-4.90usd_slippage': 2620,
    'ask_strength-gemini-orderbook-10m-5.00usd_slippage': 2621,
    'ask_strength-gemini-orderbook-10m-10.00usd_slippage': 2622,
    'ask_strength-gemini-orderbook-10m-20.00usd_slippage': 2623,
    'ask_strength-gemini-orderbook-5m-0.00usd_slippage': 2624,
    'ask_strength-gemini-orderbook-5m-0.10usd_slippage': 2625,
    'ask_strength-gemini-orderbook-5m-0.20usd_slippage': 2626,
    'ask_strength-gemini-orderbook-5m-0.30usd_slippage': 2627,
    'ask_strength-gemini-orderbook-5m-0.40usd_slippage': 2628,
    'ask_strength-gemini-orderbook-5m-0.50usd_slippage': 2629,
    'ask_strength-gemini-orderbook-5m-0.60usd_slippage': 2630,
    'ask_strength-gemini-orderbook-5m-0.70usd_slippage': 2631,
    'ask_strength-gemini-orderbook-5m-0.80usd_slippage': 2632,
    'ask_strength-gemini-orderbook-5m-0.90usd_slippage': 2633,
    'ask_strength-gemini-orderbook-5m-1.00usd_slippage': 2634,
    'ask_strength-gemini-orderbook-5m-1.10usd_slippage': 2635,
    'ask_strength-gemini-orderbook-5m-1.20usd_slippage': 2636,
    'ask_strength-gemini-orderbook-5m-1.30usd_slippage': 2637,
    'ask_strength-gemini-orderbook-5m-1.40usd_slippage': 2638,
    'ask_strength-gemini-orderbook-5m-1.50usd_slippage': 2639,
    'ask_strength-gemini-orderbook-5m-1.60usd_slippage': 2640,
    'ask_strength-gemini-orderbook-5m-1.70usd_slippage': 2641,
    'ask_strength-gemini-orderbook-5m-1.80usd_slippage': 2642,
    'ask_strength-gemini-orderbook-5m-1.90usd_slippage': 2643,
    'ask_strength-gemini-orderbook-5m-2.00usd_slippage': 2644,
    'ask_strength-gemini-orderbook-5m-2.10usd_slippage': 2645,
    'ask_strength-gemini-orderbook-5m-2.20usd_slippage': 2646,
    'ask_strength-gemini-orderbook-5m-2.30usd_slippage': 2647,
    'ask_strength-gemini-orderbook-5m-2.40usd_slippage': 2648,
    'ask_strength-gemini-orderbook-5m-2.50usd_slippage': 2649,
    'ask_strength-gemini-orderbook-5m-2.60usd_slippage': 2650,
    'ask_strength-gemini-orderbook-5m-2.70usd_slippage': 2651,
    'ask_strength-gemini-orderbook-5m-2.80usd_slippage': 2652,
    'ask_strength-gemini-orderbook-5m-2.90usd_slippage': 2653,
    'ask_strength-gemini-orderbook-5m-3.00usd_slippage': 2654,
    'ask_strength-gemini-orderbook-5m-3.10usd_slippage': 2655,
    'ask_strength-gemini-orderbook-5m-3.20usd_slippage': 2656,
    'ask_strength-gemini-orderbook-5m-3.30usd_slippage': 2657,
    'ask_strength-gemini-orderbook-5m-3.40usd_slippage': 2658,
    'ask_strength-gemini-orderbook-5m-3.50usd_slippage': 2659,
    'ask_strength-gemini-orderbook-5m-3.60usd_slippage': 2660,
    'ask_strength-gemini-orderbook-5m-3.70usd_slippage': 2661,
    'ask_strength-gemini-orderbook-5m-3.80usd_slippage': 2662,
    'ask_strength-gemini-orderbook-5m-3.90usd_slippage': 2663,
    'ask_strength-gemini-orderbook-5m-4.00usd_slippage': 2664,
    'ask_strength-gemini-orderbook-5m-4.10usd_slippage': 2665,
    'ask_strength-gemini-orderbook-5m-4.20usd_slippage': 2666,
    'ask_strength-gemini-orderbook-5m-4.30usd_slippage': 2667,
    'ask_strength-gemini-orderbook-5m-4.40usd_slippage': 2668,
    'ask_strength-gemini-orderbook-5m-4.50usd_slippage': 2669,
    'ask_strength-gemini-orderbook-5m-4.60usd_slippage': 2670,
    'ask_strength-gemini-orderbook-5m-4.70usd_slippage': 2671,
    'ask_strength-gemini-orderbook-5m-4.80usd_slippage': 2672,
    'ask_strength-gemini-orderbook-5m-4.90usd_slippage': 2673,
    'ask_strength-gemini-orderbook-5m-5.00usd_slippage': 2674,
    'ask_strength-gemini-orderbook-5m-10.00usd_slippage': 2675,
    'ask_strength-gemini-orderbook-5m-20.00usd_slippage': 2676,
    'ask_strength-gemini-orderbook-1m-0.00usd_slippage': 2677,
    'ask_strength-gemini-orderbook-1m-0.10usd_slippage': 2678,
    'ask_strength-gemini-orderbook-1m-0.20usd_slippage': 2679,
    'ask_strength-gemini-orderbook-1m-0.30usd_slippage': 2680,
    'ask_strength-gemini-orderbook-1m-0.40usd_slippage': 2681,
    'ask_strength-gemini-orderbook-1m-0.50usd_slippage': 2682,
    'ask_strength-gemini-orderbook-1m-0.60usd_slippage': 2683,
    'ask_strength-gemini-orderbook-1m-0.70usd_slippage': 2684,
    'ask_strength-gemini-orderbook-1m-0.80usd_slippage': 2685,
    'ask_strength-gemini-orderbook-1m-0.90usd_slippage': 2686,
    'ask_strength-gemini-orderbook-1m-1.00usd_slippage': 2687,
    'ask_strength-gemini-orderbook-1m-1.10usd_slippage': 2688,
    'ask_strength-gemini-orderbook-1m-1.20usd_slippage': 2689,
    'ask_strength-gemini-orderbook-1m-1.30usd_slippage': 2690,
    'ask_strength-gemini-orderbook-1m-1.40usd_slippage': 2691,
    'ask_strength-gemini-orderbook-1m-1.50usd_slippage': 2692,
    'ask_strength-gemini-orderbook-1m-1.60usd_slippage': 2693,
    'ask_strength-gemini-orderbook-1m-1.70usd_slippage': 2694,
    'ask_strength-gemini-orderbook-1m-1.80usd_slippage': 2695,
    'ask_strength-gemini-orderbook-1m-1.90usd_slippage': 2696,
    'ask_strength-gemini-orderbook-1m-2.00usd_slippage': 2697,
    'ask_strength-gemini-orderbook-1m-2.10usd_slippage': 2698,
    'ask_strength-gemini-orderbook-1m-2.20usd_slippage': 2699,
    'ask_strength-gemini-orderbook-1m-2.30usd_slippage': 2700,
    'ask_strength-gemini-orderbook-1m-2.40usd_slippage': 2701,
    'ask_strength-gemini-orderbook-1m-2.50usd_slippage': 2702,
    'ask_strength-gemini-orderbook-1m-2.60usd_slippage': 2703,
    'ask_strength-gemini-orderbook-1m-2.70usd_slippage': 2704,
    'ask_strength-gemini-orderbook-1m-2.80usd_slippage': 2705,
    'ask_strength-gemini-orderbook-1m-2.90usd_slippage': 2706,
    'ask_strength-gemini-orderbook-1m-3.00usd_slippage': 2707,
    'ask_strength-gemini-orderbook-1m-3.10usd_slippage': 2708,
    'ask_strength-gemini-orderbook-1m-3.20usd_slippage': 2709,
    'ask_strength-gemini-orderbook-1m-3.30usd_slippage': 2710,
    'ask_strength-gemini-orderbook-1m-3.40usd_slippage': 2711,
    'ask_strength-gemini-orderbook-1m-3.50usd_slippage': 2712,
    'ask_strength-gemini-orderbook-1m-3.60usd_slippage': 2713,
    'ask_strength-gemini-orderbook-1m-3.70usd_slippage': 2714,
    'ask_strength-gemini-orderbook-1m-3.80usd_slippage': 2715,
    'ask_strength-gemini-orderbook-1m-3.90usd_slippage': 2716,
    'ask_strength-gemini-orderbook-1m-4.00usd_slippage': 2717,
    'ask_strength-gemini-orderbook-1m-4.10usd_slippage': 2718,
    'ask_strength-gemini-orderbook-1m-4.20usd_slippage': 2719,
    'ask_strength-gemini-orderbook-1m-4.30usd_slippage': 2720,
    'ask_strength-gemini-orderbook-1m-4.40usd_slippage': 2721,
    'ask_strength-gemini-orderbook-1m-4.50usd_slippage': 2722,
    'ask_strength-gemini-orderbook-1m-4.60usd_slippage': 2723,
    'ask_strength-gemini-orderbook-1m-4.70usd_slippage': 2724,
    'ask_strength-gemini-orderbook-1m-4.80usd_slippage': 2725,
    'ask_strength-gemini-orderbook-1m-4.90usd_slippage': 2726,
    'ask_strength-gemini-orderbook-1m-5.00usd_slippage': 2727,
    'ask_strength-gemini-orderbook-1m-10.00usd_slippage': 2728,
    'ask_strength-gemini-orderbook-1m-20.00usd_slippage': 2729,
    'ask_strength-okcoin-orderbook-10m-0.00usd_slippage': 2730,
    'ask_strength-okcoin-orderbook-10m-0.10usd_slippage': 2731,
    'ask_strength-okcoin-orderbook-10m-0.20usd_slippage': 2732,
    'ask_strength-okcoin-orderbook-10m-0.30usd_slippage': 2733,
    'ask_strength-okcoin-orderbook-10m-0.40usd_slippage': 2734,
    'ask_strength-okcoin-orderbook-10m-0.50usd_slippage': 2735,
    'ask_strength-okcoin-orderbook-10m-0.60usd_slippage': 2736,
    'ask_strength-okcoin-orderbook-10m-0.70usd_slippage': 2737,
    'ask_strength-okcoin-orderbook-10m-0.80usd_slippage': 2738,
    'ask_strength-okcoin-orderbook-10m-0.90usd_slippage': 2739,
    'ask_strength-okcoin-orderbook-10m-1.00usd_slippage': 2740,
    'ask_strength-okcoin-orderbook-10m-1.10usd_slippage': 2741,
    'ask_strength-okcoin-orderbook-10m-1.20usd_slippage': 2742,
    'ask_strength-okcoin-orderbook-10m-1.30usd_slippage': 2743,
    'ask_strength-okcoin-orderbook-10m-1.40usd_slippage': 2744,
    'ask_strength-okcoin-orderbook-10m-1.50usd_slippage': 2745,
    'ask_strength-okcoin-orderbook-10m-1.60usd_slippage': 2746,
    'ask_strength-okcoin-orderbook-10m-1.70usd_slippage': 2747,
    'ask_strength-okcoin-orderbook-10m-1.80usd_slippage': 2748,
    'ask_strength-okcoin-orderbook-10m-1.90usd_slippage': 2749,
    'ask_strength-okcoin-orderbook-10m-2.00usd_slippage': 2750,
    'ask_strength-okcoin-orderbook-10m-2.10usd_slippage': 2751,
    'ask_strength-okcoin-orderbook-10m-2.20usd_slippage': 2752,
    'ask_strength-okcoin-orderbook-10m-2.30usd_slippage': 2753,
    'ask_strength-okcoin-orderbook-10m-2.40usd_slippage': 2754,
    'ask_strength-okcoin-orderbook-10m-2.50usd_slippage': 2755,
    'ask_strength-okcoin-orderbook-10m-2.60usd_slippage': 2756,
    'ask_strength-okcoin-orderbook-10m-2.70usd_slippage': 2757,
    'ask_strength-okcoin-orderbook-10m-2.80usd_slippage': 2758,
    'ask_strength-okcoin-orderbook-10m-2.90usd_slippage': 2759,
    'ask_strength-okcoin-orderbook-10m-3.00usd_slippage': 2760,
    'ask_strength-okcoin-orderbook-10m-3.10usd_slippage': 2761,
    'ask_strength-okcoin-orderbook-10m-3.20usd_slippage': 2762,
    'ask_strength-okcoin-orderbook-10m-3.30usd_slippage': 2763,
    'ask_strength-okcoin-orderbook-10m-3.40usd_slippage': 2764,
    'ask_strength-okcoin-orderbook-10m-3.50usd_slippage': 2765,
    'ask_strength-okcoin-orderbook-10m-3.60usd_slippage': 2766,
    'ask_strength-okcoin-orderbook-10m-3.70usd_slippage': 2767,
    'ask_strength-okcoin-orderbook-10m-3.80usd_slippage': 2768,
    'ask_strength-okcoin-orderbook-10m-3.90usd_slippage': 2769,
    'ask_strength-okcoin-orderbook-10m-4.00usd_slippage': 2770,
    'ask_strength-okcoin-orderbook-10m-4.10usd_slippage': 2771,
    'ask_strength-okcoin-orderbook-10m-4.20usd_slippage': 2772,
    'ask_strength-okcoin-orderbook-10m-4.30usd_slippage': 2773,
    'ask_strength-okcoin-orderbook-10m-4.40usd_slippage': 2774,
    'ask_strength-okcoin-orderbook-10m-4.50usd_slippage': 2775,
    'ask_strength-okcoin-orderbook-10m-4.60usd_slippage': 2776,
    'ask_strength-okcoin-orderbook-10m-4.70usd_slippage': 2777,
    'ask_strength-okcoin-orderbook-10m-4.80usd_slippage': 2778,
    'ask_strength-okcoin-orderbook-10m-4.90usd_slippage': 2779,
    'ask_strength-okcoin-orderbook-10m-5.00usd_slippage': 2780,
    'ask_strength-okcoin-orderbook-10m-10.00usd_slippage': 2781,
    'ask_strength-okcoin-orderbook-10m-20.00usd_slippage': 2782,
    'ask_strength-okcoin-orderbook-5m-0.00usd_slippage': 2783,
    'ask_strength-okcoin-orderbook-5m-0.10usd_slippage': 2784,
    'ask_strength-okcoin-orderbook-5m-0.20usd_slippage': 2785,
    'ask_strength-okcoin-orderbook-5m-0.30usd_slippage': 2786,
    'ask_strength-okcoin-orderbook-5m-0.40usd_slippage': 2787,
    'ask_strength-okcoin-orderbook-5m-0.50usd_slippage': 2788,
    'ask_strength-okcoin-orderbook-5m-0.60usd_slippage': 2789,
    'ask_strength-okcoin-orderbook-5m-0.70usd_slippage': 2790,
    'ask_strength-okcoin-orderbook-5m-0.80usd_slippage': 2791,
    'ask_strength-okcoin-orderbook-5m-0.90usd_slippage': 2792,
    'ask_strength-okcoin-orderbook-5m-1.00usd_slippage': 2793,
    'ask_strength-okcoin-orderbook-5m-1.10usd_slippage': 2794,
    'ask_strength-okcoin-orderbook-5m-1.20usd_slippage': 2795,
    'ask_strength-okcoin-orderbook-5m-1.30usd_slippage': 2796,
    'ask_strength-okcoin-orderbook-5m-1.40usd_slippage': 2797,
    'ask_strength-okcoin-orderbook-5m-1.50usd_slippage': 2798,
    'ask_strength-okcoin-orderbook-5m-1.60usd_slippage': 2799,
    'ask_strength-okcoin-orderbook-5m-1.70usd_slippage': 2800,
    'ask_strength-okcoin-orderbook-5m-1.80usd_slippage': 2801,
    'ask_strength-okcoin-orderbook-5m-1.90usd_slippage': 2802,
    'ask_strength-okcoin-orderbook-5m-2.00usd_slippage': 2803,
    'ask_strength-okcoin-orderbook-5m-2.10usd_slippage': 2804,
    'ask_strength-okcoin-orderbook-5m-2.20usd_slippage': 2805,
    'ask_strength-okcoin-orderbook-5m-2.30usd_slippage': 2806,
    'ask_strength-okcoin-orderbook-5m-2.40usd_slippage': 2807,
    'ask_strength-okcoin-orderbook-5m-2.50usd_slippage': 2808,
    'ask_strength-okcoin-orderbook-5m-2.60usd_slippage': 2809,
    'ask_strength-okcoin-orderbook-5m-2.70usd_slippage': 2810,
    'ask_strength-okcoin-orderbook-5m-2.80usd_slippage': 2811,
    'ask_strength-okcoin-orderbook-5m-2.90usd_slippage': 2812,
    'ask_strength-okcoin-orderbook-5m-3.00usd_slippage': 2813,
    'ask_strength-okcoin-orderbook-5m-3.10usd_slippage': 2814,
    'ask_strength-okcoin-orderbook-5m-3.20usd_slippage': 2815,
    'ask_strength-okcoin-orderbook-5m-3.30usd_slippage': 2816,
    'ask_strength-okcoin-orderbook-5m-3.40usd_slippage': 2817,
    'ask_strength-okcoin-orderbook-5m-3.50usd_slippage': 2818,
    'ask_strength-okcoin-orderbook-5m-3.60usd_slippage': 2819,
    'ask_strength-okcoin-orderbook-5m-3.70usd_slippage': 2820,
    'ask_strength-okcoin-orderbook-5m-3.80usd_slippage': 2821,
    'ask_strength-okcoin-orderbook-5m-3.90usd_slippage': 2822,
    'ask_strength-okcoin-orderbook-5m-4.00usd_slippage': 2823,
    'ask_strength-okcoin-orderbook-5m-4.10usd_slippage': 2824,
    'ask_strength-okcoin-orderbook-5m-4.20usd_slippage': 2825,
    'ask_strength-okcoin-orderbook-5m-4.30usd_slippage': 2826,
    'ask_strength-okcoin-orderbook-5m-4.40usd_slippage': 2827,
    'ask_strength-okcoin-orderbook-5m-4.50usd_slippage': 2828,
    'ask_strength-okcoin-orderbook-5m-4.60usd_slippage': 2829,
    'ask_strength-okcoin-orderbook-5m-4.70usd_slippage': 2830,
    'ask_strength-okcoin-orderbook-5m-4.80usd_slippage': 2831,
    'ask_strength-okcoin-orderbook-5m-4.90usd_slippage': 2832,
    'ask_strength-okcoin-orderbook-5m-5.00usd_slippage': 2833,
    'ask_strength-okcoin-orderbook-5m-10.00usd_slippage': 2834,
    'ask_strength-okcoin-orderbook-5m-20.00usd_slippage': 2835,
    'ask_strength-okcoin-orderbook-1m-0.00usd_slippage': 2836,
    'ask_strength-okcoin-orderbook-1m-0.10usd_slippage': 2837,
    'ask_strength-okcoin-orderbook-1m-0.20usd_slippage': 2838,
    'ask_strength-okcoin-orderbook-1m-0.30usd_slippage': 2839,
    'ask_strength-okcoin-orderbook-1m-0.40usd_slippage': 2840,
    'ask_strength-okcoin-orderbook-1m-0.50usd_slippage': 2841,
    'ask_strength-okcoin-orderbook-1m-0.60usd_slippage': 2842,
    'ask_strength-okcoin-orderbook-1m-0.70usd_slippage': 2843,
    'ask_strength-okcoin-orderbook-1m-0.80usd_slippage': 2844,
    'ask_strength-okcoin-orderbook-1m-0.90usd_slippage': 2845,
    'ask_strength-okcoin-orderbook-1m-1.00usd_slippage': 2846,
    'ask_strength-okcoin-orderbook-1m-1.10usd_slippage': 2847,
    'ask_strength-okcoin-orderbook-1m-1.20usd_slippage': 2848,
    'ask_strength-okcoin-orderbook-1m-1.30usd_slippage': 2849,
    'ask_strength-okcoin-orderbook-1m-1.40usd_slippage': 2850,
    'ask_strength-okcoin-orderbook-1m-1.50usd_slippage': 2851,
    'ask_strength-okcoin-orderbook-1m-1.60usd_slippage': 2852,
    'ask_strength-okcoin-orderbook-1m-1.70usd_slippage': 2853,
    'ask_strength-okcoin-orderbook-1m-1.80usd_slippage': 2854,
    'ask_strength-okcoin-orderbook-1m-1.90usd_slippage': 2855,
    'ask_strength-okcoin-orderbook-1m-2.00usd_slippage': 2856,
    'ask_strength-okcoin-orderbook-1m-2.10usd_slippage': 2857,
    'ask_strength-okcoin-orderbook-1m-2.20usd_slippage': 2858,
    'ask_strength-okcoin-orderbook-1m-2.30usd_slippage': 2859,
    'ask_strength-okcoin-orderbook-1m-2.40usd_slippage': 2860,
    'ask_strength-okcoin-orderbook-1m-2.50usd_slippage': 2861,
    'ask_strength-okcoin-orderbook-1m-2.60usd_slippage': 2862,
    'ask_strength-okcoin-orderbook-1m-2.70usd_slippage': 2863,
    'ask_strength-okcoin-orderbook-1m-2.80usd_slippage': 2864,
    'ask_strength-okcoin-orderbook-1m-2.90usd_slippage': 2865,
    'ask_strength-okcoin-orderbook-1m-3.00usd_slippage': 2866,
    'ask_strength-okcoin-orderbook-1m-3.10usd_slippage': 2867,
    'ask_strength-okcoin-orderbook-1m-3.20usd_slippage': 2868,
    'ask_strength-okcoin-orderbook-1m-3.30usd_slippage': 2869,
    'ask_strength-okcoin-orderbook-1m-3.40usd_slippage': 2870,
    'ask_strength-okcoin-orderbook-1m-3.50usd_slippage': 2871,
    'ask_strength-okcoin-orderbook-1m-3.60usd_slippage': 2872,
    'ask_strength-okcoin-orderbook-1m-3.70usd_slippage': 2873,
    'ask_strength-okcoin-orderbook-1m-3.80usd_slippage': 2874,
    'ask_strength-okcoin-orderbook-1m-3.90usd_slippage': 2875,
    'ask_strength-okcoin-orderbook-1m-4.00usd_slippage': 2876,
    'ask_strength-okcoin-orderbook-1m-4.10usd_slippage': 2877,
    'ask_strength-okcoin-orderbook-1m-4.20usd_slippage': 2878,
    'ask_strength-okcoin-orderbook-1m-4.30usd_slippage': 2879,
    'ask_strength-okcoin-orderbook-1m-4.40usd_slippage': 2880,
    'ask_strength-okcoin-orderbook-1m-4.50usd_slippage': 2881,
    'ask_strength-okcoin-orderbook-1m-4.60usd_slippage': 2882,
    'ask_strength-okcoin-orderbook-1m-4.70usd_slippage': 2883,
    'ask_strength-okcoin-orderbook-1m-4.80usd_slippage': 2884,
    'ask_strength-okcoin-orderbook-1m-4.90usd_slippage': 2885,
    'ask_strength-okcoin-orderbook-1m-5.00usd_slippage': 2886,
    'ask_strength-okcoin-orderbook-1m-10.00usd_slippage': 2887,
    'ask_strength-okcoin-orderbook-1m-20.00usd_slippage': 2888,
    'bid_strength_usd-bitstamp-orderbook-10m-0.00usd_slippage': 2889,
    'bid_strength_usd-bitstamp-orderbook-10m-0.10usd_slippage': 2890,
    'bid_strength_usd-bitstamp-orderbook-10m-0.20usd_slippage': 2891,
    'bid_strength_usd-bitstamp-orderbook-10m-0.30usd_slippage': 2892,
    'bid_strength_usd-bitstamp-orderbook-10m-0.40usd_slippage': 2893,
    'bid_strength_usd-bitstamp-orderbook-10m-0.50usd_slippage': 2894,
    'bid_strength_usd-bitstamp-orderbook-10m-0.60usd_slippage': 2895,
    'bid_strength_usd-bitstamp-orderbook-10m-0.70usd_slippage': 2896,
    'bid_strength_usd-bitstamp-orderbook-10m-0.80usd_slippage': 2897,
    'bid_strength_usd-bitstamp-orderbook-10m-0.90usd_slippage': 2898,
    'bid_strength_usd-bitstamp-orderbook-10m-1.00usd_slippage': 2899,
    'bid_strength_usd-bitstamp-orderbook-10m-1.10usd_slippage': 2900,
    'bid_strength_usd-bitstamp-orderbook-10m-1.20usd_slippage': 2901,
    'bid_strength_usd-bitstamp-orderbook-10m-1.30usd_slippage': 2902,
    'bid_strength_usd-bitstamp-orderbook-10m-1.40usd_slippage': 2903,
    'bid_strength_usd-bitstamp-orderbook-10m-1.50usd_slippage': 2904,
    'bid_strength_usd-bitstamp-orderbook-10m-1.60usd_slippage': 2905,
    'bid_strength_usd-bitstamp-orderbook-10m-1.70usd_slippage': 2906,
    'bid_strength_usd-bitstamp-orderbook-10m-1.80usd_slippage': 2907,
    'bid_strength_usd-bitstamp-orderbook-10m-1.90usd_slippage': 2908,
    'bid_strength_usd-bitstamp-orderbook-10m-2.00usd_slippage': 2909,
    'bid_strength_usd-bitstamp-orderbook-10m-2.10usd_slippage': 2910,
    'bid_strength_usd-bitstamp-orderbook-10m-2.20usd_slippage': 2911,
    'bid_strength_usd-bitstamp-orderbook-10m-2.30usd_slippage': 2912,
    'bid_strength_usd-bitstamp-orderbook-10m-2.40usd_slippage': 2913,
    'bid_strength_usd-bitstamp-orderbook-10m-2.50usd_slippage': 2914,
    'bid_strength_usd-bitstamp-orderbook-10m-2.60usd_slippage': 2915,
    'bid_strength_usd-bitstamp-orderbook-10m-2.70usd_slippage': 2916,
    'bid_strength_usd-bitstamp-orderbook-10m-2.80usd_slippage': 2917,
    'bid_strength_usd-bitstamp-orderbook-10m-2.90usd_slippage': 2918,
    'bid_strength_usd-bitstamp-orderbook-10m-3.00usd_slippage': 2919,
    'bid_strength_usd-bitstamp-orderbook-10m-3.10usd_slippage': 2920,
    'bid_strength_usd-bitstamp-orderbook-10m-3.20usd_slippage': 2921,
    'bid_strength_usd-bitstamp-orderbook-10m-3.30usd_slippage': 2922,
    'bid_strength_usd-bitstamp-orderbook-10m-3.40usd_slippage': 2923,
    'bid_strength_usd-bitstamp-orderbook-10m-3.50usd_slippage': 2924,
    'bid_strength_usd-bitstamp-orderbook-10m-3.60usd_slippage': 2925,
    'bid_strength_usd-bitstamp-orderbook-10m-3.70usd_slippage': 2926,
    'bid_strength_usd-bitstamp-orderbook-10m-3.80usd_slippage': 2927,
    'bid_strength_usd-bitstamp-orderbook-10m-3.90usd_slippage': 2928,
    'bid_strength_usd-bitstamp-orderbook-10m-4.00usd_slippage': 2929,
    'bid_strength_usd-bitstamp-orderbook-10m-4.10usd_slippage': 2930,
    'bid_strength_usd-bitstamp-orderbook-10m-4.20usd_slippage': 2931,
    'bid_strength_usd-bitstamp-orderbook-10m-4.30usd_slippage': 2932,
    'bid_strength_usd-bitstamp-orderbook-10m-4.40usd_slippage': 2933,
    'bid_strength_usd-bitstamp-orderbook-10m-4.50usd_slippage': 2934,
    'bid_strength_usd-bitstamp-orderbook-10m-4.60usd_slippage': 2935,
    'bid_strength_usd-bitstamp-orderbook-10m-4.70usd_slippage': 2936,
    'bid_strength_usd-bitstamp-orderbook-10m-4.80usd_slippage': 2937,
    'bid_strength_usd-bitstamp-orderbook-10m-4.90usd_slippage': 2938,
    'bid_strength_usd-bitstamp-orderbook-10m-5.00usd_slippage': 2939,
    'bid_strength_usd-bitstamp-orderbook-10m-10.00usd_slippage': 2940,
    'bid_strength_usd-bitstamp-orderbook-10m-20.00usd_slippage': 2941,
    'bid_strength_usd-bitstamp-orderbook-5m-0.00usd_slippage': 2942,
    'bid_strength_usd-bitstamp-orderbook-5m-0.10usd_slippage': 2943,
    'bid_strength_usd-bitstamp-orderbook-5m-0.20usd_slippage': 2944,
    'bid_strength_usd-bitstamp-orderbook-5m-0.30usd_slippage': 2945,
    'bid_strength_usd-bitstamp-orderbook-5m-0.40usd_slippage': 2946,
    'bid_strength_usd-bitstamp-orderbook-5m-0.50usd_slippage': 2947,
    'bid_strength_usd-bitstamp-orderbook-5m-0.60usd_slippage': 2948,
    'bid_strength_usd-bitstamp-orderbook-5m-0.70usd_slippage': 2949,
    'bid_strength_usd-bitstamp-orderbook-5m-0.80usd_slippage': 2950,
    'bid_strength_usd-bitstamp-orderbook-5m-0.90usd_slippage': 2951,
    'bid_strength_usd-bitstamp-orderbook-5m-1.00usd_slippage': 2952,
    'bid_strength_usd-bitstamp-orderbook-5m-1.10usd_slippage': 2953,
    'bid_strength_usd-bitstamp-orderbook-5m-1.20usd_slippage': 2954,
    'bid_strength_usd-bitstamp-orderbook-5m-1.30usd_slippage': 2955,
    'bid_strength_usd-bitstamp-orderbook-5m-1.40usd_slippage': 2956,
    'bid_strength_usd-bitstamp-orderbook-5m-1.50usd_slippage': 2957,
    'bid_strength_usd-bitstamp-orderbook-5m-1.60usd_slippage': 2958,
    'bid_strength_usd-bitstamp-orderbook-5m-1.70usd_slippage': 2959,
    'bid_strength_usd-bitstamp-orderbook-5m-1.80usd_slippage': 2960,
    'bid_strength_usd-bitstamp-orderbook-5m-1.90usd_slippage': 2961,
    'bid_strength_usd-bitstamp-orderbook-5m-2.00usd_slippage': 2962,
    'bid_strength_usd-bitstamp-orderbook-5m-2.10usd_slippage': 2963,
    'bid_strength_usd-bitstamp-orderbook-5m-2.20usd_slippage': 2964,
    'bid_strength_usd-bitstamp-orderbook-5m-2.30usd_slippage': 2965,
    'bid_strength_usd-bitstamp-orderbook-5m-2.40usd_slippage': 2966,
    'bid_strength_usd-bitstamp-orderbook-5m-2.50usd_slippage': 2967,
    'bid_strength_usd-bitstamp-orderbook-5m-2.60usd_slippage': 2968,
    'bid_strength_usd-bitstamp-orderbook-5m-2.70usd_slippage': 2969,
    'bid_strength_usd-bitstamp-orderbook-5m-2.80usd_slippage': 2970,
    'bid_strength_usd-bitstamp-orderbook-5m-2.90usd_slippage': 2971,
    'bid_strength_usd-bitstamp-orderbook-5m-3.00usd_slippage': 2972,
    'bid_strength_usd-bitstamp-orderbook-5m-3.10usd_slippage': 2973,
    'bid_strength_usd-bitstamp-orderbook-5m-3.20usd_slippage': 2974,
    'bid_strength_usd-bitstamp-orderbook-5m-3.30usd_slippage': 2975,
    'bid_strength_usd-bitstamp-orderbook-5m-3.40usd_slippage': 2976,
    'bid_strength_usd-bitstamp-orderbook-5m-3.50usd_slippage': 2977,
    'bid_strength_usd-bitstamp-orderbook-5m-3.60usd_slippage': 2978,
    'bid_strength_usd-bitstamp-orderbook-5m-3.70usd_slippage': 2979,
    'bid_strength_usd-bitstamp-orderbook-5m-3.80usd_slippage': 2980,
    'bid_strength_usd-bitstamp-orderbook-5m-3.90usd_slippage': 2981,
    'bid_strength_usd-bitstamp-orderbook-5m-4.00usd_slippage': 2982,
    'bid_strength_usd-bitstamp-orderbook-5m-4.10usd_slippage': 2983,
    'bid_strength_usd-bitstamp-orderbook-5m-4.20usd_slippage': 2984,
    'bid_strength_usd-bitstamp-orderbook-5m-4.30usd_slippage': 2985,
    'bid_strength_usd-bitstamp-orderbook-5m-4.40usd_slippage': 2986,
    'bid_strength_usd-bitstamp-orderbook-5m-4.50usd_slippage': 2987,
    'bid_strength_usd-bitstamp-orderbook-5m-4.60usd_slippage': 2988,
    'bid_strength_usd-bitstamp-orderbook-5m-4.70usd_slippage': 2989,
    'bid_strength_usd-bitstamp-orderbook-5m-4.80usd_slippage': 2990,
    'bid_strength_usd-bitstamp-orderbook-5m-4.90usd_slippage': 2991,
    'bid_strength_usd-bitstamp-orderbook-5m-5.00usd_slippage': 2992,
    'bid_strength_usd-bitstamp-orderbook-5m-10.00usd_slippage': 2993,
    'bid_strength_usd-bitstamp-orderbook-5m-20.00usd_slippage': 2994,
    'bid_strength_usd-bitstamp-orderbook-1m-0.00usd_slippage': 2995,
    'bid_strength_usd-bitstamp-orderbook-1m-0.10usd_slippage': 2996,
    'bid_strength_usd-bitstamp-orderbook-1m-0.20usd_slippage': 2997,
    'bid_strength_usd-bitstamp-orderbook-1m-0.30usd_slippage': 2998,
    'bid_strength_usd-bitstamp-orderbook-1m-0.40usd_slippage': 2999,
    'bid_strength_usd-bitstamp-orderbook-1m-0.50usd_slippage': 3000,
    'bid_strength_usd-bitstamp-orderbook-1m-0.60usd_slippage': 3001,
    'bid_strength_usd-bitstamp-orderbook-1m-0.70usd_slippage': 3002,
    'bid_strength_usd-bitstamp-orderbook-1m-0.80usd_slippage': 3003,
    'bid_strength_usd-bitstamp-orderbook-1m-0.90usd_slippage': 3004,
    'bid_strength_usd-bitstamp-orderbook-1m-1.00usd_slippage': 3005,
    'bid_strength_usd-bitstamp-orderbook-1m-1.10usd_slippage': 3006,
    'bid_strength_usd-bitstamp-orderbook-1m-1.20usd_slippage': 3007,
    'bid_strength_usd-bitstamp-orderbook-1m-1.30usd_slippage': 3008,
    'bid_strength_usd-bitstamp-orderbook-1m-1.40usd_slippage': 3009,
    'bid_strength_usd-bitstamp-orderbook-1m-1.50usd_slippage': 3010,
    'bid_strength_usd-bitstamp-orderbook-1m-1.60usd_slippage': 3011,
    'bid_strength_usd-bitstamp-orderbook-1m-1.70usd_slippage': 3012,
    'bid_strength_usd-bitstamp-orderbook-1m-1.80usd_slippage': 3013,
    'bid_strength_usd-bitstamp-orderbook-1m-1.90usd_slippage': 3014,
    'bid_strength_usd-bitstamp-orderbook-1m-2.00usd_slippage': 3015,
    'bid_strength_usd-bitstamp-orderbook-1m-2.10usd_slippage': 3016,
    'bid_strength_usd-bitstamp-orderbook-1m-2.20usd_slippage': 3017,
    'bid_strength_usd-bitstamp-orderbook-1m-2.30usd_slippage': 3018,
    'bid_strength_usd-bitstamp-orderbook-1m-2.40usd_slippage': 3019,
    'bid_strength_usd-bitstamp-orderbook-1m-2.50usd_slippage': 3020,
    'bid_strength_usd-bitstamp-orderbook-1m-2.60usd_slippage': 3021,
    'bid_strength_usd-bitstamp-orderbook-1m-2.70usd_slippage': 3022,
    'bid_strength_usd-bitstamp-orderbook-1m-2.80usd_slippage': 3023,
    'bid_strength_usd-bitstamp-orderbook-1m-2.90usd_slippage': 3024,
    'bid_strength_usd-bitstamp-orderbook-1m-3.00usd_slippage': 3025,
    'bid_strength_usd-bitstamp-orderbook-1m-3.10usd_slippage': 3026,
    'bid_strength_usd-bitstamp-orderbook-1m-3.20usd_slippage': 3027,
    'bid_strength_usd-bitstamp-orderbook-1m-3.30usd_slippage': 3028,
    'bid_strength_usd-bitstamp-orderbook-1m-3.40usd_slippage': 3029,
    'bid_strength_usd-bitstamp-orderbook-1m-3.50usd_slippage': 3030,
    'bid_strength_usd-bitstamp-orderbook-1m-3.60usd_slippage': 3031,
    'bid_strength_usd-bitstamp-orderbook-1m-3.70usd_slippage': 3032,
    'bid_strength_usd-bitstamp-orderbook-1m-3.80usd_slippage': 3033,
    'bid_strength_usd-bitstamp-orderbook-1m-3.90usd_slippage': 3034,
    'bid_strength_usd-bitstamp-orderbook-1m-4.00usd_slippage': 3035,
    'bid_strength_usd-bitstamp-orderbook-1m-4.10usd_slippage': 3036,
    'bid_strength_usd-bitstamp-orderbook-1m-4.20usd_slippage': 3037,
    'bid_strength_usd-bitstamp-orderbook-1m-4.30usd_slippage': 3038,
    'bid_strength_usd-bitstamp-orderbook-1m-4.40usd_slippage': 3039,
    'bid_strength_usd-bitstamp-orderbook-1m-4.50usd_slippage': 3040,
    'bid_strength_usd-bitstamp-orderbook-1m-4.60usd_slippage': 3041,
    'bid_strength_usd-bitstamp-orderbook-1m-4.70usd_slippage': 3042,
    'bid_strength_usd-bitstamp-orderbook-1m-4.80usd_slippage': 3043,
    'bid_strength_usd-bitstamp-orderbook-1m-4.90usd_slippage': 3044,
    'bid_strength_usd-bitstamp-orderbook-1m-5.00usd_slippage': 3045,
    'bid_strength_usd-bitstamp-orderbook-1m-10.00usd_slippage': 3046,
    'bid_strength_usd-bitstamp-orderbook-1m-20.00usd_slippage': 3047,
    'bid_strength_usd-bitfinex-orderbook-10m-0.00usd_slippage': 3048,
    'bid_strength_usd-bitfinex-orderbook-10m-0.10usd_slippage': 3049,
    'bid_strength_usd-bitfinex-orderbook-10m-0.20usd_slippage': 3050,
    'bid_strength_usd-bitfinex-orderbook-10m-0.30usd_slippage': 3051,
    'bid_strength_usd-bitfinex-orderbook-10m-0.40usd_slippage': 3052,
    'bid_strength_usd-bitfinex-orderbook-10m-0.50usd_slippage': 3053,
    'bid_strength_usd-bitfinex-orderbook-10m-0.60usd_slippage': 3054,
    'bid_strength_usd-bitfinex-orderbook-10m-0.70usd_slippage': 3055,
    'bid_strength_usd-bitfinex-orderbook-10m-0.80usd_slippage': 3056,
    'bid_strength_usd-bitfinex-orderbook-10m-0.90usd_slippage': 3057,
    'bid_strength_usd-bitfinex-orderbook-10m-1.00usd_slippage': 3058,
    'bid_strength_usd-bitfinex-orderbook-10m-1.10usd_slippage': 3059,
    'bid_strength_usd-bitfinex-orderbook-10m-1.20usd_slippage': 3060,
    'bid_strength_usd-bitfinex-orderbook-10m-1.30usd_slippage': 3061,
    'bid_strength_usd-bitfinex-orderbook-10m-1.40usd_slippage': 3062,
    'bid_strength_usd-bitfinex-orderbook-10m-1.50usd_slippage': 3063,
    'bid_strength_usd-bitfinex-orderbook-10m-1.60usd_slippage': 3064,
    'bid_strength_usd-bitfinex-orderbook-10m-1.70usd_slippage': 3065,
    'bid_strength_usd-bitfinex-orderbook-10m-1.80usd_slippage': 3066,
    'bid_strength_usd-bitfinex-orderbook-10m-1.90usd_slippage': 3067,
    'bid_strength_usd-bitfinex-orderbook-10m-2.00usd_slippage': 3068,
    'bid_strength_usd-bitfinex-orderbook-10m-2.10usd_slippage': 3069,
    'bid_strength_usd-bitfinex-orderbook-10m-2.20usd_slippage': 3070,
    'bid_strength_usd-bitfinex-orderbook-10m-2.30usd_slippage': 3071,
    'bid_strength_usd-bitfinex-orderbook-10m-2.40usd_slippage': 3072,
    'bid_strength_usd-bitfinex-orderbook-10m-2.50usd_slippage': 3073,
    'bid_strength_usd-bitfinex-orderbook-10m-2.60usd_slippage': 3074,
    'bid_strength_usd-bitfinex-orderbook-10m-2.70usd_slippage': 3075,
    'bid_strength_usd-bitfinex-orderbook-10m-2.80usd_slippage': 3076,
    'bid_strength_usd-bitfinex-orderbook-10m-2.90usd_slippage': 3077,
    'bid_strength_usd-bitfinex-orderbook-10m-3.00usd_slippage': 3078,
    'bid_strength_usd-bitfinex-orderbook-10m-3.10usd_slippage': 3079,
    'bid_strength_usd-bitfinex-orderbook-10m-3.20usd_slippage': 3080,
    'bid_strength_usd-bitfinex-orderbook-10m-3.30usd_slippage': 3081,
    'bid_strength_usd-bitfinex-orderbook-10m-3.40usd_slippage': 3082,
    'bid_strength_usd-bitfinex-orderbook-10m-3.50usd_slippage': 3083,
    'bid_strength_usd-bitfinex-orderbook-10m-3.60usd_slippage': 3084,
    'bid_strength_usd-bitfinex-orderbook-10m-3.70usd_slippage': 3085,
    'bid_strength_usd-bitfinex-orderbook-10m-3.80usd_slippage': 3086,
    'bid_strength_usd-bitfinex-orderbook-10m-3.90usd_slippage': 3087,
    'bid_strength_usd-bitfinex-orderbook-10m-4.00usd_slippage': 3088,
    'bid_strength_usd-bitfinex-orderbook-10m-4.10usd_slippage': 3089,
    'bid_strength_usd-bitfinex-orderbook-10m-4.20usd_slippage': 3090,
    'bid_strength_usd-bitfinex-orderbook-10m-4.30usd_slippage': 3091,
    'bid_strength_usd-bitfinex-orderbook-10m-4.40usd_slippage': 3092,
    'bid_strength_usd-bitfinex-orderbook-10m-4.50usd_slippage': 3093,
    'bid_strength_usd-bitfinex-orderbook-10m-4.60usd_slippage': 3094,
    'bid_strength_usd-bitfinex-orderbook-10m-4.70usd_slippage': 3095,
    'bid_strength_usd-bitfinex-orderbook-10m-4.80usd_slippage': 3096,
    'bid_strength_usd-bitfinex-orderbook-10m-4.90usd_slippage': 3097,
    'bid_strength_usd-bitfinex-orderbook-10m-5.00usd_slippage': 3098,
    'bid_strength_usd-bitfinex-orderbook-10m-10.00usd_slippage': 3099,
    'bid_strength_usd-bitfinex-orderbook-10m-20.00usd_slippage': 3100,
    'bid_strength_usd-bitfinex-orderbook-5m-0.00usd_slippage': 3101,
    'bid_strength_usd-bitfinex-orderbook-5m-0.10usd_slippage': 3102,
    'bid_strength_usd-bitfinex-orderbook-5m-0.20usd_slippage': 3103,
    'bid_strength_usd-bitfinex-orderbook-5m-0.30usd_slippage': 3104,
    'bid_strength_usd-bitfinex-orderbook-5m-0.40usd_slippage': 3105,
    'bid_strength_usd-bitfinex-orderbook-5m-0.50usd_slippage': 3106,
    'bid_strength_usd-bitfinex-orderbook-5m-0.60usd_slippage': 3107,
    'bid_strength_usd-bitfinex-orderbook-5m-0.70usd_slippage': 3108,
    'bid_strength_usd-bitfinex-orderbook-5m-0.80usd_slippage': 3109,
    'bid_strength_usd-bitfinex-orderbook-5m-0.90usd_slippage': 3110,
    'bid_strength_usd-bitfinex-orderbook-5m-1.00usd_slippage': 3111,
    'bid_strength_usd-bitfinex-orderbook-5m-1.10usd_slippage': 3112,
    'bid_strength_usd-bitfinex-orderbook-5m-1.20usd_slippage': 3113,
    'bid_strength_usd-bitfinex-orderbook-5m-1.30usd_slippage': 3114,
    'bid_strength_usd-bitfinex-orderbook-5m-1.40usd_slippage': 3115,
    'bid_strength_usd-bitfinex-orderbook-5m-1.50usd_slippage': 3116,
    'bid_strength_usd-bitfinex-orderbook-5m-1.60usd_slippage': 3117,
    'bid_strength_usd-bitfinex-orderbook-5m-1.70usd_slippage': 3118,
    'bid_strength_usd-bitfinex-orderbook-5m-1.80usd_slippage': 3119,
    'bid_strength_usd-bitfinex-orderbook-5m-1.90usd_slippage': 3120,
    'bid_strength_usd-bitfinex-orderbook-5m-2.00usd_slippage': 3121,
    'bid_strength_usd-bitfinex-orderbook-5m-2.10usd_slippage': 3122,
    'bid_strength_usd-bitfinex-orderbook-5m-2.20usd_slippage': 3123,
    'bid_strength_usd-bitfinex-orderbook-5m-2.30usd_slippage': 3124,
    'bid_strength_usd-bitfinex-orderbook-5m-2.40usd_slippage': 3125,
    'bid_strength_usd-bitfinex-orderbook-5m-2.50usd_slippage': 3126,
    'bid_strength_usd-bitfinex-orderbook-5m-2.60usd_slippage': 3127,
    'bid_strength_usd-bitfinex-orderbook-5m-2.70usd_slippage': 3128,
    'bid_strength_usd-bitfinex-orderbook-5m-2.80usd_slippage': 3129,
    'bid_strength_usd-bitfinex-orderbook-5m-2.90usd_slippage': 3130,
    'bid_strength_usd-bitfinex-orderbook-5m-3.00usd_slippage': 3131,
    'bid_strength_usd-bitfinex-orderbook-5m-3.10usd_slippage': 3132,
    'bid_strength_usd-bitfinex-orderbook-5m-3.20usd_slippage': 3133,
    'bid_strength_usd-bitfinex-orderbook-5m-3.30usd_slippage': 3134,
    'bid_strength_usd-bitfinex-orderbook-5m-3.40usd_slippage': 3135,
    'bid_strength_usd-bitfinex-orderbook-5m-3.50usd_slippage': 3136,
    'bid_strength_usd-bitfinex-orderbook-5m-3.60usd_slippage': 3137,
    'bid_strength_usd-bitfinex-orderbook-5m-3.70usd_slippage': 3138,
    'bid_strength_usd-bitfinex-orderbook-5m-3.80usd_slippage': 3139,
    'bid_strength_usd-bitfinex-orderbook-5m-3.90usd_slippage': 3140,
    'bid_strength_usd-bitfinex-orderbook-5m-4.00usd_slippage': 3141,
    'bid_strength_usd-bitfinex-orderbook-5m-4.10usd_slippage': 3142,
    'bid_strength_usd-bitfinex-orderbook-5m-4.20usd_slippage': 3143,
    'bid_strength_usd-bitfinex-orderbook-5m-4.30usd_slippage': 3144,
    'bid_strength_usd-bitfinex-orderbook-5m-4.40usd_slippage': 3145,
    'bid_strength_usd-bitfinex-orderbook-5m-4.50usd_slippage': 3146,
    'bid_strength_usd-bitfinex-orderbook-5m-4.60usd_slippage': 3147,
    'bid_strength_usd-bitfinex-orderbook-5m-4.70usd_slippage': 3148,
    'bid_strength_usd-bitfinex-orderbook-5m-4.80usd_slippage': 3149,
    'bid_strength_usd-bitfinex-orderbook-5m-4.90usd_slippage': 3150,
    'bid_strength_usd-bitfinex-orderbook-5m-5.00usd_slippage': 3151,
    'bid_strength_usd-bitfinex-orderbook-5m-10.00usd_slippage': 3152,
    'bid_strength_usd-bitfinex-orderbook-5m-20.00usd_slippage': 3153,
    'bid_strength_usd-bitfinex-orderbook-1m-0.00usd_slippage': 3154,
    'bid_strength_usd-bitfinex-orderbook-1m-0.10usd_slippage': 3155,
    'bid_strength_usd-bitfinex-orderbook-1m-0.20usd_slippage': 3156,
    'bid_strength_usd-bitfinex-orderbook-1m-0.30usd_slippage': 3157,
    'bid_strength_usd-bitfinex-orderbook-1m-0.40usd_slippage': 3158,
    'bid_strength_usd-bitfinex-orderbook-1m-0.50usd_slippage': 3159,
    'bid_strength_usd-bitfinex-orderbook-1m-0.60usd_slippage': 3160,
    'bid_strength_usd-bitfinex-orderbook-1m-0.70usd_slippage': 3161,
    'bid_strength_usd-bitfinex-orderbook-1m-0.80usd_slippage': 3162,
    'bid_strength_usd-bitfinex-orderbook-1m-0.90usd_slippage': 3163,
    'bid_strength_usd-bitfinex-orderbook-1m-1.00usd_slippage': 3164,
    'bid_strength_usd-bitfinex-orderbook-1m-1.10usd_slippage': 3165,
    'bid_strength_usd-bitfinex-orderbook-1m-1.20usd_slippage': 3166,
    'bid_strength_usd-bitfinex-orderbook-1m-1.30usd_slippage': 3167,
    'bid_strength_usd-bitfinex-orderbook-1m-1.40usd_slippage': 3168,
    'bid_strength_usd-bitfinex-orderbook-1m-1.50usd_slippage': 3169,
    'bid_strength_usd-bitfinex-orderbook-1m-1.60usd_slippage': 3170,
    'bid_strength_usd-bitfinex-orderbook-1m-1.70usd_slippage': 3171,
    'bid_strength_usd-bitfinex-orderbook-1m-1.80usd_slippage': 3172,
    'bid_strength_usd-bitfinex-orderbook-1m-1.90usd_slippage': 3173,
    'bid_strength_usd-bitfinex-orderbook-1m-2.00usd_slippage': 3174,
    'bid_strength_usd-bitfinex-orderbook-1m-2.10usd_slippage': 3175,
    'bid_strength_usd-bitfinex-orderbook-1m-2.20usd_slippage': 3176,
    'bid_strength_usd-bitfinex-orderbook-1m-2.30usd_slippage': 3177,
    'bid_strength_usd-bitfinex-orderbook-1m-2.40usd_slippage': 3178,
    'bid_strength_usd-bitfinex-orderbook-1m-2.50usd_slippage': 3179,
    'bid_strength_usd-bitfinex-orderbook-1m-2.60usd_slippage': 3180,
    'bid_strength_usd-bitfinex-orderbook-1m-2.70usd_slippage': 3181,
    'bid_strength_usd-bitfinex-orderbook-1m-2.80usd_slippage': 3182,
    'bid_strength_usd-bitfinex-orderbook-1m-2.90usd_slippage': 3183,
    'bid_strength_usd-bitfinex-orderbook-1m-3.00usd_slippage': 3184,
    'bid_strength_usd-bitfinex-orderbook-1m-3.10usd_slippage': 3185,
    'bid_strength_usd-bitfinex-orderbook-1m-3.20usd_slippage': 3186,
    'bid_strength_usd-bitfinex-orderbook-1m-3.30usd_slippage': 3187,
    'bid_strength_usd-bitfinex-orderbook-1m-3.40usd_slippage': 3188,
    'bid_strength_usd-bitfinex-orderbook-1m-3.50usd_slippage': 3189,
    'bid_strength_usd-bitfinex-orderbook-1m-3.60usd_slippage': 3190,
    'bid_strength_usd-bitfinex-orderbook-1m-3.70usd_slippage': 3191,
    'bid_strength_usd-bitfinex-orderbook-1m-3.80usd_slippage': 3192,
    'bid_strength_usd-bitfinex-orderbook-1m-3.90usd_slippage': 3193,
    'bid_strength_usd-bitfinex-orderbook-1m-4.00usd_slippage': 3194,
    'bid_strength_usd-bitfinex-orderbook-1m-4.10usd_slippage': 3195,
    'bid_strength_usd-bitfinex-orderbook-1m-4.20usd_slippage': 3196,
    'bid_strength_usd-bitfinex-orderbook-1m-4.30usd_slippage': 3197,
    'bid_strength_usd-bitfinex-orderbook-1m-4.40usd_slippage': 3198,
    'bid_strength_usd-bitfinex-orderbook-1m-4.50usd_slippage': 3199,
    'bid_strength_usd-bitfinex-orderbook-1m-4.60usd_slippage': 3200,
    'bid_strength_usd-bitfinex-orderbook-1m-4.70usd_slippage': 3201,
    'bid_strength_usd-bitfinex-orderbook-1m-4.80usd_slippage': 3202,
    'bid_strength_usd-bitfinex-orderbook-1m-4.90usd_slippage': 3203,
    'bid_strength_usd-bitfinex-orderbook-1m-5.00usd_slippage': 3204,
    'bid_strength_usd-bitfinex-orderbook-1m-10.00usd_slippage': 3205,
    'bid_strength_usd-bitfinex-orderbook-1m-20.00usd_slippage': 3206,
    'bid_strength_usd-coinbase-orderbook-10m-0.00usd_slippage': 3207,
    'bid_strength_usd-coinbase-orderbook-10m-0.10usd_slippage': 3208,
    'bid_strength_usd-coinbase-orderbook-10m-0.20usd_slippage': 3209,
    'bid_strength_usd-coinbase-orderbook-10m-0.30usd_slippage': 3210,
    'bid_strength_usd-coinbase-orderbook-10m-0.40usd_slippage': 3211,
    'bid_strength_usd-coinbase-orderbook-10m-0.50usd_slippage': 3212,
    'bid_strength_usd-coinbase-orderbook-10m-0.60usd_slippage': 3213,
    'bid_strength_usd-coinbase-orderbook-10m-0.70usd_slippage': 3214,
    'bid_strength_usd-coinbase-orderbook-10m-0.80usd_slippage': 3215,
    'bid_strength_usd-coinbase-orderbook-10m-0.90usd_slippage': 3216,
    'bid_strength_usd-coinbase-orderbook-10m-1.00usd_slippage': 3217,
    'bid_strength_usd-coinbase-orderbook-10m-1.10usd_slippage': 3218,
    'bid_strength_usd-coinbase-orderbook-10m-1.20usd_slippage': 3219,
    'bid_strength_usd-coinbase-orderbook-10m-1.30usd_slippage': 3220,
    'bid_strength_usd-coinbase-orderbook-10m-1.40usd_slippage': 3221,
    'bid_strength_usd-coinbase-orderbook-10m-1.50usd_slippage': 3222,
    'bid_strength_usd-coinbase-orderbook-10m-1.60usd_slippage': 3223,
    'bid_strength_usd-coinbase-orderbook-10m-1.70usd_slippage': 3224,
    'bid_strength_usd-coinbase-orderbook-10m-1.80usd_slippage': 3225,
    'bid_strength_usd-coinbase-orderbook-10m-1.90usd_slippage': 3226,
    'bid_strength_usd-coinbase-orderbook-10m-2.00usd_slippage': 3227,
    'bid_strength_usd-coinbase-orderbook-10m-2.10usd_slippage': 3228,
    'bid_strength_usd-coinbase-orderbook-10m-2.20usd_slippage': 3229,
    'bid_strength_usd-coinbase-orderbook-10m-2.30usd_slippage': 3230,
    'bid_strength_usd-coinbase-orderbook-10m-2.40usd_slippage': 3231,
    'bid_strength_usd-coinbase-orderbook-10m-2.50usd_slippage': 3232,
    'bid_strength_usd-coinbase-orderbook-10m-2.60usd_slippage': 3233,
    'bid_strength_usd-coinbase-orderbook-10m-2.70usd_slippage': 3234,
    'bid_strength_usd-coinbase-orderbook-10m-2.80usd_slippage': 3235,
    'bid_strength_usd-coinbase-orderbook-10m-2.90usd_slippage': 3236,
    'bid_strength_usd-coinbase-orderbook-10m-3.00usd_slippage': 3237,
    'bid_strength_usd-coinbase-orderbook-10m-3.10usd_slippage': 3238,
    'bid_strength_usd-coinbase-orderbook-10m-3.20usd_slippage': 3239,
    'bid_strength_usd-coinbase-orderbook-10m-3.30usd_slippage': 3240,
    'bid_strength_usd-coinbase-orderbook-10m-3.40usd_slippage': 3241,
    'bid_strength_usd-coinbase-orderbook-10m-3.50usd_slippage': 3242,
    'bid_strength_usd-coinbase-orderbook-10m-3.60usd_slippage': 3243,
    'bid_strength_usd-coinbase-orderbook-10m-3.70usd_slippage': 3244,
    'bid_strength_usd-coinbase-orderbook-10m-3.80usd_slippage': 3245,
    'bid_strength_usd-coinbase-orderbook-10m-3.90usd_slippage': 3246,
    'bid_strength_usd-coinbase-orderbook-10m-4.00usd_slippage': 3247,
    'bid_strength_usd-coinbase-orderbook-10m-4.10usd_slippage': 3248,
    'bid_strength_usd-coinbase-orderbook-10m-4.20usd_slippage': 3249,
    'bid_strength_usd-coinbase-orderbook-10m-4.30usd_slippage': 3250,
    'bid_strength_usd-coinbase-orderbook-10m-4.40usd_slippage': 3251,
    'bid_strength_usd-coinbase-orderbook-10m-4.50usd_slippage': 3252,
    'bid_strength_usd-coinbase-orderbook-10m-4.60usd_slippage': 3253,
    'bid_strength_usd-coinbase-orderbook-10m-4.70usd_slippage': 3254,
    'bid_strength_usd-coinbase-orderbook-10m-4.80usd_slippage': 3255,
    'bid_strength_usd-coinbase-orderbook-10m-4.90usd_slippage': 3256,
    'bid_strength_usd-coinbase-orderbook-10m-5.00usd_slippage': 3257,
    'bid_strength_usd-coinbase-orderbook-10m-10.00usd_slippage': 3258,
    'bid_strength_usd-coinbase-orderbook-10m-20.00usd_slippage': 3259,
    'bid_strength_usd-coinbase-orderbook-5m-0.00usd_slippage': 3260,
    'bid_strength_usd-coinbase-orderbook-5m-0.10usd_slippage': 3261,
    'bid_strength_usd-coinbase-orderbook-5m-0.20usd_slippage': 3262,
    'bid_strength_usd-coinbase-orderbook-5m-0.30usd_slippage': 3263,
    'bid_strength_usd-coinbase-orderbook-5m-0.40usd_slippage': 3264,
    'bid_strength_usd-coinbase-orderbook-5m-0.50usd_slippage': 3265,
    'bid_strength_usd-coinbase-orderbook-5m-0.60usd_slippage': 3266,
    'bid_strength_usd-coinbase-orderbook-5m-0.70usd_slippage': 3267,
    'bid_strength_usd-coinbase-orderbook-5m-0.80usd_slippage': 3268,
    'bid_strength_usd-coinbase-orderbook-5m-0.90usd_slippage': 3269,
    'bid_strength_usd-coinbase-orderbook-5m-1.00usd_slippage': 3270,
    'bid_strength_usd-coinbase-orderbook-5m-1.10usd_slippage': 3271,
    'bid_strength_usd-coinbase-orderbook-5m-1.20usd_slippage': 3272,
    'bid_strength_usd-coinbase-orderbook-5m-1.30usd_slippage': 3273,
    'bid_strength_usd-coinbase-orderbook-5m-1.40usd_slippage': 3274,
    'bid_strength_usd-coinbase-orderbook-5m-1.50usd_slippage': 3275,
    'bid_strength_usd-coinbase-orderbook-5m-1.60usd_slippage': 3276,
    'bid_strength_usd-coinbase-orderbook-5m-1.70usd_slippage': 3277,
    'bid_strength_usd-coinbase-orderbook-5m-1.80usd_slippage': 3278,
    'bid_strength_usd-coinbase-orderbook-5m-1.90usd_slippage': 3279,
    'bid_strength_usd-coinbase-orderbook-5m-2.00usd_slippage': 3280,
    'bid_strength_usd-coinbase-orderbook-5m-2.10usd_slippage': 3281,
    'bid_strength_usd-coinbase-orderbook-5m-2.20usd_slippage': 3282,
    'bid_strength_usd-coinbase-orderbook-5m-2.30usd_slippage': 3283,
    'bid_strength_usd-coinbase-orderbook-5m-2.40usd_slippage': 3284,
    'bid_strength_usd-coinbase-orderbook-5m-2.50usd_slippage': 3285,
    'bid_strength_usd-coinbase-orderbook-5m-2.60usd_slippage': 3286,
    'bid_strength_usd-coinbase-orderbook-5m-2.70usd_slippage': 3287,
    'bid_strength_usd-coinbase-orderbook-5m-2.80usd_slippage': 3288,
    'bid_strength_usd-coinbase-orderbook-5m-2.90usd_slippage': 3289,
    'bid_strength_usd-coinbase-orderbook-5m-3.00usd_slippage': 3290,
    'bid_strength_usd-coinbase-orderbook-5m-3.10usd_slippage': 3291,
    'bid_strength_usd-coinbase-orderbook-5m-3.20usd_slippage': 3292,
    'bid_strength_usd-coinbase-orderbook-5m-3.30usd_slippage': 3293,
    'bid_strength_usd-coinbase-orderbook-5m-3.40usd_slippage': 3294,
    'bid_strength_usd-coinbase-orderbook-5m-3.50usd_slippage': 3295,
    'bid_strength_usd-coinbase-orderbook-5m-3.60usd_slippage': 3296,
    'bid_strength_usd-coinbase-orderbook-5m-3.70usd_slippage': 3297,
    'bid_strength_usd-coinbase-orderbook-5m-3.80usd_slippage': 3298,
    'bid_strength_usd-coinbase-orderbook-5m-3.90usd_slippage': 3299,
    'bid_strength_usd-coinbase-orderbook-5m-4.00usd_slippage': 3300,
    'bid_strength_usd-coinbase-orderbook-5m-4.10usd_slippage': 3301,
    'bid_strength_usd-coinbase-orderbook-5m-4.20usd_slippage': 3302,
    'bid_strength_usd-coinbase-orderbook-5m-4.30usd_slippage': 3303,
    'bid_strength_usd-coinbase-orderbook-5m-4.40usd_slippage': 3304,
    'bid_strength_usd-coinbase-orderbook-5m-4.50usd_slippage': 3305,
    'bid_strength_usd-coinbase-orderbook-5m-4.60usd_slippage': 3306,
    'bid_strength_usd-coinbase-orderbook-5m-4.70usd_slippage': 3307,
    'bid_strength_usd-coinbase-orderbook-5m-4.80usd_slippage': 3308,
    'bid_strength_usd-coinbase-orderbook-5m-4.90usd_slippage': 3309,
    'bid_strength_usd-coinbase-orderbook-5m-5.00usd_slippage': 3310,
    'bid_strength_usd-coinbase-orderbook-5m-10.00usd_slippage': 3311,
    'bid_strength_usd-coinbase-orderbook-5m-20.00usd_slippage': 3312,
    'bid_strength_usd-coinbase-orderbook-1m-0.00usd_slippage': 3313,
    'bid_strength_usd-coinbase-orderbook-1m-0.10usd_slippage': 3314,
    'bid_strength_usd-coinbase-orderbook-1m-0.20usd_slippage': 3315,
    'bid_strength_usd-coinbase-orderbook-1m-0.30usd_slippage': 3316,
    'bid_strength_usd-coinbase-orderbook-1m-0.40usd_slippage': 3317,
    'bid_strength_usd-coinbase-orderbook-1m-0.50usd_slippage': 3318,
    'bid_strength_usd-coinbase-orderbook-1m-0.60usd_slippage': 3319,
    'bid_strength_usd-coinbase-orderbook-1m-0.70usd_slippage': 3320,
    'bid_strength_usd-coinbase-orderbook-1m-0.80usd_slippage': 3321,
    'bid_strength_usd-coinbase-orderbook-1m-0.90usd_slippage': 3322,
    'bid_strength_usd-coinbase-orderbook-1m-1.00usd_slippage': 3323,
    'bid_strength_usd-coinbase-orderbook-1m-1.10usd_slippage': 3324,
    'bid_strength_usd-coinbase-orderbook-1m-1.20usd_slippage': 3325,
    'bid_strength_usd-coinbase-orderbook-1m-1.30usd_slippage': 3326,
    'bid_strength_usd-coinbase-orderbook-1m-1.40usd_slippage': 3327,
    'bid_strength_usd-coinbase-orderbook-1m-1.50usd_slippage': 3328,
    'bid_strength_usd-coinbase-orderbook-1m-1.60usd_slippage': 3329,
    'bid_strength_usd-coinbase-orderbook-1m-1.70usd_slippage': 3330,
    'bid_strength_usd-coinbase-orderbook-1m-1.80usd_slippage': 3331,
    'bid_strength_usd-coinbase-orderbook-1m-1.90usd_slippage': 3332,
    'bid_strength_usd-coinbase-orderbook-1m-2.00usd_slippage': 3333,
    'bid_strength_usd-coinbase-orderbook-1m-2.10usd_slippage': 3334,
    'bid_strength_usd-coinbase-orderbook-1m-2.20usd_slippage': 3335,
    'bid_strength_usd-coinbase-orderbook-1m-2.30usd_slippage': 3336,
    'bid_strength_usd-coinbase-orderbook-1m-2.40usd_slippage': 3337,
    'bid_strength_usd-coinbase-orderbook-1m-2.50usd_slippage': 3338,
    'bid_strength_usd-coinbase-orderbook-1m-2.60usd_slippage': 3339,
    'bid_strength_usd-coinbase-orderbook-1m-2.70usd_slippage': 3340,
    'bid_strength_usd-coinbase-orderbook-1m-2.80usd_slippage': 3341,
    'bid_strength_usd-coinbase-orderbook-1m-2.90usd_slippage': 3342,
    'bid_strength_usd-coinbase-orderbook-1m-3.00usd_slippage': 3343,
    'bid_strength_usd-coinbase-orderbook-1m-3.10usd_slippage': 3344,
    'bid_strength_usd-coinbase-orderbook-1m-3.20usd_slippage': 3345,
    'bid_strength_usd-coinbase-orderbook-1m-3.30usd_slippage': 3346,
    'bid_strength_usd-coinbase-orderbook-1m-3.40usd_slippage': 3347,
    'bid_strength_usd-coinbase-orderbook-1m-3.50usd_slippage': 3348,
    'bid_strength_usd-coinbase-orderbook-1m-3.60usd_slippage': 3349,
    'bid_strength_usd-coinbase-orderbook-1m-3.70usd_slippage': 3350,
    'bid_strength_usd-coinbase-orderbook-1m-3.80usd_slippage': 3351,
    'bid_strength_usd-coinbase-orderbook-1m-3.90usd_slippage': 3352,
    'bid_strength_usd-coinbase-orderbook-1m-4.00usd_slippage': 3353,
    'bid_strength_usd-coinbase-orderbook-1m-4.10usd_slippage': 3354,
    'bid_strength_usd-coinbase-orderbook-1m-4.20usd_slippage': 3355,
    'bid_strength_usd-coinbase-orderbook-1m-4.30usd_slippage': 3356,
    'bid_strength_usd-coinbase-orderbook-1m-4.40usd_slippage': 3357,
    'bid_strength_usd-coinbase-orderbook-1m-4.50usd_slippage': 3358,
    'bid_strength_usd-coinbase-orderbook-1m-4.60usd_slippage': 3359,
    'bid_strength_usd-coinbase-orderbook-1m-4.70usd_slippage': 3360,
    'bid_strength_usd-coinbase-orderbook-1m-4.80usd_slippage': 3361,
    'bid_strength_usd-coinbase-orderbook-1m-4.90usd_slippage': 3362,
    'bid_strength_usd-coinbase-orderbook-1m-5.00usd_slippage': 3363,
    'bid_strength_usd-coinbase-orderbook-1m-10.00usd_slippage': 3364,
    'bid_strength_usd-coinbase-orderbook-1m-20.00usd_slippage': 3365,
    'bid_strength_usd-itbit-orderbook-10m-0.00usd_slippage': 3366,
    'bid_strength_usd-itbit-orderbook-10m-0.10usd_slippage': 3367,
    'bid_strength_usd-itbit-orderbook-10m-0.20usd_slippage': 3368,
    'bid_strength_usd-itbit-orderbook-10m-0.30usd_slippage': 3369,
    'bid_strength_usd-itbit-orderbook-10m-0.40usd_slippage': 3370,
    'bid_strength_usd-itbit-orderbook-10m-0.50usd_slippage': 3371,
    'bid_strength_usd-itbit-orderbook-10m-0.60usd_slippage': 3372,
    'bid_strength_usd-itbit-orderbook-10m-0.70usd_slippage': 3373,
    'bid_strength_usd-itbit-orderbook-10m-0.80usd_slippage': 3374,
    'bid_strength_usd-itbit-orderbook-10m-0.90usd_slippage': 3375,
    'bid_strength_usd-itbit-orderbook-10m-1.00usd_slippage': 3376,
    'bid_strength_usd-itbit-orderbook-10m-1.10usd_slippage': 3377,
    'bid_strength_usd-itbit-orderbook-10m-1.20usd_slippage': 3378,
    'bid_strength_usd-itbit-orderbook-10m-1.30usd_slippage': 3379,
    'bid_strength_usd-itbit-orderbook-10m-1.40usd_slippage': 3380,
    'bid_strength_usd-itbit-orderbook-10m-1.50usd_slippage': 3381,
    'bid_strength_usd-itbit-orderbook-10m-1.60usd_slippage': 3382,
    'bid_strength_usd-itbit-orderbook-10m-1.70usd_slippage': 3383,
    'bid_strength_usd-itbit-orderbook-10m-1.80usd_slippage': 3384,
    'bid_strength_usd-itbit-orderbook-10m-1.90usd_slippage': 3385,
    'bid_strength_usd-itbit-orderbook-10m-2.00usd_slippage': 3386,
    'bid_strength_usd-itbit-orderbook-10m-2.10usd_slippage': 3387,
    'bid_strength_usd-itbit-orderbook-10m-2.20usd_slippage': 3388,
    'bid_strength_usd-itbit-orderbook-10m-2.30usd_slippage': 3389,
    'bid_strength_usd-itbit-orderbook-10m-2.40usd_slippage': 3390,
    'bid_strength_usd-itbit-orderbook-10m-2.50usd_slippage': 3391,
    'bid_strength_usd-itbit-orderbook-10m-2.60usd_slippage': 3392,
    'bid_strength_usd-itbit-orderbook-10m-2.70usd_slippage': 3393,
    'bid_strength_usd-itbit-orderbook-10m-2.80usd_slippage': 3394,
    'bid_strength_usd-itbit-orderbook-10m-2.90usd_slippage': 3395,
    'bid_strength_usd-itbit-orderbook-10m-3.00usd_slippage': 3396,
    'bid_strength_usd-itbit-orderbook-10m-3.10usd_slippage': 3397,
    'bid_strength_usd-itbit-orderbook-10m-3.20usd_slippage': 3398,
    'bid_strength_usd-itbit-orderbook-10m-3.30usd_slippage': 3399,
    'bid_strength_usd-itbit-orderbook-10m-3.40usd_slippage': 3400,
    'bid_strength_usd-itbit-orderbook-10m-3.50usd_slippage': 3401,
    'bid_strength_usd-itbit-orderbook-10m-3.60usd_slippage': 3402,
    'bid_strength_usd-itbit-orderbook-10m-3.70usd_slippage': 3403,
    'bid_strength_usd-itbit-orderbook-10m-3.80usd_slippage': 3404,
    'bid_strength_usd-itbit-orderbook-10m-3.90usd_slippage': 3405,
    'bid_strength_usd-itbit-orderbook-10m-4.00usd_slippage': 3406,
    'bid_strength_usd-itbit-orderbook-10m-4.10usd_slippage': 3407,
    'bid_strength_usd-itbit-orderbook-10m-4.20usd_slippage': 3408,
    'bid_strength_usd-itbit-orderbook-10m-4.30usd_slippage': 3409,
    'bid_strength_usd-itbit-orderbook-10m-4.40usd_slippage': 3410,
    'bid_strength_usd-itbit-orderbook-10m-4.50usd_slippage': 3411,
    'bid_strength_usd-itbit-orderbook-10m-4.60usd_slippage': 3412,
    'bid_strength_usd-itbit-orderbook-10m-4.70usd_slippage': 3413,
    'bid_strength_usd-itbit-orderbook-10m-4.80usd_slippage': 3414,
    'bid_strength_usd-itbit-orderbook-10m-4.90usd_slippage': 3415,
    'bid_strength_usd-itbit-orderbook-10m-5.00usd_slippage': 3416,
    'bid_strength_usd-itbit-orderbook-10m-10.00usd_slippage': 3417,
    'bid_strength_usd-itbit-orderbook-10m-20.00usd_slippage': 3418,
    'bid_strength_usd-itbit-orderbook-5m-0.00usd_slippage': 3419,
    'bid_strength_usd-itbit-orderbook-5m-0.10usd_slippage': 3420,
    'bid_strength_usd-itbit-orderbook-5m-0.20usd_slippage': 3421,
    'bid_strength_usd-itbit-orderbook-5m-0.30usd_slippage': 3422,
    'bid_strength_usd-itbit-orderbook-5m-0.40usd_slippage': 3423,
    'bid_strength_usd-itbit-orderbook-5m-0.50usd_slippage': 3424,
    'bid_strength_usd-itbit-orderbook-5m-0.60usd_slippage': 3425,
    'bid_strength_usd-itbit-orderbook-5m-0.70usd_slippage': 3426,
    'bid_strength_usd-itbit-orderbook-5m-0.80usd_slippage': 3427,
    'bid_strength_usd-itbit-orderbook-5m-0.90usd_slippage': 3428,
    'bid_strength_usd-itbit-orderbook-5m-1.00usd_slippage': 3429,
    'bid_strength_usd-itbit-orderbook-5m-1.10usd_slippage': 3430,
    'bid_strength_usd-itbit-orderbook-5m-1.20usd_slippage': 3431,
    'bid_strength_usd-itbit-orderbook-5m-1.30usd_slippage': 3432,
    'bid_strength_usd-itbit-orderbook-5m-1.40usd_slippage': 3433,
    'bid_strength_usd-itbit-orderbook-5m-1.50usd_slippage': 3434,
    'bid_strength_usd-itbit-orderbook-5m-1.60usd_slippage': 3435,
    'bid_strength_usd-itbit-orderbook-5m-1.70usd_slippage': 3436,
    'bid_strength_usd-itbit-orderbook-5m-1.80usd_slippage': 3437,
    'bid_strength_usd-itbit-orderbook-5m-1.90usd_slippage': 3438,
    'bid_strength_usd-itbit-orderbook-5m-2.00usd_slippage': 3439,
    'bid_strength_usd-itbit-orderbook-5m-2.10usd_slippage': 3440,
    'bid_strength_usd-itbit-orderbook-5m-2.20usd_slippage': 3441,
    'bid_strength_usd-itbit-orderbook-5m-2.30usd_slippage': 3442,
    'bid_strength_usd-itbit-orderbook-5m-2.40usd_slippage': 3443,
    'bid_strength_usd-itbit-orderbook-5m-2.50usd_slippage': 3444,
    'bid_strength_usd-itbit-orderbook-5m-2.60usd_slippage': 3445,
    'bid_strength_usd-itbit-orderbook-5m-2.70usd_slippage': 3446,
    'bid_strength_usd-itbit-orderbook-5m-2.80usd_slippage': 3447,
    'bid_strength_usd-itbit-orderbook-5m-2.90usd_slippage': 3448,
    'bid_strength_usd-itbit-orderbook-5m-3.00usd_slippage': 3449,
    'bid_strength_usd-itbit-orderbook-5m-3.10usd_slippage': 3450,
    'bid_strength_usd-itbit-orderbook-5m-3.20usd_slippage': 3451,
    'bid_strength_usd-itbit-orderbook-5m-3.30usd_slippage': 3452,
    'bid_strength_usd-itbit-orderbook-5m-3.40usd_slippage': 3453,
    'bid_strength_usd-itbit-orderbook-5m-3.50usd_slippage': 3454,
    'bid_strength_usd-itbit-orderbook-5m-3.60usd_slippage': 3455,
    'bid_strength_usd-itbit-orderbook-5m-3.70usd_slippage': 3456,
    'bid_strength_usd-itbit-orderbook-5m-3.80usd_slippage': 3457,
    'bid_strength_usd-itbit-orderbook-5m-3.90usd_slippage': 3458,
    'bid_strength_usd-itbit-orderbook-5m-4.00usd_slippage': 3459,
    'bid_strength_usd-itbit-orderbook-5m-4.10usd_slippage': 3460,
    'bid_strength_usd-itbit-orderbook-5m-4.20usd_slippage': 3461,
    'bid_strength_usd-itbit-orderbook-5m-4.30usd_slippage': 3462,
    'bid_strength_usd-itbit-orderbook-5m-4.40usd_slippage': 3463,
    'bid_strength_usd-itbit-orderbook-5m-4.50usd_slippage': 3464,
    'bid_strength_usd-itbit-orderbook-5m-4.60usd_slippage': 3465,
    'bid_strength_usd-itbit-orderbook-5m-4.70usd_slippage': 3466,
    'bid_strength_usd-itbit-orderbook-5m-4.80usd_slippage': 3467,
    'bid_strength_usd-itbit-orderbook-5m-4.90usd_slippage': 3468,
    'bid_strength_usd-itbit-orderbook-5m-5.00usd_slippage': 3469,
    'bid_strength_usd-itbit-orderbook-5m-10.00usd_slippage': 3470,
    'bid_strength_usd-itbit-orderbook-5m-20.00usd_slippage': 3471,
    'bid_strength_usd-itbit-orderbook-1m-0.00usd_slippage': 3472,
    'bid_strength_usd-itbit-orderbook-1m-0.10usd_slippage': 3473,
    'bid_strength_usd-itbit-orderbook-1m-0.20usd_slippage': 3474,
    'bid_strength_usd-itbit-orderbook-1m-0.30usd_slippage': 3475,
    'bid_strength_usd-itbit-orderbook-1m-0.40usd_slippage': 3476,
    'bid_strength_usd-itbit-orderbook-1m-0.50usd_slippage': 3477,
    'bid_strength_usd-itbit-orderbook-1m-0.60usd_slippage': 3478,
    'bid_strength_usd-itbit-orderbook-1m-0.70usd_slippage': 3479,
    'bid_strength_usd-itbit-orderbook-1m-0.80usd_slippage': 3480,
    'bid_strength_usd-itbit-orderbook-1m-0.90usd_slippage': 3481,
    'bid_strength_usd-itbit-orderbook-1m-1.00usd_slippage': 3482,
    'bid_strength_usd-itbit-orderbook-1m-1.10usd_slippage': 3483,
    'bid_strength_usd-itbit-orderbook-1m-1.20usd_slippage': 3484,
    'bid_strength_usd-itbit-orderbook-1m-1.30usd_slippage': 3485,
    'bid_strength_usd-itbit-orderbook-1m-1.40usd_slippage': 3486,
    'bid_strength_usd-itbit-orderbook-1m-1.50usd_slippage': 3487,
    'bid_strength_usd-itbit-orderbook-1m-1.60usd_slippage': 3488,
    'bid_strength_usd-itbit-orderbook-1m-1.70usd_slippage': 3489,
    'bid_strength_usd-itbit-orderbook-1m-1.80usd_slippage': 3490,
    'bid_strength_usd-itbit-orderbook-1m-1.90usd_slippage': 3491,
    'bid_strength_usd-itbit-orderbook-1m-2.00usd_slippage': 3492,
    'bid_strength_usd-itbit-orderbook-1m-2.10usd_slippage': 3493,
    'bid_strength_usd-itbit-orderbook-1m-2.20usd_slippage': 3494,
    'bid_strength_usd-itbit-orderbook-1m-2.30usd_slippage': 3495,
    'bid_strength_usd-itbit-orderbook-1m-2.40usd_slippage': 3496,
    'bid_strength_usd-itbit-orderbook-1m-2.50usd_slippage': 3497,
    'bid_strength_usd-itbit-orderbook-1m-2.60usd_slippage': 3498,
    'bid_strength_usd-itbit-orderbook-1m-2.70usd_slippage': 3499,
    'bid_strength_usd-itbit-orderbook-1m-2.80usd_slippage': 3500,
    'bid_strength_usd-itbit-orderbook-1m-2.90usd_slippage': 3501,
    'bid_strength_usd-itbit-orderbook-1m-3.00usd_slippage': 3502,
    'bid_strength_usd-itbit-orderbook-1m-3.10usd_slippage': 3503,
    'bid_strength_usd-itbit-orderbook-1m-3.20usd_slippage': 3504,
    'bid_strength_usd-itbit-orderbook-1m-3.30usd_slippage': 3505,
    'bid_strength_usd-itbit-orderbook-1m-3.40usd_slippage': 3506,
    'bid_strength_usd-itbit-orderbook-1m-3.50usd_slippage': 3507,
    'bid_strength_usd-itbit-orderbook-1m-3.60usd_slippage': 3508,
    'bid_strength_usd-itbit-orderbook-1m-3.70usd_slippage': 3509,
    'bid_strength_usd-itbit-orderbook-1m-3.80usd_slippage': 3510,
    'bid_strength_usd-itbit-orderbook-1m-3.90usd_slippage': 3511,
    'bid_strength_usd-itbit-orderbook-1m-4.00usd_slippage': 3512,
    'bid_strength_usd-itbit-orderbook-1m-4.10usd_slippage': 3513,
    'bid_strength_usd-itbit-orderbook-1m-4.20usd_slippage': 3514,
    'bid_strength_usd-itbit-orderbook-1m-4.30usd_slippage': 3515,
    'bid_strength_usd-itbit-orderbook-1m-4.40usd_slippage': 3516,
    'bid_strength_usd-itbit-orderbook-1m-4.50usd_slippage': 3517,
    'bid_strength_usd-itbit-orderbook-1m-4.60usd_slippage': 3518,
    'bid_strength_usd-itbit-orderbook-1m-4.70usd_slippage': 3519,
    'bid_strength_usd-itbit-orderbook-1m-4.80usd_slippage': 3520,
    'bid_strength_usd-itbit-orderbook-1m-4.90usd_slippage': 3521,
    'bid_strength_usd-itbit-orderbook-1m-5.00usd_slippage': 3522,
    'bid_strength_usd-itbit-orderbook-1m-10.00usd_slippage': 3523,
    'bid_strength_usd-itbit-orderbook-1m-20.00usd_slippage': 3524,
    'bid_strength_usd-kraken-orderbook-10m-0.00usd_slippage': 3525,
    'bid_strength_usd-kraken-orderbook-10m-0.10usd_slippage': 3526,
    'bid_strength_usd-kraken-orderbook-10m-0.20usd_slippage': 3527,
    'bid_strength_usd-kraken-orderbook-10m-0.30usd_slippage': 3528,
    'bid_strength_usd-kraken-orderbook-10m-0.40usd_slippage': 3529,
    'bid_strength_usd-kraken-orderbook-10m-0.50usd_slippage': 3530,
    'bid_strength_usd-kraken-orderbook-10m-0.60usd_slippage': 3531,
    'bid_strength_usd-kraken-orderbook-10m-0.70usd_slippage': 3532,
    'bid_strength_usd-kraken-orderbook-10m-0.80usd_slippage': 3533,
    'bid_strength_usd-kraken-orderbook-10m-0.90usd_slippage': 3534,
    'bid_strength_usd-kraken-orderbook-10m-1.00usd_slippage': 3535,
    'bid_strength_usd-kraken-orderbook-10m-1.10usd_slippage': 3536,
    'bid_strength_usd-kraken-orderbook-10m-1.20usd_slippage': 3537,
    'bid_strength_usd-kraken-orderbook-10m-1.30usd_slippage': 3538,
    'bid_strength_usd-kraken-orderbook-10m-1.40usd_slippage': 3539,
    'bid_strength_usd-kraken-orderbook-10m-1.50usd_slippage': 3540,
    'bid_strength_usd-kraken-orderbook-10m-1.60usd_slippage': 3541,
    'bid_strength_usd-kraken-orderbook-10m-1.70usd_slippage': 3542,
    'bid_strength_usd-kraken-orderbook-10m-1.80usd_slippage': 3543,
    'bid_strength_usd-kraken-orderbook-10m-1.90usd_slippage': 3544,
    'bid_strength_usd-kraken-orderbook-10m-2.00usd_slippage': 3545,
    'bid_strength_usd-kraken-orderbook-10m-2.10usd_slippage': 3546,
    'bid_strength_usd-kraken-orderbook-10m-2.20usd_slippage': 3547,
    'bid_strength_usd-kraken-orderbook-10m-2.30usd_slippage': 3548,
    'bid_strength_usd-kraken-orderbook-10m-2.40usd_slippage': 3549,
    'bid_strength_usd-kraken-orderbook-10m-2.50usd_slippage': 3550,
    'bid_strength_usd-kraken-orderbook-10m-2.60usd_slippage': 3551,
    'bid_strength_usd-kraken-orderbook-10m-2.70usd_slippage': 3552,
    'bid_strength_usd-kraken-orderbook-10m-2.80usd_slippage': 3553,
    'bid_strength_usd-kraken-orderbook-10m-2.90usd_slippage': 3554,
    'bid_strength_usd-kraken-orderbook-10m-3.00usd_slippage': 3555,
    'bid_strength_usd-kraken-orderbook-10m-3.10usd_slippage': 3556,
    'bid_strength_usd-kraken-orderbook-10m-3.20usd_slippage': 3557,
    'bid_strength_usd-kraken-orderbook-10m-3.30usd_slippage': 3558,
    'bid_strength_usd-kraken-orderbook-10m-3.40usd_slippage': 3559,
    'bid_strength_usd-kraken-orderbook-10m-3.50usd_slippage': 3560,
    'bid_strength_usd-kraken-orderbook-10m-3.60usd_slippage': 3561,
    'bid_strength_usd-kraken-orderbook-10m-3.70usd_slippage': 3562,
    'bid_strength_usd-kraken-orderbook-10m-3.80usd_slippage': 3563,
    'bid_strength_usd-kraken-orderbook-10m-3.90usd_slippage': 3564,
    'bid_strength_usd-kraken-orderbook-10m-4.00usd_slippage': 3565,
    'bid_strength_usd-kraken-orderbook-10m-4.10usd_slippage': 3566,
    'bid_strength_usd-kraken-orderbook-10m-4.20usd_slippage': 3567,
    'bid_strength_usd-kraken-orderbook-10m-4.30usd_slippage': 3568,
    'bid_strength_usd-kraken-orderbook-10m-4.40usd_slippage': 3569,
    'bid_strength_usd-kraken-orderbook-10m-4.50usd_slippage': 3570,
    'bid_strength_usd-kraken-orderbook-10m-4.60usd_slippage': 3571,
    'bid_strength_usd-kraken-orderbook-10m-4.70usd_slippage': 3572,
    'bid_strength_usd-kraken-orderbook-10m-4.80usd_slippage': 3573,
    'bid_strength_usd-kraken-orderbook-10m-4.90usd_slippage': 3574,
    'bid_strength_usd-kraken-orderbook-10m-5.00usd_slippage': 3575,
    'bid_strength_usd-kraken-orderbook-10m-10.00usd_slippage': 3576,
    'bid_strength_usd-kraken-orderbook-10m-20.00usd_slippage': 3577,
    'bid_strength_usd-kraken-orderbook-5m-0.00usd_slippage': 3578,
    'bid_strength_usd-kraken-orderbook-5m-0.10usd_slippage': 3579,
    'bid_strength_usd-kraken-orderbook-5m-0.20usd_slippage': 3580,
    'bid_strength_usd-kraken-orderbook-5m-0.30usd_slippage': 3581,
    'bid_strength_usd-kraken-orderbook-5m-0.40usd_slippage': 3582,
    'bid_strength_usd-kraken-orderbook-5m-0.50usd_slippage': 3583,
    'bid_strength_usd-kraken-orderbook-5m-0.60usd_slippage': 3584,
    'bid_strength_usd-kraken-orderbook-5m-0.70usd_slippage': 3585,
    'bid_strength_usd-kraken-orderbook-5m-0.80usd_slippage': 3586,
    'bid_strength_usd-kraken-orderbook-5m-0.90usd_slippage': 3587,
    'bid_strength_usd-kraken-orderbook-5m-1.00usd_slippage': 3588,
    'bid_strength_usd-kraken-orderbook-5m-1.10usd_slippage': 3589,
    'bid_strength_usd-kraken-orderbook-5m-1.20usd_slippage': 3590,
    'bid_strength_usd-kraken-orderbook-5m-1.30usd_slippage': 3591,
    'bid_strength_usd-kraken-orderbook-5m-1.40usd_slippage': 3592,
    'bid_strength_usd-kraken-orderbook-5m-1.50usd_slippage': 3593,
    'bid_strength_usd-kraken-orderbook-5m-1.60usd_slippage': 3594,
    'bid_strength_usd-kraken-orderbook-5m-1.70usd_slippage': 3595,
    'bid_strength_usd-kraken-orderbook-5m-1.80usd_slippage': 3596,
    'bid_strength_usd-kraken-orderbook-5m-1.90usd_slippage': 3597,
    'bid_strength_usd-kraken-orderbook-5m-2.00usd_slippage': 3598,
    'bid_strength_usd-kraken-orderbook-5m-2.10usd_slippage': 3599,
    'bid_strength_usd-kraken-orderbook-5m-2.20usd_slippage': 3600,
    'bid_strength_usd-kraken-orderbook-5m-2.30usd_slippage': 3601,
    'bid_strength_usd-kraken-orderbook-5m-2.40usd_slippage': 3602,
    'bid_strength_usd-kraken-orderbook-5m-2.50usd_slippage': 3603,
    'bid_strength_usd-kraken-orderbook-5m-2.60usd_slippage': 3604,
    'bid_strength_usd-kraken-orderbook-5m-2.70usd_slippage': 3605,
    'bid_strength_usd-kraken-orderbook-5m-2.80usd_slippage': 3606,
    'bid_strength_usd-kraken-orderbook-5m-2.90usd_slippage': 3607,
    'bid_strength_usd-kraken-orderbook-5m-3.00usd_slippage': 3608,
    'bid_strength_usd-kraken-orderbook-5m-3.10usd_slippage': 3609,
    'bid_strength_usd-kraken-orderbook-5m-3.20usd_slippage': 3610,
    'bid_strength_usd-kraken-orderbook-5m-3.30usd_slippage': 3611,
    'bid_strength_usd-kraken-orderbook-5m-3.40usd_slippage': 3612,
    'bid_strength_usd-kraken-orderbook-5m-3.50usd_slippage': 3613,
    'bid_strength_usd-kraken-orderbook-5m-3.60usd_slippage': 3614,
    'bid_strength_usd-kraken-orderbook-5m-3.70usd_slippage': 3615,
    'bid_strength_usd-kraken-orderbook-5m-3.80usd_slippage': 3616,
    'bid_strength_usd-kraken-orderbook-5m-3.90usd_slippage': 3617,
    'bid_strength_usd-kraken-orderbook-5m-4.00usd_slippage': 3618,
    'bid_strength_usd-kraken-orderbook-5m-4.10usd_slippage': 3619,
    'bid_strength_usd-kraken-orderbook-5m-4.20usd_slippage': 3620,
    'bid_strength_usd-kraken-orderbook-5m-4.30usd_slippage': 3621,
    'bid_strength_usd-kraken-orderbook-5m-4.40usd_slippage': 3622,
    'bid_strength_usd-kraken-orderbook-5m-4.50usd_slippage': 3623,
    'bid_strength_usd-kraken-orderbook-5m-4.60usd_slippage': 3624,
    'bid_strength_usd-kraken-orderbook-5m-4.70usd_slippage': 3625,
    'bid_strength_usd-kraken-orderbook-5m-4.80usd_slippage': 3626,
    'bid_strength_usd-kraken-orderbook-5m-4.90usd_slippage': 3627,
    'bid_strength_usd-kraken-orderbook-5m-5.00usd_slippage': 3628,
    'bid_strength_usd-kraken-orderbook-5m-10.00usd_slippage': 3629,
    'bid_strength_usd-kraken-orderbook-5m-20.00usd_slippage': 3630,
    'bid_strength_usd-kraken-orderbook-1m-0.00usd_slippage': 3631,
    'bid_strength_usd-kraken-orderbook-1m-0.10usd_slippage': 3632,
    'bid_strength_usd-kraken-orderbook-1m-0.20usd_slippage': 3633,
    'bid_strength_usd-kraken-orderbook-1m-0.30usd_slippage': 3634,
    'bid_strength_usd-kraken-orderbook-1m-0.40usd_slippage': 3635,
    'bid_strength_usd-kraken-orderbook-1m-0.50usd_slippage': 3636,
    'bid_strength_usd-kraken-orderbook-1m-0.60usd_slippage': 3637,
    'bid_strength_usd-kraken-orderbook-1m-0.70usd_slippage': 3638,
    'bid_strength_usd-kraken-orderbook-1m-0.80usd_slippage': 3639,
    'bid_strength_usd-kraken-orderbook-1m-0.90usd_slippage': 3640,
    'bid_strength_usd-kraken-orderbook-1m-1.00usd_slippage': 3641,
    'bid_strength_usd-kraken-orderbook-1m-1.10usd_slippage': 3642,
    'bid_strength_usd-kraken-orderbook-1m-1.20usd_slippage': 3643,
    'bid_strength_usd-kraken-orderbook-1m-1.30usd_slippage': 3644,
    'bid_strength_usd-kraken-orderbook-1m-1.40usd_slippage': 3645,
    'bid_strength_usd-kraken-orderbook-1m-1.50usd_slippage': 3646,
    'bid_strength_usd-kraken-orderbook-1m-1.60usd_slippage': 3647,
    'bid_strength_usd-kraken-orderbook-1m-1.70usd_slippage': 3648,
    'bid_strength_usd-kraken-orderbook-1m-1.80usd_slippage': 3649,
    'bid_strength_usd-kraken-orderbook-1m-1.90usd_slippage': 3650,
    'bid_strength_usd-kraken-orderbook-1m-2.00usd_slippage': 3651,
    'bid_strength_usd-kraken-orderbook-1m-2.10usd_slippage': 3652,
    'bid_strength_usd-kraken-orderbook-1m-2.20usd_slippage': 3653,
    'bid_strength_usd-kraken-orderbook-1m-2.30usd_slippage': 3654,
    'bid_strength_usd-kraken-orderbook-1m-2.40usd_slippage': 3655,
    'bid_strength_usd-kraken-orderbook-1m-2.50usd_slippage': 3656,
    'bid_strength_usd-kraken-orderbook-1m-2.60usd_slippage': 3657,
    'bid_strength_usd-kraken-orderbook-1m-2.70usd_slippage': 3658,
    'bid_strength_usd-kraken-orderbook-1m-2.80usd_slippage': 3659,
    'bid_strength_usd-kraken-orderbook-1m-2.90usd_slippage': 3660,
    'bid_strength_usd-kraken-orderbook-1m-3.00usd_slippage': 3661,
    'bid_strength_usd-kraken-orderbook-1m-3.10usd_slippage': 3662,
    'bid_strength_usd-kraken-orderbook-1m-3.20usd_slippage': 3663,
    'bid_strength_usd-kraken-orderbook-1m-3.30usd_slippage': 3664,
    'bid_strength_usd-kraken-orderbook-1m-3.40usd_slippage': 3665,
    'bid_strength_usd-kraken-orderbook-1m-3.50usd_slippage': 3666,
    'bid_strength_usd-kraken-orderbook-1m-3.60usd_slippage': 3667,
    'bid_strength_usd-kraken-orderbook-1m-3.70usd_slippage': 3668,
    'bid_strength_usd-kraken-orderbook-1m-3.80usd_slippage': 3669,
    'bid_strength_usd-kraken-orderbook-1m-3.90usd_slippage': 3670,
    'bid_strength_usd-kraken-orderbook-1m-4.00usd_slippage': 3671,
    'bid_strength_usd-kraken-orderbook-1m-4.10usd_slippage': 3672,
    'bid_strength_usd-kraken-orderbook-1m-4.20usd_slippage': 3673,
    'bid_strength_usd-kraken-orderbook-1m-4.30usd_slippage': 3674,
    'bid_strength_usd-kraken-orderbook-1m-4.40usd_slippage': 3675,
    'bid_strength_usd-kraken-orderbook-1m-4.50usd_slippage': 3676,
    'bid_strength_usd-kraken-orderbook-1m-4.60usd_slippage': 3677,
    'bid_strength_usd-kraken-orderbook-1m-4.70usd_slippage': 3678,
    'bid_strength_usd-kraken-orderbook-1m-4.80usd_slippage': 3679,
    'bid_strength_usd-kraken-orderbook-1m-4.90usd_slippage': 3680,
    'bid_strength_usd-kraken-orderbook-1m-5.00usd_slippage': 3681,
    'bid_strength_usd-kraken-orderbook-1m-10.00usd_slippage': 3682,
    'bid_strength_usd-kraken-orderbook-1m-20.00usd_slippage': 3683,
    'bid_strength_usd-gemini-orderbook-10m-0.00usd_slippage': 3684,
    'bid_strength_usd-gemini-orderbook-10m-0.10usd_slippage': 3685,
    'bid_strength_usd-gemini-orderbook-10m-0.20usd_slippage': 3686,
    'bid_strength_usd-gemini-orderbook-10m-0.30usd_slippage': 3687,
    'bid_strength_usd-gemini-orderbook-10m-0.40usd_slippage': 3688,
    'bid_strength_usd-gemini-orderbook-10m-0.50usd_slippage': 3689,
    'bid_strength_usd-gemini-orderbook-10m-0.60usd_slippage': 3690,
    'bid_strength_usd-gemini-orderbook-10m-0.70usd_slippage': 3691,
    'bid_strength_usd-gemini-orderbook-10m-0.80usd_slippage': 3692,
    'bid_strength_usd-gemini-orderbook-10m-0.90usd_slippage': 3693,
    'bid_strength_usd-gemini-orderbook-10m-1.00usd_slippage': 3694,
    'bid_strength_usd-gemini-orderbook-10m-1.10usd_slippage': 3695,
    'bid_strength_usd-gemini-orderbook-10m-1.20usd_slippage': 3696,
    'bid_strength_usd-gemini-orderbook-10m-1.30usd_slippage': 3697,
    'bid_strength_usd-gemini-orderbook-10m-1.40usd_slippage': 3698,
    'bid_strength_usd-gemini-orderbook-10m-1.50usd_slippage': 3699,
    'bid_strength_usd-gemini-orderbook-10m-1.60usd_slippage': 3700,
    'bid_strength_usd-gemini-orderbook-10m-1.70usd_slippage': 3701,
    'bid_strength_usd-gemini-orderbook-10m-1.80usd_slippage': 3702,
    'bid_strength_usd-gemini-orderbook-10m-1.90usd_slippage': 3703,
    'bid_strength_usd-gemini-orderbook-10m-2.00usd_slippage': 3704,
    'bid_strength_usd-gemini-orderbook-10m-2.10usd_slippage': 3705,
    'bid_strength_usd-gemini-orderbook-10m-2.20usd_slippage': 3706,
    'bid_strength_usd-gemini-orderbook-10m-2.30usd_slippage': 3707,
    'bid_strength_usd-gemini-orderbook-10m-2.40usd_slippage': 3708,
    'bid_strength_usd-gemini-orderbook-10m-2.50usd_slippage': 3709,
    'bid_strength_usd-gemini-orderbook-10m-2.60usd_slippage': 3710,
    'bid_strength_usd-gemini-orderbook-10m-2.70usd_slippage': 3711,
    'bid_strength_usd-gemini-orderbook-10m-2.80usd_slippage': 3712,
    'bid_strength_usd-gemini-orderbook-10m-2.90usd_slippage': 3713,
    'bid_strength_usd-gemini-orderbook-10m-3.00usd_slippage': 3714,
    'bid_strength_usd-gemini-orderbook-10m-3.10usd_slippage': 3715,
    'bid_strength_usd-gemini-orderbook-10m-3.20usd_slippage': 3716,
    'bid_strength_usd-gemini-orderbook-10m-3.30usd_slippage': 3717,
    'bid_strength_usd-gemini-orderbook-10m-3.40usd_slippage': 3718,
    'bid_strength_usd-gemini-orderbook-10m-3.50usd_slippage': 3719,
    'bid_strength_usd-gemini-orderbook-10m-3.60usd_slippage': 3720,
    'bid_strength_usd-gemini-orderbook-10m-3.70usd_slippage': 3721,
    'bid_strength_usd-gemini-orderbook-10m-3.80usd_slippage': 3722,
    'bid_strength_usd-gemini-orderbook-10m-3.90usd_slippage': 3723,
    'bid_strength_usd-gemini-orderbook-10m-4.00usd_slippage': 3724,
    'bid_strength_usd-gemini-orderbook-10m-4.10usd_slippage': 3725,
    'bid_strength_usd-gemini-orderbook-10m-4.20usd_slippage': 3726,
    'bid_strength_usd-gemini-orderbook-10m-4.30usd_slippage': 3727,
    'bid_strength_usd-gemini-orderbook-10m-4.40usd_slippage': 3728,
    'bid_strength_usd-gemini-orderbook-10m-4.50usd_slippage': 3729,
    'bid_strength_usd-gemini-orderbook-10m-4.60usd_slippage': 3730,
    'bid_strength_usd-gemini-orderbook-10m-4.70usd_slippage': 3731,
    'bid_strength_usd-gemini-orderbook-10m-4.80usd_slippage': 3732,
    'bid_strength_usd-gemini-orderbook-10m-4.90usd_slippage': 3733,
    'bid_strength_usd-gemini-orderbook-10m-5.00usd_slippage': 3734,
    'bid_strength_usd-gemini-orderbook-10m-10.00usd_slippage': 3735,
    'bid_strength_usd-gemini-orderbook-10m-20.00usd_slippage': 3736,
    'bid_strength_usd-gemini-orderbook-5m-0.00usd_slippage': 3737,
    'bid_strength_usd-gemini-orderbook-5m-0.10usd_slippage': 3738,
    'bid_strength_usd-gemini-orderbook-5m-0.20usd_slippage': 3739,
    'bid_strength_usd-gemini-orderbook-5m-0.30usd_slippage': 3740,
    'bid_strength_usd-gemini-orderbook-5m-0.40usd_slippage': 3741,
    'bid_strength_usd-gemini-orderbook-5m-0.50usd_slippage': 3742,
    'bid_strength_usd-gemini-orderbook-5m-0.60usd_slippage': 3743,
    'bid_strength_usd-gemini-orderbook-5m-0.70usd_slippage': 3744,
    'bid_strength_usd-gemini-orderbook-5m-0.80usd_slippage': 3745,
    'bid_strength_usd-gemini-orderbook-5m-0.90usd_slippage': 3746,
    'bid_strength_usd-gemini-orderbook-5m-1.00usd_slippage': 3747,
    'bid_strength_usd-gemini-orderbook-5m-1.10usd_slippage': 3748,
    'bid_strength_usd-gemini-orderbook-5m-1.20usd_slippage': 3749,
    'bid_strength_usd-gemini-orderbook-5m-1.30usd_slippage': 3750,
    'bid_strength_usd-gemini-orderbook-5m-1.40usd_slippage': 3751,
    'bid_strength_usd-gemini-orderbook-5m-1.50usd_slippage': 3752,
    'bid_strength_usd-gemini-orderbook-5m-1.60usd_slippage': 3753,
    'bid_strength_usd-gemini-orderbook-5m-1.70usd_slippage': 3754,
    'bid_strength_usd-gemini-orderbook-5m-1.80usd_slippage': 3755,
    'bid_strength_usd-gemini-orderbook-5m-1.90usd_slippage': 3756,
    'bid_strength_usd-gemini-orderbook-5m-2.00usd_slippage': 3757,
    'bid_strength_usd-gemini-orderbook-5m-2.10usd_slippage': 3758,
    'bid_strength_usd-gemini-orderbook-5m-2.20usd_slippage': 3759,
    'bid_strength_usd-gemini-orderbook-5m-2.30usd_slippage': 3760,
    'bid_strength_usd-gemini-orderbook-5m-2.40usd_slippage': 3761,
    'bid_strength_usd-gemini-orderbook-5m-2.50usd_slippage': 3762,
    'bid_strength_usd-gemini-orderbook-5m-2.60usd_slippage': 3763,
    'bid_strength_usd-gemini-orderbook-5m-2.70usd_slippage': 3764,
    'bid_strength_usd-gemini-orderbook-5m-2.80usd_slippage': 3765,
    'bid_strength_usd-gemini-orderbook-5m-2.90usd_slippage': 3766,
    'bid_strength_usd-gemini-orderbook-5m-3.00usd_slippage': 3767,
    'bid_strength_usd-gemini-orderbook-5m-3.10usd_slippage': 3768,
    'bid_strength_usd-gemini-orderbook-5m-3.20usd_slippage': 3769,
    'bid_strength_usd-gemini-orderbook-5m-3.30usd_slippage': 3770,
    'bid_strength_usd-gemini-orderbook-5m-3.40usd_slippage': 3771,
    'bid_strength_usd-gemini-orderbook-5m-3.50usd_slippage': 3772,
    'bid_strength_usd-gemini-orderbook-5m-3.60usd_slippage': 3773,
    'bid_strength_usd-gemini-orderbook-5m-3.70usd_slippage': 3774,
    'bid_strength_usd-gemini-orderbook-5m-3.80usd_slippage': 3775,
    'bid_strength_usd-gemini-orderbook-5m-3.90usd_slippage': 3776,
    'bid_strength_usd-gemini-orderbook-5m-4.00usd_slippage': 3777,
    'bid_strength_usd-gemini-orderbook-5m-4.10usd_slippage': 3778,
    'bid_strength_usd-gemini-orderbook-5m-4.20usd_slippage': 3779,
    'bid_strength_usd-gemini-orderbook-5m-4.30usd_slippage': 3780,
    'bid_strength_usd-gemini-orderbook-5m-4.40usd_slippage': 3781,
    'bid_strength_usd-gemini-orderbook-5m-4.50usd_slippage': 3782,
    'bid_strength_usd-gemini-orderbook-5m-4.60usd_slippage': 3783,
    'bid_strength_usd-gemini-orderbook-5m-4.70usd_slippage': 3784,
    'bid_strength_usd-gemini-orderbook-5m-4.80usd_slippage': 3785,
    'bid_strength_usd-gemini-orderbook-5m-4.90usd_slippage': 3786,
    'bid_strength_usd-gemini-orderbook-5m-5.00usd_slippage': 3787,
    'bid_strength_usd-gemini-orderbook-5m-10.00usd_slippage': 3788,
    'bid_strength_usd-gemini-orderbook-5m-20.00usd_slippage': 3789,
    'bid_strength_usd-gemini-orderbook-1m-0.00usd_slippage': 3790,
    'bid_strength_usd-gemini-orderbook-1m-0.10usd_slippage': 3791,
    'bid_strength_usd-gemini-orderbook-1m-0.20usd_slippage': 3792,
    'bid_strength_usd-gemini-orderbook-1m-0.30usd_slippage': 3793,
    'bid_strength_usd-gemini-orderbook-1m-0.40usd_slippage': 3794,
    'bid_strength_usd-gemini-orderbook-1m-0.50usd_slippage': 3795,
    'bid_strength_usd-gemini-orderbook-1m-0.60usd_slippage': 3796,
    'bid_strength_usd-gemini-orderbook-1m-0.70usd_slippage': 3797,
    'bid_strength_usd-gemini-orderbook-1m-0.80usd_slippage': 3798,
    'bid_strength_usd-gemini-orderbook-1m-0.90usd_slippage': 3799,
    'bid_strength_usd-gemini-orderbook-1m-1.00usd_slippage': 3800,
    'bid_strength_usd-gemini-orderbook-1m-1.10usd_slippage': 3801,
    'bid_strength_usd-gemini-orderbook-1m-1.20usd_slippage': 3802,
    'bid_strength_usd-gemini-orderbook-1m-1.30usd_slippage': 3803,
    'bid_strength_usd-gemini-orderbook-1m-1.40usd_slippage': 3804,
    'bid_strength_usd-gemini-orderbook-1m-1.50usd_slippage': 3805,
    'bid_strength_usd-gemini-orderbook-1m-1.60usd_slippage': 3806,
    'bid_strength_usd-gemini-orderbook-1m-1.70usd_slippage': 3807,
    'bid_strength_usd-gemini-orderbook-1m-1.80usd_slippage': 3808,
    'bid_strength_usd-gemini-orderbook-1m-1.90usd_slippage': 3809,
    'bid_strength_usd-gemini-orderbook-1m-2.00usd_slippage': 3810,
    'bid_strength_usd-gemini-orderbook-1m-2.10usd_slippage': 3811,
    'bid_strength_usd-gemini-orderbook-1m-2.20usd_slippage': 3812,
    'bid_strength_usd-gemini-orderbook-1m-2.30usd_slippage': 3813,
    'bid_strength_usd-gemini-orderbook-1m-2.40usd_slippage': 3814,
    'bid_strength_usd-gemini-orderbook-1m-2.50usd_slippage': 3815,
    'bid_strength_usd-gemini-orderbook-1m-2.60usd_slippage': 3816,
    'bid_strength_usd-gemini-orderbook-1m-2.70usd_slippage': 3817,
    'bid_strength_usd-gemini-orderbook-1m-2.80usd_slippage': 3818,
    'bid_strength_usd-gemini-orderbook-1m-2.90usd_slippage': 3819,
    'bid_strength_usd-gemini-orderbook-1m-3.00usd_slippage': 3820,
    'bid_strength_usd-gemini-orderbook-1m-3.10usd_slippage': 3821,
    'bid_strength_usd-gemini-orderbook-1m-3.20usd_slippage': 3822,
    'bid_strength_usd-gemini-orderbook-1m-3.30usd_slippage': 3823,
    'bid_strength_usd-gemini-orderbook-1m-3.40usd_slippage': 3824,
    'bid_strength_usd-gemini-orderbook-1m-3.50usd_slippage': 3825,
    'bid_strength_usd-gemini-orderbook-1m-3.60usd_slippage': 3826,
    'bid_strength_usd-gemini-orderbook-1m-3.70usd_slippage': 3827,
    'bid_strength_usd-gemini-orderbook-1m-3.80usd_slippage': 3828,
    'bid_strength_usd-gemini-orderbook-1m-3.90usd_slippage': 3829,
    'bid_strength_usd-gemini-orderbook-1m-4.00usd_slippage': 3830,
    'bid_strength_usd-gemini-orderbook-1m-4.10usd_slippage': 3831,
    'bid_strength_usd-gemini-orderbook-1m-4.20usd_slippage': 3832,
    'bid_strength_usd-gemini-orderbook-1m-4.30usd_slippage': 3833,
    'bid_strength_usd-gemini-orderbook-1m-4.40usd_slippage': 3834,
    'bid_strength_usd-gemini-orderbook-1m-4.50usd_slippage': 3835,
    'bid_strength_usd-gemini-orderbook-1m-4.60usd_slippage': 3836,
    'bid_strength_usd-gemini-orderbook-1m-4.70usd_slippage': 3837,
    'bid_strength_usd-gemini-orderbook-1m-4.80usd_slippage': 3838,
    'bid_strength_usd-gemini-orderbook-1m-4.90usd_slippage': 3839,
    'bid_strength_usd-gemini-orderbook-1m-5.00usd_slippage': 3840,
    'bid_strength_usd-gemini-orderbook-1m-10.00usd_slippage': 3841,
    'bid_strength_usd-gemini-orderbook-1m-20.00usd_slippage': 3842,
    'bid_strength_usd-okcoin-orderbook-10m-0.00usd_slippage': 3843,
    'bid_strength_usd-okcoin-orderbook-10m-0.10usd_slippage': 3844,
    'bid_strength_usd-okcoin-orderbook-10m-0.20usd_slippage': 3845,
    'bid_strength_usd-okcoin-orderbook-10m-0.30usd_slippage': 3846,
    'bid_strength_usd-okcoin-orderbook-10m-0.40usd_slippage': 3847,
    'bid_strength_usd-okcoin-orderbook-10m-0.50usd_slippage': 3848,
    'bid_strength_usd-okcoin-orderbook-10m-0.60usd_slippage': 3849,
    'bid_strength_usd-okcoin-orderbook-10m-0.70usd_slippage': 3850,
    'bid_strength_usd-okcoin-orderbook-10m-0.80usd_slippage': 3851,
    'bid_strength_usd-okcoin-orderbook-10m-0.90usd_slippage': 3852,
    'bid_strength_usd-okcoin-orderbook-10m-1.00usd_slippage': 3853,
    'bid_strength_usd-okcoin-orderbook-10m-1.10usd_slippage': 3854,
    'bid_strength_usd-okcoin-orderbook-10m-1.20usd_slippage': 3855,
    'bid_strength_usd-okcoin-orderbook-10m-1.30usd_slippage': 3856,
    'bid_strength_usd-okcoin-orderbook-10m-1.40usd_slippage': 3857,
    'bid_strength_usd-okcoin-orderbook-10m-1.50usd_slippage': 3858,
    'bid_strength_usd-okcoin-orderbook-10m-1.60usd_slippage': 3859,
    'bid_strength_usd-okcoin-orderbook-10m-1.70usd_slippage': 3860,
    'bid_strength_usd-okcoin-orderbook-10m-1.80usd_slippage': 3861,
    'bid_strength_usd-okcoin-orderbook-10m-1.90usd_slippage': 3862,
    'bid_strength_usd-okcoin-orderbook-10m-2.00usd_slippage': 3863,
    'bid_strength_usd-okcoin-orderbook-10m-2.10usd_slippage': 3864,
    'bid_strength_usd-okcoin-orderbook-10m-2.20usd_slippage': 3865,
    'bid_strength_usd-okcoin-orderbook-10m-2.30usd_slippage': 3866,
    'bid_strength_usd-okcoin-orderbook-10m-2.40usd_slippage': 3867,
    'bid_strength_usd-okcoin-orderbook-10m-2.50usd_slippage': 3868,
    'bid_strength_usd-okcoin-orderbook-10m-2.60usd_slippage': 3869,
    'bid_strength_usd-okcoin-orderbook-10m-2.70usd_slippage': 3870,
    'bid_strength_usd-okcoin-orderbook-10m-2.80usd_slippage': 3871,
    'bid_strength_usd-okcoin-orderbook-10m-2.90usd_slippage': 3872,
    'bid_strength_usd-okcoin-orderbook-10m-3.00usd_slippage': 3873,
    'bid_strength_usd-okcoin-orderbook-10m-3.10usd_slippage': 3874,
    'bid_strength_usd-okcoin-orderbook-10m-3.20usd_slippage': 3875,
    'bid_strength_usd-okcoin-orderbook-10m-3.30usd_slippage': 3876,
    'bid_strength_usd-okcoin-orderbook-10m-3.40usd_slippage': 3877,
    'bid_strength_usd-okcoin-orderbook-10m-3.50usd_slippage': 3878,
    'bid_strength_usd-okcoin-orderbook-10m-3.60usd_slippage': 3879,
    'bid_strength_usd-okcoin-orderbook-10m-3.70usd_slippage': 3880,
    'bid_strength_usd-okcoin-orderbook-10m-3.80usd_slippage': 3881,
    'bid_strength_usd-okcoin-orderbook-10m-3.90usd_slippage': 3882,
    'bid_strength_usd-okcoin-orderbook-10m-4.00usd_slippage': 3883,
    'bid_strength_usd-okcoin-orderbook-10m-4.10usd_slippage': 3884,
    'bid_strength_usd-okcoin-orderbook-10m-4.20usd_slippage': 3885,
    'bid_strength_usd-okcoin-orderbook-10m-4.30usd_slippage': 3886,
    'bid_strength_usd-okcoin-orderbook-10m-4.40usd_slippage': 3887,
    'bid_strength_usd-okcoin-orderbook-10m-4.50usd_slippage': 3888,
    'bid_strength_usd-okcoin-orderbook-10m-4.60usd_slippage': 3889,
    'bid_strength_usd-okcoin-orderbook-10m-4.70usd_slippage': 3890,
    'bid_strength_usd-okcoin-orderbook-10m-4.80usd_slippage': 3891,
    'bid_strength_usd-okcoin-orderbook-10m-4.90usd_slippage': 3892,
    'bid_strength_usd-okcoin-orderbook-10m-5.00usd_slippage': 3893,
    'bid_strength_usd-okcoin-orderbook-10m-10.00usd_slippage': 3894,
    'bid_strength_usd-okcoin-orderbook-10m-20.00usd_slippage': 3895,
    'bid_strength_usd-okcoin-orderbook-5m-0.00usd_slippage': 3896,
    'bid_strength_usd-okcoin-orderbook-5m-0.10usd_slippage': 3897,
    'bid_strength_usd-okcoin-orderbook-5m-0.20usd_slippage': 3898,
    'bid_strength_usd-okcoin-orderbook-5m-0.30usd_slippage': 3899,
    'bid_strength_usd-okcoin-orderbook-5m-0.40usd_slippage': 3900,
    'bid_strength_usd-okcoin-orderbook-5m-0.50usd_slippage': 3901,
    'bid_strength_usd-okcoin-orderbook-5m-0.60usd_slippage': 3902,
    'bid_strength_usd-okcoin-orderbook-5m-0.70usd_slippage': 3903,
    'bid_strength_usd-okcoin-orderbook-5m-0.80usd_slippage': 3904,
    'bid_strength_usd-okcoin-orderbook-5m-0.90usd_slippage': 3905,
    'bid_strength_usd-okcoin-orderbook-5m-1.00usd_slippage': 3906,
    'bid_strength_usd-okcoin-orderbook-5m-1.10usd_slippage': 3907,
    'bid_strength_usd-okcoin-orderbook-5m-1.20usd_slippage': 3908,
    'bid_strength_usd-okcoin-orderbook-5m-1.30usd_slippage': 3909,
    'bid_strength_usd-okcoin-orderbook-5m-1.40usd_slippage': 3910,
    'bid_strength_usd-okcoin-orderbook-5m-1.50usd_slippage': 3911,
    'bid_strength_usd-okcoin-orderbook-5m-1.60usd_slippage': 3912,
    'bid_strength_usd-okcoin-orderbook-5m-1.70usd_slippage': 3913,
    'bid_strength_usd-okcoin-orderbook-5m-1.80usd_slippage': 3914,
    'bid_strength_usd-okcoin-orderbook-5m-1.90usd_slippage': 3915,
    'bid_strength_usd-okcoin-orderbook-5m-2.00usd_slippage': 3916,
    'bid_strength_usd-okcoin-orderbook-5m-2.10usd_slippage': 3917,
    'bid_strength_usd-okcoin-orderbook-5m-2.20usd_slippage': 3918,
    'bid_strength_usd-okcoin-orderbook-5m-2.30usd_slippage': 3919,
    'bid_strength_usd-okcoin-orderbook-5m-2.40usd_slippage': 3920,
    'bid_strength_usd-okcoin-orderbook-5m-2.50usd_slippage': 3921,
    'bid_strength_usd-okcoin-orderbook-5m-2.60usd_slippage': 3922,
    'bid_strength_usd-okcoin-orderbook-5m-2.70usd_slippage': 3923,
    'bid_strength_usd-okcoin-orderbook-5m-2.80usd_slippage': 3924,
    'bid_strength_usd-okcoin-orderbook-5m-2.90usd_slippage': 3925,
    'bid_strength_usd-okcoin-orderbook-5m-3.00usd_slippage': 3926,
    'bid_strength_usd-okcoin-orderbook-5m-3.10usd_slippage': 3927,
    'bid_strength_usd-okcoin-orderbook-5m-3.20usd_slippage': 3928,
    'bid_strength_usd-okcoin-orderbook-5m-3.30usd_slippage': 3929,
    'bid_strength_usd-okcoin-orderbook-5m-3.40usd_slippage': 3930,
    'bid_strength_usd-okcoin-orderbook-5m-3.50usd_slippage': 3931,
    'bid_strength_usd-okcoin-orderbook-5m-3.60usd_slippage': 3932,
    'bid_strength_usd-okcoin-orderbook-5m-3.70usd_slippage': 3933,
    'bid_strength_usd-okcoin-orderbook-5m-3.80usd_slippage': 3934,
    'bid_strength_usd-okcoin-orderbook-5m-3.90usd_slippage': 3935,
    'bid_strength_usd-okcoin-orderbook-5m-4.00usd_slippage': 3936,
    'bid_strength_usd-okcoin-orderbook-5m-4.10usd_slippage': 3937,
    'bid_strength_usd-okcoin-orderbook-5m-4.20usd_slippage': 3938,
    'bid_strength_usd-okcoin-orderbook-5m-4.30usd_slippage': 3939,
    'bid_strength_usd-okcoin-orderbook-5m-4.40usd_slippage': 3940,
    'bid_strength_usd-okcoin-orderbook-5m-4.50usd_slippage': 3941,
    'bid_strength_usd-okcoin-orderbook-5m-4.60usd_slippage': 3942,
    'bid_strength_usd-okcoin-orderbook-5m-4.70usd_slippage': 3943,
    'bid_strength_usd-okcoin-orderbook-5m-4.80usd_slippage': 3944,
    'bid_strength_usd-okcoin-orderbook-5m-4.90usd_slippage': 3945,
    'bid_strength_usd-okcoin-orderbook-5m-5.00usd_slippage': 3946,
    'bid_strength_usd-okcoin-orderbook-5m-10.00usd_slippage': 3947,
    'bid_strength_usd-okcoin-orderbook-5m-20.00usd_slippage': 3948,
    'bid_strength_usd-okcoin-orderbook-1m-0.00usd_slippage': 3949,
    'bid_strength_usd-okcoin-orderbook-1m-0.10usd_slippage': 3950,
    'bid_strength_usd-okcoin-orderbook-1m-0.20usd_slippage': 3951,
    'bid_strength_usd-okcoin-orderbook-1m-0.30usd_slippage': 3952,
    'bid_strength_usd-okcoin-orderbook-1m-0.40usd_slippage': 3953,
    'bid_strength_usd-okcoin-orderbook-1m-0.50usd_slippage': 3954,
    'bid_strength_usd-okcoin-orderbook-1m-0.60usd_slippage': 3955,
    'bid_strength_usd-okcoin-orderbook-1m-0.70usd_slippage': 3956,
    'bid_strength_usd-okcoin-orderbook-1m-0.80usd_slippage': 3957,
    'bid_strength_usd-okcoin-orderbook-1m-0.90usd_slippage': 3958,
    'bid_strength_usd-okcoin-orderbook-1m-1.00usd_slippage': 3959,
    'bid_strength_usd-okcoin-orderbook-1m-1.10usd_slippage': 3960,
    'bid_strength_usd-okcoin-orderbook-1m-1.20usd_slippage': 3961,
    'bid_strength_usd-okcoin-orderbook-1m-1.30usd_slippage': 3962,
    'bid_strength_usd-okcoin-orderbook-1m-1.40usd_slippage': 3963,
    'bid_strength_usd-okcoin-orderbook-1m-1.50usd_slippage': 3964,
    'bid_strength_usd-okcoin-orderbook-1m-1.60usd_slippage': 3965,
    'bid_strength_usd-okcoin-orderbook-1m-1.70usd_slippage': 3966,
    'bid_strength_usd-okcoin-orderbook-1m-1.80usd_slippage': 3967,
    'bid_strength_usd-okcoin-orderbook-1m-1.90usd_slippage': 3968,
    'bid_strength_usd-okcoin-orderbook-1m-2.00usd_slippage': 3969,
    'bid_strength_usd-okcoin-orderbook-1m-2.10usd_slippage': 3970,
    'bid_strength_usd-okcoin-orderbook-1m-2.20usd_slippage': 3971,
    'bid_strength_usd-okcoin-orderbook-1m-2.30usd_slippage': 3972,
    'bid_strength_usd-okcoin-orderbook-1m-2.40usd_slippage': 3973,
    'bid_strength_usd-okcoin-orderbook-1m-2.50usd_slippage': 3974,
    'bid_strength_usd-okcoin-orderbook-1m-2.60usd_slippage': 3975,
    'bid_strength_usd-okcoin-orderbook-1m-2.70usd_slippage': 3976,
    'bid_strength_usd-okcoin-orderbook-1m-2.80usd_slippage': 3977,
    'bid_strength_usd-okcoin-orderbook-1m-2.90usd_slippage': 3978,
    'bid_strength_usd-okcoin-orderbook-1m-3.00usd_slippage': 3979,
    'bid_strength_usd-okcoin-orderbook-1m-3.10usd_slippage': 3980,
    'bid_strength_usd-okcoin-orderbook-1m-3.20usd_slippage': 3981,
    'bid_strength_usd-okcoin-orderbook-1m-3.30usd_slippage': 3982,
    'bid_strength_usd-okcoin-orderbook-1m-3.40usd_slippage': 3983,
    'bid_strength_usd-okcoin-orderbook-1m-3.50usd_slippage': 3984,
    'bid_strength_usd-okcoin-orderbook-1m-3.60usd_slippage': 3985,
    'bid_strength_usd-okcoin-orderbook-1m-3.70usd_slippage': 3986,
    'bid_strength_usd-okcoin-orderbook-1m-3.80usd_slippage': 3987,
    'bid_strength_usd-okcoin-orderbook-1m-3.90usd_slippage': 3988,
    'bid_strength_usd-okcoin-orderbook-1m-4.00usd_slippage': 3989,
    'bid_strength_usd-okcoin-orderbook-1m-4.10usd_slippage': 3990,
    'bid_strength_usd-okcoin-orderbook-1m-4.20usd_slippage': 3991,
    'bid_strength_usd-okcoin-orderbook-1m-4.30usd_slippage': 3992,
    'bid_strength_usd-okcoin-orderbook-1m-4.40usd_slippage': 3993,
    'bid_strength_usd-okcoin-orderbook-1m-4.50usd_slippage': 3994,
    'bid_strength_usd-okcoin-orderbook-1m-4.60usd_slippage': 3995,
    'bid_strength_usd-okcoin-orderbook-1m-4.70usd_slippage': 3996,
    'bid_strength_usd-okcoin-orderbook-1m-4.80usd_slippage': 3997,
    'bid_strength_usd-okcoin-orderbook-1m-4.90usd_slippage': 3998,
    'bid_strength_usd-okcoin-orderbook-1m-5.00usd_slippage': 3999,
    'bid_strength_usd-okcoin-orderbook-1m-10.00usd_slippage': 4000,
    'bid_strength_usd-okcoin-orderbook-1m-20.00usd_slippage': 4001,
    'ask_strength_usd-bitstamp-orderbook-10m-0.00usd_slippage': 4002,
    'ask_strength_usd-bitstamp-orderbook-10m-0.10usd_slippage': 4003,
    'ask_strength_usd-bitstamp-orderbook-10m-0.20usd_slippage': 4004,
    'ask_strength_usd-bitstamp-orderbook-10m-0.30usd_slippage': 4005,
    'ask_strength_usd-bitstamp-orderbook-10m-0.40usd_slippage': 4006,
    'ask_strength_usd-bitstamp-orderbook-10m-0.50usd_slippage': 4007,
    'ask_strength_usd-bitstamp-orderbook-10m-0.60usd_slippage': 4008,
    'ask_strength_usd-bitstamp-orderbook-10m-0.70usd_slippage': 4009,
    'ask_strength_usd-bitstamp-orderbook-10m-0.80usd_slippage': 4010,
    'ask_strength_usd-bitstamp-orderbook-10m-0.90usd_slippage': 4011,
    'ask_strength_usd-bitstamp-orderbook-10m-1.00usd_slippage': 4012,
    'ask_strength_usd-bitstamp-orderbook-10m-1.10usd_slippage': 4013,
    'ask_strength_usd-bitstamp-orderbook-10m-1.20usd_slippage': 4014,
    'ask_strength_usd-bitstamp-orderbook-10m-1.30usd_slippage': 4015,
    'ask_strength_usd-bitstamp-orderbook-10m-1.40usd_slippage': 4016,
    'ask_strength_usd-bitstamp-orderbook-10m-1.50usd_slippage': 4017,
    'ask_strength_usd-bitstamp-orderbook-10m-1.60usd_slippage': 4018,
    'ask_strength_usd-bitstamp-orderbook-10m-1.70usd_slippage': 4019,
    'ask_strength_usd-bitstamp-orderbook-10m-1.80usd_slippage': 4020,
    'ask_strength_usd-bitstamp-orderbook-10m-1.90usd_slippage': 4021,
    'ask_strength_usd-bitstamp-orderbook-10m-2.00usd_slippage': 4022,
    'ask_strength_usd-bitstamp-orderbook-10m-2.10usd_slippage': 4023,
    'ask_strength_usd-bitstamp-orderbook-10m-2.20usd_slippage': 4024,
    'ask_strength_usd-bitstamp-orderbook-10m-2.30usd_slippage': 4025,
    'ask_strength_usd-bitstamp-orderbook-10m-2.40usd_slippage': 4026,
    'ask_strength_usd-bitstamp-orderbook-10m-2.50usd_slippage': 4027,
    'ask_strength_usd-bitstamp-orderbook-10m-2.60usd_slippage': 4028,
    'ask_strength_usd-bitstamp-orderbook-10m-2.70usd_slippage': 4029,
    'ask_strength_usd-bitstamp-orderbook-10m-2.80usd_slippage': 4030,
    'ask_strength_usd-bitstamp-orderbook-10m-2.90usd_slippage': 4031,
    'ask_strength_usd-bitstamp-orderbook-10m-3.00usd_slippage': 4032,
    'ask_strength_usd-bitstamp-orderbook-10m-3.10usd_slippage': 4033,
    'ask_strength_usd-bitstamp-orderbook-10m-3.20usd_slippage': 4034,
    'ask_strength_usd-bitstamp-orderbook-10m-3.30usd_slippage': 4035,
    'ask_strength_usd-bitstamp-orderbook-10m-3.40usd_slippage': 4036,
    'ask_strength_usd-bitstamp-orderbook-10m-3.50usd_slippage': 4037,
    'ask_strength_usd-bitstamp-orderbook-10m-3.60usd_slippage': 4038,
    'ask_strength_usd-bitstamp-orderbook-10m-3.70usd_slippage': 4039,
    'ask_strength_usd-bitstamp-orderbook-10m-3.80usd_slippage': 4040,
    'ask_strength_usd-bitstamp-orderbook-10m-3.90usd_slippage': 4041,
    'ask_strength_usd-bitstamp-orderbook-10m-4.00usd_slippage': 4042,
    'ask_strength_usd-bitstamp-orderbook-10m-4.10usd_slippage': 4043,
    'ask_strength_usd-bitstamp-orderbook-10m-4.20usd_slippage': 4044,
    'ask_strength_usd-bitstamp-orderbook-10m-4.30usd_slippage': 4045,
    'ask_strength_usd-bitstamp-orderbook-10m-4.40usd_slippage': 4046,
    'ask_strength_usd-bitstamp-orderbook-10m-4.50usd_slippage': 4047,
    'ask_strength_usd-bitstamp-orderbook-10m-4.60usd_slippage': 4048,
    'ask_strength_usd-bitstamp-orderbook-10m-4.70usd_slippage': 4049,
    'ask_strength_usd-bitstamp-orderbook-10m-4.80usd_slippage': 4050,
    'ask_strength_usd-bitstamp-orderbook-10m-4.90usd_slippage': 4051,
    'ask_strength_usd-bitstamp-orderbook-10m-5.00usd_slippage': 4052,
    'ask_strength_usd-bitstamp-orderbook-10m-10.00usd_slippage': 4053,
    'ask_strength_usd-bitstamp-orderbook-10m-20.00usd_slippage': 4054,
    'ask_strength_usd-bitstamp-orderbook-5m-0.00usd_slippage': 4055,
    'ask_strength_usd-bitstamp-orderbook-5m-0.10usd_slippage': 4056,
    'ask_strength_usd-bitstamp-orderbook-5m-0.20usd_slippage': 4057,
    'ask_strength_usd-bitstamp-orderbook-5m-0.30usd_slippage': 4058,
    'ask_strength_usd-bitstamp-orderbook-5m-0.40usd_slippage': 4059,
    'ask_strength_usd-bitstamp-orderbook-5m-0.50usd_slippage': 4060,
    'ask_strength_usd-bitstamp-orderbook-5m-0.60usd_slippage': 4061,
    'ask_strength_usd-bitstamp-orderbook-5m-0.70usd_slippage': 4062,
    'ask_strength_usd-bitstamp-orderbook-5m-0.80usd_slippage': 4063,
    'ask_strength_usd-bitstamp-orderbook-5m-0.90usd_slippage': 4064,
    'ask_strength_usd-bitstamp-orderbook-5m-1.00usd_slippage': 4065,
    'ask_strength_usd-bitstamp-orderbook-5m-1.10usd_slippage': 4066,
    'ask_strength_usd-bitstamp-orderbook-5m-1.20usd_slippage': 4067,
    'ask_strength_usd-bitstamp-orderbook-5m-1.30usd_slippage': 4068,
    'ask_strength_usd-bitstamp-orderbook-5m-1.40usd_slippage': 4069,
    'ask_strength_usd-bitstamp-orderbook-5m-1.50usd_slippage': 4070,
    'ask_strength_usd-bitstamp-orderbook-5m-1.60usd_slippage': 4071,
    'ask_strength_usd-bitstamp-orderbook-5m-1.70usd_slippage': 4072,
    'ask_strength_usd-bitstamp-orderbook-5m-1.80usd_slippage': 4073,
    'ask_strength_usd-bitstamp-orderbook-5m-1.90usd_slippage': 4074,
    'ask_strength_usd-bitstamp-orderbook-5m-2.00usd_slippage': 4075,
    'ask_strength_usd-bitstamp-orderbook-5m-2.10usd_slippage': 4076,
    'ask_strength_usd-bitstamp-orderbook-5m-2.20usd_slippage': 4077,
    'ask_strength_usd-bitstamp-orderbook-5m-2.30usd_slippage': 4078,
    'ask_strength_usd-bitstamp-orderbook-5m-2.40usd_slippage': 4079,
    'ask_strength_usd-bitstamp-orderbook-5m-2.50usd_slippage': 4080,
    'ask_strength_usd-bitstamp-orderbook-5m-2.60usd_slippage': 4081,
    'ask_strength_usd-bitstamp-orderbook-5m-2.70usd_slippage': 4082,
    'ask_strength_usd-bitstamp-orderbook-5m-2.80usd_slippage': 4083,
    'ask_strength_usd-bitstamp-orderbook-5m-2.90usd_slippage': 4084,
    'ask_strength_usd-bitstamp-orderbook-5m-3.00usd_slippage': 4085,
    'ask_strength_usd-bitstamp-orderbook-5m-3.10usd_slippage': 4086,
    'ask_strength_usd-bitstamp-orderbook-5m-3.20usd_slippage': 4087,
    'ask_strength_usd-bitstamp-orderbook-5m-3.30usd_slippage': 4088,
    'ask_strength_usd-bitstamp-orderbook-5m-3.40usd_slippage': 4089,
    'ask_strength_usd-bitstamp-orderbook-5m-3.50usd_slippage': 4090,
    'ask_strength_usd-bitstamp-orderbook-5m-3.60usd_slippage': 4091,
    'ask_strength_usd-bitstamp-orderbook-5m-3.70usd_slippage': 4092,
    'ask_strength_usd-bitstamp-orderbook-5m-3.80usd_slippage': 4093,
    'ask_strength_usd-bitstamp-orderbook-5m-3.90usd_slippage': 4094,
    'ask_strength_usd-bitstamp-orderbook-5m-4.00usd_slippage': 4095,
    'ask_strength_usd-bitstamp-orderbook-5m-4.10usd_slippage': 4096,
    'ask_strength_usd-bitstamp-orderbook-5m-4.20usd_slippage': 4097,
    'ask_strength_usd-bitstamp-orderbook-5m-4.30usd_slippage': 4098,
    'ask_strength_usd-bitstamp-orderbook-5m-4.40usd_slippage': 4099,
    'ask_strength_usd-bitstamp-orderbook-5m-4.50usd_slippage': 4100,
    'ask_strength_usd-bitstamp-orderbook-5m-4.60usd_slippage': 4101,
    'ask_strength_usd-bitstamp-orderbook-5m-4.70usd_slippage': 4102,
    'ask_strength_usd-bitstamp-orderbook-5m-4.80usd_slippage': 4103,
    'ask_strength_usd-bitstamp-orderbook-5m-4.90usd_slippage': 4104,
    'ask_strength_usd-bitstamp-orderbook-5m-5.00usd_slippage': 4105,
    'ask_strength_usd-bitstamp-orderbook-5m-10.00usd_slippage': 4106,
    'ask_strength_usd-bitstamp-orderbook-5m-20.00usd_slippage': 4107,
    'ask_strength_usd-bitstamp-orderbook-1m-0.00usd_slippage': 4108,
    'ask_strength_usd-bitstamp-orderbook-1m-0.10usd_slippage': 4109,
    'ask_strength_usd-bitstamp-orderbook-1m-0.20usd_slippage': 4110,
    'ask_strength_usd-bitstamp-orderbook-1m-0.30usd_slippage': 4111,
    'ask_strength_usd-bitstamp-orderbook-1m-0.40usd_slippage': 4112,
    'ask_strength_usd-bitstamp-orderbook-1m-0.50usd_slippage': 4113,
    'ask_strength_usd-bitstamp-orderbook-1m-0.60usd_slippage': 4114,
    'ask_strength_usd-bitstamp-orderbook-1m-0.70usd_slippage': 4115,
    'ask_strength_usd-bitstamp-orderbook-1m-0.80usd_slippage': 4116,
    'ask_strength_usd-bitstamp-orderbook-1m-0.90usd_slippage': 4117,
    'ask_strength_usd-bitstamp-orderbook-1m-1.00usd_slippage': 4118,
    'ask_strength_usd-bitstamp-orderbook-1m-1.10usd_slippage': 4119,
    'ask_strength_usd-bitstamp-orderbook-1m-1.20usd_slippage': 4120,
    'ask_strength_usd-bitstamp-orderbook-1m-1.30usd_slippage': 4121,
    'ask_strength_usd-bitstamp-orderbook-1m-1.40usd_slippage': 4122,
    'ask_strength_usd-bitstamp-orderbook-1m-1.50usd_slippage': 4123,
    'ask_strength_usd-bitstamp-orderbook-1m-1.60usd_slippage': 4124,
    'ask_strength_usd-bitstamp-orderbook-1m-1.70usd_slippage': 4125,
    'ask_strength_usd-bitstamp-orderbook-1m-1.80usd_slippage': 4126,
    'ask_strength_usd-bitstamp-orderbook-1m-1.90usd_slippage': 4127,
    'ask_strength_usd-bitstamp-orderbook-1m-2.00usd_slippage': 4128,
    'ask_strength_usd-bitstamp-orderbook-1m-2.10usd_slippage': 4129,
    'ask_strength_usd-bitstamp-orderbook-1m-2.20usd_slippage': 4130,
    'ask_strength_usd-bitstamp-orderbook-1m-2.30usd_slippage': 4131,
    'ask_strength_usd-bitstamp-orderbook-1m-2.40usd_slippage': 4132,
    'ask_strength_usd-bitstamp-orderbook-1m-2.50usd_slippage': 4133,
    'ask_strength_usd-bitstamp-orderbook-1m-2.60usd_slippage': 4134,
    'ask_strength_usd-bitstamp-orderbook-1m-2.70usd_slippage': 4135,
    'ask_strength_usd-bitstamp-orderbook-1m-2.80usd_slippage': 4136,
    'ask_strength_usd-bitstamp-orderbook-1m-2.90usd_slippage': 4137,
    'ask_strength_usd-bitstamp-orderbook-1m-3.00usd_slippage': 4138,
    'ask_strength_usd-bitstamp-orderbook-1m-3.10usd_slippage': 4139,
    'ask_strength_usd-bitstamp-orderbook-1m-3.20usd_slippage': 4140,
    'ask_strength_usd-bitstamp-orderbook-1m-3.30usd_slippage': 4141,
    'ask_strength_usd-bitstamp-orderbook-1m-3.40usd_slippage': 4142,
    'ask_strength_usd-bitstamp-orderbook-1m-3.50usd_slippage': 4143,
    'ask_strength_usd-bitstamp-orderbook-1m-3.60usd_slippage': 4144,
    'ask_strength_usd-bitstamp-orderbook-1m-3.70usd_slippage': 4145,
    'ask_strength_usd-bitstamp-orderbook-1m-3.80usd_slippage': 4146,
    'ask_strength_usd-bitstamp-orderbook-1m-3.90usd_slippage': 4147,
    'ask_strength_usd-bitstamp-orderbook-1m-4.00usd_slippage': 4148,
    'ask_strength_usd-bitstamp-orderbook-1m-4.10usd_slippage': 4149,
    'ask_strength_usd-bitstamp-orderbook-1m-4.20usd_slippage': 4150,
    'ask_strength_usd-bitstamp-orderbook-1m-4.30usd_slippage': 4151,
    'ask_strength_usd-bitstamp-orderbook-1m-4.40usd_slippage': 4152,
    'ask_strength_usd-bitstamp-orderbook-1m-4.50usd_slippage': 4153,
    'ask_strength_usd-bitstamp-orderbook-1m-4.60usd_slippage': 4154,
    'ask_strength_usd-bitstamp-orderbook-1m-4.70usd_slippage': 4155,
    'ask_strength_usd-bitstamp-orderbook-1m-4.80usd_slippage': 4156,
    'ask_strength_usd-bitstamp-orderbook-1m-4.90usd_slippage': 4157,
    'ask_strength_usd-bitstamp-orderbook-1m-5.00usd_slippage': 4158,
    'ask_strength_usd-bitstamp-orderbook-1m-10.00usd_slippage': 4159,
    'ask_strength_usd-bitstamp-orderbook-1m-20.00usd_slippage': 4160,
    'ask_strength_usd-bitfinex-orderbook-10m-0.00usd_slippage': 4161,
    'ask_strength_usd-bitfinex-orderbook-10m-0.10usd_slippage': 4162,
    'ask_strength_usd-bitfinex-orderbook-10m-0.20usd_slippage': 4163,
    'ask_strength_usd-bitfinex-orderbook-10m-0.30usd_slippage': 4164,
    'ask_strength_usd-bitfinex-orderbook-10m-0.40usd_slippage': 4165,
    'ask_strength_usd-bitfinex-orderbook-10m-0.50usd_slippage': 4166,
    'ask_strength_usd-bitfinex-orderbook-10m-0.60usd_slippage': 4167,
    'ask_strength_usd-bitfinex-orderbook-10m-0.70usd_slippage': 4168,
    'ask_strength_usd-bitfinex-orderbook-10m-0.80usd_slippage': 4169,
    'ask_strength_usd-bitfinex-orderbook-10m-0.90usd_slippage': 4170,
    'ask_strength_usd-bitfinex-orderbook-10m-1.00usd_slippage': 4171,
    'ask_strength_usd-bitfinex-orderbook-10m-1.10usd_slippage': 4172,
    'ask_strength_usd-bitfinex-orderbook-10m-1.20usd_slippage': 4173,
    'ask_strength_usd-bitfinex-orderbook-10m-1.30usd_slippage': 4174,
    'ask_strength_usd-bitfinex-orderbook-10m-1.40usd_slippage': 4175,
    'ask_strength_usd-bitfinex-orderbook-10m-1.50usd_slippage': 4176,
    'ask_strength_usd-bitfinex-orderbook-10m-1.60usd_slippage': 4177,
    'ask_strength_usd-bitfinex-orderbook-10m-1.70usd_slippage': 4178,
    'ask_strength_usd-bitfinex-orderbook-10m-1.80usd_slippage': 4179,
    'ask_strength_usd-bitfinex-orderbook-10m-1.90usd_slippage': 4180,
    'ask_strength_usd-bitfinex-orderbook-10m-2.00usd_slippage': 4181,
    'ask_strength_usd-bitfinex-orderbook-10m-2.10usd_slippage': 4182,
    'ask_strength_usd-bitfinex-orderbook-10m-2.20usd_slippage': 4183,
    'ask_strength_usd-bitfinex-orderbook-10m-2.30usd_slippage': 4184,
    'ask_strength_usd-bitfinex-orderbook-10m-2.40usd_slippage': 4185,
    'ask_strength_usd-bitfinex-orderbook-10m-2.50usd_slippage': 4186,
    'ask_strength_usd-bitfinex-orderbook-10m-2.60usd_slippage': 4187,
    'ask_strength_usd-bitfinex-orderbook-10m-2.70usd_slippage': 4188,
    'ask_strength_usd-bitfinex-orderbook-10m-2.80usd_slippage': 4189,
    'ask_strength_usd-bitfinex-orderbook-10m-2.90usd_slippage': 4190,
    'ask_strength_usd-bitfinex-orderbook-10m-3.00usd_slippage': 4191,
    'ask_strength_usd-bitfinex-orderbook-10m-3.10usd_slippage': 4192,
    'ask_strength_usd-bitfinex-orderbook-10m-3.20usd_slippage': 4193,
    'ask_strength_usd-bitfinex-orderbook-10m-3.30usd_slippage': 4194,
    'ask_strength_usd-bitfinex-orderbook-10m-3.40usd_slippage': 4195,
    'ask_strength_usd-bitfinex-orderbook-10m-3.50usd_slippage': 4196,
    'ask_strength_usd-bitfinex-orderbook-10m-3.60usd_slippage': 4197,
    'ask_strength_usd-bitfinex-orderbook-10m-3.70usd_slippage': 4198,
    'ask_strength_usd-bitfinex-orderbook-10m-3.80usd_slippage': 4199,
    'ask_strength_usd-bitfinex-orderbook-10m-3.90usd_slippage': 4200,
    'ask_strength_usd-bitfinex-orderbook-10m-4.00usd_slippage': 4201,
    'ask_strength_usd-bitfinex-orderbook-10m-4.10usd_slippage': 4202,
    'ask_strength_usd-bitfinex-orderbook-10m-4.20usd_slippage': 4203,
    'ask_strength_usd-bitfinex-orderbook-10m-4.30usd_slippage': 4204,
    'ask_strength_usd-bitfinex-orderbook-10m-4.40usd_slippage': 4205,
    'ask_strength_usd-bitfinex-orderbook-10m-4.50usd_slippage': 4206,
    'ask_strength_usd-bitfinex-orderbook-10m-4.60usd_slippage': 4207,
    'ask_strength_usd-bitfinex-orderbook-10m-4.70usd_slippage': 4208,
    'ask_strength_usd-bitfinex-orderbook-10m-4.80usd_slippage': 4209,
    'ask_strength_usd-bitfinex-orderbook-10m-4.90usd_slippage': 4210,
    'ask_strength_usd-bitfinex-orderbook-10m-5.00usd_slippage': 4211,
    'ask_strength_usd-bitfinex-orderbook-10m-10.00usd_slippage': 4212,
    'ask_strength_usd-bitfinex-orderbook-10m-20.00usd_slippage': 4213,
    'ask_strength_usd-bitfinex-orderbook-5m-0.00usd_slippage': 4214,
    'ask_strength_usd-bitfinex-orderbook-5m-0.10usd_slippage': 4215,
    'ask_strength_usd-bitfinex-orderbook-5m-0.20usd_slippage': 4216,
    'ask_strength_usd-bitfinex-orderbook-5m-0.30usd_slippage': 4217,
    'ask_strength_usd-bitfinex-orderbook-5m-0.40usd_slippage': 4218,
    'ask_strength_usd-bitfinex-orderbook-5m-0.50usd_slippage': 4219,
    'ask_strength_usd-bitfinex-orderbook-5m-0.60usd_slippage': 4220,
    'ask_strength_usd-bitfinex-orderbook-5m-0.70usd_slippage': 4221,
    'ask_strength_usd-bitfinex-orderbook-5m-0.80usd_slippage': 4222,
    'ask_strength_usd-bitfinex-orderbook-5m-0.90usd_slippage': 4223,
    'ask_strength_usd-bitfinex-orderbook-5m-1.00usd_slippage': 4224,
    'ask_strength_usd-bitfinex-orderbook-5m-1.10usd_slippage': 4225,
    'ask_strength_usd-bitfinex-orderbook-5m-1.20usd_slippage': 4226,
    'ask_strength_usd-bitfinex-orderbook-5m-1.30usd_slippage': 4227,
    'ask_strength_usd-bitfinex-orderbook-5m-1.40usd_slippage': 4228,
    'ask_strength_usd-bitfinex-orderbook-5m-1.50usd_slippage': 4229,
    'ask_strength_usd-bitfinex-orderbook-5m-1.60usd_slippage': 4230,
    'ask_strength_usd-bitfinex-orderbook-5m-1.70usd_slippage': 4231,
    'ask_strength_usd-bitfinex-orderbook-5m-1.80usd_slippage': 4232,
    'ask_strength_usd-bitfinex-orderbook-5m-1.90usd_slippage': 4233,
    'ask_strength_usd-bitfinex-orderbook-5m-2.00usd_slippage': 4234,
    'ask_strength_usd-bitfinex-orderbook-5m-2.10usd_slippage': 4235,
    'ask_strength_usd-bitfinex-orderbook-5m-2.20usd_slippage': 4236,
    'ask_strength_usd-bitfinex-orderbook-5m-2.30usd_slippage': 4237,
    'ask_strength_usd-bitfinex-orderbook-5m-2.40usd_slippage': 4238,
    'ask_strength_usd-bitfinex-orderbook-5m-2.50usd_slippage': 4239,
    'ask_strength_usd-bitfinex-orderbook-5m-2.60usd_slippage': 4240,
    'ask_strength_usd-bitfinex-orderbook-5m-2.70usd_slippage': 4241,
    'ask_strength_usd-bitfinex-orderbook-5m-2.80usd_slippage': 4242,
    'ask_strength_usd-bitfinex-orderbook-5m-2.90usd_slippage': 4243,
    'ask_strength_usd-bitfinex-orderbook-5m-3.00usd_slippage': 4244,
    'ask_strength_usd-bitfinex-orderbook-5m-3.10usd_slippage': 4245,
    'ask_strength_usd-bitfinex-orderbook-5m-3.20usd_slippage': 4246,
    'ask_strength_usd-bitfinex-orderbook-5m-3.30usd_slippage': 4247,
    'ask_strength_usd-bitfinex-orderbook-5m-3.40usd_slippage': 4248,
    'ask_strength_usd-bitfinex-orderbook-5m-3.50usd_slippage': 4249,
    'ask_strength_usd-bitfinex-orderbook-5m-3.60usd_slippage': 4250,
    'ask_strength_usd-bitfinex-orderbook-5m-3.70usd_slippage': 4251,
    'ask_strength_usd-bitfinex-orderbook-5m-3.80usd_slippage': 4252,
    'ask_strength_usd-bitfinex-orderbook-5m-3.90usd_slippage': 4253,
    'ask_strength_usd-bitfinex-orderbook-5m-4.00usd_slippage': 4254,
    'ask_strength_usd-bitfinex-orderbook-5m-4.10usd_slippage': 4255,
    'ask_strength_usd-bitfinex-orderbook-5m-4.20usd_slippage': 4256,
    'ask_strength_usd-bitfinex-orderbook-5m-4.30usd_slippage': 4257,
    'ask_strength_usd-bitfinex-orderbook-5m-4.40usd_slippage': 4258,
    'ask_strength_usd-bitfinex-orderbook-5m-4.50usd_slippage': 4259,
    'ask_strength_usd-bitfinex-orderbook-5m-4.60usd_slippage': 4260,
    'ask_strength_usd-bitfinex-orderbook-5m-4.70usd_slippage': 4261,
    'ask_strength_usd-bitfinex-orderbook-5m-4.80usd_slippage': 4262,
    'ask_strength_usd-bitfinex-orderbook-5m-4.90usd_slippage': 4263,
    'ask_strength_usd-bitfinex-orderbook-5m-5.00usd_slippage': 4264,
    'ask_strength_usd-bitfinex-orderbook-5m-10.00usd_slippage': 4265,
    'ask_strength_usd-bitfinex-orderbook-5m-20.00usd_slippage': 4266,
    'ask_strength_usd-bitfinex-orderbook-1m-0.00usd_slippage': 4267,
    'ask_strength_usd-bitfinex-orderbook-1m-0.10usd_slippage': 4268,
    'ask_strength_usd-bitfinex-orderbook-1m-0.20usd_slippage': 4269,
    'ask_strength_usd-bitfinex-orderbook-1m-0.30usd_slippage': 4270,
    'ask_strength_usd-bitfinex-orderbook-1m-0.40usd_slippage': 4271,
    'ask_strength_usd-bitfinex-orderbook-1m-0.50usd_slippage': 4272,
    'ask_strength_usd-bitfinex-orderbook-1m-0.60usd_slippage': 4273,
    'ask_strength_usd-bitfinex-orderbook-1m-0.70usd_slippage': 4274,
    'ask_strength_usd-bitfinex-orderbook-1m-0.80usd_slippage': 4275,
    'ask_strength_usd-bitfinex-orderbook-1m-0.90usd_slippage': 4276,
    'ask_strength_usd-bitfinex-orderbook-1m-1.00usd_slippage': 4277,
    'ask_strength_usd-bitfinex-orderbook-1m-1.10usd_slippage': 4278,
    'ask_strength_usd-bitfinex-orderbook-1m-1.20usd_slippage': 4279,
    'ask_strength_usd-bitfinex-orderbook-1m-1.30usd_slippage': 4280,
    'ask_strength_usd-bitfinex-orderbook-1m-1.40usd_slippage': 4281,
    'ask_strength_usd-bitfinex-orderbook-1m-1.50usd_slippage': 4282,
    'ask_strength_usd-bitfinex-orderbook-1m-1.60usd_slippage': 4283,
    'ask_strength_usd-bitfinex-orderbook-1m-1.70usd_slippage': 4284,
    'ask_strength_usd-bitfinex-orderbook-1m-1.80usd_slippage': 4285,
    'ask_strength_usd-bitfinex-orderbook-1m-1.90usd_slippage': 4286,
    'ask_strength_usd-bitfinex-orderbook-1m-2.00usd_slippage': 4287,
    'ask_strength_usd-bitfinex-orderbook-1m-2.10usd_slippage': 4288,
    'ask_strength_usd-bitfinex-orderbook-1m-2.20usd_slippage': 4289,
    'ask_strength_usd-bitfinex-orderbook-1m-2.30usd_slippage': 4290,
    'ask_strength_usd-bitfinex-orderbook-1m-2.40usd_slippage': 4291,
    'ask_strength_usd-bitfinex-orderbook-1m-2.50usd_slippage': 4292,
    'ask_strength_usd-bitfinex-orderbook-1m-2.60usd_slippage': 4293,
    'ask_strength_usd-bitfinex-orderbook-1m-2.70usd_slippage': 4294,
    'ask_strength_usd-bitfinex-orderbook-1m-2.80usd_slippage': 4295,
    'ask_strength_usd-bitfinex-orderbook-1m-2.90usd_slippage': 4296,
    'ask_strength_usd-bitfinex-orderbook-1m-3.00usd_slippage': 4297,
    'ask_strength_usd-bitfinex-orderbook-1m-3.10usd_slippage': 4298,
    'ask_strength_usd-bitfinex-orderbook-1m-3.20usd_slippage': 4299,
    'ask_strength_usd-bitfinex-orderbook-1m-3.30usd_slippage': 4300,
    'ask_strength_usd-bitfinex-orderbook-1m-3.40usd_slippage': 4301,
    'ask_strength_usd-bitfinex-orderbook-1m-3.50usd_slippage': 4302,
    'ask_strength_usd-bitfinex-orderbook-1m-3.60usd_slippage': 4303,
    'ask_strength_usd-bitfinex-orderbook-1m-3.70usd_slippage': 4304,
    'ask_strength_usd-bitfinex-orderbook-1m-3.80usd_slippage': 4305,
    'ask_strength_usd-bitfinex-orderbook-1m-3.90usd_slippage': 4306,
    'ask_strength_usd-bitfinex-orderbook-1m-4.00usd_slippage': 4307,
    'ask_strength_usd-bitfinex-orderbook-1m-4.10usd_slippage': 4308,
    'ask_strength_usd-bitfinex-orderbook-1m-4.20usd_slippage': 4309,
    'ask_strength_usd-bitfinex-orderbook-1m-4.30usd_slippage': 4310,
    'ask_strength_usd-bitfinex-orderbook-1m-4.40usd_slippage': 4311,
    'ask_strength_usd-bitfinex-orderbook-1m-4.50usd_slippage': 4312,
    'ask_strength_usd-bitfinex-orderbook-1m-4.60usd_slippage': 4313,
    'ask_strength_usd-bitfinex-orderbook-1m-4.70usd_slippage': 4314,
    'ask_strength_usd-bitfinex-orderbook-1m-4.80usd_slippage': 4315,
    'ask_strength_usd-bitfinex-orderbook-1m-4.90usd_slippage': 4316,
    'ask_strength_usd-bitfinex-orderbook-1m-5.00usd_slippage': 4317,
    'ask_strength_usd-bitfinex-orderbook-1m-10.00usd_slippage': 4318,
    'ask_strength_usd-bitfinex-orderbook-1m-20.00usd_slippage': 4319,
    'ask_strength_usd-coinbase-orderbook-10m-0.00usd_slippage': 4320,
    'ask_strength_usd-coinbase-orderbook-10m-0.10usd_slippage': 4321,
    'ask_strength_usd-coinbase-orderbook-10m-0.20usd_slippage': 4322,
    'ask_strength_usd-coinbase-orderbook-10m-0.30usd_slippage': 4323,
    'ask_strength_usd-coinbase-orderbook-10m-0.40usd_slippage': 4324,
    'ask_strength_usd-coinbase-orderbook-10m-0.50usd_slippage': 4325,
    'ask_strength_usd-coinbase-orderbook-10m-0.60usd_slippage': 4326,
    'ask_strength_usd-coinbase-orderbook-10m-0.70usd_slippage': 4327,
    'ask_strength_usd-coinbase-orderbook-10m-0.80usd_slippage': 4328,
    'ask_strength_usd-coinbase-orderbook-10m-0.90usd_slippage': 4329,
    'ask_strength_usd-coinbase-orderbook-10m-1.00usd_slippage': 4330,
    'ask_strength_usd-coinbase-orderbook-10m-1.10usd_slippage': 4331,
    'ask_strength_usd-coinbase-orderbook-10m-1.20usd_slippage': 4332,
    'ask_strength_usd-coinbase-orderbook-10m-1.30usd_slippage': 4333,
    'ask_strength_usd-coinbase-orderbook-10m-1.40usd_slippage': 4334,
    'ask_strength_usd-coinbase-orderbook-10m-1.50usd_slippage': 4335,
    'ask_strength_usd-coinbase-orderbook-10m-1.60usd_slippage': 4336,
    'ask_strength_usd-coinbase-orderbook-10m-1.70usd_slippage': 4337,
    'ask_strength_usd-coinbase-orderbook-10m-1.80usd_slippage': 4338,
    'ask_strength_usd-coinbase-orderbook-10m-1.90usd_slippage': 4339,
    'ask_strength_usd-coinbase-orderbook-10m-2.00usd_slippage': 4340,
    'ask_strength_usd-coinbase-orderbook-10m-2.10usd_slippage': 4341,
    'ask_strength_usd-coinbase-orderbook-10m-2.20usd_slippage': 4342,
    'ask_strength_usd-coinbase-orderbook-10m-2.30usd_slippage': 4343,
    'ask_strength_usd-coinbase-orderbook-10m-2.40usd_slippage': 4344,
    'ask_strength_usd-coinbase-orderbook-10m-2.50usd_slippage': 4345,
    'ask_strength_usd-coinbase-orderbook-10m-2.60usd_slippage': 4346,
    'ask_strength_usd-coinbase-orderbook-10m-2.70usd_slippage': 4347,
    'ask_strength_usd-coinbase-orderbook-10m-2.80usd_slippage': 4348,
    'ask_strength_usd-coinbase-orderbook-10m-2.90usd_slippage': 4349,
    'ask_strength_usd-coinbase-orderbook-10m-3.00usd_slippage': 4350,
    'ask_strength_usd-coinbase-orderbook-10m-3.10usd_slippage': 4351,
    'ask_strength_usd-coinbase-orderbook-10m-3.20usd_slippage': 4352,
    'ask_strength_usd-coinbase-orderbook-10m-3.30usd_slippage': 4353,
    'ask_strength_usd-coinbase-orderbook-10m-3.40usd_slippage': 4354,
    'ask_strength_usd-coinbase-orderbook-10m-3.50usd_slippage': 4355,
    'ask_strength_usd-coinbase-orderbook-10m-3.60usd_slippage': 4356,
    'ask_strength_usd-coinbase-orderbook-10m-3.70usd_slippage': 4357,
    'ask_strength_usd-coinbase-orderbook-10m-3.80usd_slippage': 4358,
    'ask_strength_usd-coinbase-orderbook-10m-3.90usd_slippage': 4359,
    'ask_strength_usd-coinbase-orderbook-10m-4.00usd_slippage': 4360,
    'ask_strength_usd-coinbase-orderbook-10m-4.10usd_slippage': 4361,
    'ask_strength_usd-coinbase-orderbook-10m-4.20usd_slippage': 4362,
    'ask_strength_usd-coinbase-orderbook-10m-4.30usd_slippage': 4363,
    'ask_strength_usd-coinbase-orderbook-10m-4.40usd_slippage': 4364,
    'ask_strength_usd-coinbase-orderbook-10m-4.50usd_slippage': 4365,
    'ask_strength_usd-coinbase-orderbook-10m-4.60usd_slippage': 4366,
    'ask_strength_usd-coinbase-orderbook-10m-4.70usd_slippage': 4367,
    'ask_strength_usd-coinbase-orderbook-10m-4.80usd_slippage': 4368,
    'ask_strength_usd-coinbase-orderbook-10m-4.90usd_slippage': 4369,
    'ask_strength_usd-coinbase-orderbook-10m-5.00usd_slippage': 4370,
    'ask_strength_usd-coinbase-orderbook-10m-10.00usd_slippage': 4371,
    'ask_strength_usd-coinbase-orderbook-10m-20.00usd_slippage': 4372,
    'ask_strength_usd-coinbase-orderbook-5m-0.00usd_slippage': 4373,
    'ask_strength_usd-coinbase-orderbook-5m-0.10usd_slippage': 4374,
    'ask_strength_usd-coinbase-orderbook-5m-0.20usd_slippage': 4375,
    'ask_strength_usd-coinbase-orderbook-5m-0.30usd_slippage': 4376,
    'ask_strength_usd-coinbase-orderbook-5m-0.40usd_slippage': 4377,
    'ask_strength_usd-coinbase-orderbook-5m-0.50usd_slippage': 4378,
    'ask_strength_usd-coinbase-orderbook-5m-0.60usd_slippage': 4379,
    'ask_strength_usd-coinbase-orderbook-5m-0.70usd_slippage': 4380,
    'ask_strength_usd-coinbase-orderbook-5m-0.80usd_slippage': 4381,
    'ask_strength_usd-coinbase-orderbook-5m-0.90usd_slippage': 4382,
    'ask_strength_usd-coinbase-orderbook-5m-1.00usd_slippage': 4383,
    'ask_strength_usd-coinbase-orderbook-5m-1.10usd_slippage': 4384,
    'ask_strength_usd-coinbase-orderbook-5m-1.20usd_slippage': 4385,
    'ask_strength_usd-coinbase-orderbook-5m-1.30usd_slippage': 4386,
    'ask_strength_usd-coinbase-orderbook-5m-1.40usd_slippage': 4387,
    'ask_strength_usd-coinbase-orderbook-5m-1.50usd_slippage': 4388,
    'ask_strength_usd-coinbase-orderbook-5m-1.60usd_slippage': 4389,
    'ask_strength_usd-coinbase-orderbook-5m-1.70usd_slippage': 4390,
    'ask_strength_usd-coinbase-orderbook-5m-1.80usd_slippage': 4391,
    'ask_strength_usd-coinbase-orderbook-5m-1.90usd_slippage': 4392,
    'ask_strength_usd-coinbase-orderbook-5m-2.00usd_slippage': 4393,
    'ask_strength_usd-coinbase-orderbook-5m-2.10usd_slippage': 4394,
    'ask_strength_usd-coinbase-orderbook-5m-2.20usd_slippage': 4395,
    'ask_strength_usd-coinbase-orderbook-5m-2.30usd_slippage': 4396,
    'ask_strength_usd-coinbase-orderbook-5m-2.40usd_slippage': 4397,
    'ask_strength_usd-coinbase-orderbook-5m-2.50usd_slippage': 4398,
    'ask_strength_usd-coinbase-orderbook-5m-2.60usd_slippage': 4399,
    'ask_strength_usd-coinbase-orderbook-5m-2.70usd_slippage': 4400,
    'ask_strength_usd-coinbase-orderbook-5m-2.80usd_slippage': 4401,
    'ask_strength_usd-coinbase-orderbook-5m-2.90usd_slippage': 4402,
    'ask_strength_usd-coinbase-orderbook-5m-3.00usd_slippage': 4403,
    'ask_strength_usd-coinbase-orderbook-5m-3.10usd_slippage': 4404,
    'ask_strength_usd-coinbase-orderbook-5m-3.20usd_slippage': 4405,
    'ask_strength_usd-coinbase-orderbook-5m-3.30usd_slippage': 4406,
    'ask_strength_usd-coinbase-orderbook-5m-3.40usd_slippage': 4407,
    'ask_strength_usd-coinbase-orderbook-5m-3.50usd_slippage': 4408,
    'ask_strength_usd-coinbase-orderbook-5m-3.60usd_slippage': 4409,
    'ask_strength_usd-coinbase-orderbook-5m-3.70usd_slippage': 4410,
    'ask_strength_usd-coinbase-orderbook-5m-3.80usd_slippage': 4411,
    'ask_strength_usd-coinbase-orderbook-5m-3.90usd_slippage': 4412,
    'ask_strength_usd-coinbase-orderbook-5m-4.00usd_slippage': 4413,
    'ask_strength_usd-coinbase-orderbook-5m-4.10usd_slippage': 4414,
    'ask_strength_usd-coinbase-orderbook-5m-4.20usd_slippage': 4415,
    'ask_strength_usd-coinbase-orderbook-5m-4.30usd_slippage': 4416,
    'ask_strength_usd-coinbase-orderbook-5m-4.40usd_slippage': 4417,
    'ask_strength_usd-coinbase-orderbook-5m-4.50usd_slippage': 4418,
    'ask_strength_usd-coinbase-orderbook-5m-4.60usd_slippage': 4419,
    'ask_strength_usd-coinbase-orderbook-5m-4.70usd_slippage': 4420,
    'ask_strength_usd-coinbase-orderbook-5m-4.80usd_slippage': 4421,
    'ask_strength_usd-coinbase-orderbook-5m-4.90usd_slippage': 4422,
    'ask_strength_usd-coinbase-orderbook-5m-5.00usd_slippage': 4423,
    'ask_strength_usd-coinbase-orderbook-5m-10.00usd_slippage': 4424,
    'ask_strength_usd-coinbase-orderbook-5m-20.00usd_slippage': 4425,
    'ask_strength_usd-coinbase-orderbook-1m-0.00usd_slippage': 4426,
    'ask_strength_usd-coinbase-orderbook-1m-0.10usd_slippage': 4427,
    'ask_strength_usd-coinbase-orderbook-1m-0.20usd_slippage': 4428,
    'ask_strength_usd-coinbase-orderbook-1m-0.30usd_slippage': 4429,
    'ask_strength_usd-coinbase-orderbook-1m-0.40usd_slippage': 4430,
    'ask_strength_usd-coinbase-orderbook-1m-0.50usd_slippage': 4431,
    'ask_strength_usd-coinbase-orderbook-1m-0.60usd_slippage': 4432,
    'ask_strength_usd-coinbase-orderbook-1m-0.70usd_slippage': 4433,
    'ask_strength_usd-coinbase-orderbook-1m-0.80usd_slippage': 4434,
    'ask_strength_usd-coinbase-orderbook-1m-0.90usd_slippage': 4435,
    'ask_strength_usd-coinbase-orderbook-1m-1.00usd_slippage': 4436,
    'ask_strength_usd-coinbase-orderbook-1m-1.10usd_slippage': 4437,
    'ask_strength_usd-coinbase-orderbook-1m-1.20usd_slippage': 4438,
    'ask_strength_usd-coinbase-orderbook-1m-1.30usd_slippage': 4439,
    'ask_strength_usd-coinbase-orderbook-1m-1.40usd_slippage': 4440,
    'ask_strength_usd-coinbase-orderbook-1m-1.50usd_slippage': 4441,
    'ask_strength_usd-coinbase-orderbook-1m-1.60usd_slippage': 4442,
    'ask_strength_usd-coinbase-orderbook-1m-1.70usd_slippage': 4443,
    'ask_strength_usd-coinbase-orderbook-1m-1.80usd_slippage': 4444,
    'ask_strength_usd-coinbase-orderbook-1m-1.90usd_slippage': 4445,
    'ask_strength_usd-coinbase-orderbook-1m-2.00usd_slippage': 4446,
    'ask_strength_usd-coinbase-orderbook-1m-2.10usd_slippage': 4447,
    'ask_strength_usd-coinbase-orderbook-1m-2.20usd_slippage': 4448,
    'ask_strength_usd-coinbase-orderbook-1m-2.30usd_slippage': 4449,
    'ask_strength_usd-coinbase-orderbook-1m-2.40usd_slippage': 4450,
    'ask_strength_usd-coinbase-orderbook-1m-2.50usd_slippage': 4451,
    'ask_strength_usd-coinbase-orderbook-1m-2.60usd_slippage': 4452,
    'ask_strength_usd-coinbase-orderbook-1m-2.70usd_slippage': 4453,
    'ask_strength_usd-coinbase-orderbook-1m-2.80usd_slippage': 4454,
    'ask_strength_usd-coinbase-orderbook-1m-2.90usd_slippage': 4455,
    'ask_strength_usd-coinbase-orderbook-1m-3.00usd_slippage': 4456,
    'ask_strength_usd-coinbase-orderbook-1m-3.10usd_slippage': 4457,
    'ask_strength_usd-coinbase-orderbook-1m-3.20usd_slippage': 4458,
    'ask_strength_usd-coinbase-orderbook-1m-3.30usd_slippage': 4459,
    'ask_strength_usd-coinbase-orderbook-1m-3.40usd_slippage': 4460,
    'ask_strength_usd-coinbase-orderbook-1m-3.50usd_slippage': 4461,
    'ask_strength_usd-coinbase-orderbook-1m-3.60usd_slippage': 4462,
    'ask_strength_usd-coinbase-orderbook-1m-3.70usd_slippage': 4463,
    'ask_strength_usd-coinbase-orderbook-1m-3.80usd_slippage': 4464,
    'ask_strength_usd-coinbase-orderbook-1m-3.90usd_slippage': 4465,
    'ask_strength_usd-coinbase-orderbook-1m-4.00usd_slippage': 4466,
    'ask_strength_usd-coinbase-orderbook-1m-4.10usd_slippage': 4467,
    'ask_strength_usd-coinbase-orderbook-1m-4.20usd_slippage': 4468,
    'ask_strength_usd-coinbase-orderbook-1m-4.30usd_slippage': 4469,
    'ask_strength_usd-coinbase-orderbook-1m-4.40usd_slippage': 4470,
    'ask_strength_usd-coinbase-orderbook-1m-4.50usd_slippage': 4471,
    'ask_strength_usd-coinbase-orderbook-1m-4.60usd_slippage': 4472,
    'ask_strength_usd-coinbase-orderbook-1m-4.70usd_slippage': 4473,
    'ask_strength_usd-coinbase-orderbook-1m-4.80usd_slippage': 4474,
    'ask_strength_usd-coinbase-orderbook-1m-4.90usd_slippage': 4475,
    'ask_strength_usd-coinbase-orderbook-1m-5.00usd_slippage': 4476,
    'ask_strength_usd-coinbase-orderbook-1m-10.00usd_slippage': 4477,
    'ask_strength_usd-coinbase-orderbook-1m-20.00usd_slippage': 4478,
    'ask_strength_usd-itbit-orderbook-10m-0.00usd_slippage': 4479,
    'ask_strength_usd-itbit-orderbook-10m-0.10usd_slippage': 4480,
    'ask_strength_usd-itbit-orderbook-10m-0.20usd_slippage': 4481,
    'ask_strength_usd-itbit-orderbook-10m-0.30usd_slippage': 4482,
    'ask_strength_usd-itbit-orderbook-10m-0.40usd_slippage': 4483,
    'ask_strength_usd-itbit-orderbook-10m-0.50usd_slippage': 4484,
    'ask_strength_usd-itbit-orderbook-10m-0.60usd_slippage': 4485,
    'ask_strength_usd-itbit-orderbook-10m-0.70usd_slippage': 4486,
    'ask_strength_usd-itbit-orderbook-10m-0.80usd_slippage': 4487,
    'ask_strength_usd-itbit-orderbook-10m-0.90usd_slippage': 4488,
    'ask_strength_usd-itbit-orderbook-10m-1.00usd_slippage': 4489,
    'ask_strength_usd-itbit-orderbook-10m-1.10usd_slippage': 4490,
    'ask_strength_usd-itbit-orderbook-10m-1.20usd_slippage': 4491,
    'ask_strength_usd-itbit-orderbook-10m-1.30usd_slippage': 4492,
    'ask_strength_usd-itbit-orderbook-10m-1.40usd_slippage': 4493,
    'ask_strength_usd-itbit-orderbook-10m-1.50usd_slippage': 4494,
    'ask_strength_usd-itbit-orderbook-10m-1.60usd_slippage': 4495,
    'ask_strength_usd-itbit-orderbook-10m-1.70usd_slippage': 4496,
    'ask_strength_usd-itbit-orderbook-10m-1.80usd_slippage': 4497,
    'ask_strength_usd-itbit-orderbook-10m-1.90usd_slippage': 4498,
    'ask_strength_usd-itbit-orderbook-10m-2.00usd_slippage': 4499,
    'ask_strength_usd-itbit-orderbook-10m-2.10usd_slippage': 4500,
    'ask_strength_usd-itbit-orderbook-10m-2.20usd_slippage': 4501,
    'ask_strength_usd-itbit-orderbook-10m-2.30usd_slippage': 4502,
    'ask_strength_usd-itbit-orderbook-10m-2.40usd_slippage': 4503,
    'ask_strength_usd-itbit-orderbook-10m-2.50usd_slippage': 4504,
    'ask_strength_usd-itbit-orderbook-10m-2.60usd_slippage': 4505,
    'ask_strength_usd-itbit-orderbook-10m-2.70usd_slippage': 4506,
    'ask_strength_usd-itbit-orderbook-10m-2.80usd_slippage': 4507,
    'ask_strength_usd-itbit-orderbook-10m-2.90usd_slippage': 4508,
    'ask_strength_usd-itbit-orderbook-10m-3.00usd_slippage': 4509,
    'ask_strength_usd-itbit-orderbook-10m-3.10usd_slippage': 4510,
    'ask_strength_usd-itbit-orderbook-10m-3.20usd_slippage': 4511,
    'ask_strength_usd-itbit-orderbook-10m-3.30usd_slippage': 4512,
    'ask_strength_usd-itbit-orderbook-10m-3.40usd_slippage': 4513,
    'ask_strength_usd-itbit-orderbook-10m-3.50usd_slippage': 4514,
    'ask_strength_usd-itbit-orderbook-10m-3.60usd_slippage': 4515,
    'ask_strength_usd-itbit-orderbook-10m-3.70usd_slippage': 4516,
    'ask_strength_usd-itbit-orderbook-10m-3.80usd_slippage': 4517,
    'ask_strength_usd-itbit-orderbook-10m-3.90usd_slippage': 4518,
    'ask_strength_usd-itbit-orderbook-10m-4.00usd_slippage': 4519,
    'ask_strength_usd-itbit-orderbook-10m-4.10usd_slippage': 4520,
    'ask_strength_usd-itbit-orderbook-10m-4.20usd_slippage': 4521,
    'ask_strength_usd-itbit-orderbook-10m-4.30usd_slippage': 4522,
    'ask_strength_usd-itbit-orderbook-10m-4.40usd_slippage': 4523,
    'ask_strength_usd-itbit-orderbook-10m-4.50usd_slippage': 4524,
    'ask_strength_usd-itbit-orderbook-10m-4.60usd_slippage': 4525,
    'ask_strength_usd-itbit-orderbook-10m-4.70usd_slippage': 4526,
    'ask_strength_usd-itbit-orderbook-10m-4.80usd_slippage': 4527,
    'ask_strength_usd-itbit-orderbook-10m-4.90usd_slippage': 4528,
    'ask_strength_usd-itbit-orderbook-10m-5.00usd_slippage': 4529,
    'ask_strength_usd-itbit-orderbook-10m-10.00usd_slippage': 4530,
    'ask_strength_usd-itbit-orderbook-10m-20.00usd_slippage': 4531,
    'ask_strength_usd-itbit-orderbook-5m-0.00usd_slippage': 4532,
    'ask_strength_usd-itbit-orderbook-5m-0.10usd_slippage': 4533,
    'ask_strength_usd-itbit-orderbook-5m-0.20usd_slippage': 4534,
    'ask_strength_usd-itbit-orderbook-5m-0.30usd_slippage': 4535,
    'ask_strength_usd-itbit-orderbook-5m-0.40usd_slippage': 4536,
    'ask_strength_usd-itbit-orderbook-5m-0.50usd_slippage': 4537,
    'ask_strength_usd-itbit-orderbook-5m-0.60usd_slippage': 4538,
    'ask_strength_usd-itbit-orderbook-5m-0.70usd_slippage': 4539,
    'ask_strength_usd-itbit-orderbook-5m-0.80usd_slippage': 4540,
    'ask_strength_usd-itbit-orderbook-5m-0.90usd_slippage': 4541,
    'ask_strength_usd-itbit-orderbook-5m-1.00usd_slippage': 4542,
    'ask_strength_usd-itbit-orderbook-5m-1.10usd_slippage': 4543,
    'ask_strength_usd-itbit-orderbook-5m-1.20usd_slippage': 4544,
    'ask_strength_usd-itbit-orderbook-5m-1.30usd_slippage': 4545,
    'ask_strength_usd-itbit-orderbook-5m-1.40usd_slippage': 4546,
    'ask_strength_usd-itbit-orderbook-5m-1.50usd_slippage': 4547,
    'ask_strength_usd-itbit-orderbook-5m-1.60usd_slippage': 4548,
    'ask_strength_usd-itbit-orderbook-5m-1.70usd_slippage': 4549,
    'ask_strength_usd-itbit-orderbook-5m-1.80usd_slippage': 4550,
    'ask_strength_usd-itbit-orderbook-5m-1.90usd_slippage': 4551,
    'ask_strength_usd-itbit-orderbook-5m-2.00usd_slippage': 4552,
    'ask_strength_usd-itbit-orderbook-5m-2.10usd_slippage': 4553,
    'ask_strength_usd-itbit-orderbook-5m-2.20usd_slippage': 4554,
    'ask_strength_usd-itbit-orderbook-5m-2.30usd_slippage': 4555,
    'ask_strength_usd-itbit-orderbook-5m-2.40usd_slippage': 4556,
    'ask_strength_usd-itbit-orderbook-5m-2.50usd_slippage': 4557,
    'ask_strength_usd-itbit-orderbook-5m-2.60usd_slippage': 4558,
    'ask_strength_usd-itbit-orderbook-5m-2.70usd_slippage': 4559,
    'ask_strength_usd-itbit-orderbook-5m-2.80usd_slippage': 4560,
    'ask_strength_usd-itbit-orderbook-5m-2.90usd_slippage': 4561,
    'ask_strength_usd-itbit-orderbook-5m-3.00usd_slippage': 4562,
    'ask_strength_usd-itbit-orderbook-5m-3.10usd_slippage': 4563,
    'ask_strength_usd-itbit-orderbook-5m-3.20usd_slippage': 4564,
    'ask_strength_usd-itbit-orderbook-5m-3.30usd_slippage': 4565,
    'ask_strength_usd-itbit-orderbook-5m-3.40usd_slippage': 4566,
    'ask_strength_usd-itbit-orderbook-5m-3.50usd_slippage': 4567,
    'ask_strength_usd-itbit-orderbook-5m-3.60usd_slippage': 4568,
    'ask_strength_usd-itbit-orderbook-5m-3.70usd_slippage': 4569,
    'ask_strength_usd-itbit-orderbook-5m-3.80usd_slippage': 4570,
    'ask_strength_usd-itbit-orderbook-5m-3.90usd_slippage': 4571,
    'ask_strength_usd-itbit-orderbook-5m-4.00usd_slippage': 4572,
    'ask_strength_usd-itbit-orderbook-5m-4.10usd_slippage': 4573,
    'ask_strength_usd-itbit-orderbook-5m-4.20usd_slippage': 4574,
    'ask_strength_usd-itbit-orderbook-5m-4.30usd_slippage': 4575,
    'ask_strength_usd-itbit-orderbook-5m-4.40usd_slippage': 4576,
    'ask_strength_usd-itbit-orderbook-5m-4.50usd_slippage': 4577,
    'ask_strength_usd-itbit-orderbook-5m-4.60usd_slippage': 4578,
    'ask_strength_usd-itbit-orderbook-5m-4.70usd_slippage': 4579,
    'ask_strength_usd-itbit-orderbook-5m-4.80usd_slippage': 4580,
    'ask_strength_usd-itbit-orderbook-5m-4.90usd_slippage': 4581,
    'ask_strength_usd-itbit-orderbook-5m-5.00usd_slippage': 4582,
    'ask_strength_usd-itbit-orderbook-5m-10.00usd_slippage': 4583,
    'ask_strength_usd-itbit-orderbook-5m-20.00usd_slippage': 4584,
    'ask_strength_usd-itbit-orderbook-1m-0.00usd_slippage': 4585,
    'ask_strength_usd-itbit-orderbook-1m-0.10usd_slippage': 4586,
    'ask_strength_usd-itbit-orderbook-1m-0.20usd_slippage': 4587,
    'ask_strength_usd-itbit-orderbook-1m-0.30usd_slippage': 4588,
    'ask_strength_usd-itbit-orderbook-1m-0.40usd_slippage': 4589,
    'ask_strength_usd-itbit-orderbook-1m-0.50usd_slippage': 4590,
    'ask_strength_usd-itbit-orderbook-1m-0.60usd_slippage': 4591,
    'ask_strength_usd-itbit-orderbook-1m-0.70usd_slippage': 4592,
    'ask_strength_usd-itbit-orderbook-1m-0.80usd_slippage': 4593,
    'ask_strength_usd-itbit-orderbook-1m-0.90usd_slippage': 4594,
    'ask_strength_usd-itbit-orderbook-1m-1.00usd_slippage': 4595,
    'ask_strength_usd-itbit-orderbook-1m-1.10usd_slippage': 4596,
    'ask_strength_usd-itbit-orderbook-1m-1.20usd_slippage': 4597,
    'ask_strength_usd-itbit-orderbook-1m-1.30usd_slippage': 4598,
    'ask_strength_usd-itbit-orderbook-1m-1.40usd_slippage': 4599,
    'ask_strength_usd-itbit-orderbook-1m-1.50usd_slippage': 4600,
    'ask_strength_usd-itbit-orderbook-1m-1.60usd_slippage': 4601,
    'ask_strength_usd-itbit-orderbook-1m-1.70usd_slippage': 4602,
    'ask_strength_usd-itbit-orderbook-1m-1.80usd_slippage': 4603,
    'ask_strength_usd-itbit-orderbook-1m-1.90usd_slippage': 4604,
    'ask_strength_usd-itbit-orderbook-1m-2.00usd_slippage': 4605,
    'ask_strength_usd-itbit-orderbook-1m-2.10usd_slippage': 4606,
    'ask_strength_usd-itbit-orderbook-1m-2.20usd_slippage': 4607,
    'ask_strength_usd-itbit-orderbook-1m-2.30usd_slippage': 4608,
    'ask_strength_usd-itbit-orderbook-1m-2.40usd_slippage': 4609,
    'ask_strength_usd-itbit-orderbook-1m-2.50usd_slippage': 4610,
    'ask_strength_usd-itbit-orderbook-1m-2.60usd_slippage': 4611,
    'ask_strength_usd-itbit-orderbook-1m-2.70usd_slippage': 4612,
    'ask_strength_usd-itbit-orderbook-1m-2.80usd_slippage': 4613,
    'ask_strength_usd-itbit-orderbook-1m-2.90usd_slippage': 4614,
    'ask_strength_usd-itbit-orderbook-1m-3.00usd_slippage': 4615,
    'ask_strength_usd-itbit-orderbook-1m-3.10usd_slippage': 4616,
    'ask_strength_usd-itbit-orderbook-1m-3.20usd_slippage': 4617,
    'ask_strength_usd-itbit-orderbook-1m-3.30usd_slippage': 4618,
    'ask_strength_usd-itbit-orderbook-1m-3.40usd_slippage': 4619,
    'ask_strength_usd-itbit-orderbook-1m-3.50usd_slippage': 4620,
    'ask_strength_usd-itbit-orderbook-1m-3.60usd_slippage': 4621,
    'ask_strength_usd-itbit-orderbook-1m-3.70usd_slippage': 4622,
    'ask_strength_usd-itbit-orderbook-1m-3.80usd_slippage': 4623,
    'ask_strength_usd-itbit-orderbook-1m-3.90usd_slippage': 4624,
    'ask_strength_usd-itbit-orderbook-1m-4.00usd_slippage': 4625,
    'ask_strength_usd-itbit-orderbook-1m-4.10usd_slippage': 4626,
    'ask_strength_usd-itbit-orderbook-1m-4.20usd_slippage': 4627,
    'ask_strength_usd-itbit-orderbook-1m-4.30usd_slippage': 4628,
    'ask_strength_usd-itbit-orderbook-1m-4.40usd_slippage': 4629,
    'ask_strength_usd-itbit-orderbook-1m-4.50usd_slippage': 4630,
    'ask_strength_usd-itbit-orderbook-1m-4.60usd_slippage': 4631,
    'ask_strength_usd-itbit-orderbook-1m-4.70usd_slippage': 4632,
    'ask_strength_usd-itbit-orderbook-1m-4.80usd_slippage': 4633,
    'ask_strength_usd-itbit-orderbook-1m-4.90usd_slippage': 4634,
    'ask_strength_usd-itbit-orderbook-1m-5.00usd_slippage': 4635,
    'ask_strength_usd-itbit-orderbook-1m-10.00usd_slippage': 4636,
    'ask_strength_usd-itbit-orderbook-1m-20.00usd_slippage': 4637,
    'ask_strength_usd-kraken-orderbook-10m-0.00usd_slippage': 4638,
    'ask_strength_usd-kraken-orderbook-10m-0.10usd_slippage': 4639,
    'ask_strength_usd-kraken-orderbook-10m-0.20usd_slippage': 4640,
    'ask_strength_usd-kraken-orderbook-10m-0.30usd_slippage': 4641,
    'ask_strength_usd-kraken-orderbook-10m-0.40usd_slippage': 4642,
    'ask_strength_usd-kraken-orderbook-10m-0.50usd_slippage': 4643,
    'ask_strength_usd-kraken-orderbook-10m-0.60usd_slippage': 4644,
    'ask_strength_usd-kraken-orderbook-10m-0.70usd_slippage': 4645,
    'ask_strength_usd-kraken-orderbook-10m-0.80usd_slippage': 4646,
    'ask_strength_usd-kraken-orderbook-10m-0.90usd_slippage': 4647,
    'ask_strength_usd-kraken-orderbook-10m-1.00usd_slippage': 4648,
    'ask_strength_usd-kraken-orderbook-10m-1.10usd_slippage': 4649,
    'ask_strength_usd-kraken-orderbook-10m-1.20usd_slippage': 4650,
    'ask_strength_usd-kraken-orderbook-10m-1.30usd_slippage': 4651,
    'ask_strength_usd-kraken-orderbook-10m-1.40usd_slippage': 4652,
    'ask_strength_usd-kraken-orderbook-10m-1.50usd_slippage': 4653,
    'ask_strength_usd-kraken-orderbook-10m-1.60usd_slippage': 4654,
    'ask_strength_usd-kraken-orderbook-10m-1.70usd_slippage': 4655,
    'ask_strength_usd-kraken-orderbook-10m-1.80usd_slippage': 4656,
    'ask_strength_usd-kraken-orderbook-10m-1.90usd_slippage': 4657,
    'ask_strength_usd-kraken-orderbook-10m-2.00usd_slippage': 4658,
    'ask_strength_usd-kraken-orderbook-10m-2.10usd_slippage': 4659,
    'ask_strength_usd-kraken-orderbook-10m-2.20usd_slippage': 4660,
    'ask_strength_usd-kraken-orderbook-10m-2.30usd_slippage': 4661,
    'ask_strength_usd-kraken-orderbook-10m-2.40usd_slippage': 4662,
    # 1hr quote-based series', 0, 1, 5, 10, 20 btc depths
    'bid-bitstamp-orderbook-1hr-0.00btc_depth': 4663,
    'bid-bitstamp-orderbook-1hr-1.00btc_depth': 4664,
    'bid-bitstamp-orderbook-1hr-5.00btc_depth': 4665,
    'bid-bitstamp-orderbook-1hr-10.00btc_depth': 4666,
    'bid-bitstamp-orderbook-1hr-20.00btc_depth': 4667,
    'bid-bitfinex-orderbook-1hr-0.00btc_depth': 4668,
    'bid-bitfinex-orderbook-1hr-1.00btc_depth': 4669,
    'bid-bitfinex-orderbook-1hr-5.00btc_depth': 4670,
    'bid-bitfinex-orderbook-1hr-10.00btc_depth': 4671,
    'bid-bitfinex-orderbook-1hr-20.00btc_depth': 4672,
    'bid-coinbase-orderbook-1hr-0.00btc_depth': 4673,
    'bid-coinbase-orderbook-1hr-1.00btc_depth': 4674,
    'bid-coinbase-orderbook-1hr-5.00btc_depth': 4675,
    'bid-coinbase-orderbook-1hr-10.00btc_depth': 4676,
    'bid-coinbase-orderbook-1hr-20.00btc_depth': 4677,
    'bid-itbit-orderbook-1hr-0.00btc_depth': 4678,
    'bid-itbit-orderbook-1hr-1.00btc_depth': 4679,
    'bid-itbit-orderbook-1hr-5.00btc_depth': 4680,
    'bid-itbit-orderbook-1hr-10.00btc_depth': 4681,
    'bid-itbit-orderbook-1hr-20.00btc_depth': 4682,
    'bid-kraken-orderbook-1hr-0.00btc_depth': 4683,
    'bid-kraken-orderbook-1hr-1.00btc_depth': 4684,
    'bid-kraken-orderbook-1hr-5.00btc_depth': 4685,
    'bid-kraken-orderbook-1hr-10.00btc_depth': 4686,
    'bid-kraken-orderbook-1hr-20.00btc_depth': 4687,
    'bid-gemini-orderbook-1hr-0.00btc_depth': 4688,
    'bid-gemini-orderbook-1hr-1.00btc_depth': 4689,
    'bid-gemini-orderbook-1hr-5.00btc_depth': 4690,
    'bid-gemini-orderbook-1hr-10.00btc_depth': 4691,
    'bid-gemini-orderbook-1hr-20.00btc_depth': 4692,
    'bid-okcoin-orderbook-1hr-0.00btc_depth': 4693,
    'bid-okcoin-orderbook-1hr-1.00btc_depth': 4694,
    'bid-okcoin-orderbook-1hr-5.00btc_depth': 4695,
    'bid-okcoin-orderbook-1hr-10.00btc_depth': 4696,
    'bid-okcoin-orderbook-1hr-20.00btc_depth': 4697,
    'ask-bitstamp-orderbook-1hr-0.00btc_depth': 4698,
    'ask-bitstamp-orderbook-1hr-1.00btc_depth': 4699,
    'ask-bitstamp-orderbook-1hr-5.00btc_depth': 4700,
    'ask-bitstamp-orderbook-1hr-10.00btc_depth': 4701,
    'ask-bitstamp-orderbook-1hr-20.00btc_depth': 4702,
    'ask-bitfinex-orderbook-1hr-0.00btc_depth': 4703,
    'ask-bitfinex-orderbook-1hr-1.00btc_depth': 4704,
    'ask-bitfinex-orderbook-1hr-5.00btc_depth': 4705,
    'ask-bitfinex-orderbook-1hr-10.00btc_depth': 4706,
    'ask-bitfinex-orderbook-1hr-20.00btc_depth': 4707,
    'ask-coinbase-orderbook-1hr-0.00btc_depth': 4708,
    'ask-coinbase-orderbook-1hr-1.00btc_depth': 4709,
    'ask-coinbase-orderbook-1hr-5.00btc_depth': 4710,
    'ask-coinbase-orderbook-1hr-10.00btc_depth': 4711,
    'ask-coinbase-orderbook-1hr-20.00btc_depth': 4712,
    'ask-itbit-orderbook-1hr-0.00btc_depth': 4713,
    'ask-itbit-orderbook-1hr-1.00btc_depth': 4714,
    'ask-itbit-orderbook-1hr-5.00btc_depth': 4715,
    'ask-itbit-orderbook-1hr-10.00btc_depth': 4716,
    'ask-itbit-orderbook-1hr-20.00btc_depth': 4717,
    'ask-kraken-orderbook-1hr-0.00btc_depth': 4718,
    'ask-kraken-orderbook-1hr-1.00btc_depth': 4719,
    'ask-kraken-orderbook-1hr-5.00btc_depth': 4720,
    'ask-kraken-orderbook-1hr-10.00btc_depth': 4721,
    'ask-kraken-orderbook-1hr-20.00btc_depth': 4722,
    'ask-gemini-orderbook-1hr-0.00btc_depth': 4723,
    'ask-gemini-orderbook-1hr-1.00btc_depth': 4724,
    'ask-gemini-orderbook-1hr-5.00btc_depth': 4725,
    'ask-gemini-orderbook-1hr-10.00btc_depth': 4726,
    'ask-gemini-orderbook-1hr-20.00btc_depth': 4727,
    'ask-okcoin-orderbook-1hr-0.00btc_depth': 4728,
    'ask-okcoin-orderbook-1hr-1.00btc_depth': 4729,
    'ask-okcoin-orderbook-1hr-5.00btc_depth': 4730,
    'ask-okcoin-orderbook-1hr-10.00btc_depth': 4731,
    'ask-okcoin-orderbook-1hr-20.00btc_depth': 4732,
    'spread-bitstamp-orderbook-1hr-0.00btc_depth': 4733,
    'spread-bitstamp-orderbook-1hr-1.00btc_depth': 4734,
    'spread-bitstamp-orderbook-1hr-5.00btc_depth': 4735,
    'spread-bitstamp-orderbook-1hr-10.00btc_depth': 4736,
    'spread-bitstamp-orderbook-1hr-20.00btc_depth': 4737,
    'spread-bitfinex-orderbook-1hr-0.00btc_depth': 4738,
    'spread-bitfinex-orderbook-1hr-1.00btc_depth': 4739,
    'spread-bitfinex-orderbook-1hr-5.00btc_depth': 4740,
    'spread-bitfinex-orderbook-1hr-10.00btc_depth': 4741,
    'spread-bitfinex-orderbook-1hr-20.00btc_depth': 4742,
    'spread-coinbase-orderbook-1hr-0.00btc_depth': 4743,
    'spread-coinbase-orderbook-1hr-1.00btc_depth': 4744,
    'spread-coinbase-orderbook-1hr-5.00btc_depth': 4745,
    'spread-coinbase-orderbook-1hr-10.00btc_depth': 4746,
    'spread-coinbase-orderbook-1hr-20.00btc_depth': 4747,
    'spread-itbit-orderbook-1hr-0.00btc_depth': 4748,
    'spread-itbit-orderbook-1hr-1.00btc_depth': 4749,
    'spread-itbit-orderbook-1hr-5.00btc_depth': 4750,
    'spread-itbit-orderbook-1hr-10.00btc_depth': 4751,
    'spread-itbit-orderbook-1hr-20.00btc_depth': 4752,
    'spread-kraken-orderbook-1hr-0.00btc_depth': 4753,
    'spread-kraken-orderbook-1hr-1.00btc_depth': 4754,
    'spread-kraken-orderbook-1hr-5.00btc_depth': 4755,
    'spread-kraken-orderbook-1hr-10.00btc_depth': 4756,
    'spread-kraken-orderbook-1hr-20.00btc_depth': 4757,
    'spread-gemini-orderbook-1hr-0.00btc_depth': 4758,
    'spread-gemini-orderbook-1hr-1.00btc_depth': 4759,
    'spread-gemini-orderbook-1hr-5.00btc_depth': 4760,
    'spread-gemini-orderbook-1hr-10.00btc_depth': 4761,
    'spread-gemini-orderbook-1hr-20.00btc_depth': 4762,
    'spread-okcoin-orderbook-1hr-0.00btc_depth': 4763,
    'spread-okcoin-orderbook-1hr-1.00btc_depth': 4764,
    'spread-okcoin-orderbook-1hr-5.00btc_depth': 4765,
    'spread-okcoin-orderbook-1hr-10.00btc_depth': 4766,
    'spread-okcoin-orderbook-1hr-20.00btc_depth': 4767,
    'midpoint-bitstamp-orderbook-1hr-0.00btc_depth': 4768,
    'midpoint-bitstamp-orderbook-1hr-1.00btc_depth': 4769,
    'midpoint-bitstamp-orderbook-1hr-5.00btc_depth': 4770,
    'midpoint-bitstamp-orderbook-1hr-10.00btc_depth': 4771,
    'midpoint-bitstamp-orderbook-1hr-20.00btc_depth': 4772,
    'midpoint-bitfinex-orderbook-1hr-0.00btc_depth': 4773,
    'midpoint-bitfinex-orderbook-1hr-1.00btc_depth': 4774,
    'midpoint-bitfinex-orderbook-1hr-5.00btc_depth': 4775,
    'midpoint-bitfinex-orderbook-1hr-10.00btc_depth': 4776,
    'midpoint-bitfinex-orderbook-1hr-20.00btc_depth': 4777,
    'midpoint-coinbase-orderbook-1hr-0.00btc_depth': 4778,
    'midpoint-coinbase-orderbook-1hr-1.00btc_depth': 4779,
    'midpoint-coinbase-orderbook-1hr-5.00btc_depth': 4780,
    'midpoint-coinbase-orderbook-1hr-10.00btc_depth': 4781,
    'midpoint-coinbase-orderbook-1hr-20.00btc_depth': 4782,
    'midpoint-itbit-orderbook-1hr-0.00btc_depth': 4783,
    'midpoint-itbit-orderbook-1hr-1.00btc_depth': 4784,
    'midpoint-itbit-orderbook-1hr-5.00btc_depth': 4785,
    'midpoint-itbit-orderbook-1hr-10.00btc_depth': 4786,
    'midpoint-itbit-orderbook-1hr-20.00btc_depth': 4787,
    'midpoint-kraken-orderbook-1hr-0.00btc_depth': 4788,
    'midpoint-kraken-orderbook-1hr-1.00btc_depth': 4789,
    'midpoint-kraken-orderbook-1hr-5.00btc_depth': 4790,
    'midpoint-kraken-orderbook-1hr-10.00btc_depth': 4791,
    'midpoint-kraken-orderbook-1hr-20.00btc_depth': 4792,
    'midpoint-gemini-orderbook-1hr-0.00btc_depth': 4793,
    'midpoint-gemini-orderbook-1hr-1.00btc_depth': 4794,
    'midpoint-gemini-orderbook-1hr-5.00btc_depth': 4795,
    'midpoint-gemini-orderbook-1hr-10.00btc_depth': 4796,
    'midpoint-gemini-orderbook-1hr-20.00btc_depth': 4797,
    'midpoint-okcoin-orderbook-1hr-0.00btc_depth': 4798,
    'midpoint-okcoin-orderbook-1hr-1.00btc_depth': 4799,
    'midpoint-okcoin-orderbook-1hr-5.00btc_depth': 4800,
    'midpoint-okcoin-orderbook-1hr-10.00btc_depth': 4801,
    'midpoint-okcoin-orderbook-1hr-20.00btc_depth': 4802,
    ###
    'ask_strength_usd-kraken-orderbook-10m-2.50usd_slippage': 4999,
    'ask_strength_usd-kraken-orderbook-10m-2.60usd_slippage': 5000,
    'ask_strength_usd-kraken-orderbook-10m-2.70usd_slippage': 5001,
    'ask_strength_usd-kraken-orderbook-10m-2.80usd_slippage': 5002,
    'ask_strength_usd-kraken-orderbook-10m-2.90usd_slippage': 5003,
    'ask_strength_usd-kraken-orderbook-10m-3.00usd_slippage': 5004,
    'ask_strength_usd-kraken-orderbook-10m-3.10usd_slippage': 5005,
    'ask_strength_usd-kraken-orderbook-10m-3.20usd_slippage': 5006,
    'ask_strength_usd-kraken-orderbook-10m-3.30usd_slippage': 5007,
    'ask_strength_usd-kraken-orderbook-10m-3.40usd_slippage': 5008,
    'ask_strength_usd-kraken-orderbook-10m-3.50usd_slippage': 5009,
    'ask_strength_usd-kraken-orderbook-10m-3.60usd_slippage': 5010,
    'ask_strength_usd-kraken-orderbook-10m-3.70usd_slippage': 5011,
    'ask_strength_usd-kraken-orderbook-10m-3.80usd_slippage': 5012,
    'ask_strength_usd-kraken-orderbook-10m-3.90usd_slippage': 5013,
    'ask_strength_usd-kraken-orderbook-10m-4.00usd_slippage': 5014,
    'ask_strength_usd-kraken-orderbook-10m-4.10usd_slippage': 5015,
    'ask_strength_usd-kraken-orderbook-10m-4.20usd_slippage': 5016,
    'ask_strength_usd-kraken-orderbook-10m-4.30usd_slippage': 5017,
    'ask_strength_usd-kraken-orderbook-10m-4.40usd_slippage': 5018,
    'ask_strength_usd-kraken-orderbook-10m-4.50usd_slippage': 5019,
    'ask_strength_usd-kraken-orderbook-10m-4.60usd_slippage': 5020,
    'ask_strength_usd-kraken-orderbook-10m-4.70usd_slippage': 5021,
    'ask_strength_usd-kraken-orderbook-10m-4.80usd_slippage': 5022,
    'ask_strength_usd-kraken-orderbook-10m-4.90usd_slippage': 5023,
    'ask_strength_usd-kraken-orderbook-10m-5.00usd_slippage': 5024,
    'ask_strength_usd-kraken-orderbook-10m-10.00usd_slippage': 5025,
    'ask_strength_usd-kraken-orderbook-10m-20.00usd_slippage': 5026,
    'ask_strength_usd-kraken-orderbook-5m-2.50usd_slippage': 5027,
    'ask_strength_usd-kraken-orderbook-5m-2.60usd_slippage': 5028,
    'ask_strength_usd-kraken-orderbook-5m-2.70usd_slippage': 5029,
    'ask_strength_usd-kraken-orderbook-5m-2.80usd_slippage': 5030,
    'ask_strength_usd-kraken-orderbook-5m-2.90usd_slippage': 5031,
    'ask_strength_usd-kraken-orderbook-5m-3.00usd_slippage': 5032,
    'ask_strength_usd-kraken-orderbook-5m-3.10usd_slippage': 5033,
    'ask_strength_usd-kraken-orderbook-5m-3.20usd_slippage': 5034,
    'ask_strength_usd-kraken-orderbook-5m-3.30usd_slippage': 5035,
    'ask_strength_usd-kraken-orderbook-5m-3.40usd_slippage': 5036,
    'ask_strength_usd-kraken-orderbook-5m-3.50usd_slippage': 5037,
    'ask_strength_usd-kraken-orderbook-5m-3.60usd_slippage': 5038,
    'ask_strength_usd-kraken-orderbook-5m-3.70usd_slippage': 5039,
    'ask_strength_usd-kraken-orderbook-5m-3.80usd_slippage': 5040,
    'ask_strength_usd-kraken-orderbook-5m-3.90usd_slippage': 5041,
    'ask_strength_usd-kraken-orderbook-5m-4.00usd_slippage': 5042,
    'ask_strength_usd-kraken-orderbook-5m-4.10usd_slippage': 5043,
    'ask_strength_usd-kraken-orderbook-5m-4.20usd_slippage': 5044,
    'ask_strength_usd-kraken-orderbook-5m-4.30usd_slippage': 5045,
    'ask_strength_usd-kraken-orderbook-5m-4.40usd_slippage': 5046,
    'ask_strength_usd-kraken-orderbook-5m-4.50usd_slippage': 5047,
    'ask_strength_usd-kraken-orderbook-5m-4.60usd_slippage': 5048,
    'ask_strength_usd-kraken-orderbook-5m-4.70usd_slippage': 5049,
    'ask_strength_usd-kraken-orderbook-5m-4.80usd_slippage': 5050,
    'ask_strength_usd-kraken-orderbook-5m-4.90usd_slippage': 5051,
    'ask_strength_usd-kraken-orderbook-5m-5.00usd_slippage': 5052,
    'ask_strength_usd-kraken-orderbook-5m-10.00usd_slippage': 5053,
    'ask_strength_usd-kraken-orderbook-5m-20.00usd_slippage': 5054,
    'ask_strength_usd-kraken-orderbook-1m-2.50usd_slippage': 5055,
    'ask_strength_usd-kraken-orderbook-1m-2.60usd_slippage': 5056,
    'ask_strength_usd-kraken-orderbook-1m-2.70usd_slippage': 5057,
    'ask_strength_usd-kraken-orderbook-1m-2.80usd_slippage': 5058,
    'ask_strength_usd-kraken-orderbook-1m-2.90usd_slippage': 5059,
    'ask_strength_usd-kraken-orderbook-1m-3.00usd_slippage': 5060,
    'ask_strength_usd-kraken-orderbook-1m-3.10usd_slippage': 5061,
    'ask_strength_usd-kraken-orderbook-1m-3.20usd_slippage': 5062,
    'ask_strength_usd-kraken-orderbook-1m-3.30usd_slippage': 5063,
    'ask_strength_usd-kraken-orderbook-1m-3.40usd_slippage': 5064,
    'ask_strength_usd-kraken-orderbook-1m-3.50usd_slippage': 5065,
    'ask_strength_usd-kraken-orderbook-1m-3.60usd_slippage': 5066,
    'ask_strength_usd-kraken-orderbook-1m-3.70usd_slippage': 5067,
    'ask_strength_usd-kraken-orderbook-1m-3.80usd_slippage': 5068,
    'ask_strength_usd-kraken-orderbook-1m-3.90usd_slippage': 5069,
    'ask_strength_usd-kraken-orderbook-1m-4.00usd_slippage': 5070,
    'ask_strength_usd-kraken-orderbook-1m-4.10usd_slippage': 5071,
    'ask_strength_usd-kraken-orderbook-1m-4.20usd_slippage': 5072,
    'ask_strength_usd-kraken-orderbook-1m-4.30usd_slippage': 5073,
    'ask_strength_usd-kraken-orderbook-1m-4.40usd_slippage': 5074,
    'ask_strength_usd-kraken-orderbook-1m-4.50usd_slippage': 5075,
    'ask_strength_usd-kraken-orderbook-1m-4.60usd_slippage': 5076,
    'ask_strength_usd-kraken-orderbook-1m-4.70usd_slippage': 5077,
    'ask_strength_usd-kraken-orderbook-1m-4.80usd_slippage': 5078,
    'ask_strength_usd-kraken-orderbook-1m-4.90usd_slippage': 5079,
    'ask_strength_usd-kraken-orderbook-1m-5.00usd_slippage': 5080,
    'ask_strength_usd-kraken-orderbook-1m-10.00usd_slippage': 5081,
    'ask_strength_usd-kraken-orderbook-1m-20.00usd_slippage': 5082,
    'ask_strength_usd-gemini-orderbook-10m-0.00usd_slippage': 5083,
    'ask_strength_usd-gemini-orderbook-10m-0.10usd_slippage': 5084,
    'ask_strength_usd-gemini-orderbook-10m-0.20usd_slippage': 5085,
    'ask_strength_usd-gemini-orderbook-10m-0.30usd_slippage': 5086,
    'ask_strength_usd-gemini-orderbook-10m-0.40usd_slippage': 5087,
    'ask_strength_usd-gemini-orderbook-10m-0.50usd_slippage': 5088,
    'ask_strength_usd-gemini-orderbook-10m-0.60usd_slippage': 5089,
    'ask_strength_usd-gemini-orderbook-10m-0.70usd_slippage': 5090,
    'ask_strength_usd-gemini-orderbook-10m-0.80usd_slippage': 5091,
    'ask_strength_usd-gemini-orderbook-10m-0.90usd_slippage': 5092,
    'ask_strength_usd-gemini-orderbook-10m-1.00usd_slippage': 5093,
    'ask_strength_usd-gemini-orderbook-10m-1.10usd_slippage': 5094,
    'ask_strength_usd-gemini-orderbook-10m-1.20usd_slippage': 5095,
    'ask_strength_usd-gemini-orderbook-10m-1.30usd_slippage': 5096,
    'ask_strength_usd-gemini-orderbook-10m-1.40usd_slippage': 5097,
    'ask_strength_usd-gemini-orderbook-10m-1.50usd_slippage': 5098,
    'ask_strength_usd-gemini-orderbook-10m-1.60usd_slippage': 5099,
    'ask_strength_usd-gemini-orderbook-10m-1.70usd_slippage': 5100,
    'ask_strength_usd-gemini-orderbook-10m-1.80usd_slippage': 5101,
    'ask_strength_usd-gemini-orderbook-10m-1.90usd_slippage': 5102,
    'ask_strength_usd-gemini-orderbook-10m-2.00usd_slippage': 5103,
    'ask_strength_usd-gemini-orderbook-10m-2.10usd_slippage': 5104,
    'ask_strength_usd-gemini-orderbook-10m-2.20usd_slippage': 5105,
    'ask_strength_usd-gemini-orderbook-10m-2.30usd_slippage': 5106,
    'ask_strength_usd-gemini-orderbook-10m-2.40usd_slippage': 5107,
    'ask_strength_usd-gemini-orderbook-10m-2.50usd_slippage': 5108,
    'ask_strength_usd-gemini-orderbook-10m-2.60usd_slippage': 5109,
    'ask_strength_usd-gemini-orderbook-10m-2.70usd_slippage': 5110,
    'ask_strength_usd-gemini-orderbook-10m-2.80usd_slippage': 5111,
    'ask_strength_usd-gemini-orderbook-10m-2.90usd_slippage': 5112,
    'ask_strength_usd-gemini-orderbook-10m-3.00usd_slippage': 5113,
    'ask_strength_usd-gemini-orderbook-10m-3.10usd_slippage': 5114,
    'ask_strength_usd-gemini-orderbook-10m-3.20usd_slippage': 5115,
    'ask_strength_usd-gemini-orderbook-10m-3.30usd_slippage': 5116,
    'ask_strength_usd-gemini-orderbook-10m-3.40usd_slippage': 5117,
    'ask_strength_usd-gemini-orderbook-10m-3.50usd_slippage': 5118,
    'ask_strength_usd-gemini-orderbook-10m-3.60usd_slippage': 5119,
    'ask_strength_usd-gemini-orderbook-10m-3.70usd_slippage': 5120,
    'ask_strength_usd-gemini-orderbook-10m-3.80usd_slippage': 5121,
    'ask_strength_usd-gemini-orderbook-10m-3.90usd_slippage': 5122,
    'ask_strength_usd-gemini-orderbook-10m-4.00usd_slippage': 5123,
    'ask_strength_usd-gemini-orderbook-10m-4.10usd_slippage': 5124,
    'ask_strength_usd-gemini-orderbook-10m-4.20usd_slippage': 5125,
    'ask_strength_usd-gemini-orderbook-10m-4.30usd_slippage': 5126,
    'ask_strength_usd-gemini-orderbook-10m-4.40usd_slippage': 5127,
    'ask_strength_usd-gemini-orderbook-10m-4.50usd_slippage': 5128,
    'ask_strength_usd-gemini-orderbook-10m-4.60usd_slippage': 5129,
    'ask_strength_usd-gemini-orderbook-10m-4.70usd_slippage': 5130,
    'ask_strength_usd-gemini-orderbook-10m-4.80usd_slippage': 5131,
    'ask_strength_usd-gemini-orderbook-10m-4.90usd_slippage': 5132,
    'ask_strength_usd-gemini-orderbook-10m-5.00usd_slippage': 5133,
    'ask_strength_usd-gemini-orderbook-10m-10.00usd_slippage': 5134,
    'ask_strength_usd-gemini-orderbook-10m-20.00usd_slippage': 5135,
    'ask_strength_usd-gemini-orderbook-5m-0.00usd_slippage': 5136,
    'ask_strength_usd-gemini-orderbook-5m-0.10usd_slippage': 5137,
    'ask_strength_usd-gemini-orderbook-5m-0.20usd_slippage': 5138,
    'ask_strength_usd-gemini-orderbook-5m-0.30usd_slippage': 5139,
    'ask_strength_usd-gemini-orderbook-5m-0.40usd_slippage': 5140,
    'ask_strength_usd-gemini-orderbook-5m-0.50usd_slippage': 5141,
    'ask_strength_usd-gemini-orderbook-5m-0.60usd_slippage': 5142,
    'ask_strength_usd-gemini-orderbook-5m-0.70usd_slippage': 5143,
    'ask_strength_usd-gemini-orderbook-5m-0.80usd_slippage': 5144,
    'ask_strength_usd-gemini-orderbook-5m-0.90usd_slippage': 5145,
    'ask_strength_usd-gemini-orderbook-5m-1.00usd_slippage': 5146,
    'ask_strength_usd-gemini-orderbook-5m-1.10usd_slippage': 5147,
    'ask_strength_usd-gemini-orderbook-5m-1.20usd_slippage': 5148,
    'ask_strength_usd-gemini-orderbook-5m-1.30usd_slippage': 5149,
    'ask_strength_usd-gemini-orderbook-5m-1.40usd_slippage': 5150,
    'ask_strength_usd-gemini-orderbook-5m-1.50usd_slippage': 5151,
    'ask_strength_usd-gemini-orderbook-5m-1.60usd_slippage': 5152,
    'ask_strength_usd-gemini-orderbook-5m-1.70usd_slippage': 5153,
    'ask_strength_usd-gemini-orderbook-5m-1.80usd_slippage': 5154,
    'ask_strength_usd-gemini-orderbook-5m-1.90usd_slippage': 5155,
    'ask_strength_usd-gemini-orderbook-5m-2.00usd_slippage': 5156,
    'ask_strength_usd-gemini-orderbook-5m-2.10usd_slippage': 5157,
    'ask_strength_usd-gemini-orderbook-5m-2.20usd_slippage': 5158,
    'ask_strength_usd-gemini-orderbook-5m-2.30usd_slippage': 5159,
    'ask_strength_usd-gemini-orderbook-5m-2.40usd_slippage': 5160,
    'ask_strength_usd-gemini-orderbook-5m-2.50usd_slippage': 5161,
    'ask_strength_usd-gemini-orderbook-5m-2.60usd_slippage': 5162,
    'ask_strength_usd-gemini-orderbook-5m-2.70usd_slippage': 5163,
    'ask_strength_usd-gemini-orderbook-5m-2.80usd_slippage': 5164,
    'ask_strength_usd-gemini-orderbook-5m-2.90usd_slippage': 5165,
    'ask_strength_usd-gemini-orderbook-5m-3.00usd_slippage': 5166,
    'ask_strength_usd-gemini-orderbook-5m-3.10usd_slippage': 5167,
    'ask_strength_usd-gemini-orderbook-5m-3.20usd_slippage': 5168,
    'ask_strength_usd-gemini-orderbook-5m-3.30usd_slippage': 5169,
    'ask_strength_usd-gemini-orderbook-5m-3.40usd_slippage': 5170,
    'ask_strength_usd-gemini-orderbook-5m-3.50usd_slippage': 5171,
    'ask_strength_usd-gemini-orderbook-5m-3.60usd_slippage': 5172,
    'ask_strength_usd-gemini-orderbook-5m-3.70usd_slippage': 5173,
    'ask_strength_usd-gemini-orderbook-5m-3.80usd_slippage': 5174,
    'ask_strength_usd-gemini-orderbook-5m-3.90usd_slippage': 5175,
    'ask_strength_usd-gemini-orderbook-5m-4.00usd_slippage': 5176,
    'ask_strength_usd-gemini-orderbook-5m-4.10usd_slippage': 5177,
    'ask_strength_usd-gemini-orderbook-5m-4.20usd_slippage': 5178,
    'ask_strength_usd-gemini-orderbook-5m-4.30usd_slippage': 5179,
    'ask_strength_usd-gemini-orderbook-5m-4.40usd_slippage': 5180,
    'ask_strength_usd-gemini-orderbook-5m-4.50usd_slippage': 5181,
    'ask_strength_usd-gemini-orderbook-5m-4.60usd_slippage': 5182,
    'ask_strength_usd-gemini-orderbook-5m-4.70usd_slippage': 5183,
    'ask_strength_usd-gemini-orderbook-5m-4.80usd_slippage': 5184,
    'ask_strength_usd-gemini-orderbook-5m-4.90usd_slippage': 5185,
    'ask_strength_usd-gemini-orderbook-5m-5.00usd_slippage': 5186,
    'ask_strength_usd-gemini-orderbook-5m-10.00usd_slippage': 5187,
    'ask_strength_usd-gemini-orderbook-5m-20.00usd_slippage': 5188,
    'ask_strength_usd-gemini-orderbook-1m-0.00usd_slippage': 5189,
    'ask_strength_usd-gemini-orderbook-1m-0.10usd_slippage': 5190,
    'ask_strength_usd-gemini-orderbook-1m-0.20usd_slippage': 5191,
    'ask_strength_usd-gemini-orderbook-1m-0.30usd_slippage': 5192,
    'ask_strength_usd-gemini-orderbook-1m-0.40usd_slippage': 5193,
    'ask_strength_usd-gemini-orderbook-1m-0.50usd_slippage': 5194,
    'ask_strength_usd-gemini-orderbook-1m-0.60usd_slippage': 5195,
    'ask_strength_usd-gemini-orderbook-1m-0.70usd_slippage': 5196,
    'ask_strength_usd-gemini-orderbook-1m-0.80usd_slippage': 5197,
    'ask_strength_usd-gemini-orderbook-1m-0.90usd_slippage': 5198,
    'ask_strength_usd-gemini-orderbook-1m-1.00usd_slippage': 5199,
    'ask_strength_usd-gemini-orderbook-1m-1.10usd_slippage': 5200,
    'ask_strength_usd-gemini-orderbook-1m-1.20usd_slippage': 5201,
    'ask_strength_usd-gemini-orderbook-1m-1.30usd_slippage': 5202,
    'ask_strength_usd-gemini-orderbook-1m-1.40usd_slippage': 5203,
    'ask_strength_usd-gemini-orderbook-1m-1.50usd_slippage': 5204,
    'ask_strength_usd-gemini-orderbook-1m-1.60usd_slippage': 5205,
    'ask_strength_usd-gemini-orderbook-1m-1.70usd_slippage': 5206,
    'ask_strength_usd-gemini-orderbook-1m-1.80usd_slippage': 5207,
    'ask_strength_usd-gemini-orderbook-1m-1.90usd_slippage': 5208,
    'ask_strength_usd-gemini-orderbook-1m-2.00usd_slippage': 5209,
    'ask_strength_usd-gemini-orderbook-1m-2.10usd_slippage': 5210,
    'ask_strength_usd-gemini-orderbook-1m-2.20usd_slippage': 5211,
    'ask_strength_usd-gemini-orderbook-1m-2.30usd_slippage': 5212,
    'ask_strength_usd-gemini-orderbook-1m-2.40usd_slippage': 5213,
    'ask_strength_usd-gemini-orderbook-1m-2.50usd_slippage': 5214,
    'ask_strength_usd-gemini-orderbook-1m-2.60usd_slippage': 5215,
    'ask_strength_usd-gemini-orderbook-1m-2.70usd_slippage': 5216,
    'ask_strength_usd-gemini-orderbook-1m-2.80usd_slippage': 5217,
    'ask_strength_usd-gemini-orderbook-1m-2.90usd_slippage': 5218,
    'ask_strength_usd-gemini-orderbook-1m-3.00usd_slippage': 5219,
    'ask_strength_usd-gemini-orderbook-1m-3.10usd_slippage': 5220,
    'ask_strength_usd-gemini-orderbook-1m-3.20usd_slippage': 5221,
    'ask_strength_usd-gemini-orderbook-1m-3.30usd_slippage': 5222,
    'ask_strength_usd-gemini-orderbook-1m-3.40usd_slippage': 5223,
    'ask_strength_usd-gemini-orderbook-1m-3.50usd_slippage': 5224,
    'ask_strength_usd-gemini-orderbook-1m-3.60usd_slippage': 5225,
    'ask_strength_usd-gemini-orderbook-1m-3.70usd_slippage': 5226,
    'ask_strength_usd-gemini-orderbook-1m-3.80usd_slippage': 5227,
    'ask_strength_usd-gemini-orderbook-1m-3.90usd_slippage': 5228,
    'ask_strength_usd-gemini-orderbook-1m-4.00usd_slippage': 5229,
    'ask_strength_usd-gemini-orderbook-1m-4.10usd_slippage': 5230,
    'ask_strength_usd-gemini-orderbook-1m-4.20usd_slippage': 5231,
    'ask_strength_usd-gemini-orderbook-1m-4.30usd_slippage': 5232,
    'ask_strength_usd-gemini-orderbook-1m-4.40usd_slippage': 5233,
    'ask_strength_usd-gemini-orderbook-1m-4.50usd_slippage': 5234,
    'ask_strength_usd-gemini-orderbook-1m-4.60usd_slippage': 5235,
    'ask_strength_usd-gemini-orderbook-1m-4.70usd_slippage': 5236,
    'ask_strength_usd-gemini-orderbook-1m-4.80usd_slippage': 5237,
    'ask_strength_usd-gemini-orderbook-1m-4.90usd_slippage': 5238,
    'ask_strength_usd-gemini-orderbook-1m-5.00usd_slippage': 5239,
    'ask_strength_usd-gemini-orderbook-1m-10.00usd_slippage': 5240,
    'ask_strength_usd-gemini-orderbook-1m-20.00usd_slippage': 5241,
    'ask_strength_usd-okcoin-orderbook-10m-0.00usd_slippage': 5242,
    'ask_strength_usd-okcoin-orderbook-10m-0.10usd_slippage': 5243,
    'ask_strength_usd-okcoin-orderbook-10m-0.20usd_slippage': 5244,
    'ask_strength_usd-okcoin-orderbook-10m-0.30usd_slippage': 5245,
    'ask_strength_usd-okcoin-orderbook-10m-0.40usd_slippage': 5246,
    'ask_strength_usd-okcoin-orderbook-10m-0.50usd_slippage': 5247,
    'ask_strength_usd-okcoin-orderbook-10m-0.60usd_slippage': 5248,
    'ask_strength_usd-okcoin-orderbook-10m-0.70usd_slippage': 5249,
    'ask_strength_usd-okcoin-orderbook-10m-0.80usd_slippage': 5250,
    'ask_strength_usd-okcoin-orderbook-10m-0.90usd_slippage': 5251,
    'ask_strength_usd-okcoin-orderbook-10m-1.00usd_slippage': 5252,
    'ask_strength_usd-okcoin-orderbook-10m-1.10usd_slippage': 5253,
    'ask_strength_usd-okcoin-orderbook-10m-1.20usd_slippage': 5254,
    'ask_strength_usd-okcoin-orderbook-10m-1.30usd_slippage': 5255,
    'ask_strength_usd-okcoin-orderbook-10m-1.40usd_slippage': 5256,
    'ask_strength_usd-okcoin-orderbook-10m-1.50usd_slippage': 5257,
    'ask_strength_usd-okcoin-orderbook-10m-1.60usd_slippage': 5258,
    'ask_strength_usd-okcoin-orderbook-10m-1.70usd_slippage': 5259,
    'ask_strength_usd-okcoin-orderbook-10m-1.80usd_slippage': 5260,
    'ask_strength_usd-okcoin-orderbook-10m-1.90usd_slippage': 5261,
    'ask_strength_usd-okcoin-orderbook-10m-2.00usd_slippage': 5262,
    'ask_strength_usd-okcoin-orderbook-10m-2.10usd_slippage': 5263,
    'ask_strength_usd-okcoin-orderbook-10m-2.20usd_slippage': 5264,
    'ask_strength_usd-okcoin-orderbook-10m-2.30usd_slippage': 5265,
    'ask_strength_usd-okcoin-orderbook-10m-2.40usd_slippage': 5266,
    'ask_strength_usd-okcoin-orderbook-10m-2.50usd_slippage': 5267,
    'ask_strength_usd-okcoin-orderbook-10m-2.60usd_slippage': 5268,
    'ask_strength_usd-okcoin-orderbook-10m-2.70usd_slippage': 5269,
    'ask_strength_usd-okcoin-orderbook-10m-2.80usd_slippage': 5270,
    'ask_strength_usd-okcoin-orderbook-10m-2.90usd_slippage': 5271,
    'ask_strength_usd-okcoin-orderbook-10m-3.00usd_slippage': 5272,
    'ask_strength_usd-okcoin-orderbook-10m-3.10usd_slippage': 5273,
    'ask_strength_usd-okcoin-orderbook-10m-3.20usd_slippage': 5274,
    'ask_strength_usd-okcoin-orderbook-10m-3.30usd_slippage': 5275,
    'ask_strength_usd-okcoin-orderbook-10m-3.40usd_slippage': 5276,
    'ask_strength_usd-okcoin-orderbook-10m-3.50usd_slippage': 5277,
    'ask_strength_usd-okcoin-orderbook-10m-3.60usd_slippage': 5278,
    'ask_strength_usd-okcoin-orderbook-10m-3.70usd_slippage': 5279,
    'ask_strength_usd-okcoin-orderbook-10m-3.80usd_slippage': 5280,
    'ask_strength_usd-okcoin-orderbook-10m-3.90usd_slippage': 5281,
    'ask_strength_usd-okcoin-orderbook-10m-4.00usd_slippage': 5282,
    'ask_strength_usd-okcoin-orderbook-10m-4.10usd_slippage': 5283,
    'ask_strength_usd-okcoin-orderbook-10m-4.20usd_slippage': 5284,
    'ask_strength_usd-okcoin-orderbook-10m-4.30usd_slippage': 5285,
    'ask_strength_usd-okcoin-orderbook-10m-4.40usd_slippage': 5286,
    'ask_strength_usd-okcoin-orderbook-10m-4.50usd_slippage': 5287,
    'ask_strength_usd-okcoin-orderbook-10m-4.60usd_slippage': 5288,
    'ask_strength_usd-okcoin-orderbook-10m-4.70usd_slippage': 5289,
    'ask_strength_usd-okcoin-orderbook-10m-4.80usd_slippage': 5290,
    'ask_strength_usd-okcoin-orderbook-10m-4.90usd_slippage': 5291,
    'ask_strength_usd-okcoin-orderbook-10m-5.00usd_slippage': 5292,
    'ask_strength_usd-okcoin-orderbook-10m-10.00usd_slippage': 5293,
    'ask_strength_usd-okcoin-orderbook-10m-20.00usd_slippage': 5294,
    'ask_strength_usd-okcoin-orderbook-5m-0.00usd_slippage': 5295,
    'ask_strength_usd-okcoin-orderbook-5m-0.10usd_slippage': 5296,
    'ask_strength_usd-okcoin-orderbook-5m-0.20usd_slippage': 5297,
    'ask_strength_usd-okcoin-orderbook-5m-0.30usd_slippage': 5298,
    'ask_strength_usd-okcoin-orderbook-5m-0.40usd_slippage': 5299,
    'ask_strength_usd-okcoin-orderbook-5m-0.50usd_slippage': 5300,
    'ask_strength_usd-okcoin-orderbook-5m-0.60usd_slippage': 5301,
    'ask_strength_usd-okcoin-orderbook-5m-0.70usd_slippage': 5302,
    'ask_strength_usd-okcoin-orderbook-5m-0.80usd_slippage': 5303,
    'ask_strength_usd-okcoin-orderbook-5m-0.90usd_slippage': 5304,
    'ask_strength_usd-okcoin-orderbook-5m-1.00usd_slippage': 5305,
    'ask_strength_usd-okcoin-orderbook-5m-1.10usd_slippage': 5306,
    'ask_strength_usd-okcoin-orderbook-5m-1.20usd_slippage': 5307,
    'ask_strength_usd-okcoin-orderbook-5m-1.30usd_slippage': 5308,
    'ask_strength_usd-okcoin-orderbook-5m-1.40usd_slippage': 5309,
    'ask_strength_usd-okcoin-orderbook-5m-1.50usd_slippage': 5310,
    'ask_strength_usd-okcoin-orderbook-5m-1.60usd_slippage': 5311,
    'ask_strength_usd-okcoin-orderbook-5m-1.70usd_slippage': 5312,
    'ask_strength_usd-okcoin-orderbook-5m-1.80usd_slippage': 5313,
    'ask_strength_usd-okcoin-orderbook-5m-1.90usd_slippage': 5314,
    'ask_strength_usd-okcoin-orderbook-5m-2.00usd_slippage': 5315,
    'ask_strength_usd-okcoin-orderbook-5m-2.10usd_slippage': 5316,
    'ask_strength_usd-okcoin-orderbook-5m-2.20usd_slippage': 5317,
    'ask_strength_usd-okcoin-orderbook-5m-2.30usd_slippage': 5318,
    'ask_strength_usd-okcoin-orderbook-5m-2.40usd_slippage': 5319,
    'ask_strength_usd-okcoin-orderbook-5m-2.50usd_slippage': 5320,
    'ask_strength_usd-okcoin-orderbook-5m-2.60usd_slippage': 5321,
    'ask_strength_usd-okcoin-orderbook-5m-2.70usd_slippage': 5322,
    'ask_strength_usd-okcoin-orderbook-5m-2.80usd_slippage': 5323,
    'ask_strength_usd-okcoin-orderbook-5m-2.90usd_slippage': 5324,
    'ask_strength_usd-okcoin-orderbook-5m-3.00usd_slippage': 5325,
    'ask_strength_usd-okcoin-orderbook-5m-3.10usd_slippage': 5326,
    'ask_strength_usd-okcoin-orderbook-5m-3.20usd_slippage': 5327,
    'ask_strength_usd-okcoin-orderbook-5m-3.30usd_slippage': 5328,
    'ask_strength_usd-okcoin-orderbook-5m-3.40usd_slippage': 5329,
    'ask_strength_usd-okcoin-orderbook-5m-3.50usd_slippage': 5330,
    'ask_strength_usd-okcoin-orderbook-5m-3.60usd_slippage': 5331,
    'ask_strength_usd-okcoin-orderbook-5m-3.70usd_slippage': 5332,
    'ask_strength_usd-okcoin-orderbook-5m-3.80usd_slippage': 5333,
    'ask_strength_usd-okcoin-orderbook-5m-3.90usd_slippage': 5334,
    'ask_strength_usd-okcoin-orderbook-5m-4.00usd_slippage': 5335,
    'ask_strength_usd-okcoin-orderbook-5m-4.10usd_slippage': 5336,
    'ask_strength_usd-okcoin-orderbook-5m-4.20usd_slippage': 5337,
    'ask_strength_usd-okcoin-orderbook-5m-4.30usd_slippage': 5338,
    'ask_strength_usd-okcoin-orderbook-5m-4.40usd_slippage': 5339,
    'ask_strength_usd-okcoin-orderbook-5m-4.50usd_slippage': 5340,
    'ask_strength_usd-okcoin-orderbook-5m-4.60usd_slippage': 5341,
    'ask_strength_usd-okcoin-orderbook-5m-4.70usd_slippage': 5342,
    'ask_strength_usd-okcoin-orderbook-5m-4.80usd_slippage': 5343,
    'ask_strength_usd-okcoin-orderbook-5m-4.90usd_slippage': 5344,
    'ask_strength_usd-okcoin-orderbook-5m-5.00usd_slippage': 5345,
    'ask_strength_usd-okcoin-orderbook-5m-10.00usd_slippage': 5346,
    'ask_strength_usd-okcoin-orderbook-5m-20.00usd_slippage': 5347,
    'ask_strength_usd-okcoin-orderbook-1m-0.00usd_slippage': 5348,
    'ask_strength_usd-okcoin-orderbook-1m-0.10usd_slippage': 5349,
    'ask_strength_usd-okcoin-orderbook-1m-0.20usd_slippage': 5350,
    'ask_strength_usd-okcoin-orderbook-1m-0.30usd_slippage': 5351,
    'ask_strength_usd-okcoin-orderbook-1m-0.40usd_slippage': 5352,
    'ask_strength_usd-okcoin-orderbook-1m-0.50usd_slippage': 5353,
    'ask_strength_usd-okcoin-orderbook-1m-0.60usd_slippage': 5354,
    'ask_strength_usd-okcoin-orderbook-1m-0.70usd_slippage': 5355,
    'ask_strength_usd-okcoin-orderbook-1m-0.80usd_slippage': 5356,
    'ask_strength_usd-okcoin-orderbook-1m-0.90usd_slippage': 5357,
    'ask_strength_usd-okcoin-orderbook-1m-1.00usd_slippage': 5358,
    'ask_strength_usd-okcoin-orderbook-1m-1.10usd_slippage': 5359,
    'ask_strength_usd-okcoin-orderbook-1m-1.20usd_slippage': 5360,
    'ask_strength_usd-okcoin-orderbook-1m-1.30usd_slippage': 5361,
    'ask_strength_usd-okcoin-orderbook-1m-1.40usd_slippage': 5362,
    'ask_strength_usd-okcoin-orderbook-1m-1.50usd_slippage': 5363,
    'ask_strength_usd-okcoin-orderbook-1m-1.60usd_slippage': 5364,
    'ask_strength_usd-okcoin-orderbook-1m-1.70usd_slippage': 5365,
    'ask_strength_usd-okcoin-orderbook-1m-1.80usd_slippage': 5366,
    'ask_strength_usd-okcoin-orderbook-1m-1.90usd_slippage': 5367,
    'ask_strength_usd-okcoin-orderbook-1m-2.00usd_slippage': 5368,
    'ask_strength_usd-okcoin-orderbook-1m-2.10usd_slippage': 5369,
    'ask_strength_usd-okcoin-orderbook-1m-2.20usd_slippage': 5370,
    'ask_strength_usd-okcoin-orderbook-1m-2.30usd_slippage': 5371,
    'ask_strength_usd-okcoin-orderbook-1m-2.40usd_slippage': 5372,
    'ask_strength_usd-okcoin-orderbook-1m-2.50usd_slippage': 5373,
    'ask_strength_usd-okcoin-orderbook-1m-2.60usd_slippage': 5374,
    'ask_strength_usd-okcoin-orderbook-1m-2.70usd_slippage': 5375,
    'ask_strength_usd-okcoin-orderbook-1m-2.80usd_slippage': 5376,
    'ask_strength_usd-okcoin-orderbook-1m-2.90usd_slippage': 5377,
    'ask_strength_usd-okcoin-orderbook-1m-3.00usd_slippage': 5378,
    'ask_strength_usd-okcoin-orderbook-1m-3.10usd_slippage': 5379,
    'ask_strength_usd-okcoin-orderbook-1m-3.20usd_slippage': 5380,
    'ask_strength_usd-okcoin-orderbook-1m-3.30usd_slippage': 5381,
    'ask_strength_usd-okcoin-orderbook-1m-3.40usd_slippage': 5382,
    'ask_strength_usd-okcoin-orderbook-1m-3.50usd_slippage': 5383,
    'ask_strength_usd-okcoin-orderbook-1m-3.60usd_slippage': 5384,
    'ask_strength_usd-okcoin-orderbook-1m-3.70usd_slippage': 5385,
    'ask_strength_usd-okcoin-orderbook-1m-3.80usd_slippage': 5386,
    'ask_strength_usd-okcoin-orderbook-1m-3.90usd_slippage': 5387,
    'ask_strength_usd-okcoin-orderbook-1m-4.00usd_slippage': 5388,
    'ask_strength_usd-okcoin-orderbook-1m-4.10usd_slippage': 5389,
    'ask_strength_usd-okcoin-orderbook-1m-4.20usd_slippage': 5390,
    'ask_strength_usd-okcoin-orderbook-1m-4.30usd_slippage': 5391,
    'ask_strength_usd-okcoin-orderbook-1m-4.40usd_slippage': 5392,
    'ask_strength_usd-okcoin-orderbook-1m-4.50usd_slippage': 5393,
    'ask_strength_usd-okcoin-orderbook-1m-4.60usd_slippage': 5394,
    'ask_strength_usd-okcoin-orderbook-1m-4.70usd_slippage': 5395,
    'ask_strength_usd-okcoin-orderbook-1m-4.80usd_slippage': 5396,
    'ask_strength_usd-okcoin-orderbook-1m-4.90usd_slippage': 5397,
    'ask_strength_usd-okcoin-orderbook-1m-5.00usd_slippage': 5398,
    'ask_strength_usd-okcoin-orderbook-1m-10.00usd_slippage': 5399,
    'ask_strength_usd-okcoin-orderbook-1m-20.00usd_slippage': 5400,
    # 1hr strength series': 0, 1, 2, 4, 6, 8, 10, usd slippages
    'bid_strength-bitstamp-orderbook-1hr-0.00usd_slippage': 4803,
    'bid_strength-bitstamp-orderbook-1hr-1.00usd_slippage': 4804,
    'bid_strength-bitstamp-orderbook-1hr-2.00usd_slippage': 4805,
    'bid_strength-bitstamp-orderbook-1hr-4.00usd_slippage': 4806,
    'bid_strength-bitstamp-orderbook-1hr-6.00usd_slippage': 4807,
    'bid_strength-bitstamp-orderbook-1hr-8.00usd_slippage': 4808,
    'bid_strength-bitstamp-orderbook-1hr-10.00usd_slippage': 4809,
    'bid_strength-bitfinex-orderbook-1hr-0.00usd_slippage': 4810,
    'bid_strength-bitfinex-orderbook-1hr-1.00usd_slippage': 4811,
    'bid_strength-bitfinex-orderbook-1hr-2.00usd_slippage': 4812,
    'bid_strength-bitfinex-orderbook-1hr-4.00usd_slippage': 4813,
    'bid_strength-bitfinex-orderbook-1hr-6.00usd_slippage': 4814,
    'bid_strength-bitfinex-orderbook-1hr-8.00usd_slippage': 4815,
    'bid_strength-bitfinex-orderbook-1hr-10.00usd_slippage': 4816,
    'bid_strength-coinbase-orderbook-1hr-0.00usd_slippage': 4817,
    'bid_strength-coinbase-orderbook-1hr-1.00usd_slippage': 4818,
    'bid_strength-coinbase-orderbook-1hr-2.00usd_slippage': 4819,
    'bid_strength-coinbase-orderbook-1hr-4.00usd_slippage': 4820,
    'bid_strength-coinbase-orderbook-1hr-6.00usd_slippage': 4821,
    'bid_strength-coinbase-orderbook-1hr-8.00usd_slippage': 4822,
    'bid_strength-coinbase-orderbook-1hr-10.00usd_slippage': 4823,
    'bid_strength-itbit-orderbook-1hr-0.00usd_slippage': 4824,
    'bid_strength-itbit-orderbook-1hr-1.00usd_slippage': 4825,
    'bid_strength-itbit-orderbook-1hr-2.00usd_slippage': 4826,
    'bid_strength-itbit-orderbook-1hr-4.00usd_slippage': 4827,
    'bid_strength-itbit-orderbook-1hr-6.00usd_slippage': 4828,
    'bid_strength-itbit-orderbook-1hr-8.00usd_slippage': 4829,
    'bid_strength-itbit-orderbook-1hr-10.00usd_slippage': 4830,
    'bid_strength-kraken-orderbook-1hr-0.00usd_slippage': 4831,
    'bid_strength-kraken-orderbook-1hr-1.00usd_slippage': 4832,
    'bid_strength-kraken-orderbook-1hr-2.00usd_slippage': 4833,
    'bid_strength-kraken-orderbook-1hr-4.00usd_slippage': 4834,
    'bid_strength-kraken-orderbook-1hr-6.00usd_slippage': 4835,
    'bid_strength-kraken-orderbook-1hr-8.00usd_slippage': 4836,
    'bid_strength-kraken-orderbook-1hr-10.00usd_slippage': 4837,
    'bid_strength-gemini-orderbook-1hr-0.00usd_slippage': 4838,
    'bid_strength-gemini-orderbook-1hr-1.00usd_slippage': 4839,
    'bid_strength-gemini-orderbook-1hr-2.00usd_slippage': 4840,
    'bid_strength-gemini-orderbook-1hr-4.00usd_slippage': 4841,
    'bid_strength-gemini-orderbook-1hr-6.00usd_slippage': 4842,
    'bid_strength-gemini-orderbook-1hr-8.00usd_slippage': 4843,
    'bid_strength-gemini-orderbook-1hr-10.00usd_slippage': 4844,
    'bid_strength-okcoin-orderbook-1hr-0.00usd_slippage': 4845,
    'bid_strength-okcoin-orderbook-1hr-1.00usd_slippage': 4846,
    'bid_strength-okcoin-orderbook-1hr-2.00usd_slippage': 4847,
    'bid_strength-okcoin-orderbook-1hr-4.00usd_slippage': 4848,
    'bid_strength-okcoin-orderbook-1hr-6.00usd_slippage': 4849,
    'bid_strength-okcoin-orderbook-1hr-8.00usd_slippage': 4850,
    'bid_strength-okcoin-orderbook-1hr-10.00usd_slippage': 4851,
    'ask_strength-bitstamp-orderbook-1hr-0.00usd_slippage': 4852,
    'ask_strength-bitstamp-orderbook-1hr-1.00usd_slippage': 4853,
    'ask_strength-bitstamp-orderbook-1hr-2.00usd_slippage': 4854,
    'ask_strength-bitstamp-orderbook-1hr-4.00usd_slippage': 4855,
    'ask_strength-bitstamp-orderbook-1hr-6.00usd_slippage': 4856,
    'ask_strength-bitstamp-orderbook-1hr-8.00usd_slippage': 4857,
    'ask_strength-bitstamp-orderbook-1hr-10.00usd_slippage': 4858,
    'ask_strength-bitfinex-orderbook-1hr-0.00usd_slippage': 4859,
    'ask_strength-bitfinex-orderbook-1hr-1.00usd_slippage': 4860,
    'ask_strength-bitfinex-orderbook-1hr-2.00usd_slippage': 4861,
    'ask_strength-bitfinex-orderbook-1hr-4.00usd_slippage': 4862,
    'ask_strength-bitfinex-orderbook-1hr-6.00usd_slippage': 4863,
    'ask_strength-bitfinex-orderbook-1hr-8.00usd_slippage': 4864,
    'ask_strength-bitfinex-orderbook-1hr-10.00usd_slippage': 4865,
    'ask_strength-coinbase-orderbook-1hr-0.00usd_slippage': 4866,
    'ask_strength-coinbase-orderbook-1hr-1.00usd_slippage': 4867,
    'ask_strength-coinbase-orderbook-1hr-2.00usd_slippage': 4868,
    'ask_strength-coinbase-orderbook-1hr-4.00usd_slippage': 4869,
    'ask_strength-coinbase-orderbook-1hr-6.00usd_slippage': 4870,
    'ask_strength-coinbase-orderbook-1hr-8.00usd_slippage': 4871,
    'ask_strength-coinbase-orderbook-1hr-10.00usd_slippage': 4872,
    'ask_strength-itbit-orderbook-1hr-0.00usd_slippage': 4873,
    'ask_strength-itbit-orderbook-1hr-1.00usd_slippage': 4874,
    'ask_strength-itbit-orderbook-1hr-2.00usd_slippage': 4875,
    'ask_strength-itbit-orderbook-1hr-4.00usd_slippage': 4876,
    'ask_strength-itbit-orderbook-1hr-6.00usd_slippage': 4877,
    'ask_strength-itbit-orderbook-1hr-8.00usd_slippage': 4878,
    'ask_strength-itbit-orderbook-1hr-10.00usd_slippage': 4879,
    'ask_strength-kraken-orderbook-1hr-0.00usd_slippage': 4880,
    'ask_strength-kraken-orderbook-1hr-1.00usd_slippage': 4881,
    'ask_strength-kraken-orderbook-1hr-2.00usd_slippage': 4882,
    'ask_strength-kraken-orderbook-1hr-4.00usd_slippage': 4883,
    'ask_strength-kraken-orderbook-1hr-6.00usd_slippage': 4884,
    'ask_strength-kraken-orderbook-1hr-8.00usd_slippage': 4885,
    'ask_strength-kraken-orderbook-1hr-10.00usd_slippage': 4886,
    'ask_strength-gemini-orderbook-1hr-0.00usd_slippage': 4887,
    'ask_strength-gemini-orderbook-1hr-1.00usd_slippage': 4888,
    'ask_strength-gemini-orderbook-1hr-2.00usd_slippage': 4889,
    'ask_strength-gemini-orderbook-1hr-4.00usd_slippage': 4890,
    'ask_strength-gemini-orderbook-1hr-6.00usd_slippage': 4891,
    'ask_strength-gemini-orderbook-1hr-8.00usd_slippage': 4892,
    'ask_strength-gemini-orderbook-1hr-10.00usd_slippage': 4893,
    'ask_strength-okcoin-orderbook-1hr-0.00usd_slippage': 4894,
    'ask_strength-okcoin-orderbook-1hr-1.00usd_slippage': 4895,
    'ask_strength-okcoin-orderbook-1hr-2.00usd_slippage': 4896,
    'ask_strength-okcoin-orderbook-1hr-4.00usd_slippage': 4897,
    'ask_strength-okcoin-orderbook-1hr-6.00usd_slippage': 4898,
    'ask_strength-okcoin-orderbook-1hr-8.00usd_slippage': 4899,
    'ask_strength-okcoin-orderbook-1hr-10.00usd_slippage': 4900,
    'bid_strength_usd-bitstamp-orderbook-1hr-0.00usd_slippage': 4901,
    'bid_strength_usd-bitstamp-orderbook-1hr-1.00usd_slippage': 4902,
    'bid_strength_usd-bitstamp-orderbook-1hr-2.00usd_slippage': 4903,
    'bid_strength_usd-bitstamp-orderbook-1hr-4.00usd_slippage': 4904,
    'bid_strength_usd-bitstamp-orderbook-1hr-6.00usd_slippage': 4905,
    'bid_strength_usd-bitstamp-orderbook-1hr-8.00usd_slippage': 4906,
    'bid_strength_usd-bitstamp-orderbook-1hr-10.00usd_slippage': 4907,
    'bid_strength_usd-bitfinex-orderbook-1hr-0.00usd_slippage': 4908,
    'bid_strength_usd-bitfinex-orderbook-1hr-1.00usd_slippage': 4909,
    'bid_strength_usd-bitfinex-orderbook-1hr-2.00usd_slippage': 4910,
    'bid_strength_usd-bitfinex-orderbook-1hr-4.00usd_slippage': 4911,
    'bid_strength_usd-bitfinex-orderbook-1hr-6.00usd_slippage': 4912,
    'bid_strength_usd-bitfinex-orderbook-1hr-8.00usd_slippage': 4913,
    'bid_strength_usd-bitfinex-orderbook-1hr-10.00usd_slippage': 4914,
    'bid_strength_usd-coinbase-orderbook-1hr-0.00usd_slippage': 4915,
    'bid_strength_usd-coinbase-orderbook-1hr-1.00usd_slippage': 4916,
    'bid_strength_usd-coinbase-orderbook-1hr-2.00usd_slippage': 4917,
    'bid_strength_usd-coinbase-orderbook-1hr-4.00usd_slippage': 4918,
    'bid_strength_usd-coinbase-orderbook-1hr-6.00usd_slippage': 4919,
    'bid_strength_usd-coinbase-orderbook-1hr-8.00usd_slippage': 4920,
    'bid_strength_usd-coinbase-orderbook-1hr-10.00usd_slippage': 4921,
    'bid_strength_usd-itbit-orderbook-1hr-0.00usd_slippage': 4922,
    'bid_strength_usd-itbit-orderbook-1hr-1.00usd_slippage': 4923,
    'bid_strength_usd-itbit-orderbook-1hr-2.00usd_slippage': 4924,
    'bid_strength_usd-itbit-orderbook-1hr-4.00usd_slippage': 4925,
    'bid_strength_usd-itbit-orderbook-1hr-6.00usd_slippage': 4926,
    'bid_strength_usd-itbit-orderbook-1hr-8.00usd_slippage': 4927,
    'bid_strength_usd-itbit-orderbook-1hr-10.00usd_slippage': 4928,
    'bid_strength_usd-kraken-orderbook-1hr-0.00usd_slippage': 4929,
    'bid_strength_usd-kraken-orderbook-1hr-1.00usd_slippage': 4930,
    'bid_strength_usd-kraken-orderbook-1hr-2.00usd_slippage': 4931,
    'bid_strength_usd-kraken-orderbook-1hr-4.00usd_slippage': 4932,
    'bid_strength_usd-kraken-orderbook-1hr-6.00usd_slippage': 4933,
    'bid_strength_usd-kraken-orderbook-1hr-8.00usd_slippage': 4934,
    'bid_strength_usd-kraken-orderbook-1hr-10.00usd_slippage': 4935,
    'bid_strength_usd-gemini-orderbook-1hr-0.00usd_slippage': 4936,
    'bid_strength_usd-gemini-orderbook-1hr-1.00usd_slippage': 4937,
    'bid_strength_usd-gemini-orderbook-1hr-2.00usd_slippage': 4938,
    'bid_strength_usd-gemini-orderbook-1hr-4.00usd_slippage': 4939,
    'bid_strength_usd-gemini-orderbook-1hr-6.00usd_slippage': 4940,
    'bid_strength_usd-gemini-orderbook-1hr-8.00usd_slippage': 4941,
    'bid_strength_usd-gemini-orderbook-1hr-10.00usd_slippage': 4942,
    'bid_strength_usd-okcoin-orderbook-1hr-0.00usd_slippage': 4943,
    'bid_strength_usd-okcoin-orderbook-1hr-1.00usd_slippage': 4944,
    'bid_strength_usd-okcoin-orderbook-1hr-2.00usd_slippage': 4945,
    'bid_strength_usd-okcoin-orderbook-1hr-4.00usd_slippage': 4946,
    'bid_strength_usd-okcoin-orderbook-1hr-6.00usd_slippage': 4947,
    'bid_strength_usd-okcoin-orderbook-1hr-8.00usd_slippage': 4948,
    'bid_strength_usd-okcoin-orderbook-1hr-10.00usd_slippage': 4949,
    'ask_strength_usd-bitstamp-orderbook-1hr-0.00usd_slippage': 4950,
    'ask_strength_usd-bitstamp-orderbook-1hr-1.00usd_slippage': 4951,
    'ask_strength_usd-bitstamp-orderbook-1hr-2.00usd_slippage': 4952,
    'ask_strength_usd-bitstamp-orderbook-1hr-4.00usd_slippage': 4953,
    'ask_strength_usd-bitstamp-orderbook-1hr-6.00usd_slippage': 4954,
    'ask_strength_usd-bitstamp-orderbook-1hr-8.00usd_slippage': 4955,
    'ask_strength_usd-bitstamp-orderbook-1hr-10.00usd_slippage': 4956,
    'ask_strength_usd-bitfinex-orderbook-1hr-0.00usd_slippage': 4957,
    'ask_strength_usd-bitfinex-orderbook-1hr-1.00usd_slippage': 4958,
    'ask_strength_usd-bitfinex-orderbook-1hr-2.00usd_slippage': 4959,
    'ask_strength_usd-bitfinex-orderbook-1hr-4.00usd_slippage': 4960,
    'ask_strength_usd-bitfinex-orderbook-1hr-6.00usd_slippage': 4961,
    'ask_strength_usd-bitfinex-orderbook-1hr-8.00usd_slippage': 4962,
    'ask_strength_usd-bitfinex-orderbook-1hr-10.00usd_slippage': 4963,
    'ask_strength_usd-coinbase-orderbook-1hr-0.00usd_slippage': 4964,
    'ask_strength_usd-coinbase-orderbook-1hr-1.00usd_slippage': 4965,
    'ask_strength_usd-coinbase-orderbook-1hr-2.00usd_slippage': 4966,
    'ask_strength_usd-coinbase-orderbook-1hr-4.00usd_slippage': 4967,
    'ask_strength_usd-coinbase-orderbook-1hr-6.00usd_slippage': 4968,
    'ask_strength_usd-coinbase-orderbook-1hr-8.00usd_slippage': 4969,
    'ask_strength_usd-coinbase-orderbook-1hr-10.00usd_slippage': 4970,
    'ask_strength_usd-itbit-orderbook-1hr-0.00usd_slippage': 4971,
    'ask_strength_usd-itbit-orderbook-1hr-1.00usd_slippage': 4972,
    'ask_strength_usd-itbit-orderbook-1hr-2.00usd_slippage': 4973,
    'ask_strength_usd-itbit-orderbook-1hr-4.00usd_slippage': 4974,
    'ask_strength_usd-itbit-orderbook-1hr-6.00usd_slippage': 4975,
    'ask_strength_usd-itbit-orderbook-1hr-8.00usd_slippage': 4976,
    'ask_strength_usd-itbit-orderbook-1hr-10.00usd_slippage': 4977,
    'ask_strength_usd-kraken-orderbook-1hr-0.00usd_slippage': 4978,
    'ask_strength_usd-kraken-orderbook-1hr-1.00usd_slippage': 4979,
    'ask_strength_usd-kraken-orderbook-1hr-2.00usd_slippage': 4980,
    'ask_strength_usd-kraken-orderbook-1hr-4.00usd_slippage': 4981,
    'ask_strength_usd-kraken-orderbook-1hr-6.00usd_slippage': 4982,
    'ask_strength_usd-kraken-orderbook-1hr-8.00usd_slippage': 4983,
    'ask_strength_usd-kraken-orderbook-1hr-10.00usd_slippage': 4984,
    'ask_strength_usd-gemini-orderbook-1hr-0.00usd_slippage': 4985,
    'ask_strength_usd-gemini-orderbook-1hr-1.00usd_slippage': 4986,
    'ask_strength_usd-gemini-orderbook-1hr-2.00usd_slippage': 4987,
    'ask_strength_usd-gemini-orderbook-1hr-4.00usd_slippage': 4988,
    'ask_strength_usd-gemini-orderbook-1hr-6.00usd_slippage': 4989,
    'ask_strength_usd-gemini-orderbook-1hr-8.00usd_slippage': 4990,
    'ask_strength_usd-gemini-orderbook-1hr-10.00usd_slippage': 4991,
    'ask_strength_usd-okcoin-orderbook-1hr-0.00usd_slippage': 4992,
    'ask_strength_usd-okcoin-orderbook-1hr-1.00usd_slippage': 4993,
    'ask_strength_usd-okcoin-orderbook-1hr-2.00usd_slippage': 4994,
    'ask_strength_usd-okcoin-orderbook-1hr-4.00usd_slippage': 4995,
    'ask_strength_usd-okcoin-orderbook-1hr-6.00usd_slippage': 4996,
    'ask_strength_usd-okcoin-orderbook-1hr-8.00usd_slippage': 4997,
    'ask_strength_usd-okcoin-orderbook-1hr-10.00usd_slippage': 4998,
    ### Arbitrage series'. Arbitrage revenue and arbitrage volume at the USD midpoint.
    'signed_arb_revenue_usd-bitfinex-bitstamp-orderbook-1m': 5401,
    'signed_arb_revenue_usd-bitfinex-bitstamp-orderbook-5m': 5402,
    'signed_arb_revenue_usd-bitfinex-bitstamp-orderbook-10m': 5403,
    'signed_arb_revenue_usd-bitfinex-coinbase-orderbook-1m': 5404,
    'signed_arb_revenue_usd-bitfinex-coinbase-orderbook-5m': 5405,
    'signed_arb_revenue_usd-bitfinex-coinbase-orderbook-10m': 5406,
    'signed_arb_revenue_usd-bitfinex-itbit-orderbook-1m': 5407,
    'signed_arb_revenue_usd-bitfinex-itbit-orderbook-5m': 5408,
    'signed_arb_revenue_usd-bitfinex-itbit-orderbook-10m': 5409,
    'signed_arb_revenue_usd-bitfinex-kraken-orderbook-1m': 5410,
    'signed_arb_revenue_usd-bitfinex-kraken-orderbook-5m': 5411,
    'signed_arb_revenue_usd-bitfinex-kraken-orderbook-10m': 5412,
    'signed_arb_revenue_usd-bitfinex-gemini-orderbook-1m': 5413,
    'signed_arb_revenue_usd-bitfinex-gemini-orderbook-5m': 5414,
    'signed_arb_revenue_usd-bitfinex-gemini-orderbook-10m': 5415,
    'signed_arb_revenue_usd-bitfinex-okcoin-orderbook-1m': 5416,
    'signed_arb_revenue_usd-bitfinex-okcoin-orderbook-5m': 5417,
    'signed_arb_revenue_usd-bitfinex-okcoin-orderbook-10m': 5418,
    'signed_arb_revenue_usd-bitstamp-bitfinex-orderbook-1m': 5419,
    'signed_arb_revenue_usd-bitstamp-bitfinex-orderbook-5m': 5420,
    'signed_arb_revenue_usd-bitstamp-bitfinex-orderbook-10m': 5421,
    'signed_arb_revenue_usd-bitstamp-coinbase-orderbook-1m': 5422,
    'signed_arb_revenue_usd-bitstamp-coinbase-orderbook-5m': 5423,
    'signed_arb_revenue_usd-bitstamp-coinbase-orderbook-10m': 5424,
    'signed_arb_revenue_usd-bitstamp-itbit-orderbook-1m': 5425,
    'signed_arb_revenue_usd-bitstamp-itbit-orderbook-5m': 5426,
    'signed_arb_revenue_usd-bitstamp-itbit-orderbook-10m': 5427,
    'signed_arb_revenue_usd-bitstamp-kraken-orderbook-1m': 5428,
    'signed_arb_revenue_usd-bitstamp-kraken-orderbook-5m': 5429,
    'signed_arb_revenue_usd-bitstamp-kraken-orderbook-10m': 5430,
    'signed_arb_revenue_usd-bitstamp-gemini-orderbook-1m': 5431,
    'signed_arb_revenue_usd-bitstamp-gemini-orderbook-5m': 5432,
    'signed_arb_revenue_usd-bitstamp-gemini-orderbook-10m': 5433,
    'signed_arb_revenue_usd-bitstamp-okcoin-orderbook-1m': 5434,
    'signed_arb_revenue_usd-bitstamp-okcoin-orderbook-5m': 5435,
    'signed_arb_revenue_usd-bitstamp-okcoin-orderbook-10m': 5436,
    'signed_arb_revenue_usd-coinbase-bitfinex-orderbook-1m': 5437,
    'signed_arb_revenue_usd-coinbase-bitfinex-orderbook-5m': 5438,
    'signed_arb_revenue_usd-coinbase-bitfinex-orderbook-10m': 5439,
    'signed_arb_revenue_usd-coinbase-bitstamp-orderbook-1m': 5440,
    'signed_arb_revenue_usd-coinbase-bitstamp-orderbook-5m': 5441,
    'signed_arb_revenue_usd-coinbase-bitstamp-orderbook-10m': 5442,
    'signed_arb_revenue_usd-coinbase-itbit-orderbook-1m': 5443,
    'signed_arb_revenue_usd-coinbase-itbit-orderbook-5m': 5444,
    'signed_arb_revenue_usd-coinbase-itbit-orderbook-10m': 5445,
    'signed_arb_revenue_usd-coinbase-kraken-orderbook-1m': 5446,
    'signed_arb_revenue_usd-coinbase-kraken-orderbook-5m': 5447,
    'signed_arb_revenue_usd-coinbase-kraken-orderbook-10m': 5448,
    'signed_arb_revenue_usd-coinbase-gemini-orderbook-1m': 5449,
    'signed_arb_revenue_usd-coinbase-gemini-orderbook-5m': 5450,
    'signed_arb_revenue_usd-coinbase-gemini-orderbook-10m': 5451,
    'signed_arb_revenue_usd-coinbase-okcoin-orderbook-1m': 5452,
    'signed_arb_revenue_usd-coinbase-okcoin-orderbook-5m': 5453,
    'signed_arb_revenue_usd-coinbase-okcoin-orderbook-10m': 5454,
    'signed_arb_revenue_usd-itbit-bitfinex-orderbook-1m': 5455,
    'signed_arb_revenue_usd-itbit-bitfinex-orderbook-5m': 5456,
    'signed_arb_revenue_usd-itbit-bitfinex-orderbook-10m': 5457,
    'signed_arb_revenue_usd-itbit-bitstamp-orderbook-1m': 5458,
    'signed_arb_revenue_usd-itbit-bitstamp-orderbook-5m': 5459,
    'signed_arb_revenue_usd-itbit-bitstamp-orderbook-10m': 5460,
    'signed_arb_revenue_usd-itbit-coinbase-orderbook-1m': 5461,
    'signed_arb_revenue_usd-itbit-coinbase-orderbook-5m': 5462,
    'signed_arb_revenue_usd-itbit-coinbase-orderbook-10m': 5463,
    'signed_arb_revenue_usd-itbit-kraken-orderbook-1m': 5464,
    'signed_arb_revenue_usd-itbit-kraken-orderbook-5m': 5465,
    'signed_arb_revenue_usd-itbit-kraken-orderbook-10m': 5466,
    'signed_arb_revenue_usd-itbit-gemini-orderbook-1m': 5467,
    'signed_arb_revenue_usd-itbit-gemini-orderbook-5m': 5468,
    'signed_arb_revenue_usd-itbit-gemini-orderbook-10m': 5469,
    'signed_arb_revenue_usd-itbit-okcoin-orderbook-1m': 5470,
    'signed_arb_revenue_usd-itbit-okcoin-orderbook-5m': 5471,
    'signed_arb_revenue_usd-itbit-okcoin-orderbook-10m': 5472,
    'signed_arb_revenue_usd-kraken-bitfinex-orderbook-1m': 5473,
    'signed_arb_revenue_usd-kraken-bitfinex-orderbook-5m': 5474,
    'signed_arb_revenue_usd-kraken-bitfinex-orderbook-10m': 5475,
    'signed_arb_revenue_usd-kraken-bitstamp-orderbook-1m': 5476,
    'signed_arb_revenue_usd-kraken-bitstamp-orderbook-5m': 5477,
    'signed_arb_revenue_usd-kraken-bitstamp-orderbook-10m': 5478,
    'signed_arb_revenue_usd-kraken-coinbase-orderbook-1m': 5479,
    'signed_arb_revenue_usd-kraken-coinbase-orderbook-5m': 5480,
    'signed_arb_revenue_usd-kraken-coinbase-orderbook-10m': 5481,
    'signed_arb_revenue_usd-kraken-itbit-orderbook-1m': 5482,
    'signed_arb_revenue_usd-kraken-itbit-orderbook-5m': 5483,
    'signed_arb_revenue_usd-kraken-itbit-orderbook-10m': 5484,
    'signed_arb_revenue_usd-kraken-gemini-orderbook-1m': 5485,
    'signed_arb_revenue_usd-kraken-gemini-orderbook-5m': 5486,
    'signed_arb_revenue_usd-kraken-gemini-orderbook-10m': 5487,
    'signed_arb_revenue_usd-kraken-okcoin-orderbook-1m': 5488,
    'signed_arb_revenue_usd-kraken-okcoin-orderbook-5m': 5489,
    'signed_arb_revenue_usd-kraken-okcoin-orderbook-10m': 5490,
    'signed_arb_revenue_usd-gemini-bitfinex-orderbook-1m': 5491,
    'signed_arb_revenue_usd-gemini-bitfinex-orderbook-5m': 5492,
    'signed_arb_revenue_usd-gemini-bitfinex-orderbook-10m': 5493,
    'signed_arb_revenue_usd-gemini-bitstamp-orderbook-1m': 5494,
    'signed_arb_revenue_usd-gemini-bitstamp-orderbook-5m': 5495,
    'signed_arb_revenue_usd-gemini-bitstamp-orderbook-10m': 5496,
    'signed_arb_revenue_usd-gemini-coinbase-orderbook-1m': 5497,
    'signed_arb_revenue_usd-gemini-coinbase-orderbook-5m': 5498,
    'signed_arb_revenue_usd-gemini-coinbase-orderbook-10m': 5499,
    'signed_arb_revenue_usd-gemini-itbit-orderbook-1m': 5500,
    'signed_arb_revenue_usd-gemini-itbit-orderbook-5m': 5501,
    'signed_arb_revenue_usd-gemini-itbit-orderbook-10m': 5502,
    'signed_arb_revenue_usd-gemini-kraken-orderbook-1m': 5503,
    'signed_arb_revenue_usd-gemini-kraken-orderbook-5m': 5504,
    'signed_arb_revenue_usd-gemini-kraken-orderbook-10m': 5505,
    'signed_arb_revenue_usd-gemini-okcoin-orderbook-1m': 5506,
    'signed_arb_revenue_usd-gemini-okcoin-orderbook-5m': 5507,
    'signed_arb_revenue_usd-gemini-okcoin-orderbook-10m': 5508,
    'signed_arb_revenue_usd-okcoin-bitfinex-orderbook-1m': 5509,
    'signed_arb_revenue_usd-okcoin-bitfinex-orderbook-5m': 5510,
    'signed_arb_revenue_usd-okcoin-bitfinex-orderbook-10m': 5511,
    'signed_arb_revenue_usd-okcoin-bitstamp-orderbook-1m': 5512,
    'signed_arb_revenue_usd-okcoin-bitstamp-orderbook-5m': 5513,
    'signed_arb_revenue_usd-okcoin-bitstamp-orderbook-10m': 5514,
    'signed_arb_revenue_usd-okcoin-coinbase-orderbook-1m': 5515,
    'signed_arb_revenue_usd-okcoin-coinbase-orderbook-5m': 5516,
    'signed_arb_revenue_usd-okcoin-coinbase-orderbook-10m': 5517,
    'signed_arb_revenue_usd-okcoin-itbit-orderbook-1m': 5518,
    'signed_arb_revenue_usd-okcoin-itbit-orderbook-5m': 5519,
    'signed_arb_revenue_usd-okcoin-itbit-orderbook-10m': 5520,
    'signed_arb_revenue_usd-okcoin-kraken-orderbook-1m': 5521,
    'signed_arb_revenue_usd-okcoin-kraken-orderbook-5m': 5522,
    'signed_arb_revenue_usd-okcoin-kraken-orderbook-10m': 5523,
    'signed_arb_revenue_usd-okcoin-gemini-orderbook-1m': 5524,
    'signed_arb_revenue_usd-okcoin-gemini-orderbook-5m': 5525,
    'signed_arb_revenue_usd-okcoin-gemini-orderbook-10m': 5526,
    'signed_arb_volume_usd-bitfinex-bitstamp-orderbook-1m': 5527,
    'signed_arb_volume_usd-bitfinex-bitstamp-orderbook-5m': 5528,
    'signed_arb_volume_usd-bitfinex-bitstamp-orderbook-10m': 5529,
    'signed_arb_volume_usd-bitfinex-coinbase-orderbook-1m': 5530,
    'signed_arb_volume_usd-bitfinex-coinbase-orderbook-5m': 5531,
    'signed_arb_volume_usd-bitfinex-coinbase-orderbook-10m': 5532,
    'signed_arb_volume_usd-bitfinex-itbit-orderbook-1m': 5533,
    'signed_arb_volume_usd-bitfinex-itbit-orderbook-5m': 5534,
    'signed_arb_volume_usd-bitfinex-itbit-orderbook-10m': 5535,
    'signed_arb_volume_usd-bitfinex-kraken-orderbook-1m': 5536,
    'signed_arb_volume_usd-bitfinex-kraken-orderbook-5m': 5537,
    'signed_arb_volume_usd-bitfinex-kraken-orderbook-10m': 5538,
    'signed_arb_volume_usd-bitfinex-gemini-orderbook-1m': 5539,
    'signed_arb_volume_usd-bitfinex-gemini-orderbook-5m': 5540,
    'signed_arb_volume_usd-bitfinex-gemini-orderbook-10m': 5541,
    'signed_arb_volume_usd-bitfinex-okcoin-orderbook-1m': 5542,
    'signed_arb_volume_usd-bitfinex-okcoin-orderbook-5m': 5543,
    'signed_arb_volume_usd-bitfinex-okcoin-orderbook-10m': 5544,
    'signed_arb_volume_usd-bitstamp-bitfinex-orderbook-1m': 5545,
    'signed_arb_volume_usd-bitstamp-bitfinex-orderbook-5m': 5546,
    'signed_arb_volume_usd-bitstamp-bitfinex-orderbook-10m': 5547,
    'signed_arb_volume_usd-bitstamp-coinbase-orderbook-1m': 5548,
    'signed_arb_volume_usd-bitstamp-coinbase-orderbook-5m': 5549,
    'signed_arb_volume_usd-bitstamp-coinbase-orderbook-10m': 5550,
    'signed_arb_volume_usd-bitstamp-itbit-orderbook-1m': 5551,
    'signed_arb_volume_usd-bitstamp-itbit-orderbook-5m': 5552,
    'signed_arb_volume_usd-bitstamp-itbit-orderbook-10m': 5553,
    'signed_arb_volume_usd-bitstamp-kraken-orderbook-1m': 5554,
    'signed_arb_volume_usd-bitstamp-kraken-orderbook-5m': 5555,
    'signed_arb_volume_usd-bitstamp-kraken-orderbook-10m': 5556,
    'signed_arb_volume_usd-bitstamp-gemini-orderbook-1m': 5557,
    'signed_arb_volume_usd-bitstamp-gemini-orderbook-5m': 5558,
    'signed_arb_volume_usd-bitstamp-gemini-orderbook-10m': 5559,
    'signed_arb_volume_usd-bitstamp-okcoin-orderbook-1m': 5560,
    'signed_arb_volume_usd-bitstamp-okcoin-orderbook-5m': 5561,
    'signed_arb_volume_usd-bitstamp-okcoin-orderbook-10m': 5562,
    'signed_arb_volume_usd-coinbase-bitfinex-orderbook-1m': 5563,
    'signed_arb_volume_usd-coinbase-bitfinex-orderbook-5m': 5564,
    'signed_arb_volume_usd-coinbase-bitfinex-orderbook-10m': 5565,
    'signed_arb_volume_usd-coinbase-bitstamp-orderbook-1m': 5566,
    'signed_arb_volume_usd-coinbase-bitstamp-orderbook-5m': 5567,
    'signed_arb_volume_usd-coinbase-bitstamp-orderbook-10m': 5568,
    'signed_arb_volume_usd-coinbase-itbit-orderbook-1m': 5569,
    'signed_arb_volume_usd-coinbase-itbit-orderbook-5m': 5570,
    'signed_arb_volume_usd-coinbase-itbit-orderbook-10m': 5571,
    'signed_arb_volume_usd-coinbase-kraken-orderbook-1m': 5572,
    'signed_arb_volume_usd-coinbase-kraken-orderbook-5m': 5573,
    'signed_arb_volume_usd-coinbase-kraken-orderbook-10m': 5574,
    'signed_arb_volume_usd-coinbase-gemini-orderbook-1m': 5575,
    'signed_arb_volume_usd-coinbase-gemini-orderbook-5m': 5576,
    'signed_arb_volume_usd-coinbase-gemini-orderbook-10m': 5577,
    'signed_arb_volume_usd-coinbase-okcoin-orderbook-1m': 5578,
    'signed_arb_volume_usd-coinbase-okcoin-orderbook-5m': 5579,
    'signed_arb_volume_usd-coinbase-okcoin-orderbook-10m': 5580,
    'signed_arb_volume_usd-itbit-bitfinex-orderbook-1m': 5581,
    'signed_arb_volume_usd-itbit-bitfinex-orderbook-5m': 5582,
    'signed_arb_volume_usd-itbit-bitfinex-orderbook-10m': 5583,
    'signed_arb_volume_usd-itbit-bitstamp-orderbook-1m': 5584,
    'signed_arb_volume_usd-itbit-bitstamp-orderbook-5m': 5585,
    'signed_arb_volume_usd-itbit-bitstamp-orderbook-10m': 5586,
    'signed_arb_volume_usd-itbit-coinbase-orderbook-1m': 5587,
    'signed_arb_volume_usd-itbit-coinbase-orderbook-5m': 5588,
    'signed_arb_volume_usd-itbit-coinbase-orderbook-10m': 5589,
    'signed_arb_volume_usd-itbit-kraken-orderbook-1m': 5590,
    'signed_arb_volume_usd-itbit-kraken-orderbook-5m': 5591,
    'signed_arb_volume_usd-itbit-kraken-orderbook-10m': 5592,
    'signed_arb_volume_usd-itbit-gemini-orderbook-1m': 5593,
    'signed_arb_volume_usd-itbit-gemini-orderbook-5m': 5594,
    'signed_arb_volume_usd-itbit-gemini-orderbook-10m': 5595,
    'signed_arb_volume_usd-itbit-okcoin-orderbook-1m': 5596,
    'signed_arb_volume_usd-itbit-okcoin-orderbook-5m': 5597,
    'signed_arb_volume_usd-itbit-okcoin-orderbook-10m': 5598,
    'signed_arb_volume_usd-kraken-bitfinex-orderbook-1m': 5599,
    'signed_arb_volume_usd-kraken-bitfinex-orderbook-5m': 5600,
    'signed_arb_volume_usd-kraken-bitfinex-orderbook-10m': 5601,
    'signed_arb_volume_usd-kraken-bitstamp-orderbook-1m': 5602,
    'signed_arb_volume_usd-kraken-bitstamp-orderbook-5m': 5603,
    'signed_arb_volume_usd-kraken-bitstamp-orderbook-10m': 5604,
    'signed_arb_volume_usd-kraken-coinbase-orderbook-1m': 5605,
    'signed_arb_volume_usd-kraken-coinbase-orderbook-5m': 5606,
    'signed_arb_volume_usd-kraken-coinbase-orderbook-10m': 5607,
    'signed_arb_volume_usd-kraken-itbit-orderbook-1m': 5608,
    'signed_arb_volume_usd-kraken-itbit-orderbook-5m': 5609,
    'signed_arb_volume_usd-kraken-itbit-orderbook-10m': 5610,
    'signed_arb_volume_usd-kraken-gemini-orderbook-1m': 5611,
    'signed_arb_volume_usd-kraken-gemini-orderbook-5m': 5612,
    'signed_arb_volume_usd-kraken-gemini-orderbook-10m': 5613,
    'signed_arb_volume_usd-kraken-okcoin-orderbook-1m': 5614,
    'signed_arb_volume_usd-kraken-okcoin-orderbook-5m': 5615,
    'signed_arb_volume_usd-kraken-okcoin-orderbook-10m': 5616,
    'signed_arb_volume_usd-gemini-bitfinex-orderbook-1m': 5617,
    'signed_arb_volume_usd-gemini-bitfinex-orderbook-5m': 5618,
    'signed_arb_volume_usd-gemini-bitfinex-orderbook-10m': 5619,
    'signed_arb_volume_usd-gemini-bitstamp-orderbook-1m': 5620,
    'signed_arb_volume_usd-gemini-bitstamp-orderbook-5m': 5621,
    'signed_arb_volume_usd-gemini-bitstamp-orderbook-10m': 5622,
    'signed_arb_volume_usd-gemini-coinbase-orderbook-1m': 5623,
    'signed_arb_volume_usd-gemini-coinbase-orderbook-5m': 5624,
    'signed_arb_volume_usd-gemini-coinbase-orderbook-10m': 5625,
    'signed_arb_volume_usd-gemini-itbit-orderbook-1m': 5626,
    'signed_arb_volume_usd-gemini-itbit-orderbook-5m': 5627,
    'signed_arb_volume_usd-gemini-itbit-orderbook-10m': 5628,
    'signed_arb_volume_usd-gemini-kraken-orderbook-1m': 5629,
    'signed_arb_volume_usd-gemini-kraken-orderbook-5m': 5630,
    'signed_arb_volume_usd-gemini-kraken-orderbook-10m': 5631,
    'signed_arb_volume_usd-gemini-okcoin-orderbook-1m': 5632,
    'signed_arb_volume_usd-gemini-okcoin-orderbook-5m': 5633,
    'signed_arb_volume_usd-gemini-okcoin-orderbook-10m': 5634,
    'signed_arb_volume_usd-okcoin-bitfinex-orderbook-1m': 5635,
    'signed_arb_volume_usd-okcoin-bitfinex-orderbook-5m': 5636,
    'signed_arb_volume_usd-okcoin-bitfinex-orderbook-10m': 5637,
    'signed_arb_volume_usd-okcoin-bitstamp-orderbook-1m': 5638,
    'signed_arb_volume_usd-okcoin-bitstamp-orderbook-5m': 5639,
    'signed_arb_volume_usd-okcoin-bitstamp-orderbook-10m': 5640,
    'signed_arb_volume_usd-okcoin-coinbase-orderbook-1m': 5641,
    'signed_arb_volume_usd-okcoin-coinbase-orderbook-5m': 5642,
    'signed_arb_volume_usd-okcoin-coinbase-orderbook-10m': 5643,
    'signed_arb_volume_usd-okcoin-itbit-orderbook-1m': 5644,
    'signed_arb_volume_usd-okcoin-itbit-orderbook-5m': 5645,
    'signed_arb_volume_usd-okcoin-itbit-orderbook-10m': 5646,
    'signed_arb_volume_usd-okcoin-kraken-orderbook-1m': 5647,
    'signed_arb_volume_usd-okcoin-kraken-orderbook-5m': 5648,
    'signed_arb_volume_usd-okcoin-kraken-orderbook-10m': 5649,
    'signed_arb_volume_usd-okcoin-gemini-orderbook-1m': 5650,
    'signed_arb_volume_usd-okcoin-gemini-orderbook-5m': 5651,
    'signed_arb_volume_usd-okcoin-gemini-orderbook-10m': 5652,
}

# Currently, next_series_number is 5652.

def test_metric_type_uniqueness():
    vals = metric_types_map.values()
    distinct_vals = list(distinct(vals))

    assert len(distinct_vals) == len(vals)


def test_metric_type_completeness():
    """
    Test that the value map is complete above 243, our arbitrary cutoff from Apr. 14th.
    """

    vals = metric_types_map.values()
    complete_min = 243
    complete_max = max(vals)
    complete_range = range(complete_min, complete_max + 1)

    # All values in the complete range are in vals.
    assert set(complete_range) - set(vals) == set()


def get_metric_type_int(metric_type):

    test_metric_type_uniqueness()

    if metric_type in metric_types_map.values():
        return metric_type
    elif metric_type in metric_types_map:
        return metric_types_map[metric_type]
    else:
        raise MetricTypeException('Metric type is not in the dictionary')

def generate_initial_series_name_list():
    count = 3

    series_types_orderbook_data = ['midpoint', 'bid', 'ask', 'spread']
    depths_on_orderbook_data = [
        'orderbook-1hr-0btc_depth',
        'orderbook-1hr-1btc_depth',
        'orderbook-1hr-20btc_depth'
    ]

    for st in series_types_orderbook_data:
        for exchange in all_exchanges:
            for depth in depths_on_orderbook_data:
                print "    '%s-%s-%s': %s," % (st, exchange, depth, count)
                count += 1

    series_types_on_trade_data = ['volume', 'vwap']
    info_on_trade_data = ['trades-1hr']

    for st in series_types_on_trade_data:
        for exchange in all_exchanges:
            for info in info_on_trade_data:
                print "    '%s-%s-%s': %s," % (st, exchange, info, count)
                count += 1
